/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * Author: Tudor Toma
 *
 **/

/****************************************************************************************************/

Prompt == FUNCTION FC_FIX_ASSIGN_STRING ==
CREATE OR REPLACE FUNCTION FC_FIX_ASSIGN_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , h_assign_string IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_start_D DATE;
   h_finish_D DATE;
   h_minutes PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR C_FACT IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, ACTUAL_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 1
         UNION ALL
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM PMO_TMP_TDOY
         ORDER BY 1;

      CURSOR C_FETC IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, CASE WHEN ACTUAL_MIN = 0 THEN SCHED_MIN ELSE ACTUAL_MIN END
         FROM TASK_DAY_OF_YEAR
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR BETWEEN h_start_D AND h_finish_D
         UNION ALL
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM PMO_TMP_TDOY
         ORDER BY 1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_ASSIGN_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_ASSIGN_STRING'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_ASSIGN_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || '''' || h_assign_string || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_ASSIGN_STRING', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_ASSIGN_STRING(48) : cplan_fun.sql(61) : ' || SYSTIMESTAMP);
   END IF; 

   ora_temp_sc_state := PMO_DB2M.SC_STATE;
   null_return := FC_INI_ASSIGN_STR(1);
   PMO_DB2M.SC_STATE := ora_temp_sc_state;
   h_assign_string := '';
   --trace_printf("**__1 h_bitflag: %d, h_start_DT: %s, h_finish_DT: %s\n", h_bitflag,h_start_DT,h_finish_DT);
   
   If h_bitflag = 1 Then
      OPEN C_FACT;
      
      LOOP
         FETCH C_FACT INTO h_julian_day, h_minutes;
         IF C_FACT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         ora_temp_sc_state := PMO_DB2M.SC_STATE;
         null_return := FC_ASSIGN_STRING(h_julian_day, h_minutes);
         PMO_DB2M.SC_STATE := ora_temp_sc_state;
      END LOOP;
      
      BEGIN
         CLOSE C_FACT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_ASSIGN_STRING', 83+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_ASSIGN_STRING(84) : cplan_fun.sql(97) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_start_D := SubStr(h_start_DT, 1, 10);
      h_finish_D := SubStr(h_finish_DT, 1, 10);
      
      OPEN C_FETC;
      
      LOOP
         FETCH C_FETC INTO h_julian_day, h_minutes;
         IF C_FETC%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      END LOOP;
      
      BEGIN
         CLOSE C_FETC ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_ASSIGN_STRING', 115+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_ASSIGN_STRING(116) : cplan_fun.sql(129) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_temp_sc_state := FC_GET_ASSIGN_STR(h_assign_string, 0);
   --trace_printf("%s\n", h_assign_string);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_ASSIGN_STRING', 125+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_ASSIGN_STRING(126) : cplan_fun.sql(139) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_ASSIGN_STRING;
ALTER FUNCTION FC_FIX_ASSIGN_STRING COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_DEL_NOACTUAL ==
CREATE OR REPLACE FUNCTION FC_DEL_NOACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_prj_assignment_id VARCHAR2(32);
   h_minutes PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CDNA IS
         SELECT DAY_YEAR
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;

      CURSOR CPA0 IS
         SELECT DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DEL_NOACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DEL_NOACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_DEL_NOACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_NOACTUAL', 36+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DEL_NOACTUAL(37) : cplan_fun.sql(186) : ' || SYSTIMESTAMP);
   END IF; 

   --short h_tsk_assignment_id_Ind = 0;
   --short h_project_id_Ind = 0;
   
   If h_bitflag = 0 Then
      OPEN CDNA;
      
      LOOP
         FETCH CDNA INTO h_dayofyear;
         IF CDNA%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_temp_var := 0;
         h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
      END LOOP;
      
      BEGIN
         CLOSE CDNA ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 69+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(70) : cplan_fun.sql(219) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_bitflag = 1 Then
      BEGIN
         DELETE FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 89+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(90) : cplan_fun.sql(239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_bitflag = 2 Then
      BEGIN
         SELECT PROJECT_ASSIGNMENT_ID INTO h_prj_assignment_id
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 104+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(105) : cplan_fun.sql(254) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- fix task assignments from affected to project
      
      OPEN CPA0;
      
      LOOP
         FETCH CPA0 INTO h_dayofyear, h_minutes, h_bitID;
         IF CPA0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, TASK_ID = h_project_id, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 137+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(138) : cplan_fun.sql(287) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_dayofyear: %d %s, h_minutes: %d\n",SC_STATE,h_dayofyear_Ind,h_dayofyear,h_minutes);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id, PARENT_ID = h_prj_assignment_id, TASK_ID = h_project_id
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 157+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(158) : cplan_fun.sql(307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 175+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(176) : cplan_fun.sql(325) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CPA0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(191) : cplan_fun.sql(340) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_NOACTUAL', 197+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_NOACTUAL(198) : cplan_fun.sql(347) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DEL_NOACTUAL;
ALTER FUNCTION FC_DEL_NOACTUAL COMPILE DEBUG;

--########################################################################

Prompt == FUNCTION FC_SRCH_DURATION ==
CREATE OR REPLACE FUNCTION FC_SRCH_DURATION
(
     sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_every PLS_INTEGER
   , sh_h_duration PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_unit PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_every PLS_INTEGER := sh_h_every;
   h_duration PLS_INTEGER := sh_h_duration;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_unit PLS_INTEGER := sh_h_unit;
   h_temp_sc_state PLS_INTEGER := 0;
   j PLS_INTEGER;
   i_week PLS_INTEGER;
   i PLS_INTEGER;
   ii PLS_INTEGER;
   i_days PLS_INTEGER;
   i_every PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SRCH_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SRCH_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_SRCH_DURATION(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_every), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SRCH_DURATION', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SRCH_DURATION(36) : cplan_fun.sql(393) : ' || SYSTIMESTAMP);
   END IF; 

   If (Coalesce(h_bitflag, 0) = 4) OR (Coalesce(h_bitflag, 0) > 6) Then
      PMO_DB2M.SC_STATE := 1200;
   Else
      i_week := 0;
      i := 0;
      ii := 0;
      i_days := 0;
      i_every := 0;
      
      If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) > 0 Then
         i := 1;
         ii := 7;
         i_days := 0;
         WHILE ii > 0 LOOP
            If (BITAND(Coalesce(h_contour, 0), i)) = i Then
               i_days := i_days + 1;
            End If;
            i := PMO_FUNC.ShiftLeft(i, 1);
            ii := ii - 1;
         END LOOP;
      Else
         i_days := 1;
      End If;
      
      i_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), 1920)), 7);
      If i_week > 0 Then
         i := 0;
         j := 0;
         WHILE j < 4 LOOP
            If (BITAND(i_week, 1)) <> 0 Then
               i := i + 1;
            End If;
            
            i_week := (PMO_FUNC.ShiftRight(i_week, 1));
            If i_week = 0 Then
               EXIT;
            End If;
            
            j := j + 1;
         END LOOP;
         
         i_week := i;
      End If;
      
      If Coalesce(h_every, 0) > 0 Then
         If Coalesce(h_occurrence, 0) = 0 Then
            h_duration := Coalesce(h_duration, 0) + 1;
         End If;
         
         --trace_printf("0_ h_bitflag = %d, h_duration = %d, h_every = %d, i_week = %d, i_days = %d\n", h_bitflag,h_duration,h_every,i_week,i_days);
         If Coalesce(h_bitflag, 0) = 1 Then
            h_duration := Coalesce(h_duration, 0) * Coalesce(h_every, 0);
         ElsIf Coalesce(h_bitflag, 0) = 2 Then
            h_duration := Coalesce(h_duration, 0) * (Trunc((Coalesce(h_every, 0) * 7) / i_days));
         ElsIf Coalesce(h_bitflag, 0) < 6 Then
            If (Coalesce(h_unit, 0) = 2) OR (Coalesce(h_occurrence, 0) > 0) Then
               h_duration := 31 * (Coalesce(h_duration, 0) + Coalesce(h_every, 0) - 1);
            ElsIf i_week = 0 Then
               h_duration := Trunc(Coalesce(h_duration, 0) * ((Coalesce(h_every, 0) * 31) / (4.5 * i_days)));
            Else
               h_duration := Coalesce(h_duration, 0) * (Trunc((Coalesce(h_every, 0) * 31) / (i_week * i_days)));
            End If;
         End If;
         --trace_printf("1_ h_duration = %d, h_every = %d, i_week = %d, i_days = %d, h_unit: %d\n", h_duration,h_every,i_week,i_days,h_unit);
      Else
         i_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_1FFE0000)), 17);
         -- July, August, Decembre
         
         If i_every > 0 Then
            i := 0;
            j := 0;
            WHILE j < 12 LOOP
               If (BITAND(i_every, 1)) <> 0 Then
                  i := i + 1;
               End If;
               
               i_every := (PMO_FUNC.ShiftRight(i_every, 1));
               
               If i_every = 0 Then
                  EXIT;
               End If;
               
               j := j + 1;
            END LOOP;
            
            i_every := 10 - j;
            
            If i > 0 Then
               If i_week = 0 Then
                  h_duration := Trunc(Coalesce(h_duration, 0) / (4.5 * i_days * i) + 1);
               Else
                  h_duration := Trunc(Coalesce(h_duration, 0) / (i_week * i_days * i)) + 1;
               End If;
            End If;
            h_duration := Coalesce(h_duration, 0) * 365;
            h_duration := Coalesce(h_duration, 0) - i_every * 30;
         End If;
         --trace_printf("2_ h_duration = %d, i_every = %d, i_week = %d, i_days = %d, i = %d\n", h_duration,i_every,i_week,i_days,i);
      End If;
      
      PMO_DB2M.SC_STATE := Coalesce(h_duration, 0);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SRCH_DURATION', 142+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SRCH_DURATION(143) : cplan_fun.sql(500) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SRCH_DURATION;
ALTER FUNCTION FC_SRCH_DURATION COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_GET_RES_RATES ==
CREATE OR REPLACE FUNCTION FC_GET_RES_RATES
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_and_roles IN OUT NOCOPY VARCHAR2
   , h_and_skills IN OUT NOCOPY VARCHAR2
   , data_flag IN OUT NOCOPY PLS_INTEGER
   , reg_cost IN OUT NOCOPY NUMBER
   , spec_cost IN OUT NOCOPY NUMBER
   , reg_sale IN OUT NOCOPY NUMBER
   , spec_sale IN OUT NOCOPY NUMBER
) RETURN PLS_INTEGER
AS
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   token VARCHAR2(32);
   numItems PLS_INTEGER;
   i PLS_INTEGER;
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_RES_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_RES_RATES'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_RES_RATES(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || '''' || h_and_roles || ''''
          || ', ' || '''' || h_and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(data_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_RES_RATES', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_RES_RATES(40) : cplan_fun.sql(550) : ' || SYSTIMESTAMP);
   END IF; 

   h_reg_cost := 0.00;
   h_reg_sale := 0.00;
   h_spec_cost := 0.00;
   h_spec_sale := 0.00;
   
   --trace_printf("\n h_and_roles-----> %s\n", h_and_roles);
   --trace_printf("\n h_and_skills-----> %s\n", h_and_skills);
   
   If (Coalesce(Length(h_and_roles), 0) >= 32 OR Coalesce(Length(h_and_skills), 0) >= 32) 
         AND (NOT SubStr(h_res_type, 1, 1) = '1') Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_RATE';
      PMO_DB2M.SC_STATE := 0;
      
      token := SubStr(token, 1, 32);
      
      numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
      
      stmt := '';
      
      stmt := 
            'INSERT INTO PMO_TMP_RES_RATE (RES_ID,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
             || ' SELECT ''' || h_res_id || ''',';
      stwhere := 
            ' CASE WHEN (R1.REGULAR_COST = 0 OR R1.REGULAR_COST IS NULL) AND (R.REGULAR_COST = 0 OR R.REGULAR_COST IS NULL) THEN 0 '
             || ' WHEN (R1.REGULAR_COST = 0 OR  R1.REGULAR_COST IS NULL) THEN R.REGULAR_COST '
             || ' WHEN (R.REGULAR_COST = 0  OR R.REGULAR_COST IS NULL) THEN R1.REGULAR_COST '
             || ' ELSE R1.REGULAR_COST END AS REGULAR_COST, '
             || ' CASE WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) AND (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL)  THEN 0 '
             || ' WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) THEN R.REGULAR_SALE '
             || ' WHEN (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL) THEN R1.REGULAR_SALE '
             || ' ELSE R1.REGULAR_SALE END AS REGULAR_SALE, '
             || ' CASE WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) AND (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL)  THEN 0 '
             || ' WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) THEN R.SPECIAL_COST '
             || ' WHEN (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL) THEN R1.SPECIAL_COST '
             || ' ELSE R1.SPECIAL_COST END AS SPECIAL_COST, '
             || ' CASE WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) AND (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL)  THEN 0 '
             || ' WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) THEN R.SPECIAL_SALE '
             || ' WHEN (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL) THEN R1.SPECIAL_SALE '
             || ' ELSE R1.SPECIAL_SALE END AS SPECIAL_SALE, '
             || ' t1.DEFAULT_FLAG FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
             || ' ( t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
             || '   AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         --          pmo_substr(token, h_and_roles, i*32, 32);
         token := SubStr(h_and_roles, i * 32 + 1, 32);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      
      --trace_printf("\n stmt roles------> %s", (char *) stmt);
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 119+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(120) : cplan_fun.sql(630) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      --Parse Skills list
      If Coalesce(Length(h_and_skills), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
      End If;
      
      If numItems > 0 Then
         stmt := '';
         stmt := 
               'INSERT INTO PMO_TMP_RES_RATE (RES_ID,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                || ' SELECT ''' || h_res_id || ''', ';
         
         stwhere := 
               ' case when (r1.REGULAR_COST is null or r1.REGULAR_COST=0) then COALESCE(r.REGULAR_COST,0) else r1.REGULAR_COST end AS REGULAR_COST, '
                || ' case when (r1.REGULAR_SALE is null or r1.REGULAR_SALE=0) then COALESCE(r.REGULAR_SALE,0) else r1.REGULAR_SALE end AS REGULAR_SALE, '
                || ' case when (r1.SPECIAL_COST is null or r1.SPECIAL_COST=0) then COALESCE(r.SPECIAL_COST,0) else r1.SPECIAL_COST end AS SPECIAL_COST, '
                || ' case when (r1.SPECIAL_SALE is null or r1.SPECIAL_SALE=0) then COALESCE(r.SPECIAL_SALE,0) else r1.SPECIAL_SALE end AS SPECIAL_SALE, ''0'' '
                || ' FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
         stwhere := stwhere || h_res_id;
         stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                || ' (t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK  '
                || '  AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                || ' AND t3.ELEMENT_ID IN (''';
         
         i := 0;
         WHILE i < numItems LOOP
            If i > 0 Then
               stwhere := stwhere || ''',''';
            End If;
            
            --              pmo_substr(token, h_and_skills, i*32, 32);
            token := SubStr(h_and_skills, i * 32 + 1, 32);
            
            stwhere := stwhere || token;
            i := i + 1;
         END LOOP;
         stwhere := stwhere || ''')';
         stmt := stmt || stwhere;
         
         --trace_printf("\n stmt skills------> %s", (char *) stmt);
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 179+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(180) : cplan_fun.sql(690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
      End If;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0006';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN_0006
         SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, 0
         FROM PMO_TMP_RES_RATE WHERE DEFAULT_FLAG = '1';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 204+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(205) : cplan_fun.sql(715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN_0006
         SELECT  RES_ID,                             MAX(REGULAR_COST) AS REGULAR_COST,
         MAX(REGULAR_SALE) AS REGULAR_SALE,  MAX(SPECIAL_COST) AS SPECIAL_COST,
         MAX(SPECIAL_SALE) AS SPECIAL_SALE,  0
         FROM PMO_TMP_RES_RATE
         WHERE RES_ID NOT IN (SELECT RES_ID
         FROM PMO_TMP_RES_RATE WHERE DEFAULT_FLAG = '1')
         GROUP BY RES_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 228+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(229) : cplan_fun.sql(739) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST
         WHEN a.REGULAR_COST>0 THEN a.REGULAR_COST
         else def.REGULAR_COST    END,2),
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.REGULAR_RATE
         WHEN a.REGULAR_SALE>0 THEN a.REGULAR_SALE
         else def.REGULAR_SALE   END,2),
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.SPECIAL_COST
         WHEN a.REGULAR_COST>0 THEN a.SPECIAL_COST
         else def.SPECIAL_COST  END,2),
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.SPECIAL_RATE
         WHEN a.REGULAR_SALE>0 THEN a.SPECIAL_SALE
         else def.SPECIAL_SALE  END,2)
         
         INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
         
         FROM PMO_TMP_CPLN_0006 def
         JOIN RESOURCES r ON def.RES_ID=r.RESOURCE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=def.RES_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 260+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(261) : cplan_fun.sql(771) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\n h_project_id ------> %s", h_project_id);
      
      --trace_printf("\n h_reg_cost ------> %f", h_reg_cost);
      --trace_printf("\n h_reg_sale ------> %f", h_reg_sale);
      --trace_printf("\n h_spec_cost------> %f", h_spec_cost);
      --trace_printf("\n h_spec_sale------> %f", h_spec_sale);
   Else
      -- project assignments level
      BEGIN
         SELECT REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
         INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
         FROM PROJECT_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 284+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(285) : cplan_fun.sql(795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("PA__ SC_STATE: %d, h_project_id: %s, h_res_id: %s\n", SC_STATE,h_project_id,h_res_id);
      --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
      --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
      --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
      --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_reg_cost, 0) + Coalesce(
            h_reg_sale, 0) > 0.0) Then
         data_flag := 1;
      Else
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT COALESCE(MAX(R.REGULAR_COST),0.00), COALESCE(MAX(R.REGULAR_SALE),0.00),
               COALESCE(MAX(R.SPECIAL_COST),0.00), COALESCE(MAX(R.SPECIAL_SALE),0.00)
               INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
               FROM (PROFILES P JOIN TMT_COMPETENCIES C ON P.PROJECT_ID = h_project_id AND P.ROLE_ID = C.ELEMENT_ID)
               JOIN RATES R ON R.RATE_ID = C.RATE_ID
               WHERE P.PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 313+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(314) : cplan_fun.sql(824) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("PROF_SC_STATE = %d ********************************\n",SC_STATE);
            --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
            --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
            --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
            --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
         Else
            -- get resource rate
            BEGIN
               SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
               INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
               FROM RESOURCES rs LEFT OUTER JOIN RATES r
               ON rs.RATE_ID=r.RATE_ID
               WHERE rs.RESOURCE_ID= h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 337+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(338) : cplan_fun.sql(848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("0_SC_STATE = %d ********************************\n",SC_STATE);
            --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
            --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
            --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
            --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
            
            If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                  h_spec_sale, 0)) = 0.00 Then
               -- get the default rate in resource level
               BEGIN
                  SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
                  INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TMT_COMPETENCIES t1
                  LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID
                  WHERE t1.RESOURCE_ID = h_res_id AND t1.DEFAULT_FLAG='1';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 363+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(364) : cplan_fun.sql(874) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("1_SC_STATE = %d ********************************\n",SC_STATE);
               --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
               --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
               --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
               --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
               
               If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                     h_spec_sale, 0)) = 0.00 Then
                  -- get the default value from parent level (copetency level)
                  BEGIN
                     SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
                     INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                     FROM TMT_COMPETENCIES t1
                     JOIN TMT_COMPETENCIES P ON P.ELEMENT_ID = T1.PARENT_ID
                     LEFT OUTER JOIN RATES r ON P.RATE_ID=r.RATE_ID
                     WHERE t1.RESOURCE_ID = h_res_id AND t1.DEFAULT_FLAG='1';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 390+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(391) : cplan_fun.sql(901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                        h_spec_sale, 0)) = 0.00 Then
                     BEGIN
                        SELECT COALESCE(MAX(r.REGULAR_COST),0.00), COALESCE(MAX(r.REGULAR_SALE),0.00), COALESCE(MAX(r.SPECIAL_COST),0.00), COALESCE(MAX(r.SPECIAL_SALE),0.00)
                        INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                        FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID
                        WHERE t1.RESOURCE_ID = h_res_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 408+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(409) : cplan_fun.sql(919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     --trace_printf("2_SC_STATE = %d ********************************\n",SC_STATE);
                     --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
                     --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
                     --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
                     --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
                     
                     If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                           h_spec_sale, 0)) = 0.00 Then
                        BEGIN
                           SELECT COALESCE(MAX(r.REGULAR_COST),0.00), COALESCE(MAX(r.REGULAR_SALE),0.00), COALESCE(MAX(r.SPECIAL_COST),0.00), COALESCE(MAX(r.SPECIAL_SALE),0.00)
                           INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                           FROM TMT_COMPETENCIES t1 JOIN TMT_COMPETENCIES t2
                           ON t2.RESOURCE_ID = h_res_id AND t1.ELEMENT_ID=t2.PARENT_ID LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 432+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(433) : cplan_fun.sql(943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        --trace_printf("3_SC_STATE = %d ********************************\n",SC_STATE);
                        --trace_printf("h_reg_cost = %d:%.4f\n",h_reg_cost_Ind,h_reg_cost);
                        --trace_printf("h_reg_sale = %d:%.4f\n",h_reg_sale_Ind,h_reg_sale);
                        --trace_printf("h_spec_cost = %d:%.4f\n",h_spec_cost_Ind,h_spec_cost);
                        --trace_printf("h_spec_sale = %d:%.4f\n",h_spec_sale_Ind,h_spec_sale);
                     End If;
                  End If;
                  
                  data_flag := 2;
               End If;
            End If;
         End If;
      End If;
   End If;
   
   reg_cost := Coalesce(h_reg_cost, 0);
   reg_sale := Coalesce(h_reg_sale, 0);
   spec_cost := Coalesce(h_spec_cost, 0);
   spec_sale := Coalesce(h_spec_sale, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_RES_RATES', 458+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_RES_RATES(459) : cplan_fun.sql(969) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_RES_RATES;
ALTER FUNCTION FC_GET_RES_RATES COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_CHECK_PA ==
CREATE OR REPLACE FUNCTION FC_CHECK_PA
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , sh_h_etc PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_etc PLS_INTEGER := sh_h_etc;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_total PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CHECK_PA') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CHECK_PA'), '-- ' || SYSTIMESTAMP, ':r := FC_CHECK_PA(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_etc), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CHECK_PA(27) : cplan_fun.sql(1006) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_finish_date: %d %s, h_etc: %d\n",h_finish_date_Ind,h_finish_date,h_etc); trace_flush();
   h_total := 0;
   h_start_DT := NULL;
   h_finish_DT := NULL;
   
   BEGIN
      SELECT COALESCE(SCHED_START,PLANNED_START), COALESCE(SCHED_FINISH,PLANNED_FINISH) 
      INTO h_start_DT, h_finish_DT
      FROM TASK_ASSIGNMENTS
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHECK_PA', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHECK_PA(46) : cplan_fun.sql(1025) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_finish_date IS NOT NULL) Then
      If (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) < 0) OR (PMO_ORASTD.strncmp(h_finish_DT
            , h_finish_date, 10) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 54+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(55) : cplan_fun.sql(1034) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
   ElsIf (h_start_date IS NOT NULL) Then
      If (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) < 0) OR ((PMO_ORASTD.strncmp(h_finish_DT
            , h_start_date, 10) < 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 62+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(63) : cplan_fun.sql(1042) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
      
      --if ((strncmp(h_start_date,h_start_DT,10) != 0)
      --{
      -- InitializeHost(SC_PARAM(h_start_date), h_start_DT);
      --}
   End If;
   
   BEGIN
      SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND (TSK_ASSIGNMENT_ID = h_prj_assignment_id OR TSK_ASSIGNMENT_ID = h_tsk_assignment_id);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHECK_PA', 83+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHECK_PA(84) : cplan_fun.sql(1063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_total, 0) < Coalesce(h_etc, 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 90+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(91) : cplan_fun.sql(1070) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 96+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(97) : cplan_fun.sql(1076) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CHECK_PA;
ALTER FUNCTION FC_CHECK_PA COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_DECLARE_TDY ==
CREATE OR REPLACE FUNCTION FC_DECLARE_TDY RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DECLARE_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DECLARE_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_DECLARE_TDY();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_TDY', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DECLARE_TDY(9) : cplan_fun.sql(1095) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_TDY', 20+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DECLARE_TDY(21) : cplan_fun.sql(1107) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DECLARE_TDY;
ALTER FUNCTION FC_DECLARE_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_TDY ==
CREATE OR REPLACE FUNCTION FC_CREATE_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_consider PLS_INTEGER
   , sh_h_cri PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_update PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_consider PLS_INTEGER := sh_h_consider;
   h_cri PLS_INTEGER := sh_h_cri;
   h_priority PLS_INTEGER := sh_h_priority;
   h_update PLS_INTEGER := sh_h_update;
   stmt VARCHAR2(7999);
   h_count PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_TDY(32) : cplan_fun.sql(1153) : ' || SYSTIMESTAMP);
   END IF; 

   --timeLog (file,"1__ FC_CREATE_TDY:");
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   --timeLog (file,"2__ FC_CREATE_TDY:");//trace_flush();
   
   --trace_printf("h_cri: %d, h_consider: %d, h_update: %d, h_res_id: %s\n",h_cri,h_consider,h_update,h_res_id);//trace_flush();
   If (Coalesce(h_consider, 0) = 0) OR (Coalesce(h_consider, 0) = 7) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY(48) : cplan_fun.sql(1169) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_cri, 0) >= PMO_CNST.SC_BIT_30 Then
      h_cri := (BITAND(Coalesce(h_cri, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove search / assign flag
   End If;
   --trace_printf("h_cri: %d, h_project_id: %s, h_start_date: %s\n",h_cri,h_project_id,h_start_date);//trace_flush();
   
   CASE h_consider
      WHEN 1 THEN
         -- Consider SCHED inside current project
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID=h_project_id AND RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 75+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(76) : cplan_fun.sql(1197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 2 THEN
         -- Consider SCHED outside current project
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID<>h_project_id AND RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 97+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(98) : cplan_fun.sql(1219) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 3 THEN
         -- Consider SCHED all projects
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 119+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(120) : cplan_fun.sql(1241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 4 THEN
         -- Consider PLANNED  SCHED inside current project
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.PROJECT_ID=''' || h_project_id || ''' AND t.RESOURCE_ID = ''' || h_res_id 
                        || ''' AND y.PROJECT_ID=''' || h_project_id || ''' AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR '
                   || 'WHERE PROJECT_ID=''' || h_project_id || ''' AND RESOURCE_ID=''' || h_res_id 
                        || ''' AND DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 154+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(155) : cplan_fun.sql(1276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 5 THEN
         -- Consider PLANNED  SCHED outside current project
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.PROJECT_ID<>''' || h_project_id || ''' AND t.RESOURCE_ID = ''' || h_res_id 
                        || ''' AND y.PROJECT_ID<>''' || h_project_id || ''' AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID<>''' 
                  || h_project_id || ''' AND '
                   || 'RESOURCE_ID=''' || h_res_id || ''' AND DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 191+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(192) : cplan_fun.sql(1313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 6 THEN
         -- Consider PLANNED  SCHED all projects
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.RESOURCE_ID = ''' || h_res_id || ''' AND y.RESOURCE_ID=''' || h_res_id 
                        || ''' AND t.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' 
                        || h_start_date || ''', 1, 10))';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR '
                   || 'WHERE RESOURCE_ID=''' || h_res_id || ''' AND DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 226+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(227) : cplan_fun.sql(1348) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 13 THEN
         -- Consider SCHED outside current project if priority >= then me
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
            FROM TASK_DAY_OF_YEAR y, TMT_WBS w
            WHERE w.ELEMENT_ID<>h_project_id AND w.PRIORITY>=h_priority AND y.PROJECT_ID=w.ELEMENT_ID AND
            y.RESOURCE_ID = h_res_id AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND y.BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 250+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(251) : cplan_fun.sql(1372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 14 THEN
         -- Consider SCHED outside current project if priority >= then me  SCHED inside
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
            FROM TASK_DAY_OF_YEAR y, TMT_WBS w
            WHERE w.PRIORITY>=h_priority AND y.PROJECT_ID=w.ELEMENT_ID AND
            y.RESOURCE_ID = h_res_id AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND y.BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 273+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(274) : cplan_fun.sql(1395) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 15 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TMT_WBS w, TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.RESOURCE_ID = ''' || h_res_id || ''' AND w.ELEMENT_ID<>''' || h_project_id 
                        || ''' AND w.PRIORITY>=' || TO_CHAR(Coalesce(h_priority, 0)) 
                        || ' AND y.PROJECT_ID=w.ELEMENT_ID AND y.RESOURCE_ID=''' || h_res_id 
                        || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) AND t.PROJECT_ID=w.ELEMENT_ID AND w.TYPE_ID IN (' 
                        || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ')';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED FROM TASK_DAY_OF_YEAR y, TMT_WBS w '
                   || 'WHERE y.RESOURCE_ID=''' || h_res_id || ''' AND y.DAY_YEAR >= TO_DATE(SubStr(''' 
                        || h_start_date || ''', 1, 10)) AND w.ELEMENT_ID<>''' || h_project_id 
                        || ''' AND w.PRIORITY>=' || TO_CHAR(Coalesce(h_priority, 0)) || ' AND '
                   || 'y.PROJECT_ID=w.ELEMENT_ID AND w.TYPE_ID IN (' || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ')';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 313+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(314) : cplan_fun.sql(1435) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 16 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me  PLANNED  SCHED inside
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TMT_WBS w, TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.RESOURCE_ID = ''' || h_res_id || ''' AND w.PRIORITY>=' || TO_CHAR(Coalesce(
                        h_priority, 0)) || ' AND y.PROJECT_ID=w.ELEMENT_ID AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) AND t.PROJECT_ID=w.ELEMENT_ID';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED FROM TASK_DAY_OF_YEAR y, TMT_WBS w '
                   || 'WHERE w.PRIORITY>=' || TO_CHAR(Coalesce(h_priority, 0)) 
                        || ' AND y.PROJECT_ID=w.ELEMENT_ID AND y.RESOURCE_ID=''' || h_res_id 
                        || ''' AND '
                   || 'y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 352+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(353) : cplan_fun.sql(1474) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 17 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me  SCHED inside current project
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TMT_WBS w JOIN TASK_ASSIGNMENTS t ON w.ELEMENT_ID<>''' || h_project_id 
                        || ''' AND w.PRIORITY>=' || TO_CHAR(Coalesce(h_priority, 0)) 
                        || ' AND t.PROJECT_ID=w.ELEMENT_ID AND t.TABLE_TYPE = 1 '
                   || 'JOIN TASK_DAY_OF_YEAR y ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID AND '
                   || 'y.RESOURCE_ID=''' || h_res_id || ''' AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) UNION ALL '
                   || 'SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED FROM TASK_DAY_OF_YEAR y '
                   || ' JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.PROJECT_ID=''' || h_project_id || ''' AND t.RESOURCE_ID=''' || h_res_id 
                        || ''' AND t.TABLE_TYPE = 1 AND t.BIT_FLAG=''2'' AND '
                   || 'y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y, TMT_WBS w '
                   || 'WHERE y.RESOURCE_ID=''' || h_res_id || ''' AND y.DAY_YEAR >= TO_DATE(SubStr(''' 
                        || h_start_date || ''', 1, 10)) AND w.PRIORITY>=' || TO_CHAR(Coalesce(
                        h_priority, 0)) || ' AND w.ELEMENT_ID<>''' || h_project_id 
                        || ''' AND y.PROJECT_ID=w.ELEMENT_ID '
                   || 'UNION ALL '
                   || 'SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID=''' 
                        || h_project_id || ''' AND RESOURCE_ID=''' || h_res_id || ''' AND '
                   || 'DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10)) AND BIT_FLAG=''2''';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 401+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(402) : cplan_fun.sql(1523) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 22 THEN
         -- Consider scheduled all projects    planned outer current project
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG = '2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 424+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(425) : cplan_fun.sql(1546) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.PROJECT_ID<>''' || h_project_id || ''' AND t.RESOURCE_ID = ''' || h_res_id 
                        || ''' AND y.PROJECT_ID<>''' || h_project_id || ''' AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR '
                   || 'WHERE PROJECT_ID<>''' || h_project_id || ''' AND RESOURCE_ID=''' || h_res_id 
                        || ''' AND DAY_YEAR = TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 456+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(457) : cplan_fun.sql(1578) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 23 THEN
         -- Consider PLANNED inside current project  SCHED all projects
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 479+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(480) : cplan_fun.sql(1601) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'JOIN RESOURCE_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.ELEMENT_TYPE<38 '
                   || 'WHERE t.PROJECT_ID=''' || h_project_id || ''' AND t.RESOURCE_ID = ''' || h_res_id 
                        || ''' AND y.PROJECT_ID=''' || h_project_id || ''' AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND '
                   || 't.TABLE_TYPE = 1 AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) AND y.BIT_FLAG = ''0''';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID=''' || h_project_id || ''' AND RESOURCE_ID=''' 
                        || h_res_id || ''' AND DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) AND BIT_FLAG = ''0''';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 514+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(515) : cplan_fun.sql(1636) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
      
      WHEN 10 THEN
         -- search from project assignments
         If Coalesce(h_update, 0) = 3 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT td.TSK_ASSIGNMENT_ID, td.DAY_YEAR, td.SCHED_MIN, td.BIT_FLAG, td.BIT_SCHED
               FROM TASK_ASSIGNMENTS ta JOIN TASK_DAY_OF_YEAR td ON td.PROJECT_ID = ta.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = td.TSK_ASSIGNMENT_ID
               WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID=h_project_id AND ta.RESOURCE_ID = h_res_id AND td.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND td.BIT_FLAG = '0';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 539+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(540) : cplan_fun.sql(1661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT td.TSK_ASSIGNMENT_ID, td.DAY_YEAR, td.SCHED_MIN, td.BIT_FLAG, td.BIT_SCHED
               FROM TASK_ASSIGNMENTS ta JOIN TASK_DAY_OF_YEAR td ON td.PROJECT_ID = ta.PROJECT_ID AND ta.TSK_ASSIGNMENT_ID = td.TSK_ASSIGNMENT_ID
               WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID=h_project_id AND ta.RESOURCE_ID = h_res_id AND td.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND td.BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 559+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(560) : cplan_fun.sql(1681) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 12 THEN
         -- profile
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID=h_project_id AND RESOURCE_ID = h_res_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 581+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(582) : cplan_fun.sql(1703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      ELSE
         -- Consider SCHED outside current project if priority >= then me  PLANNED  SCHED inside current project
         
         If Coalesce(h_cri, 0) = 1 Then
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TMT_WBS w, TASK_DAY_OF_YEAR y JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.RESOURCE_ID = ''' || h_res_id || ''' AND t.TABLE_TYPE = 1 AND y.RESOURCE_ID=''' 
                        || h_res_id || ''' AND y.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date 
                        || ''', 1, 10)) AND '
                   || 'w.PRIORITY>=' || TO_CHAR(Coalesce(h_priority, 0)) 
                        || ' AND t.PROJECT_ID=w.ELEMENT_ID AND y.PROJECT_ID=w.ELEMENT_ID AND y.BIT_FLAG=''2'' AND t.BIT_FLAG=''2'' '
                   || 'UNION ALL SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED FROM TASK_DAY_OF_YEAR y '
                   || 'JOIN TASK_ASSIGNMENTS t ON y.PROJECT_ID=t.PROJECT_ID AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID '
                   || 'WHERE t.PROJECT_ID=''' || h_project_id || ''' AND t.RESOURCE_ID=''' || h_res_id || ''' AND '
                   || 'DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10)) AND t.TABLE_TYPE = 1';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED '
                   || 'FROM TASK_DAY_OF_YEAR y, TMT_WBS w WHERE y.RESOURCE_ID=''' || h_res_id || ''' AND '
                   || 'y.DAY_YEAR = TO_DATE(SubStr(''' || h_start_date || ''', 1, 10)) AND w.PRIORITY>=' 
                        || TO_CHAR(Coalesce(h_priority, 0)) 
                        || ' AND y.PROJECT_ID=w.ELEMENT_ID AND y.BIT_FLAG=''2'' UNION ALL '
                   || 'SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID=''' 
                        || h_project_id || ''' AND RESOURCE_ID=''' || h_res_id || ''' AND '
                   || 'DAY_YEAR >= TO_DATE(SubStr(''' || h_start_date || ''', 1, 10))';
         End If;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 627+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(628) : cplan_fun.sql(1749) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
         
   END CASE;
   --trace_printf("%s\n", (char *) stmt);//trace_flush();
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 638+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY(639) : cplan_fun.sql(1760) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_TDY;
ALTER FUNCTION FC_CREATE_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR for Using LEVELING information from LEVEL_TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_TDY_LEVEL ==
CREATE OR REPLACE FUNCTION FC_CREATE_TDY_LEVEL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_consider PLS_INTEGER
   , sh_h_cri PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_update PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_consider PLS_INTEGER := sh_h_consider;
   h_cri PLS_INTEGER := sh_h_cri;
   h_priority PLS_INTEGER := sh_h_priority;
   h_update PLS_INTEGER := sh_h_update;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_count PLS_INTEGER := 0;
   h_priority_min PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_min_start_date TIMESTAMP;
   h_tsk_assignment_id VARCHAR2(32);
   h_dayofyear DATE;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR L_MINDT IS
         SELECT * FROM (SELECT MIN(t.LEVEL_START) as minStart
         FROM TASK_ASSIGNMENTS a JOIN Lev_tmt_wbs t ON t.PROJECT_ID = a.PROJECT_ID AND t.ELEMENT_ID = a.TASK_ID
         WHERE a.RESOURCE_ID = h_res_id AND t.LEVEL_START is not Null
         UNION ALL
         SELECT MIN(BAK_START_DT) as minStart FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = TASK_ID and RESOURCE_ID = h_res_id AND PROJECT_ID IN (
         SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00)
         ORDER BY minStart ) WHERE ROWNUM<=1;

      CURSOR CL_ACT IS
         SELECT 
         TSK_ASSIGNMENT_ID, DAY_YEAR, ACTUAL_MIN, BIT_SCHED 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND ACTUAL_FLAG = 1 
         AND SCHED_MIN <> ACTUAL_MIN;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_TDY_LEVEL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_TDY_LEVEL'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_TDY_LEVEL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY_LEVEL', 57+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_TDY_LEVEL(58) : cplan_fun.sql(1832) : ' || SYSTIMESTAMP);
   END IF; 

   h_min_start_date := NULL;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   --trace_printf("***FC_CREATE_TDY_LEVEL h_update: %d\n",h_update);
   If (BITAND(Coalesce(h_update, 0), 256)) <> 256 Then
      /* Check if leveling Assignments or just Calculate[MSP-F9]
      		 if Calculate [MSP-F9] skip this code and don't Populate the table. */
      
      BEGIN
         SELECT COUNT(RESOURCE_ID) INTO h_count
         FROM PMO_TMP_TDY_LEVEL
         WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(86) : cplan_fun.sql(1860) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("\n\n ==> In.FC_CREATE_TDY, h_count: %d, Proj_ID:%s; ResID:%s; SD:%s; Consider:%d\n",h_count,h_project_id,h_res_id,h_start_date,h_consider);
      
      --  stmt.Clear();
      If Coalesce(h_count, 0) = 0 Then
         OPEN L_MINDT;
         
         FETCH L_MINDT INTO h_min_start_date;
         IF L_MINDT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("2.i ==> SC_STATE: %d, h_min_start_date: %s\n", SC_STATE,h_min_start_date);
         
         BEGIN
            CLOSE L_MINDT ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 111+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(112) : cplan_fun.sql(1886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         /*BEGIN_SQL
         					SELECT MIN(t.LEVEL_START) INTO HOST(h_min_start_date)
         					FROM TASK_ASSIGNMENTS a JOIN TMT_WBS t ON t.ELEMENT_ID = a.TASK_ID
         					WHERE a.RESOURCE_ID = HOST(h_res_id) and t.LEVEL_START is not Null;
         				END_SQL_NOCHECK;*/
         
         /* Loading the following Records:
         						 Actual  Priority     Sched   Planned
         						----------------------------------------
         							x                   x               Outside project from the projList
         																Project in the ProjLiat
         																Current Project
         
         				*/
         BEGIN
            SELECT MIN(PRIORITY) INTO h_priority_min
            FROM PMO_TMP_PROJECTS_00;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 139+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(140) : cplan_fun.sql(1914) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_TDY_LEVEL (
            PROJECT_ID,
            RESOURCE_ID,
            PRIORITY,
            ACTUAL_FLAG,
            TSK_ASSIGNMENT_ID,
            DAY_YEAR,
            SCHED_MIN,
            BIT_FLAG,
            BIT_SCHED,
            PROJ_ASSIGN_FLAG)
            SELECT
            y.PROJECT_ID,
            y.RESOURCE_ID,
            w.PRIORITY,
            y.ACTUAL_FLAG,
            y.TSK_ASSIGNMENT_ID,
            y.DAY_YEAR,
            y.SCHED_MIN,
            y.BIT_FLAG,
            y.BIT_SCHED,
            case when (t.project_assignment_id is not NuLL) AND (t.PROJECT_ID = t.TASK_ID) then 1 else 0 end as PROJ_ASSIGN_FLAG
            FROM TASK_ASSIGNMENTS t, TMT_WBS w, TASK_DAY_OF_YEAR y
            WHERE   0=0
            AND NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE t.PROJECT_ID=ELEMENT_ID)
            AND t.RESOURCE_ID = h_res_id
            AND t.TABLE_TYPE = 1
            AND t.PROJECT_ID=w.ELEMENT_ID
            AND w.PRIORITY >= h_priority_min
            AND y.PROJECT_ID=t.PROJECT_ID
            AND y.TSK_ASSIGNMENT_ID=t.TSK_ASSIGNMENT_ID
            AND y.DAY_YEAR >= TO_DATE(SubStr(h_min_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 187+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(188) : cplan_fun.sql(1962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2.1 ==> SC_STATE: %d, h_min_start_date: %s, h_priority: %d\n", SC_STATE,h_min_start_date,h_priority);
      End If;
      
      -- PMO_TMP_TDY_LEVEL initially
      BEGIN
         INSERT INTO PMO_TMP_TDY
         SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
         FROM  PMO_TMP_TDY_LEVEL
         WHERE RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND (
         (PROJECT_ID = h_project_id AND PROJ_ASSIGN_FLAG = 0)
         OR (PRIORITY >= h_priority
         AND NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE PROJECT_ID=ELEMENT_ID))
         OR (PROJECT_ID <> h_project_id
         AND EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE PROJECT_ID=ELEMENT_ID))
         )
         UNION ALL
         SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
         FROM  TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND ACTUAL_FLAG = 1 
         AND SCHED_MIN = ACTUAL_MIN;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 226+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(227) : cplan_fun.sql(2001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2.2 ==> SC_STATE: %d, h_start_date: %s, h_priority: %d\n", SC_STATE,h_start_date,h_priority);
      
      OPEN CL_ACT;
      
      LOOP
         FETCH CL_ACT INTO h_tsk_assignment_id, h_dayofyear, h_available, h_bitID1;
         IF CL_ACT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If Coalesce(h_available, 0) > 0 Then
            PMO_STPU.clearFinishDayStruct(Coalesce(h_available, 0), h_bitID1);
            BEGIN
               INSERT INTO PMO_TMP_TDY (TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
               VALUES(h_tsk_assignment_id, h_dayofyear, h_available, h_bitID1, '2');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 260+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(261) : cplan_fun.sql(2035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CL_ACT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 275+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(276) : cplan_fun.sql(2050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY_LEVEL', 283+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY_LEVEL(284) : cplan_fun.sql(2058) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_TDY_LEVEL;
ALTER FUNCTION FC_CREATE_TDY_LEVEL COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_DECLARE_PMO_TMP_SEARCH ==
CREATE OR REPLACE FUNCTION FC_DECLARE_PMO_TMP_SEARCH RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DECLARE_PMO_TMP_SEARCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DECLARE_PMO_TMP_SEARCH'), '-- ' || SYSTIMESTAMP, ':r := FC_DECLARE_PMO_TMP_SEARCH();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_PMO_TMP_SEARCH', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DECLARE_PMO_TMP_SEARCH(9) : cplan_fun.sql(2081) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SEARCH';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_PMO_TMP_SEARCH', 16+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DECLARE_PMO_TMP_SEARCH(17) : cplan_fun.sql(2089) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DECLARE_PMO_TMP_SEARCH;
ALTER FUNCTION FC_DECLARE_PMO_TMP_SEARCH COMPILE DEBUG;

/***************************************************************************************************
SC_CHAR(h_element_id, SC_H_ID);

Modification needed in other .cpps:
VIEWS
	CALENDAR_NAME ==>
	SELECT ELEMENT_NAME
	FROM TMT_CALENDARS T1 LEFT OUTER JOIN TMT_CALENDARS T2 ON T1.ELEMENT_ID = T2.PARENT_ID
	WHERE T2.ELEMENT_ID = HOST(h_id) ;
RESOURCE
	CALENDAR_NAME ==>
	SELECT ELEMENT_NAME
	FROM TMT_CALENDARS T1 LEFT OUTER JOIN TMT_CALENDARS T2 ON T1.ELEMENT_ID = T2.PARENT_ID
	WHERE T2.ELEMENT_ID = HOST(h_resource_id) ;

***************************************************************************************************/

Prompt == FUNCTION POPULATE_TMP_CALENDAR ==
CREATE OR REPLACE FUNCTION POPULATE_TMP_CALENDAR
(
     element_id001 IN OUT NOCOPY VARCHAR2
   , calendar_id001 IN OUT NOCOPY VARCHAR2
   , SD001 IN OUT NOCOPY DATE
   , FD001 IN OUT NOCOPY DATE
   , sh_duration001 PLS_INTEGER
) RETURN PLS_INTEGER
AS
   duration001 PLS_INTEGER := sh_duration001;
   h_current_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_friday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_monday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_saturday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_sunday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_thursday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_tuesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_wednesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_FD DATE;
   h_SD DATE;
   h_calendar_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_exception_date DATE;
   h_dayofyear DATE;
   h_rank VARCHAR2(200);
   h_count PLS_INTEGER := 0;
   h_current_daynumber PLS_INTEGER := 0;
   h_current_min PLS_INTEGER := 0;
   h_current_weekday PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_friday_min PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_monday_min PLS_INTEGER := 0;
   h_saturday_min PLS_INTEGER := 0;
   h_sunday_min PLS_INTEGER := 0;
   h_thursday_min PLS_INTEGER := 0;
   h_tuesday_min PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_wednesday_min PLS_INTEGER := 0;
   h_week_iso PLS_INTEGER := 0;
   h_month_iso PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_start_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_end_year PLS_INTEGER := 0;
   h_end_month PLS_INTEGER := 0;
   iSF PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CSD1 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_element_id AND DAY_YEAR BETWEEN h_dayofyear AND  h_SD
         ) WHERE ROWNUM<=1;

      CURSOR C_EXCEPTS IS
         SELECT EXCEPTION_DATE, EXCEPTION_MIN, EXCEPTION_BIT
         FROM CALENDAR_EXCEPTIONS
         WHERE CALENDAR_ID IN
         (
         SELECT ELEMENT_ID FROM TMT_CALENDARS
         WHERE RANK LIKE SUBSTR(h_rank, 1, 8) || '%'
         AND SUBSTR( h_rank, 1, LENGTH(RANK) ) = RANK
         )
         AND EXCEPTION_DATE BETWEEN TO_DATE(SubStr(TO_DATE(SubStr(h_SD, 1, 10)), 1, 10)) AND TO_DATE(SubStr(TO_DATE(SubStr(h_FD, 1, 10)), 1, 10));

      CURSOR C_DUPLIC IS
         SELECT ELEMENT_ID, DAY_YEAR, COUNT(ELEMENT_ID) FROM PMO_TMP_CDY
         GROUP BY ELEMENT_ID, DAY_YEAR HAVING COUNT(ELEMENT_ID) > 1;

      CURSOR C_DUP1 IS
         SELECT * FROM (SELECT AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
         FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = h_dayofyear
         ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('POPULATE_TMP_CALENDAR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('POPULATE_TMP_CALENDAR'), '-- ' || SYSTIMESTAMP, ':r := POPULATE_TMP_CALENDAR(' || 
       '''' || element_id001 || ''''
          || ', ' || '''' || calendar_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || SD001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || FD001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_duration001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 84+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: POPULATE_TMP_CALENDAR(85) : cplan_fun.sql(2199) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --  SC_LONG(h_bitflag);
   
   h_element_id := element_id001;
   h_calendar_id := calendar_id001;
   h_SD := SD001;
   h_FD := FD001;
   h_duration := duration001;
   
   --trace_printf("\nSTART_ h_SD = %d %s, h_FD = %d %s, h_duration = %d\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD,h_duration);
   
   -- iSF = 0 fordward, iSF = 1 backward
   If Coalesce(h_duration, 0) >= PMO_CNST.SC_BIT_30 Then
      iSF := 1;
      h_duration := (BITAND(Coalesce(h_duration, 0), PMO_CNST.SC_BIT_NOT_30));
      --      InitializeHost(SC_PARAM(h_bitflag), 1);
   Else
      iSF := 0;
      --      InitializeHost(SC_PARAM(h_bitflag), 0);
   End If;
   
   If ((h_SD IS NULL)) OR (Coalesce(Length(h_SD), 0) = 0) Then
      BEGIN
         SELECT TO_DATE(SYSDATE) INTO h_SD FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 117+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(118) : cplan_fun.sql(2232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_start_year := 0;
      
      -- This macro exits with an error if fiscal calendar is too short
      BEGIN
         BEGIN
            SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_start_year, h_start_month 
                    FROM FISCAL_CALENDAR WHERE h_SD BETWEEN START_DATE AND FINISH_DATE;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 135+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(136) : cplan_fun.sql(2250) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 141+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(142) : cplan_fun.sql(2256) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 144+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(145) : cplan_fun.sql(2259) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_SD, False);
         End If;
      END;
      
      BEGIN
         SELECT START_DATE INTO h_SD FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_start_year AND FISCAL_MONTH = 1;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 158+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(159) : cplan_fun.sql(2273) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_start_year := 0;
      
      -- This macro exits with an error if fiscal calendar is too short
      BEGIN
         BEGIN
            SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_start_year, h_start_month 
                    FROM FISCAL_CALENDAR WHERE h_SD BETWEEN START_DATE AND FINISH_DATE;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 176+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(177) : cplan_fun.sql(2291) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 182+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(183) : cplan_fun.sql(2297) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 185+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(186) : cplan_fun.sql(2300) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_SD, False);
         End If;
      END;
      
      BEGIN
         SELECT START_DATE INTO h_dayofyear FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_start_year AND FISCAL_MONTH = 1;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 199+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(200) : cplan_fun.sql(2314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (PMO_ORASTD.strcmp(h_dayofyear, h_SD) < 0) Then
         OPEN CSD1;
         
         FETCH CSD1 INTO h_dayofyear;
         IF CSD1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            h_SD := h_dayofyear;
         End If;
         
         BEGIN
            CLOSE CSD1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 226+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(227) : cplan_fun.sql(2341) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If ((h_FD IS NULL)) OR (Coalesce(Length(h_FD), 0) = 0) Then
      BEGIN
         SELECT TO_DATE(SubStr(h_SD, 1, 10)) + (h_duration + 400)      INTO h_FD FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 242+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(243) : cplan_fun.sql(2357) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.YEAR(TO_DATE(SubStr(h_FD, 1, 10))) INTO h_count FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 255+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(256) : cplan_fun.sql(2370) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_EXTEND_FISCAL_CALENDAR(h_count, NULL);
      
      h_FD := '2100-01-01';
   ElsIf iSF = 0 Then
      h_end_year := 0;
      
      -- This macro exits with an error if fiscal calendar is too short
      BEGIN
         BEGIN
            SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_end_year, h_end_month 
                    FROM FISCAL_CALENDAR WHERE h_FD BETWEEN START_DATE AND FINISH_DATE;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 277+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(278) : cplan_fun.sql(2392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 283+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(284) : cplan_fun.sql(2398) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 286+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(287) : cplan_fun.sql(2401) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_FD, False);
         End If;
      END;
      
      If Coalesce(h_end_year, 0) <> 0 Then
         BEGIN
            SELECT FINISH_DATE INTO h_FD FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_end_year AND FISCAL_MONTH = 12;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 301+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(302) : cplan_fun.sql(2416) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         SELECT SETTING_VALUE INTO h_count FROM TMT_SETTINGS WHERE ELEMENT_ID = 'CPLAN_PROJECT_CALENDAR';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 315+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(316) : cplan_fun.sql(2430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_count IS NULL) Then
         h_count := 365;
      End If;
      
      If Coalesce(h_duration, 0) < Coalesce(h_count, 0) Then
         h_duration := h_count;
      End If;
   End If;
   --trace_printf("*** iSF: %d, h_SD = %s, h_FD = %s, h_duration = %d, h_calendar_id = %s\n", iSF,h_SD,h_FD,h_duration,h_calendar_id);
   
   If Coalesce(Length(h_calendar_id), 0) = 0 Then
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 347+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(348) : cplan_fun.sql(2462) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID = h_calendar_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 368+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(369) : cplan_fun.sql(2483) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT
            RTRIM(RANK),
            MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
            MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
            INTO
            h_rank,
            h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
            h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
            FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 390+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(391) : cplan_fun.sql(2505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   --trace_printf("SC_STATE = %d, h_monday_min = %d\n", SC_STATE, h_monday_min);
   
   -- Insert records day by day
   -- ********************************************************************
   -- IMPORTANT SETTING, PMOv4 SETS SUNDAY AS THE FIRST DAY OF THE WEEK
   -- ********************************************************************
   
   -- Get start_date (weekday value)
   BEGIN
      SELECT PMO_ORASTD.DAYOFWEEK(TO_DATE(SubStr(h_SD, 1, 10))) INTO h_current_weekday FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 412+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(413) : cplan_fun.sql(2527) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_FD = %d;%s, h_duration = %d\n", SC_STATE, h_FD_Ind, h_FD, h_duration);
   
   -- Get total_day_count
   If (Coalesce(h_duration, 0) = 0) AND (iSF = 1) Then
      -- Assuming the start_date and finish_date are in good format
      BEGIN
         SELECT PMO_ORASTD.DAYS(TO_DATE(SubStr(h_FD, 1, 10))) - PMO_ORASTD.DAYS(TO_DATE(SubStr(h_SD, 1, 10))) INTO h_duration FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 429+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(430) : cplan_fun.sql(2544) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1___________*\n");
      
      If Coalesce(h_duration, 0) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 437+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(438) : cplan_fun.sql(2552) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
      End If;
   End If;
   
   h_current_daynumber := 0;
   h_count := 0;
   
   h_dayofyear := h_SD;
   --trace_printf("*** h_duration = %d, h_count = %d, h_current_weekday = %d\n", h_duration, h_count, h_current_weekday);
   
   LOOP
      CASE h_current_weekday
         WHEN 1 THEN
            h_current_bit := h_sunday_bit;
            h_current_min := Coalesce(h_sunday_min, 0);
         
         WHEN 2 THEN
            h_current_bit := h_monday_bit;
            h_current_min := Coalesce(h_monday_min, 0);
         
         WHEN 3 THEN
            h_current_bit := h_tuesday_bit;
            h_current_min := Coalesce(h_tuesday_min, 0);
         
         WHEN 4 THEN
            h_current_bit := h_wednesday_bit;
            h_current_min := Coalesce(h_wednesday_min, 0);
         
         WHEN 5 THEN
            h_current_bit := h_thursday_bit;
            h_current_min := Coalesce(h_thursday_min, 0);
         
         WHEN 6 THEN
            h_current_bit := h_friday_bit;
            h_current_min := Coalesce(h_friday_min, 0);
         
         WHEN 7 THEN
            h_current_bit := h_saturday_bit;
            h_current_min := Coalesce(h_saturday_min, 0);
         ELSE
            NULL;
      END CASE;
      
      --trace_printf("*** h_current_min = %d\n", h_current_min);
      -- insert into DayOfYear Table
      BEGIN
         SELECT  TO_DATE(SubStr(h_SD, 1, 10)) + h_current_daynumber      INTO h_dayofyear FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 492+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(493) : cplan_fun.sql(2607) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("**_ h_count = %d, h_current_daynumber: %d, h_FD: %s, h_dayofyear: %s\n",h_count,h_current_daynumber,h_FD,h_dayofyear);
      If PMO_ORASTD.strncmp(h_FD, h_dayofyear, 10) < 0 Then
         EXIT;
      End If;
      
      If Coalesce(h_current_min, 0) > 0 Then
         BEGIN
            SELECT PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear)  INTO h_week_iso, h_month_iso
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 512+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(513) : cplan_fun.sql(2627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY( ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 1, PMO_ORASTD.YEAR(h_dayofyear)+1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 531+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(532) : cplan_fun.sql(2646) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 12, PMO_ORASTD.YEAR(h_dayofyear)-1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 549+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(550) : cplan_fun.sql(2664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear), PMO_ORASTD.YEAR(h_dayofyear));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 567+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(568) : cplan_fun.sql(2682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            NULL;
         End If;
         
         If (PMO_ORASTD.strncmp(h_FD, '2100-01-01', 10) = 0) AND (Coalesce(h_count, 0) >= Coalesce(
               h_duration, 0)) Then
            --trace_printf("1_ h_FD: %d:%s, h_count: %d, h_duration: %d\n",h_FD_Ind,h_FD,h_count,h_duration);
            -- This macro exits with an error if fiscal calendar is too short
            BEGIN
               BEGIN
                  SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_end_year, h_end_month 
                          FROM FISCAL_CALENDAR WHERE h_dayofyear BETWEEN START_DATE AND FINISH_DATE;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 588+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(589) : cplan_fun.sql(2703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 594+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(595) : cplan_fun.sql(2709) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 597+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(598) : cplan_fun.sql(2712) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_dayofyear
                        , False);
               End If;
            END;
            
            BEGIN
               SELECT FINISH_DATE INTO h_FD FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_end_year AND FISCAL_MONTH = 12;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 612+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(613) : cplan_fun.sql(2727) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("2_ h_dayofyear: %s, h_FD: %d:%s\n",h_dayofyear,h_FD_Ind,h_FD);
         End If;
         
         h_count := Coalesce(h_count, 0) + 1;
      End If;
      
      h_current_weekday := Coalesce(h_current_weekday, 0) + 1;
      
      If Coalesce(h_current_weekday, 0) = 8 Then
         h_current_weekday := 1;
      End If;
      
      h_current_daynumber := Coalesce(h_current_daynumber, 0) + 1;
   END LOOP;
   
   If PMO_ORASTD.strncmp(h_FD, '2100-01-01', 10) = 0 Then
      h_FD := h_dayofyear;
   End If;
   
   -- ********************************************************************
   -- Insert exceptions
   -- ********************************************************************
   If Coalesce(Length(h_calendar_id), 0) > 0 Then
      If Coalesce(Length(h_FD), 0) = 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_SD, 1, 10)) + h_duration      INTO h_FD
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 649+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(650) : cplan_fun.sql(2764) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- if there are exception...
      
      OPEN C_EXCEPTS;
      
      LOOP
         FETCH C_EXCEPTS INTO h_exception_date, h_current_min, h_current_bit;
         IF C_EXCEPTS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If Coalesce(h_current_min, 0) > 0 Then
            BEGIN
               SELECT PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date)
               INTO h_week_iso, h_month_iso
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 681+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(682) : cplan_fun.sql(2796) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
               --removed ASSIGNED_MIN from insertion
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 1, PMO_ORASTD.YEAR(h_exception_date)+1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 701+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(702) : cplan_fun.sql(2816) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 12, PMO_ORASTD.YEAR(h_exception_date)-1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 719+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(720) : cplan_fun.sql(2834) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, PMO_ORASTD.DAYOFWEEK(h_exception_date), h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date), PMO_ORASTD.YEAR(h_exception_date));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 737+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(738) : cplan_fun.sql(2852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("exceptions: h_exception_date = %s, h_current_min = %d\n", h_exception_date,h_current_min);
         ElsIf Coalesce(h_current_min, 0) = 0 Then
            BEGIN
               DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = TO_DATE(SubStr(h_exception_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 756+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(757) : cplan_fun.sql(2871) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_EXCEPTS ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 771+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(772) : cplan_fun.sql(2886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("END_1 iSF: %d, h_SD: %s, h_FD: %s\n", iSF,h_SD,h_FD);
   --timeLog (file,"INSERT INTO CALENDAR_DAY_OF_YEAR:");
   
   --  if( h_bitflag == 0 )
   --  {
   If iSF = 1 Then
      If PMO_ORASTD.strncmp(h_FD, FD001, 10) > 0 Then
         h_dayofyear := FD001;
         BEGIN
            DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_dayofyear, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 796+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(797) : cplan_fun.sql(2911) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_FD: %s, FD: %s\n", SC_STATE,h_FD,FD);
         
         h_FD := FD001;
      End If;
   Else
      FD001 := h_FD;
   End If;
   
   SD001 := h_SD;
   FD001 := h_FD;
   
   --trace_printf("END_2 h_SD: %s, h_FD: %s\n", h_SD,h_FD);
   
   -- if there are duplicates...
   
   OPEN C_DUPLIC;
   
   LOOP
      FETCH C_DUPLIC INTO h_element_id, h_dayofyear, h_count;
      IF C_DUPLIC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      --trace_printf("DUPLICATE: h_element_id: %s, h_dayofyear: %s, h_count: %d\n", h_element_id,h_dayofyear,h_count);
      
      OPEN C_DUP1;
      
      FETCH C_DUP1 INTO h_current_min, h_current_bit, h_week_iso, h_month_iso, h_count;
      IF C_DUP1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      BEGIN
         CLOSE C_DUP1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 846+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(847) : cplan_fun.sql(2961) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = h_dayofyear;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 863+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(864) : cplan_fun.sql(2978) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
         VALUES(h_element_id, h_dayofyear, PMO_ORASTD.DAYOFWEEK(h_dayofyear), h_current_min, h_current_bit, h_week_iso, h_month_iso, h_count);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 881+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(882) : cplan_fun.sql(2996) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_DUPLIC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 895+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(896) : cplan_fun.sql(3010) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO CALENDAR_DAY_OF_YEAR (ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
      SELECT * FROM PMO_TMP_CDY WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_SD, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10)) AND DAY_YEAR NOT IN (
      SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_SD, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10)));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 914+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(915) : cplan_fun.sql(3029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --  }
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 921+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(922) : cplan_fun.sql(3036) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION POPULATE_TMP_CALENDAR;
ALTER FUNCTION POPULATE_TMP_CALENDAR COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_CDY

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_CDY ==
CREATE OR REPLACE FUNCTION FC_CREATE_CDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_DT VARCHAR2(26);
   h_calendar_id VARCHAR2(32);
   h_days PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CSCDY1 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ) WHERE ROWNUM<=1;

      CURSOR CFCDY0 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ) WHERE ROWNUM<=1;

      CURSOR CFCDY1 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= h_finish_D
         ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_CDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_CDY'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_CDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 42+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_CDY(43) : cplan_fun.sql(3093) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_project_id: %s\nh_start_date: %s, h_finish_date: %s, h_duration: %d\n", h_project_id,h_start_date,h_finish_date,h_duration);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   --timeLog (file,"DECLARE TEMPORARY TABLE:");
   --trace_printf("START_SC_STATE = %d, len of finish date = %d\n", SC_STATE,strlen(h_finish_date));
   
   If ((h_start_date IS NULL)) OR (Coalesce(Length(h_start_date), 0) = 0) Then
      BEGIN
         SELECT MIN(DAY_YEAR) INTO h_start_date
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 63+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(64) : cplan_fun.sql(3114) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 70+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(71) : cplan_fun.sql(3121) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
   End If;
   
   h_calendar_id := h_project_id;
   h_dayofyear := NULL;
   
   If ((h_finish_date IS NULL)) OR (Coalesce(Length(h_finish_date), 0) = 0) Then
      h_finish_D := '';
      h_days := 0;
      
      OPEN CSCDY1;
      
      FETCH CSCDY1 INTO h_dayofyear;
      IF CSCDY1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d:%s, h_dayofyear: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_dayofyear_Ind,h_dayofyear);
      
      BEGIN
         CLOSE CSCDY1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 101+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(102) : cplan_fun.sql(3152) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strncmp(h_dayofyear, h_start_date, 10) > 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1      INTO h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 116+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(117) : cplan_fun.sql(3167) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9.0_SC_STATE: %d, h_finish_D: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D);
         
         If PMO_ORASTD.strncmp(h_finish_D, h_start_date, 10) < 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_date
               FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 132+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(133) : cplan_fun.sql(3183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT MIN(DAY_YEAR), COUNT(*) INTO h_dayofyear, h_days FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_project_id AND  DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 146+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(147) : cplan_fun.sql(3197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_days := Coalesce(h_duration, 0) + 1;
         End If;
      Else
         BEGIN
            SELECT MIN(DAY_YEAR), COUNT(*) INTO h_dayofyear, h_days FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND  DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 163+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(164) : cplan_fun.sql(3214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --timeLog (file,"1_ SELECT COUNTER() = h_duration:");
      --trace_printf("AA_SC_STATE = %d, h_dayofyear: %s, h_start_date = %s, h_duration = %d, h_days = %d\n", SC_STATE,h_dayofyear,h_start_date,h_duration,h_days);
      
      If Coalesce(h_days, 0) < Coalesce(h_duration, 0) Then
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 186+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(187) : cplan_fun.sql(3237) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("AB_SC_STATE = %d, h_start_date = %s\n", SC_STATE,h_start_date);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear FROM PMO_TMP_CDY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(202) : cplan_fun.sql(3252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_days := Coalesce(h_duration, 0);
         Else
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 217+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(218) : cplan_fun.sql(3268) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_ORASTD.strcmp(h_dayofyear, h_start_date) < 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DAYS(h_start_date) - PMO_ORASTD.DAYS(h_dayofyear) + h_duration INTO h_days
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 232+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(233) : cplan_fun.sql(3283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_days);
         
         --trace_printf("BB_SC_STATE = %d, h_dayofyear = %d:%s, h_finish_D = %d:%s, h_days: %d\n", SC_STATE,strlen(h_dayofyear),h_dayofyear, strlen(h_finish_D),h_finish_D,h_days);
         --trace_printf("*** h_dayofyear = %s\n", h_dayofyear);
      ElsIf PMO_ORASTD.strncmp(h_dayofyear, h_start_date, 10) > 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_start_date, 1, 10)), TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     
            INTO h_dayofyear, h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 255+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(256) : cplan_fun.sql(3306) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9_SC_STATE: %d, h_dayofyear: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_dayofyear_Ind,h_dayofyear,h_finish_D_Ind,h_finish_D);
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR ((h_dayofyear IS NULL)) 
               OR ((h_finish_D IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 264+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(265) : cplan_fun.sql(3315) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
         
         h_days := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_days);
         
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 287+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(288) : cplan_fun.sql(3338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("10_SC_STATE: %d, h_start_date: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_D_Ind,h_finish_D);
      Else
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 307+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(308) : cplan_fun.sql(3358) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("11_SC_STATE: %d, h_start_date: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_D_Ind,h_finish_D);
      End If;
      
      BEGIN
         SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) INTO h_finish_date FROM PMO_TMP_CDY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 322+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(323) : cplan_fun.sql(3373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_duration := 0;
      h_start_D := NULL;
      
      OPEN CFCDY0;
      
      FETCH CFCDY0 INTO h_start_D;
      IF CFCDY0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d:%s, h_start_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_start_D_Ind,h_start_D);
      
      h_temp_sc_state := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CFCDY0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 350+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(351) : cplan_fun.sql(3401) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_start_DT := h_start_date;
      If (Coalesce(h_temp_sc_state, 0) = 0) AND ((h_start_D IS NOT NULL)) AND (PMO_ORASTD.strncmp(
             h_start_D, h_start_date, 10) > 0) Then
         -- extend backwards
         h_start_D := SubStr(h_start_date, 1, 10);
         BEGIN
            SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1      INTO h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 369+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(370) : cplan_fun.sql(3420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9.1_SC_STATE: %d, h_finish_D: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D);
         
         h_days := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_start_D, h_finish_D
                                         , h_duration);
         
         BEGIN
            SELECT TO_DATE(SubStr(h_finish_D, 1, 10)) + 1      INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 388+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(389) : cplan_fun.sql(3439) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_dayofyear := NULL;
      h_finish_D := SubStr(h_finish_date, 1, 10);
      
      OPEN CFCDY1;
      
      FETCH CFCDY1 INTO h_dayofyear;
      IF CFCDY1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_finish_D: %d:%s, h_dayofyear: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D,h_dayofyear_Ind,h_dayofyear);
      
      h_temp_sc_state := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CFCDY1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 417+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(418) : cplan_fun.sql(3468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("*** h_temp_sc_state = %d\n", h_temp_sc_state);
      
      If (Coalesce(h_temp_sc_state, 0) <> 0) OR ((h_dayofyear IS NULL)) Then
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND
            DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 439+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(440) : cplan_fun.sql(3490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("22_SC_STATE: %d, h_start_date: %d:%s, h_finish_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
      Else
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 459+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(460) : cplan_fun.sql(3510) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("23_SC_STATE: %d, h_start_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date);
         
         BEGIN
            SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 473+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(474) : cplan_fun.sql(3524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CC_SC_STATE = %d, h_dayofyear = %s\n", SC_STATE,h_dayofyear);
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_duration);
         --trace_printf("*** h_dayofyear = %s\n", h_dayofyear);
      End If;
   End If;
   --trace_printf("END: %d, h_start_date: %d:%s, h_finish_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 488+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(489) : cplan_fun.sql(3539) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_CDY;
ALTER FUNCTION FC_CREATE_CDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_CDY_LEVEL ==
CREATE OR REPLACE FUNCTION FC_CREATE_CDY_LEVEL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_duration PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_start_D DATE;
   h_finish_D DATE;
   h_finish_date TIMESTAMP;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_CDY_LEVEL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_CDY_LEVEL'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_CDY_LEVEL(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY_LEVEL', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_CDY_LEVEL(25) : cplan_fun.sql(3578) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("\nh_bitflag: %d, h_project_id: %s, h_start_date: %s\n",h_bitflag, h_project_id,h_start_date);
   
   If Coalesce(h_bitflag, 0) = 0 Then
      h_finish_date := NULL;
      h_temp_sc_state := FC_CREATE_CDY(h_project_id, h_start_date, h_finish_date, h_duration);
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_start_D FROM PMO_TMP_CDY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY_LEVEL', 41+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY_LEVEL(42) : cplan_fun.sql(3595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_D := '';
      --trace_printf("h_duration: %d, h_start_D: %s\n",h_duration, h_start_D);
      h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_project_id, h_start_D, h_finish_D
                                         , h_duration);
      --trace_printf("2_h_temp_sc_state: %d, h_start_D: %s, h_finish_D: %s\n",h_temp_sc_state,h_start_D,h_finish_D);
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY_LEVEL', 55+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY_LEVEL(56) : cplan_fun.sql(3609) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_CDY_LEVEL;
ALTER FUNCTION FC_CREATE_CDY_LEVEL COMPILE DEBUG;

--########################################################################

-- EXTEND CALENDAR

--########################################################################

Prompt == FUNCTION EXTEND_CAL ==
CREATE OR REPLACE FUNCTION EXTEND_CAL
(
     id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_project_id VARCHAR2(32);
   h_dayofyear DATE;
   h_finish_D DATE;
   h_type_id PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('EXTEND_CAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('EXTEND_CAL'), '-- ' || SYSTIMESTAMP, ':r := EXTEND_CAL(' || 
       '''' || id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_CAL', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: EXTEND_CAL(19) : cplan_fun.sql(3642) : ' || SYSTIMESTAMP);
   END IF; 

   h_project_id := id;
   
   -- Check for project calendar/resource
   BEGIN
      SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear
      FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_CAL', 33+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_CAL(34) : cplan_fun.sql(3657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_dayofyear IS NOT NULL)) Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_CAL', 47+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_CAL(48) : cplan_fun.sql(3671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         h_finish_D := '';
         h_temp_var := 0;
         h_temp_sc_state := POPULATE_CALENDAR(h_project_id, h_type_id, h_dayofyear, h_finish_D, h_temp_var);
      End If;
   Else
      PMO_DB2M.SC_STATE := -1;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_CAL', 62+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: EXTEND_CAL(63) : cplan_fun.sql(3686) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION EXTEND_CAL;
ALTER FUNCTION EXTEND_CAL COMPILE DEBUG;

--########################################################################

-- EXTEND RESOURCE / CALENDAR

--########################################################################

Prompt == FUNCTION EXTEND_PRCAL ==
CREATE OR REPLACE FUNCTION EXTEND_PRCAL
(
     id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_type PLS_INTEGER := Coalesce(sh_h_type, 0);
   h_id VARCHAR2(32);
   h_dayofyear DATE;
   h_FD DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_type_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_ERC1 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('EXTEND_PRCAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('EXTEND_PRCAL'), '-- ' || SYSTIMESTAMP, ':r := EXTEND_PRCAL(' || 
       '''' || id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: EXTEND_PRCAL(39) : cplan_fun.sql(3739) : ' || SYSTIMESTAMP);
   END IF; 

   h_type_id := h_type;
   h_id := id;
   h_dayofyear := SubStr(h_start_DT, 1, 10);
   h_temp_sc_state := -1;
   
   --trace_printf("\nh_id: %d%s, h_dayofyear: %d:%s\n", h_id_Ind, h_id, h_dayofyear_Ind, h_dayofyear);
   --trace_printf("h_type_id = %d, h_duration = %d, h_start_DT = %s, h_finish_DT = %s\n", h_type_id, h_duration, h_start_DT, h_finish_DT);
   If Coalesce(h_duration, 0) > 0 Then
      BEGIN
         SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     , TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
         INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 59+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(60) : cplan_fun.sql(3760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("CDY__1 SC_STATE: %d, h_start_D = %s, h_finish_D = %s, h_dayofyear = %d:%s\n", SC_STATE,h_start_D,h_finish_D,h_dayofyear_Ind,h_dayofyear);trace_flush();
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR ((h_start_D IS NULL)) 
            OR ((h_finish_D IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 68+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(69) : cplan_fun.sql(3769) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
      
      If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
         h_temp_var := 0;
         
         --trace_printf("A.a1__ h_dayofyear = %s, h_start_D = %s, h_duration = %d\n", h_dayofyear,h_start_D,h_duration); trace_flush();
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_dayofyear, h_start_D, h_temp_var);
      ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) > 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + h_duration      INTO h_FD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 87+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(88) : cplan_fun.sql(3788) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_count := 0;
         
         --trace_printf("A.a2__ h_finish_D = %s, h_FD = %s, h_count = %d\n", h_finish_D,h_FD,h_count); trace_flush();
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_FD, h_count);
      End If;
      
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_count
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id AND DAY_YEAR > h_dayofyear;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 106+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(107) : cplan_fun.sql(3807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("***** h_count = %d, h_finish_DT: %d %s\n", h_count, h_finish_DT_Ind,h_finish_DT); trace_flush();
      
      If Coalesce(h_count, 0) < Coalesce(h_duration, 0) Then
         h_FD := '';
         
         --trace_printf("A.b__ h_finish_D = %s, h_duration = %d\n", h_finish_D,h_duration); trace_flush();
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_FD, h_duration);
      End If;
      
      stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = ''' || h_id 
            || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_dayofyear || ''', 1, 10)) 
' || Chr(13) 
            || ') WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_duration, 0));
      
      OPEN C_ERC1 FOR stmt;
      
      LOOP
         FETCH C_ERC1 INTO h_finish_DT;
         IF C_ERC1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_ERC1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 145+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(146) : cplan_fun.sql(3846) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     , TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
         INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 160+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(161) : cplan_fun.sql(3861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("CDY__2 h_start_D = %s, h_finish_D = %s\n", h_start_D,h_finish_D); trace_flush();
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR ((h_start_D IS NULL)) 
            OR ((h_finish_D IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 169+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(170) : cplan_fun.sql(3870) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
      
      If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
         --trace_printf("B.b1__ h_dayofyear = %s, h_start_D = %s, h_duration = %d\n", h_dayofyear,h_start_D,h_duration); trace_flush();
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_dayofyear, h_start_D, h_duration);
      End If;
      
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) >= 0 Then
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         
         --trace_printf("C.c__ h_dayofyear = %s, h_finish_D = %s, h_duration = %d\n", h_dayofyear,h_finish_D,h_duration); trace_flush();
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_dayofyear, h_duration);
      End If;
   End If;
   
   If (isProjectType(Coalesce(h_type_id, 0))) <> 0 AND (Coalesce(h_temp_sc_state, 0) = 0) Then
      BEGIN
         SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) INTO h_start_D FROM PMO_TMP_CDY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 196+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(197) : cplan_fun.sql(3897) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) INTO h_finish_DT FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 211+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(212) : cplan_fun.sql(3912) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("C.*__ h_start_D = %s, h_finish_D = %s, h_duration = %d\n", h_start_D,h_finish_D,h_duration); trace_flush();
         If PMO_ORASTD.strncmp(h_start_D, h_finish_D, 10) < 0 Then
            BEGIN
               INSERT INTO PMO_TMP_CDY
               SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
               FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_id AND DAY_YEAR > TO_DATE(SubStr(h_start_D, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 233+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(234) : cplan_fun.sql(3934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("====> INSERT INTO session.PMO_TMP_CDY: SC_STATE: %d\n", SC_STATE);
         End If;
      End If;
   End If;
   
   --trace_printf("END__ h_start_DT: %d %s, h_finish_DT: %d %s, h_duration: %d\n", h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration);
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 246+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(247) : cplan_fun.sql(3947) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION EXTEND_PRCAL;
ALTER FUNCTION EXTEND_PRCAL COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_CONSIDER_DAY ==
CREATE OR REPLACE FUNCTION FC_CONSIDER_DAY
(
     h_dayofyear IN OUT NOCOPY DATE
   , sh_h_total PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_minPerDay PLS_INTEGER
   , h_bitID IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_total PLS_INTEGER := Coalesce(sh_h_total, 0);
   h_type PLS_INTEGER := Coalesce(sh_h_type, 0);
   h_minPerDay PLS_INTEGER := Coalesce(sh_h_minPerDay, 0);
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_available PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC16 IS
         SELECT SCHED_MIN, BIT_SCHED FROM PMO_TMP_TDY
         WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10)) ORDER BY SCHED_MIN DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CONSIDER_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CONSIDER_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_CONSIDER_DAY(' || 
       COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_total), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minPerDay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CONSIDER_DAY', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CONSIDER_DAY(33) : cplan_fun.sql(3990) : ' || SYSTIMESTAMP);
   END IF; 

   h_bit_minutes := 0;
   
   --trace_printf("CONSIDER: h_minPerDay = %d, h_dayofyear = %s, h_total = %d, h_type: %d, h_bit_minutes: %d\n",h_minPerDay, h_dayofyear,h_total,h_type,h_bit_minutes);
   
   BEGIN
      SELECT SCHED_MIN INTO h_bit_minutes FROM PMO_TMP_CPLNF_0001 WHERE DAY_YEAR = h_dayofyear;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CONSIDER_DAY', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CONSIDER_DAY(48) : cplan_fun.sql(4005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE = %d, h_bit_minutes: %d\n", SC_STATE,h_bit_minutes);
   
   /*
   		the function return the Amount of assigned min.
   
   		h_total:        Calendar Avalibility per day
   		h_bit_minutes:  Agregate Assignments per day
   		h_minPerDay:    Minimum Assign per day.
   	*/
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If (h_total - Coalesce(h_bit_minutes, 0)) >= h_minPerDay Then
         If Coalesce(h_bit_minutes, 0) < h_total Then
            If h_type > 0 Then
               OPEN CC16;
               
               LOOP
                  FETCH CC16 INTO h_available, h_bitID1;
                  IF CC16%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
                  --trace_printf("## h_available = %d\n", h_available);
                  
                  If Coalesce(h_available, 0) > 0 Then
                     h_bitID := PMO_STPU.BIT_ANDNOT_DAY(h_bitID, h_bitID1);
                  End If;
               END LOOP;
               
               BEGIN
                  CLOSE CC16 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CONSIDER_DAY', 93+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CONSIDER_DAY(94) : cplan_fun.sql(4051) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            h_bit_minutes := h_total;
         End If;
      Else
         h_bit_minutes := h_total;
         -- return that this day is not avalibal (all day assigned)
      End If;
   End If;
   --trace_printf("h_bit_minutes = %d\n", h_bit_minutes);
   
   PMO_DB2M.SC_STATE := Coalesce(h_bit_minutes, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CONSIDER_DAY', 111+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CONSIDER_DAY(112) : cplan_fun.sql(4069) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CONSIDER_DAY;
ALTER FUNCTION FC_CONSIDER_DAY COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_FIX_SUMMARY_TS ==
CREATE OR REPLACE FUNCTION FC_FIX_SUMMARY_TS
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_day_year DATE;
   h_mls_percent PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
-- Declare Cursors
      CURSOR C_TS IS
         SELECT WEEK_OF FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND WORK_TODATE >= h_work_todate
         ORDER BY WEEK_OF;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_SUMMARY_TS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_SUMMARY_TS'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_SUMMARY_TS(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_SUMMARY_TS(29) : cplan_fun.sql(4108) : ' || SYSTIMESTAMP);
   END IF; 

   h_sched_work := 0;
   h_work_todate := 0;
   BEGIN
      SELECT SCHED_WORK, WORK_TODATE, MILESTONE_PERCENT, REGULAR_COST
      INTO h_sched_work,h_work_todate,h_mls_percent, h_reg_cost
      FROM TASK_ASSIGNMENTS
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(45) : cplan_fun.sql(4124) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 50+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_SUMMARY_TS(51) : cplan_fun.sql(4130) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   --fix baseline_work per week information
   --  h_temp_sc_state = FC_GENERATE_EW(SC_PARAM(h_tsk_assignment_id));
   
   --locate affected summary_timesheets
   
   OPEN C_TS;
   
   LOOP
      FETCH C_TS INTO h_day_year;
      IF C_TS%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      --accumulate baseline_work_todate
      BEGIN
         SELECT COALESCE(SUM(SCHED_MIN),0) INTO h_work
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR <= h_day_year;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(86) : cplan_fun.sql(4165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\nh_tsk_assignment_id: %d, h_day_year %s", h_tsk_assignment_id,h_day_year);
      --trace_printf("\nh_work: %d, %d", h_work_Ind, h_work);
      --trace_printf("\nh_sched_work: %d, %d", h_sched_work_Ind, h_sched_work);
      --trace_printf("\nh_work_todate: %d, %d", h_work_todate_Ind, h_work_todate);
      --trace_printf("\nh_mls_percent: %d, %d", h_mls_percent_Ind, h_mls_percent);
      
      --update affected summary_timesheets
      If Coalesce(h_sched_work, 0) > Coalesce(h_work_todate, 0) Then
         BEGIN
            UPDATE SUMMARY_TIMESHEETS SET
            TOTAL_SCHED_EFFORT = h_sched_work,
            ETC = h_sched_work - h_work_todate,
            EV_ETC = (h_sched_work - h_work_todate) * h_reg_cost,
            MILESTONE_PERCENT = h_mls_percent,
            EAC = ACWP + ((h_sched_work - h_work_todate)/60.00 * BASELINE_REGULAR_COST),
            BASELINE_WORK_TODATE = h_work,
            BCWS = h_work/60.00 * BASELINE_REGULAR_COST,
            BCWP = BAC * (h_mls_percent/10000.00)
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 120+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(121) : cplan_fun.sql(4200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE SUMMARY_TIMESHEETS SET
            TOTAL_SCHED_EFFORT = h_sched_work,
            ETC = 0,
            EV_ETC = 0.00,
            MILESTONE_PERCENT = h_mls_percent,
            EAC = ACWP,
            BASELINE_WORK_TODATE = h_work,
            BCWS = h_work/60.00 * BASELINE_REGULAR_COST,
            BCWP = BAC * (h_mls_percent/10000.00)
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(148) : cplan_fun.sql(4227) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_TS ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 162+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(163) : cplan_fun.sql(4242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET
      EAC = CASE WHEN SCHED_WORK < WORK_TODATE THEN ACWP ELSE BCWS END,
      EV_ETC = ((ETC/60.00)*REGULAR_COST)
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 182+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(183) : cplan_fun.sql(4262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 188+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_SUMMARY_TS(189) : cplan_fun.sql(4268) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_SUMMARY_TS;
ALTER FUNCTION FC_FIX_SUMMARY_TS COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_DCONSIDER_DAY ==
CREATE OR REPLACE FUNCTION FC_DCONSIDER_DAY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_day_year IN OUT NOCOPY DATE
   , h_bitID IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_available PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_project_id_Ind PLS_INTEGER NOT NULL := 0;
   h_tsk_assignment_id_Ind PLS_INTEGER NOT NULL := 0;
   h_prj_assignment_id_Ind PLS_INTEGER NOT NULL := 0;
   h_day_year_Ind PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CCD1 IS
         SELECT SCHED_MIN, BIT_SCHED FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND TSK_ASSIGNMENT_ID <> h_tsk_assignment_id
         AND TSK_ASSIGNMENT_ID <> PARENT_ID AND DAY_YEAR = TO_DATE(SubStr(h_day_year, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DCONSIDER_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DCONSIDER_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_DCONSIDER_DAY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_day_year || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DCONSIDER_DAY', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DCONSIDER_DAY(35) : cplan_fun.sql(4313) : ' || SYSTIMESTAMP);
   END IF; 

   h_bitID1 := PMO_CNST.SC_ZERO_BIT;
   
   OPEN CCD1;
   
   LOOP
      FETCH CCD1 INTO h_available, h_bitID1;
      IF CCD1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If Coalesce(h_available, 0) > 0 Then
         h_bit_minutes := Coalesce(h_bit_minutes, 0) + Coalesce(h_available, 0);
         h_bitID := PMO_STPU.BIT_ANDNOT_DAY(h_bitID, h_bitID1);
      End If;
   END LOOP;
   BEGIN
      CLOSE CCD1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DCONSIDER_DAY', 65+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DCONSIDER_DAY(66) : cplan_fun.sql(4344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := Coalesce(h_bit_minutes, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DCONSIDER_DAY', 73+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DCONSIDER_DAY(74) : cplan_fun.sql(4352) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DCONSIDER_DAY;
ALTER FUNCTION FC_DCONSIDER_DAY COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_INSERT_DAY_OF_YEAR ==
CREATE OR REPLACE FUNCTION FC_INSERT_DAY_OF_YEAR
(
     sh_assign_to_prj PLS_INTEGER
   , sh_h_work_type PLS_INTEGER
   , sh_h_minutes PLS_INTEGER
   , sh_h_update PLS_INTEGER
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
   , h_first_time IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , h_bitID1 IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   assign_to_prj PLS_INTEGER := sh_assign_to_prj;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_update PLS_INTEGER := sh_h_update;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_consider PLS_INTEGER := sh_h_consider;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_bit_flag VARCHAR2(1);
   h_bit_minutes PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_min_work_per_day PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_bitflag1 PLS_INTEGER := 0;
   i_update_pa PLS_INTEGER NOT NULL := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_INSERT_DAY_OF_YEAR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_INSERT_DAY_OF_YEAR'), '-- ' || SYSTIMESTAMP, ':r := FC_INSERT_DAY_OF_YEAR(' || 
       COALESCE(TO_CHAR(sh_assign_to_prj), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL')
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || '''' || h_first_time || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID1), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_DAY_OF_YEAR', 61+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_INSERT_DAY_OF_YEAR(62) : cplan_fun.sql(4424) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_CHAR(tempStr, SC_H_DESCRIPTION);
   
   h_type := 1;
   h_finish_time := '';
   h_min_work_per_day := h_minutes;
   --trace_printf("*** h_tsk_assignment_id: %s, h_bitflag: %d, assign_to_prj: %d, h_dayofyear: %s, h_minutes: %d, h_consider: %d\n",h_tsk_assignment_id,h_bitflag,assign_to_prj,h_dayofyear,h_minutes,h_consider);
   --trace_printf("h_project_id: %s, h_task_id: %d:%s, h_res_id: %d:%s\n",h_project_id,h_task_id_Ind,h_task_id,h_res_id_Ind,h_res_id);
   
   If Coalesce(h_minutes, 0) < 5 Then
      h_min_work_per_day := 5;
      -- can't assign less then 5 minutes
   Else
      h_temp_var := Coalesce(h_minutes, 0) MOD 5;
      If Coalesce(h_temp_var, 0) <> 0 Then
         h_min_work_per_day := Coalesce(h_min_work_per_day, 0) + 5 - Coalesce(h_temp_var, 0);
      End If;
   End If;
   
   If Coalesce(assign_to_prj, 0) = 0 Then
      h_from_prj := 1;
   End If;
   -- not from PA
   
   If Coalesce(h_work_type, 0) = 2 Then
      h_bit_flag := '2';
   Else
      h_bit_flag := '0';
   End If;
   --trace_printf("*** h_min_work_per_day: %d, h_work_type: %d, h_bit_flag: %s\n",h_min_work_per_day,h_work_type,h_bit_flag);
   
   If (BITAND(Coalesce(h_bitflag, 0), 255)) = 10 Then
      h_bitID := h_bitID1;
      h_update := 0;
      
      h_total := PMO_STPU.getNumberOfBits(h_bitID) * 5;
      If (h_prj_assignment_id IS NULL) Then
         h_prj_assignment_id := h_tsk_assignment_id;
         h_from_prj := Coalesce(assign_to_prj, 0);
      Else
         h_from_prj := 0;
      End If;
      --trace_printf("h_total: %d, h_prj_assignment_id: %d:%s, h_update: %d, h_from_prj: %d\n",h_total,h_prj_assignment_id_Ind,h_prj_assignment_id,h_update,h_from_prj);
   ElsIf Coalesce(h_bitflag, 0) > 9 Then
      h_bitID := h_bitID1;
      h_update := 0;
      
      --trace_printf("h_prj_assignment_id_Ind = %d:%s, h_update: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_update,h_from_prj);
      h_total := PMO_STPU.getNumberOfBits(h_bitID) * 5;
      If (h_prj_assignment_id IS NULL) Then
         h_prj_assignment_id := h_tsk_assignment_id;
         h_from_prj := Coalesce(assign_to_prj, 0);
      Else
         h_from_prj := 0;
      End If;
   Else
      If Coalesce(h_consider, 0) = 0 Then
         If Coalesce(h_work_type, 0) = 2 Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, PMO_ORASTD.TIME(SCHED_START),PMO_ORASTD.TIME(SCHED_FINISH),
               CASE WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) = h_dayofyear THEN 1
               WHEN TO_DATE(SubStr(SCHED_FINISH, 1, 10)) = h_dayofyear THEN 2
               ELSE 0 END
               INTO h_prj_assignment_id, h_start_time, h_finish_time, h_bitflag1
               FROM TASK_ASSIGNMENTS 
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 135+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(136) : cplan_fun.sql(4498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, PMO_ORASTD.TIME(PLANNED_START),PMO_ORASTD.TIME(PLANNED_FINISH),
               CASE WHEN TO_DATE(SubStr(PLANNED_START, 1, 10)) = h_dayofyear THEN 1
               WHEN TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) = h_dayofyear THEN 2
               ELSE 0 END
               INTO h_prj_assignment_id, h_start_time, h_finish_time, h_bitflag1
               FROM TASK_ASSIGNMENTS 
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 154+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(155) : cplan_fun.sql(4517) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If (h_first_time IS NOT NULL) Then
            h_start_time := h_first_time;
         Else
            h_start_time := '';
         End If;
         
         BEGIN
            SELECT PROJECT_ASSIGNMENT_ID INTO h_prj_assignment_id 
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = h_bit_flag;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 176+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(177) : cplan_fun.sql(4539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("SC_STATE = %d, h_prj_assignment_id_Ind = %d:%s, h_from_prj: %d\n",SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id,h_from_prj);
      If (h_prj_assignment_id IS NULL) Then
         BEGIN
            SELECT SCHED_MIN, BIT_SCHED INTO h_exists, h_bitID 
            FROM PMO_TMP_TDY/*TASK_DAY_OF_YEAR*/
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = h_bit_flag;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 194+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(195) : cplan_fun.sql(4557) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_prj_assignment_id := h_tsk_assignment_id;
         h_from_prj := 2;
         -- not from PA
      ElsIf Coalesce(h_work_type, 0) = 2 Then
         h_update := 0;
         If (Coalesce(h_consider, 0) = 0) OR (Coalesce(h_consider, 0) = 7) Then
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_exists, h_bitID 
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 217+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(218) : cplan_fun.sql(4580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_exists, h_bitID 
               FROM PMO_TMP_TDY/*TASK_DAY_OF_YEAR*/
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 233+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(234) : cplan_fun.sql(4596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --trace_printf("SC_STATE = %d, h_consider = %d, h_dayofyear: %s\n",SC_STATE,h_consider);
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (Coalesce(h_consider, 0) <> 0) Then
            h_prj_assignment_id := h_tsk_assignment_id;
         End If;
      ElsIf Coalesce(h_work_type, 0) = 0 Then
         h_update := 0;
         If (Coalesce(h_consider, 0) = 0) OR (Coalesce(h_consider, 0) = 7) Then
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_exists, h_bitID 
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 258+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(259) : cplan_fun.sql(4621) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_exists, h_bitID 
               FROM PMO_TMP_TDY/*TASK_DAY_OF_YEAR*/
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 274+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(275) : cplan_fun.sql(4637) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("SC_STATE = %d, h_prj_assignment_id_Ind = %d:%s, h_dayofyear: %s\n",SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id,h_dayofyear);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (Coalesce(h_consider, 0) <> 0) Then
            h_prj_assignment_id := h_tsk_assignment_id;
         End If;
      End If;
      
      --trace_printf("h_consider = %d, h_dayofyear = %s, h_min_work_per_day = %d, h_exists = %d, h_from_prj = %d\n",h_consider,h_dayofyear,h_min_work_per_day,h_exists,h_from_prj);
      If (Coalesce(h_from_prj, 0) = 0) AND (Coalesce(h_minutes, 0) <= Coalesce(h_exists, 0)) Then
         --trace_printf("1__h_first_time = %d:%s, h_start_time = %s\n", h_first_time_Ind,h_first_time,h_start_time);
         If (h_first_time IS NOT NULL) Then
            If PMO_ORASTD.strncmp(h_start_time, h_first_time, 5) < 0 Then
               PMO_STPU.cleardayStruct(h_start_time, h_first_time, h_bitID);
            Else
               BEGIN
                  SELECT PMO_STPU.FIRST_AVAIL_TIME(h_bitID) INTO h_start_time FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 301+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(302) : cplan_fun.sql(4664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("1_*** h_start_time = %s\n", h_start_time);
               If PMO_ORASTD.strncmp(h_start_time, h_first_time, 5) < 0 Then
                  PMO_STPU.cleardayStruct(h_start_time, h_first_time, h_bitID);
               End If;
            End If;
         End If;
         
         h_bitID2 := h_bitID;
         h_total := PMO_STPU.getNumberOfBits(h_bitID2) * 5;
         --trace_printf("1_a h_total: %d, h_exists: %d\n", h_total,h_exists);
         
         If Coalesce(h_total, 0) < Coalesce(h_exists, 0) Then
            i_update_pa := 1;
            h_temp_var := Coalesce(h_exists, 0) - Coalesce(h_min_work_per_day, 0);
            PMO_STPU.clearStartDayStruct(Coalesce(h_temp_var, 0), h_bitID2);
            
            h_total := Coalesce(h_exists, 0);
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID=h_project_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 335+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(336) : cplan_fun.sql(4698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            -- RES_CAL_TT
            BEGIN
               SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 350+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(351) : cplan_fun.sql(4713) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --strcpy(tempStr,"");
         --printTimes((unsigned char *)h_bitID.h_value,tempStr);
         --trace_printf("0_*** SC_STATE: %d, tempStr = %s\n",SC_STATE,tempStr); trace_flush();
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            h_bitID := PMO_CNST.SC_ZERO_BIT;
         End If;
         
         --trace_printf("2__h_first_time = %d:%s, h_start_time = %s\n",h_first_time_Ind,h_first_time,h_start_time);
         If (h_first_time IS NOT NULL) Then
            If PMO_ORASTD.strncmp(h_start_time, h_first_time, 5) < 0 Then
               PMO_STPU.cleardayStruct(h_start_time, h_first_time, h_bitID);
            Else
               BEGIN
                  SELECT PMO_STPU.FIRST_AVAIL_TIME(h_bitID) INTO h_start_time FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 377+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(378) : cplan_fun.sql(4740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("2_*** h_start_time = %s\n", h_start_time);
               If PMO_ORASTD.strncmp(h_start_time, h_first_time, 5) < 0 Then
                  PMO_STPU.cleardayStruct(h_start_time, h_first_time, h_bitID);
               End If;
            End If;
         End If;
         
         h_total := PMO_STPU.getNumberOfBits(h_bitID) * 5;
         
         --trace_printf("h_minutes = %d, h_total = %d\n", h_minutes, h_total);
         If Coalesce(h_minutes, 0) < Coalesce(h_total, 0) Then
            --trace_printf("h_consider = %d, h_bitflag1 = %d, h_start_time = %s, h_finish_time = %s\n", h_consider, h_bitflag1, h_start_time, h_finish_time);
            If (Coalesce(h_consider, 0) = 0) AND (Coalesce(h_bitflag1, 0) <> 0) Then
               PMO_STPU.clearFinishDayStruct(Coalesce(h_min_work_per_day, 0), h_bitID);
               /*if (h_bitflag1 == 1)  // h_dayofyear = SD
               					{
               						BEGIN_SQL
               							SELECT TIME(HOST(h_start_time)) + HOST(h_minutes) MINUTES
               							INTO HOST(h_finish_time)
               							FROM DUAL;
               						END_SQL;
               
               						filldayStruct(h_start_time, h_finish_time, (unsigned char *)h_bitID.h_value);
               					}
               					else if (h_bitflag1 == 2)   // h_dayofyear = FD
               					{
               						BEGIN_SQL
               							SELECT TIME(HOST(h_finish_time)) - HOST(h_minutes) MINUTES  INTO HOST(h_start_time)
               							FROM DUAL;
               						END_SQL;
               
               						filldayStruct(h_start_time, h_finish_time, (unsigned char *)h_bitID.h_value);
               					}*/
               
               --strcpy(tempStr,"");
               --printTimes((unsigned char *)h_bitID.h_value,tempStr);
               --trace_printf("2_*** tempStr = %s\n",tempStr); trace_flush();
            Else
               h_bit_minutes := FC_CONSIDER_DAY(h_dayofyear, Coalesce(h_total, 0), Coalesce(h_type, 0), 0, h_bitID);
               
               If Coalesce(h_bit_minutes, 0) > 0 Then
                  h_total := Coalesce(h_total, 0) - Coalesce(h_bit_minutes, 0);
               End If;
            End If;
         End If;
      End If;
   End If;
   
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   --trace_printf("h_tsk_assignment_id = %s, _Ind = %d\n",h_tsk_assignment_id,h_tsk_assignment_id_Ind);
   --trace_printf("h_prj_assignment_id =%s, _Ind = %d\n",h_prj_assignment_id,h_prj_assignment_id_Ind);
   --trace_printf("h_bitflag1 = %d, h_update = %d, h_total = %d, tempStr = %s\n",h_bitflag1,h_update,h_total,tempStr); trace_flush();
   
   -- FIX 7/9/2002: When MSP calls for update assignment 1 h work, PMOffice puts 1 day work (bug ISS-341, George)
   If (Coalesce(h_bitflag1, 0) = 0) AND (Coalesce(h_minutes, 0) <= Coalesce(h_total, 0)) Then
      PMO_STPU.clearFinishDayStruct(Coalesce(h_min_work_per_day, 0), h_bitID);
   End If;
   
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   --trace_printf("h_bitflag: %d, h_minutes: %d, h_total: %d, h_update: %d, tempStr: %s\n",h_bitflag,h_minutes,h_total,h_update,tempStr);
   
   If PMO_ORASTD.strcmp(h_tsk_assignment_id, h_prj_assignment_id) = 0 Then
      h_from_prj := 1;
   End If;
   h_exists := 0;
   If (BITAND(Coalesce(h_bitflag, 0), 255)) = 10 Then
      BEGIN
         INSERT INTO PMO_TMP_TDOY (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
         VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_prj_assignment_id, h_dayofyear, h_minutes, h_bitID, h_bit_flag);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 463+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(464) : cplan_fun.sql(4826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_update, 0) = 0 Then
      BEGIN
         SELECT SCHED_MIN INTO h_exists 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR = h_dayofyear 
         AND BIT_FLAG = h_bit_flag;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 481+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(482) : cplan_fun.sql(4844) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_from_prj, 0) = 0 Then
         If Coalesce(h_minutes, 0) < Coalesce(h_total, 0) Then
            If Coalesce(h_exists, 0) = 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_TDOY/*TASK_DAY_OF_YEAR*/ (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
                  VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_prj_assignment_id, h_dayofyear, h_minutes, h_bitID, h_bit_flag);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 502+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(503) : cplan_fun.sql(4865) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 520+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(521) : cplan_fun.sql(4883) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("1_* SC_STATE = %d, h_exists = %d\n",SC_STATE,h_exists);
            
            If PMO_ORASTD.strcmp(h_prj_assignment_id, h_tsk_assignment_id) <> 0 Then
               If i_update_pa = 0 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = CASE WHEN SCHED_MIN > h_minutes THEN SCHED_MIN - h_minutes ELSE 0 END, BIT_SCHED = PMO_STPU.BIT_ANDNOT_DAY(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 542+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(543) : cplan_fun.sql(4905) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf i_update_pa = 1 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = CASE WHEN SCHED_MIN > h_minutes THEN SCHED_MIN - h_minutes ELSE 0 END, BIT_SCHED = h_bitID2
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 560+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(561) : cplan_fun.sql(4923) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = CASE WHEN SCHED_MIN > h_minutes THEN SCHED_MIN - h_minutes ELSE 0 END
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 578+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(579) : cplan_fun.sql(4941) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("2_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
            End If;
         Else
            If Coalesce(h_exists, 0) = 0 Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_tsk_assignment_id, SCHED_MIN = h_minutes, TASK_ID = h_task_id, BIT_FLAG = h_bit_flag
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 600+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(601) : cplan_fun.sql(4963) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("3_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
            Else
               -- delete PA remaining
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 619+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(620) : cplan_fun.sql(4982) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- merge PA remaining to TA
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 638+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(639) : cplan_fun.sql(5001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("4_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
            End If;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS AND ((Coalesce(h_work_type, 0) = 0) OR (Coalesce(
                  h_consider, 0) <> 0)) Then
               BEGIN
                  INSERT INTO PMO_TMP_TDOY/*TASK_DAY_OF_YEAR*/ (PROJECT_ID,TASK_ID,RESOURCE_ID,TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
                  VALUES(h_project_id,h_task_id, h_res_id,h_tsk_assignment_id, h_prj_assignment_id,h_dayofyear, h_minutes, h_bitID, h_bit_flag);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 660+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(661) : cplan_fun.sql(5023) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("5_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
            End If;
         End If;
      Else
         --trace_printf("h_work_type: %d:%d,h_res_id: %d:%s, h_project_id: %d:%s, h_dayofyear: %d:%s\n",h_work_type_Ind,h_work_type,h_res_id_Ind,h_res_id,h_project_id_Ind,h_project_id,h_dayofyear_Ind,h_dayofyear);//trace_flush();
         --trace_printf("h_tsk_assignment_id: %d:%s, h_prj_assignment_id: %d:%s, h_minutes: %d:%d, h_bitID: %d\n",h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id,h_minutes_Ind,h_minutes,h_bitID_Ind);//trace_flush();
         If Coalesce(h_exists, 0) = 0 Then
            BEGIN
               INSERT INTO PMO_TMP_TDOY/*TASK_DAY_OF_YEAR*/ (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
               VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_prj_assignment_id, h_dayofyear, h_minutes, h_bitID, h_bit_flag);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 684+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(685) : cplan_fun.sql(5047) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 702+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(703) : cplan_fun.sql(5065) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("10_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d, h_bit_flag: %s\n",SC_STATE,h_dayofyear,h_minutes,h_bit_flag);
      End If;
   ElsIf Coalesce(h_work_type, 0) = 0 Then
      BEGIN
         UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_minutes
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 723+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(724) : cplan_fun.sql(5086) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("6_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            INSERT INTO PMO_TMP_TDOY/*TASK_DAY_OF_YEAR*/ (PROJECT_ID,TASK_ID,RESOURCE_ID,TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
            VALUES(h_project_id,h_task_id, h_res_id,h_tsk_assignment_id, h_prj_assignment_id,h_dayofyear, h_minutes, h_bitID, '0');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 743+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(744) : cplan_fun.sql(5106) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("7_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
      End If;
   ElsIf Coalesce(h_work_type, 0) = 2 Then
      BEGIN
         UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_minutes, BIT_SCHED = h_bitID, BIT_FLAG = '2'
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 763+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(764) : cplan_fun.sql(5126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("8_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            INSERT INTO PMO_TMP_TDOY/*TASK_DAY_OF_YEAR*/ (PROJECT_ID,TASK_ID,RESOURCE_ID,TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
            VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_prj_assignment_id, h_dayofyear, h_minutes, h_bitID, '2');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 783+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(784) : cplan_fun.sql(5146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9_* SC_STATE = %d, h_dayofyear = %s, h_minutes = %d\n",SC_STATE,h_dayofyear,h_minutes);
      End If;
   End If;
   
   h_bitID1 := h_bitID;
   
   --trace_printf("END_* SC_STATE = %d, h_tsk_assignment_id: %s\n",SC_STATE, h_tsk_assignment_id);//trace_flush();
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_DAY_OF_YEAR', 800+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_INSERT_DAY_OF_YEAR(801) : cplan_fun.sql(5163) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_INSERT_DAY_OF_YEAR;
ALTER FUNCTION FC_INSERT_DAY_OF_YEAR COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION CHECK_ASSIGN ==
CREATE OR REPLACE FUNCTION CHECK_ASSIGN
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_consider PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_work PLS_INTEGER := sh_h_work;
   h_effort PLS_INTEGER := sh_h_effort;
   h_consider PLS_INTEGER := sh_h_consider;
   h_constr_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   stwhere VARCHAR2(2999);
   h_temp_sc_state PLS_INTEGER := 0;
   h_leveling PLS_INTEGER := 0;
   h_level_DT VARCHAR2(26);
   t_start2 PLS_INTEGER := 0;
   t_total2 NUMBER := 0;
   h_available PLS_INTEGER := 0;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   stmt VARCHAR2(7999);
-- Declare Cursors
      CURSOR CCA16a IS
         SELECT * FROM (SELECT MIN(d.DAY_YEAR), COUNT(d.DAY_YEAR) FROM CALENDAR_DAY_OF_YEAR d
         LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = d.DAY_YEAR
         WHERE d.ELEMENT_ID=h_res_id AND d.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
         AND (d.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)) > 0 
         ) WHERE ROWNUM<=1;

      CURSOR CCA16b IS
         SELECT * FROM (SELECT MIN(d.DAY_YEAR) FROM CALENDAR_DAY_OF_YEAR d
         LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = d.DAY_YEAR
         WHERE d.ELEMENT_ID=h_res_id AND d.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND
         (d.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)) >= h_work 
         ) WHERE ROWNUM<=1;

      -- Dynamic Cursor Declaration
      CCDY0 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('CHECK_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('CHECK_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := CHECK_ASSIGN(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN', 63+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: CHECK_ASSIGN(64) : cplan_fun.sql(5237) : ' || SYSTIMESTAMP);
   END IF; 

   stwhere := '';
   --  InitializeHost(SC_PARAM(h_dayofyear), "");
   h_dayofyear := NULL;
   h_constr_DT := h_start_DT;
   
   If Coalesce(h_consider, 0) >= PMO_CNST.SC_BIT_30 Then
      h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
      
      h_leveling := 1;
      --      h_work = 30;
      h_level_DT := h_start_DT;
   End If;
   --trace_printf("h_leveling: %d, h_work: %d, h_start_DT: %d:%s, h_finish_DT: %d:%s, h_duration: %d\n",h_leveling,h_work,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration);
   
   --t_start2 = clock();
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(h_work, 0) = 0 Then
      BEGIN
         INSERT INTO PMO_TMP_CPLNF_0001 SELECT DAY_YEAR, SUM(SCHED_MIN)
         FROM PMO_TMP_TDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
         GROUP BY DAY_YEAR;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 100+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(101) : cplan_fun.sql(5274) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         OPEN CCA16a;
         
         FETCH CCA16a INTO h_dayofyear, h_total;
         IF CCA16a%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("\nCA.3:h_dayofyear = %d %s, h_start_DT = %d, %s, h_total: %d\n", h_dayofyear_Ind,h_dayofyear,h_start_DT_Ind,h_start_DT,h_total);
         
         BEGIN
            CLOSE CCA16a ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 124+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(125) : cplan_fun.sql(5298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (Coalesce(h_total, 0) > 0) AND ((h_dayofyear IS NOT NULL)) Then
            --trace_printf("\nCA.4:");
            If Coalesce(Length(h_start_DT), 0) > 10 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 141+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(142) : cplan_fun.sql(5315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
                  FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = h_dayofyear;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 155+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(156) : cplan_fun.sql(5329) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR) + 1    , PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
               FROM PMO_TMP_CPLNF_0001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 170+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(171) : cplan_fun.sql(5344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("\nCA.5:SC_STATE: %d, h_start_DT: %s\n", SC_STATE,h_start_DT);
            /*
            				BEGIN_SQL
            					SELECT MAX(DAY_YEAR) + 1 DAY INTO HOST(h_dayofyear) FROM session.PMO_TMP_CDY;
            				END_SQL;
            //trace_printf("SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
            
            				BEGIN_SQL
            					INSERT INTO session.PMO_TMP_CDY
            					//removed ASSIGNED_MIN from insertion
            					SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            					FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = HOST(h_project_id) AND DAY_YEAR >= DATE(HOST(h_dayofyear));
            				END_SQL;
            //trace_printf("1__? SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
            
            				if ( SC_NOSUCCESS )
            				{
            					return -1;
            				}*/
         End If;
      ElsIf PMO_DB2M.SC_STATE < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN', 196+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN(197) : cplan_fun.sql(5370) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
   Else
      BEGIN
         INSERT INTO PMO_TMP_CPLNF_0001 SELECT DAY_YEAR, SUM(SCHED_MIN) AS SCHED_MIN
         FROM PMO_TMP_TDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) GROUP BY DAY_YEAR;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 214+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(215) : cplan_fun.sql(5388) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         OPEN CCA16b;
         
         FETCH CCA16b INTO h_dayofyear;
         IF CCA16b%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         h_total := PMO_DB2M.SC_STATE;
         BEGIN
            CLOSE CCA16b ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 238+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(239) : cplan_fun.sql(5412) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("\nCA.0:h_total: %d, h_dayofyear: %d %s\n",h_total,h_dayofyear_Ind,h_dayofyear);
         If (Coalesce(h_total, 0) = 0) AND ((h_dayofyear IS NOT NULL)) Then
            --trace_printf("\nCA.1:h_count: %d, h_res_id: %s, h_start_DT: %s, h_finish_DT: %s\n",h_count,h_res_id,h_start_DT,h_finish_DT);
            If Coalesce(Length(h_start_DT), 0) > 10 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 256+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(257) : cplan_fun.sql(5430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = h_dayofyear;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 271+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(272) : cplan_fun.sql(5445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            --trace_printf("\nCA.2:h_count: %d, h_res_id: %s, h_start_DT: %s, h_finish_DT: %s\n",h_count,h_res_id,h_start_DT,h_finish_DT);
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR) + 1    , PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
               FROM PMO_TMP_CPLNF_0001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 287+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(288) : cplan_fun.sql(5461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_start_DT: %s\n", SC_STATE,h_start_DT);
            /*
            				BEGIN_SQL
            					SELECT MAX(DAY_YEAR) + 1 DAY INTO HOST(h_dayofyear) FROM session.PMO_TMP_CDY;
            				END_SQL;
            //trace_printf("SC_STATE: %d, h_project_id: %s, h_dayofyear: %s\n", SC_STATE,h_project_id,h_dayofyear);
            
            				BEGIN_SQL
            					INSERT INTO session.PMO_TMP_CDY
            					SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            					FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = HOST(h_project_id) AND DAY_YEAR >= DATE(HOST(h_dayofyear));
            				END_SQL;
            //trace_printf("2__? SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
            
            				if ( SC_NOSUCCESS )
            				{
            					return -1;
            				}*/
         End If;
      End If;
   End If;
   
   --trace_printf("h_leveling = %d, h_start_DT = %s, h_level_DT = %s\n", h_leveling,h_start_DT,h_level_DT);
   If Coalesce(h_leveling, 0) = 1 Then
      -- if the new Start_DT > LevelDate requested (by leveler newtwork) reset the start time to 0:00:00
      BEGIN
         SELECT (case when (PMO_ORASTD.DAYS(h_start_DT) - PMO_ORASTD.DAYS(h_level_DT)) > 0 then PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)) , PMO_ORASTD.TIME('00:00:00'))
         else TO_TIMESTAMP(h_start_DT) end)
         INTO h_start_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 327+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(328) : cplan_fun.sql(5501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("LEV_*** SC_STATE = %d, h_start_DT = %s\n", SC_STATE,h_start_DT);
   End If;
   
   If (Coalesce(Length(h_finish_DT), 0) > 0) AND (PMO_ORASTD.strncmp(h_start_DT, h_constr_DT
         , 10) > 0) Then
      If Coalesce(Length(h_finish_DT), 0) > 10 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(h_finish_DT) + (PMO_ORASTD.DAYS(h_start_DT) - PMO_ORASTD.DAYS(h_constr_DT))     
            INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 347+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(348) : cplan_fun.sql(5521) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)) + (PMO_ORASTD.DAYS(h_start_DT) - PMO_ORASTD.DAYS(h_constr_DT))     
            INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 362+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(363) : cplan_fun.sql(5536) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   /*
   if (h_leveling == 1){
   t_total2 = (double)(clock() - t_start2) / CLOCKS_PER_SEC;
   BEGIN_SQL
   	UPDATE session.PMO_TMP_TIMEING SET DURATION = DURATION + HOST(t_total2) WHERE NAME = 'CHECK_ASSIGN/1';
   END_SQL;}
   t_start2 = clock();
   */
   --trace_printf("h_duration = %d, h_consider = %d, h_start_DT = %s, h_finish_DT = %s\n", h_duration,h_consider,h_start_DT,h_finish_DT);
   PMO_DB2M.SC_STATE := 0;
   If (PMO_ORASTD.strncmp(h_start_DT, h_constr_DT, 10) > 0) AND (Coalesce(h_duration, 0) > 0) 
         AND (Coalesce(Length(h_finish_DT), 0) > 0) Then
      h_total := 0;
      BEGIN
         SELECT COALESCE(SUM(SCHED_MIN),0) INTO h_total
         FROM PMO_TMP_CPLNF_0001
         WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 392+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(393) : cplan_fun.sql(5566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE = %d, h_start_DT: %s, h_total = %d, h_res_id: %s\n", SC_STATE, h_start_DT, h_total,h_res_id);
      
      h_totpr := 0;
      BEGIN
         SELECT MAX(DAY_YEAR) + 1     , COALESCE(SUM(AVAILABLE_MIN),0) - h_total
         INTO h_dayofyear, h_totpr
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_res_id AND DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 410+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(411) : cplan_fun.sql(5584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_totpr, 0) < Coalesce(h_effort, 0) Then
         /*BEGIN_SQL
         				SELECT MAX(DAY_YEAR) + 1 DAYS INTO HOST(h_dayofyear) FROM session.PMO_TMP_CDY;
         			END_SQL;*/
         
         -- Extend project calendar
         h_finish_DT := '';
         h_temp_sc_state := EXTEND_PRCAL(h_res_id, h_dayofyear, h_finish_DT, h_duration
                                , PMO_CNST.SC_RESOURCE);
      Else
         If Coalesce(h_total, 0) = 0 Then
            h_dayofyear := SubStr(h_start_DT, 1, 10);
         Else
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_dayofyear FROM PMO_TMP_CPLNF_0001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(437) : cplan_fun.sql(5610) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("*** h_total: %d, h_dayofyear: %d:%s\n", h_total,h_dayofyear_Ind,h_dayofyear);
         
         stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                || 'WHERE ELEMENT_ID = ''' || h_res_id || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_dayofyear 
                     || ''', 1, 10)) '
                || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_duration, 0));
         
         OPEN CCDY0 FOR stmt;
         
         LOOP
            FETCH CCDY0 INTO h_finish_DT;
            IF CCDY0%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CCDY0 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 470+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(471) : cplan_fun.sql(5644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("===> h_finish_DT = %d:%s\n", h_finish_DT_Ind,h_finish_DT);
      End If;
      --trace_printf("h_total: %d, h_effort: %d, h_finish_DT = %d:%s\n", h_total,h_effort, h_finish_DT_Ind,h_finish_DT);
   End If;
   /*
   if (h_leveling == 1){
   t_total2 = (double)(clock() - t_start2) / CLOCKS_PER_SEC;
   BEGIN_SQL
   	UPDATE session.PMO_TMP_TIMEING SET DURATION = DURATION + HOST(t_total2) WHERE NAME = 'CHECK_ASSIGN/2';
   END_SQL;}*/
   --trace_printf("SC_STATE = %d, h_effort = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", SC_STATE,h_effort,h_total,h_start_DT,h_finish_DT);//trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN', 487+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN(488) : cplan_fun.sql(5661) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION CHECK_ASSIGN;
ALTER FUNCTION CHECK_ASSIGN COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION CHECK_ASSIGN_PROF ==
CREATE OR REPLACE FUNCTION CHECK_ASSIGN_PROF
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_consider PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_work PLS_INTEGER := sh_h_work;
   h_effort PLS_INTEGER := sh_h_effort;
   h_consider PLS_INTEGER := sh_h_consider;
   h_constr_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   stwhere VARCHAR2(2999);
   h_temp_sc_state PLS_INTEGER := 0;
   h_leveling PLS_INTEGER := 0;
   h_level_DT VARCHAR2(26);
-- Declare Cursors
      CURSOR CCA15a IS
         SELECT * FROM (SELECT MIN(p.DAY_YEAR) FROM PMO_TMP_CDY p
         WHERE p.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND p.AVAILABLE_MIN > 0 
         ) WHERE ROWNUM<=1;

      CURSOR CCA17a IS
         SELECT * FROM (SELECT MIN(p.DAY_YEAR) FROM PMO_TMP_CDY p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR
         WHERE p.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND p.AVAILABLE_MIN > COALESCE(ta.SCHED_MIN,0) 
         ) WHERE ROWNUM<=1;

      CURSOR CCA17b IS
         SELECT * FROM (SELECT MIN(p.DAY_YEAR) FROM PMO_TMP_CDY p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR
         WHERE p.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND (p.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)) >= h_work 
         ) WHERE ROWNUM<=1;

      CURSOR CCDY3 IS
         SELECT DAY_YEAR FROM PMO_TMP_CDY
         WHERE DAY_YEAR > TO_DATE(SubStr(h_dayofyear, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('CHECK_ASSIGN_PROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('CHECK_ASSIGN_PROF'), '-- ' || SYSTIMESTAMP, ':r := CHECK_ASSIGN_PROF(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN_PROF', 59+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: CHECK_ASSIGN_PROF(60) : cplan_fun.sql(5731) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_work: %d, h_consider: %d, h_start_DT: %d:%s, h_finish_DT: %d:%s\n",h_work,h_consider,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
   stwhere := '';
   h_dayofyear := '';
   h_count := Coalesce(Length(h_finish_DT), 0);
   If Coalesce(h_count, 0) > 0 Then
      h_constr_DT := h_finish_DT;
   End If;
   
   If Coalesce(h_consider, 0) >= PMO_CNST.SC_BIT_30 Then
      h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
      
      h_leveling := 1;
      --      h_work = 30;
      h_level_DT := h_start_DT;
   End If;
   
   If Coalesce(h_work, 0) = 0 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLNF_0001
         SELECT DAY_YEAR,
         SUM(SCHED_MIN) AS SCHED_MIN
         FROM PMO_TMP_TDY
         WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
         GROUP BY DAY_YEAR;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 100+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(101) : cplan_fun.sql(5772) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("PMO_TMP_CPLNF_0001: SC_STATE: %d, h_start_DT: %s\n", SC_STATE,h_start_DT);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         OPEN CCA15a;
         
         FETCH CCA15a INTO h_dayofyear;
         IF CCA15a%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         h_total := PMO_DB2M.SC_STATE;
         BEGIN
            CLOSE CCA15a ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 125+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(126) : cplan_fun.sql(5797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         OPEN CCA17a;
         
         FETCH CCA17a INTO h_dayofyear;
         IF CCA17a%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         h_total := PMO_DB2M.SC_STATE;
         BEGIN
            CLOSE CCA17a ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 148+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(149) : cplan_fun.sql(5820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("h_total: %d, h_dayofyear = %d, %s,h_start_DT = %d, %s\n", h_total,h_dayofyear_Ind,h_dayofyear,h_start_DT_Ind,h_start_DT);
      
      If (Coalesce(h_total, 0) = 0) AND ((h_dayofyear IS NOT NULL)) Then
         If Coalesce(Length(h_start_DT), 0) > 10 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 166+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(167) : cplan_fun.sql(5838) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
               FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(181) : cplan_fun.sql(5852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR) + 1    , PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
            FROM PMO_TMP_CPLNF_0001;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(196) : cplan_fun.sql(5867) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_start_DT: %s\n", SC_STATE,h_start_DT);
         
         BEGIN
            SELECT MAX(DAY_YEAR) + 1     INTO h_dayofyear FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 209+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(210) : cplan_fun.sql(5881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
         
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10))
            AND DAY_YEAR NOT IN (SELECT DAY_YEAR FROM PMO_TMP_CDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 230+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(231) : cplan_fun.sql(5902) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3__? SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN_PROF', 238+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN_PROF(239) : cplan_fun.sql(5910) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
      End If;
   Else
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLNF_0001
         SELECT DAY_YEAR, SUM(SCHED_MIN) AS SCHED_MIN
         FROM PMO_TMP_TDY
         WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
         GROUP BY DAY_YEAR;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 264+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(265) : cplan_fun.sql(5936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN CCA17b;
      
      FETCH CCA17b INTO h_dayofyear;
      IF CCA17b%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      h_total := PMO_DB2M.SC_STATE;
      BEGIN
         CLOSE CCA17b ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 287+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(288) : cplan_fun.sql(5959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (Coalesce(h_total, 0) = 0) AND ((h_dayofyear IS NOT NULL)) Then
         If Coalesce(Length(h_start_DT), 0) > 10 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 303+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(304) : cplan_fun.sql(5975) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
               FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 317+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(318) : cplan_fun.sql(5989) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR) + 1    , PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT
            FROM PMO_TMP_CPLNF_0001;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 332+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(333) : cplan_fun.sql(6004) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_start_DT: %s\n", SC_STATE,h_start_DT);
         
         BEGIN
            SELECT MAX(DAY_YEAR) + 1     INTO h_dayofyear FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 346+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(347) : cplan_fun.sql(6018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_project_id: %s, h_dayofyear: %s\n", SC_STATE,h_project_id,h_dayofyear);
         
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10))
            AND DAY_YEAR NOT IN (SELECT DAY_YEAR FROM PMO_TMP_CDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 367+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(368) : cplan_fun.sql(6039) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4__? SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN_PROF', 375+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN_PROF(376) : cplan_fun.sql(6047) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
      End If;
   End If;
   
   If Coalesce(h_leveling, 0) = 1 Then
      -- if the new Start_DT > LevelDate requested (by leveler newtwork) reset the start time to 0:00:00
      BEGIN
         SELECT (case when (PMO_ORASTD.DAYS(h_start_DT) - PMO_ORASTD.DAYS(h_level_DT)) > 0 then PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)) , PMO_ORASTD.TIME('00:00:00'))
         else TO_TIMESTAMP(h_start_DT) end)
         INTO h_start_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 395+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(396) : cplan_fun.sql(6067) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("LEV_*** SC_STATE = %d, h_start_DT = %s\n", SC_STATE,h_start_DT);
   End If;
   
   --trace_printf("h_duration = %d, h_consider = %d, h_start_DT = %s, h_finish_DT = %s\n", h_duration,h_consider,h_start_DT,h_finish_DT);
   PMO_DB2M.SC_STATE := 0;
   If Coalesce(h_duration, 0) > 0 Then
      h_total := 0;
      BEGIN
         SELECT COALESCE(SUM(SCHED_MIN),0) INTO h_total
         FROM PMO_TMP_CPLNF_0001
         WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 416+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(417) : cplan_fun.sql(6088) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE = %d, h_dayofyear: %s, h_total = %d\n", SC_STATE, h_dayofyear, h_total);
      
      h_totpr := 0;
      BEGIN
         SELECT COALESCE(SUM(AVAILABLE_MIN),0) - h_total INTO h_totpr FROM PMO_TMP_CDY
         WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 432+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(433) : cplan_fun.sql(6104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE = %d, h_totpr: %d, h_effort: %d, h_finish_DT = %d:%s\n", SC_STATE,h_totpr,h_effort, h_finish_DT_Ind,h_finish_DT);
      
      If Coalesce(h_totpr, 0) < Coalesce(h_effort, 0) Then
         BEGIN
            SELECT MAX(DAY_YEAR) + 1      INTO h_dayofyear FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 447+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(448) : cplan_fun.sql(6119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- Extend project calendar
         h_finish_DT := '';
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_project_id, h_dayofyear, h_finish_DT
                                         , h_duration);
      Else
         If Coalesce(h_total, 0) = 0 Then
            h_dayofyear := SubStr(h_start_DT, 1, 10);
         Else
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_dayofyear FROM PMO_TMP_CPLNF_0001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 468+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(469) : cplan_fun.sql(6140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("*** h_total: %d, h_dayofyear: %d:%s\n", h_total,h_dayofyear_Ind,h_dayofyear);
         
         h_totpr := 0;
         
         OPEN CCDY3;
         
         WHILE Coalesce(h_totpr, 0) < Coalesce(h_duration, 0) LOOP
            FETCH CCDY3 INTO h_finish_DT;
            IF CCDY3%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            h_totpr := Coalesce(h_totpr, 0) + 1;
         END LOOP;
         
         BEGIN
            CLOSE CCDY3 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 501+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(502) : cplan_fun.sql(6173) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_totpr = %d, h_duration: %d, h_finish_DT = %d:%s\n", h_totpr,h_duration,h_finish_DT_Ind,h_finish_DT);
         If Coalesce(h_totpr, 0) < Coalesce(h_duration, 0) Then
            BEGIN
               SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)) + 1      INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 516+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(517) : cplan_fun.sql(6188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_dayofyear := '';
            h_totpr := 0;
            h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_project_id, h_finish_DT, h_dayofyear
                                         , h_totpr);
            --trace_printf("*** h_dayofyear = %s\n", h_dayofyear);
            
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) INTO h_finish_DT FROM PMO_TMP_CDY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN_PROF', 535+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN_PROF(536) : cplan_fun.sql(6207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   --trace_printf("SC_STATE = %d, h_effort = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", SC_STATE,h_effort,h_total,h_start_DT,h_finish_DT);//trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN_PROF', 546+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN_PROF(547) : cplan_fun.sql(6218) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION CHECK_ASSIGN_PROF;
ALTER FUNCTION CHECK_ASSIGN_PROF COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION AssignResource ==
CREATE OR REPLACE FUNCTION AssignResource
(
     sh_h_table_type PLS_INTEGER
   , sh_type_id001 PLS_INTEGER
   , id IN OUT NOCOPY VARCHAR2
   , res_id001 IN OUT NOCOPY VARCHAR2
   , rec_user001 IN OUT NOCOPY VARCHAR2
   , h_ra_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   type_id001 PLS_INTEGER := sh_type_id001;
   h_full_name VARCHAR2(255);
   h_profile_name VARCHAR2(255);
   h_project_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_security_id CHAR(32);
   h_participant PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_tmp_security_id VARCHAR2(32);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfer_flag PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('AssignResource') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('AssignResource'), '-- ' || SYSTIMESTAMP, ':r := AssignResource(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_type_id001), 'NULL')
          || ', ' || '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || ''''
          || ', ' || '''' || h_ra_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 46+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: AssignResource(47) : cplan_fun.sql(6275) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- to hold TRANSFER_FLAG corresponding to the transferd requirment
   
   --short h_ra_id_Ind = 0;
   
   h_resource_id := res_id001;
   h_element_id := id;
   h_rec_user := rec_user001;
   h_type_id := type_id001;
   h_participant := PMO_CNST.SC_PARTICIPANT;
   h_full_name := '';
   h_profile_name := '';
   
   BEGIN
      SELECT NICK_NAME INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 71+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(72) : cplan_fun.sql(6300) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("\n 3743 h_full_name %s,type_id: %d\n",h_full_name,type_id);//trace_flush();
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_resource_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 86+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(87) : cplan_fun.sql(6315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 93+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(94) : cplan_fun.sql(6322) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
      
      If isChangeToTask(Coalesce(type_id001, 0)) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 100+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(101) : cplan_fun.sql(6329) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      h_res_type := '1';
   Else
      h_res_type := '0';
   End If;
   
   --trace_printf("\n 3758 h_res_type %s",h_res_type);//trace_flush();
   BEGIN
      SELECT RES_ASSIGNMENT_ID INTO h_res_assignment_id FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = h_element_id AND RESOURCE_TYPE = h_res_type
      AND (RESOURCE_ID = h_resource_id OR PROFILE_ID = h_resource_id);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 121+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(122) : cplan_fun.sql(6350) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_res_assignment_id: %d:%s, h_resource_id: %s\n",SC_STATE,h_res_assignment_id_Ind,h_res_assignment_id,h_resource_id);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      h_ra_id := h_res_assignment_id;
      
      If (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) OR (Coalesce(
             h_table_type, 0) = PMO_CNST.SC_TABLE_CRI) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 133+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(134) : cplan_fun.sql(6362) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 138+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(139) : cplan_fun.sql(6367) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_ALREADY_ASSIGNED;
      End If;
   End If;
   
   If ((h_table_type IS NULL)) OR (Coalesce(h_table_type, 0) = 0) Then
      h_table_type := GetTable(Coalesce(h_type_id, 0));
   End If;
   --trace_printf("h_table_type: %d:%d, h_type_id: %d, h_full_name: %s\n",h_table_type_Ind,h_table_type,h_type_id,h_full_name);
   
   h_transfer_flag := 0;
   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_POL THEN
         --TMT_POOLS
         BEGIN
            SELECT RANK INTO h_rank FROM TMT_POOLS WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 161+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(162) : cplan_fun.sql(6390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 168+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(169) : cplan_fun.sql(6397) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         BEGIN
            SELECT ELEMENT_ID INTO h_project_id FROM TMT_POOLS WHERE RANK = SUBSTR( h_rank, 1, 8) AND PARENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 181+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(182) : cplan_fun.sql(6410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 188+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(189) : cplan_fun.sql(6417) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_PARTICIPANT_GROUP, h_security_id);
         --          InitializeHost(SC_PARAM(h_security_id), "SCPOOLPARTICIPANT");
         
         
      
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         -- For transfered Requirement the type_id in TMT_WBS is 2 or SC_DELIVERABLE, to recognize a Deliverable from a Transfered
         -- requirement, the algorithm need TRANSFER_FLAG from TMT_WBS, if this Flag is 'R' then the object is a transfered requirement
         -- otherwise it is a DELIVERABLE
         -- Fetch TRANSFER_FLAG in TMT_WBS coressponding to the selected element
         BEGIN
            SELECT PROJECT_ID, ASSIGN_COUNT, CASE WHEN TRANSFER_FLAG IS NuLL THEN 0 ELSE 1 END
            INTO h_project_id, h_assign_count, h_transfer_flag
            FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 213+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(214) : cplan_fun.sql(6442) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE = %d, h_element_id = %s, h_assign_count = %d\n", SC_STATE,h_element_id,h_assign_count);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 221+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(222) : cplan_fun.sql(6450) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         BEGIN
            SELECT PROJECT_ID, ASSIGN_COUNT, CASE WHEN WBS_ID IS NuLL THEN 0 ELSE 1 END
            INTO h_project_id, h_assign_count, h_transfer_flag
            FROM TMT_CRI WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 239+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(240) : cplan_fun.sql(6468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 246+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(247) : cplan_fun.sql(6475) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            SELECT PROJECT_ID INTO h_project_id
            FROM TMT_DOCUMENTS   WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 263+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(264) : cplan_fun.sql(6492) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 270+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(271) : cplan_fun.sql(6499) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_ASSIGNED_SCORECARDS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 289+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(290) : cplan_fun.sql(6518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 296+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(297) : cplan_fun.sql(6525) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_POOL_DOCUMENTS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 315+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(316) : cplan_fun.sql(6544) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 322+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(323) : cplan_fun.sql(6551) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_RESOURCE_DOCUMENTS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 341+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(342) : cplan_fun.sql(6570) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 348+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(349) : cplan_fun.sql(6577) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_CLIENT_DOCUMENTS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 367+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(368) : cplan_fun.sql(6596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 374+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(375) : cplan_fun.sql(6603) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_ASSET_DOCUMENTS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 393+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(394) : cplan_fun.sql(6622) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 400+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(401) : cplan_fun.sql(6629) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            SELECT PROJECT_ID
            INTO h_project_id
            FROM TMT_VENDOR_DOCUMENTS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 419+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(420) : cplan_fun.sql(6648) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 426+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(427) : cplan_fun.sql(6655) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
      ELSE
         NULL;
   END CASE;
   
   --trace_printf("h_res_type = %s, h_assign_count = %d\n", h_res_type,h_assign_count);
   If (SubStr(h_res_type, 1, 1) = '0') AND (Coalesce(h_assign_count, 0) < 2) Then
      BEGIN
         SELECT COUNT(*) INTO h_assign_count FROM RESOURCE_ASSIGNMENTS
         WHERE ELEMENT_ID = h_element_id AND RESOURCE_TYPE = h_res_type;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 445+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(446) : cplan_fun.sql(6674) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE = %d, h_assign_count = %d\n", SC_STATE,h_assign_count);
      
      If Coalesce(h_assign_count, 0) = 1 Then
         -- for transfered requirement, SC_WBS_CHANGE_REQUEST, SC_WBS_RISK, SC_WBS_ISSUE
         -- the new resource will be added to the resource assignment list, not replaced by default resource
         -- also, an element by type SC_DELIVERABLE and  TRANSFER_FLAG = 'R' is a transfered requirement
         If (Coalesce(h_type_id, 0) = PMO_CNST.SC_DELIVERABLE) OR (Coalesce(
               h_type_id, 0) = PMO_CNST.SC_WORK_PRODUCT) OR (Coalesce(
               h_type_id, 0) = PMO_CNST.SC_SUMMARY_TASK) Then
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            --trace_printf("\n 5288 h_security_id %s",h_security_id);//trace_flush();
            
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';'))
               INTO h_res_assignment_id,h_profile_name FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.ELEMENT_ID = h_element_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND
               ra.SECURITY_ID = h_security_id AND ra.RESOURCE_ID = tw.CREATED_BY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 473+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(474) : cplan_fun.sql(6702) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE = %d, h_res_assignment_id = %s, h_res_assignment_id_Ind = %d\n", SC_STATE,h_res_assignment_id,h_res_assignment_id_Ind);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_res_assignment_id IS NOT NULL)) Then
               h_participant := PMO_CNST.SC_DLV_MANAGER;
               
               h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id);
               
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 496+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(497) : cplan_fun.sql(6725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE = %d, h_res_assignment_id = %s, DELETED...\n", SC_STATE,h_res_assignment_id);
            End If;
         ElsIf (Coalesce(h_type_id, 0) = PMO_CNST.SC_TASK) OR (Coalesce(
                h_type_id, 0) = PMO_CNST.SC_MILESTONE) Then
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            --trace_printf("\n 5314 h_security_id %s",h_security_id);//trace_flush();
            
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')), tw.ASSIGN_COUNT
               INTO h_res_assignment_id,h_profile_name, h_count
               FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.ELEMENT_ID = h_element_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND
               ra.SECURITY_ID = h_security_id AND ra.RESOURCE_ID = tw.CREATED_BY AND NOT EXISTS(
               SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 521+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(522) : cplan_fun.sql(6750) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE = %d, h_res_assignment_id_Ind = %d, h_res_assignment_id = %s\nh_element_id = %s, h_profile_name = %s\n",SC_STATE,h_res_assignment_id_Ind,h_res_assignment_id,h_element_id,h_profile_name);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_res_assignment_id IS NOT NULL)) Then
               h_participant := PMO_CNST.SC_TASK_RESPONSIBLE;
               
               h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id);
               
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 544+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(545) : cplan_fun.sql(6773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DELETE: SC_STATE = %d, h_res_assignment_id = %s\n",SC_STATE,h_res_assignment_id);
            End If;
         ElsIf (Coalesce(h_type_id, 0) = PMO_CNST.SC_CRI_MILESTONE) OR (Coalesce(
                h_type_id, 0) = PMO_CNST.SC_CRI_TASK) Then
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            --trace_printf("\n 5342 h_security_id %s",h_security_id);//trace_flush();
            
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')), tw.ASSIGN_COUNT
               INTO h_res_assignment_id,h_profile_name, h_count
               FROM TMT_CRI tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.ELEMENT_ID = h_element_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND
               ra.SECURITY_ID = h_security_id AND ra.RESOURCE_ID = tw.CREATED_BY AND NOT EXISTS(
               SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 569+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(570) : cplan_fun.sql(6798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_res_assignment_id IS NOT NULL)) Then
               h_participant := PMO_CNST.SC_TASK_RESPONSIBLE;
               
               h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id);
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 590+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(591) : cplan_fun.sql(6819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf (Coalesce(h_type_id, 0) = PMO_CNST.SC_CRI_SUMMARY_TASK) OR (Coalesce(
                h_type_id, 0) = PMO_CNST.SC_CRI_DELIVERABLE) Then
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            --trace_printf("\n 5370 h_security_id %s",h_security_id);//trace_flush();
            
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';'))
               INTO h_res_assignment_id,h_profile_name FROM TMT_CRI tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.ELEMENT_ID = h_element_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND
               ra.SECURITY_ID = h_security_id AND ra.RESOURCE_ID = tw.CREATED_BY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 612+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(613) : cplan_fun.sql(6841) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_res_assignment_id IS NOT NULL)) Then
               h_participant := PMO_CNST.SC_DLV_MANAGER;
               
               h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id);
               
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 634+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(635) : cplan_fun.sql(6863) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            h_profile_name := NULL;
         End If;
      End If;
   End If;
   --trace_printf("h_table_type: %d, h_participant: %d, h_type_id: %d, h_profile_name: %d:%s, h_full_name: %d:%s\n", h_table_type,h_participant,h_type_id,h_profile_name_Ind,h_profile_name,h_full_name_Ind,h_full_name);
   
   --  h_table_type = GetTable(h_type_id);
   
   /*  DO NOTHING IN DB2 BUT SCREWS UP ORACLE TRANSLATION
   
   	if ( (strlen(h_profile_name) > 0)  (strlen(h_profile_name) < strlen(h_full_name)) )
   	{
   		strncpy(h_full_name, h_full_name, strlen(h_profile_name)-1);
   	}
   */
   
   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         If (Coalesce(h_participant, 0) = PMO_CNST.SC_DLV_MANAGER) OR (Coalesce(
               h_participant, 0) = PMO_CNST.SC_TASK_RESPONSIBLE) Then
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES =
               CASE
               WHEN InStr( ASSIGN_NAMES, h_profile_name) > 0
               THEN REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name || ';')
               ELSE h_full_name || ';' || ASSIGN_NAMES
               END
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 678+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(679) : cplan_fun.sql(6907) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES =
               CASE
               WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
               ELSE
               CASE
               WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
               ELSE ASSIGN_NAMES || h_full_name || ';'
               END
               END
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 704+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(705) : cplan_fun.sql(6933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("WBS_SC_STATE: %d, h_participant: %d, h_element_id: %s, h_res_type: %s\n",SC_STATE,h_participant,h_element_id,h_res_type);
         
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         If Coalesce(h_participant, 0) = PMO_CNST.SC_DLV_MANAGER Then
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES =
               CASE
               WHEN InStr( ASSIGN_NAMES, h_profile_name) > 0
               THEN REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name || ';')
               ELSE h_full_name || ';' || ASSIGN_NAMES
               END
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 733+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(734) : cplan_fun.sql(6962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_participant, 0) = PMO_CNST.SC_TASK_RESPONSIBLE Then
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TMT_CRI SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_profile_name) > 0
                  THEN REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name || ';')
                  ELSE h_full_name || ';' || ASSIGN_NAMES
                  END
                  WHERE ELEMENT_ID = h_element_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 757+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(758) : cplan_fun.sql(6986) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_CRI SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_profile_name) > 0
                  THEN REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name || ';')
                  ELSE h_full_name || ';' || ASSIGN_NAMES
                  END
                  WHERE ELEMENT_ID = h_element_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 780+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(781) : cplan_fun.sql(7009) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES =
               CASE
               WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
               ELSE
               CASE
               WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
               ELSE ASSIGN_NAMES || h_full_name || ';'
               END
               END
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 807+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(808) : cplan_fun.sql(7036) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("CRI_SC_STATE: %d, h_participant: %d, h_element_id: %s, h_res_type: %s\n",SC_STATE,h_participant,h_element_id,h_res_type);
         
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            UPDATE TMT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 838+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(839) : cplan_fun.sql(7067) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("doc_SC_STATE: %d, h_participant: %d, h_element_id: %s, h_res_type: %s\n",SC_STATE,h_participant,h_element_id,h_res_type);
         
      
      WHEN PMO_CNST.SC_TABLE_POL THEN
         BEGIN
            UPDATE TMT_POOLS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE CASE WHEN ASSIGN_NAMES is Null THEN  h_full_name || ';'
            ELSE ASSIGN_NAMES || h_full_name || ';' END
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 868+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(869) : cplan_fun.sql(7097) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            UPDATE TMT_POOL_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 896+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(897) : cplan_fun.sql(7125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            UPDATE TMT_RESOURCE_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 924+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(925) : cplan_fun.sql(7153) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            UPDATE TMT_CLIENT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 952+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(953) : cplan_fun.sql(7181) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            UPDATE TMT_ASSET_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 980+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(981) : cplan_fun.sql(7209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            UPDATE TMT_VENDOR_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 1008+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(1009) : cplan_fun.sql(7237) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 1036+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(1037) : cplan_fun.sql(7265) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("doc_SC_STATE: %d, h_participant: %d, h_element_id: %s, h_res_type: %s\n",SC_STATE,h_participant,h_element_id,h_res_type);
      ELSE
         NULL;
   END CASE;
   --trace_printf("h_participant: %d, h_type_id: %d, h_resource_id: %s, h_security_id: %s, h_table_type: %d\n",h_participant,h_type_id,h_resource_id,h_security_id,h_table_type);//trace_flush();
   
   If Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
      h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_PARTICIPANT_GROUP, h_security_id);
   End If;
   --trace_printf("\n 5175 h_table_type: %d, h_transfer_flag: %d",h_table_type,h_transfer_flag);//trace_flush();
   --trace_printf("\n 5175 h_type_id: %d, h_security_id: %s\n",h_type_id,h_security_id);//trace_flush();
   
   If Coalesce(h_transfer_flag, 0) = 1 Then
      h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_element_id, h_type_id, h_security_id, h_cri_security_id);
   Else
      h_cri_security_id := NULL;
   End If;
   
   h_temp_participant := PMO_CNST.SC_PARTICIPANT;
   h_temp_security_id := h_security_id;
   h_temp_bitflag := 1;
   
   h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_resource_id, h_type_id, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
   
   BEGIN
      INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
      RESOURCE_ID,PROFILE_ID,PARTICIPANT,SECURITY_ID, CRI_SECURITY_ID, REC_USER,REC_DATETIME,TABLE_TYPE)
      VALUES ( h_ra_id,h_project_id,h_element_id,h_type_id,h_res_type,
      CASE WHEN h_res_type = '0' THEN h_resource_id ELSE NuLL END,
      CASE WHEN h_res_type = '1' THEN h_resource_id ELSE NuLL END,
      h_participant, h_security_id, h_cri_security_id, h_rec_user, SYSTIMESTAMP, h_table_type);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 1083+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(1084) : cplan_fun.sql(7312) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_participant: %d, h_security_id: %s, h_cri_security_id: %s\n",SC_STATE,h_participant,h_security_id, h_cri_security_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 1090+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: AssignResource(1091) : cplan_fun.sql(7319) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION AssignResource;
ALTER FUNCTION AssignResource COMPILE DEBUG;

/*************************************************************************************************************************************************************/

Prompt == FUNCTION U_TSKRES2 ==
CREATE OR REPLACE FUNCTION U_TSKRES2
(
     assign_id IN OUT NOCOPY VARCHAR2
   , project_id001 IN OUT NOCOPY VARCHAR2
   , task_id001 IN OUT NOCOPY VARCHAR2
   , res_id001 IN OUT NOCOPY VARCHAR2
   , and_roles IN OUT NOCOPY VARCHAR2
   , and_skills IN OUT NOCOPY VARCHAR2
   , start_date001 IN OUT NOCOPY VARCHAR2
   , finish_date001 IN OUT NOCOPY VARCHAR2
   , sh_effort001 PLS_INTEGER
   , sh_work001 PLS_INTEGER
   , sh_eunit PLS_INTEGER
   , sh_unit PLS_INTEGER
   , res_type IN OUT NOCOPY VARCHAR2
   , sh_work_type PLS_INTEGER
   , sh_bitflag001 PLS_INTEGER
   , sh_contour001 PLS_INTEGER
   , sh_contour_type001 PLS_INTEGER
   , sh_occurrence PLS_INTEGER
   , sh_consider001 PLS_INTEGER
   , assign_string001 IN OUT NOCOPY VARCHAR2
   , rec_user001 IN OUT NOCOPY VARCHAR2
   , sh_hrs_per_day PLS_INTEGER
   , sh_p_reg_sale NUMBER := 0.00
   , sh_p_spec_sale NUMBER := 0.00
) RETURN PLS_INTEGER
AS
   effort001 PLS_INTEGER := sh_effort001;
   work001 PLS_INTEGER := sh_work001;
   eunit PLS_INTEGER := sh_eunit;
   unit PLS_INTEGER := sh_unit;
   work_type PLS_INTEGER := sh_work_type;
   bitflag001 PLS_INTEGER := sh_bitflag001;
   contour001 PLS_INTEGER := sh_contour001;
   contour_type001 PLS_INTEGER := sh_contour_type001;
   occurrence PLS_INTEGER := sh_occurrence;
   consider001 PLS_INTEGER := sh_consider001;
   hrs_per_day PLS_INTEGER := sh_hrs_per_day;
   p_reg_sale NUMBER := Coalesce(sh_p_reg_sale, 0);
   p_spec_sale NUMBER := Coalesce(sh_p_spec_sale, 0);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_created_by VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_PA_SD DATE;
   h_PA_FD DATE;
   h_dayofyear DATE;
   h_first_time VARCHAR2(8);
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_lev_start_DT VARCHAR2(26);
   h_role_id VARCHAR2(32);
   h_and_roles VARCHAR2(4000);
   h_and_skills VARCHAR2(4000);
   h_res_type VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h1_tsk_assignment_id VARCHAR2(32);
   h1_project_id VARCHAR2(32);
   h_res1_id VARCHAR2(32);
   h_min VARCHAR2(19);
   tempStr VARCHAR2(255);
   h_labor_code VARCHAR2(50);
   h_full_name VARCHAR2(255);
   h_creator_name VARCHAR2(255);
   h_timecode_1_id VARCHAR2(32);
   h_timecode_2_id VARCHAR2(32);
   h_same_labor PLS_INTEGER := 0;
   h_fs_project PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_transfered_cri PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_assigned PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_planned_work PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_spec_cost NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_rate_echange NUMBER := 1.00;
   h_prj_rate_echange NUMBER := 1.00;
   h_budget NUMBER := 0.00;
   h_assign_string VARCHAR2(8192);
   h_proj_assign_flag PLS_INTEGER := 0;
   h_change_flag PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_u_etc PLS_INTEGER := 0;
   h_rate_adjustment PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_sc_state PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_security_id VARCHAR2(32);
   h_temp_security_id VARCHAR2(32);
   h_pa_timephase VARCHAR2(1);
   h_rate_edited PLS_INTEGER := 0;
   h_old_type_id PLS_INTEGER := 0;
   h_bitID RAW(36) := PMO_CNST.SC_ZERO_BIT;
   h_bitID0 RAW(36) := PMO_CNST.SC_ZERO_BIT;
   assign_to_prj PLS_INTEGER := 0;
   l_data_flag PLS_INTEGER;
   token LONG;
   pos LONG;
   result PLS_INTEGER;
   numItems PLS_INTEGER;
-- Declare Cursors
      CURSOR C43 IS
         SELECT * FROM (SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND PROJECT_ID <> TASK_ID AND RESOURCE_ID = h_res_id AND PROJECT_ASSIGNMENT_ID IS NuLL
         ) WHERE ROWNUM<=1;

      CURSOR C41 IS
         SELECT PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '0';

      CURSOR C40 IS
         SELECT PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := U_TSKRES2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || project_id001 || ''''
          || ', ' || '''' || task_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_unit), 'NULL')
          || ', ' || '''' || res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_contour_type001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_consider001), 'NULL')
          || ', ' || '''' || assign_string001 || ''''
          || ', ' || '''' || rec_user001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 182+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_TSKRES2(183) : cplan_fun.sql(7512) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_update := 0;
   h_table_type := 0;
   h_task_type := 0;
   
   h_timecode_1_id := NULL;
   h_timecode_2_id := NULL;
   h_lev_start_DT := NULL;
   h_prj_assignment_id := NULL;
   
   h_project_id := project_id001;
   h_task_id := task_id001;
   h_res_id := res_id001;
   h_res_type := res_type;
   h_start_date := start_date001;
   h_finish_date := finish_date001;
   h_assign_string := assign_string001;
   h_rec_user := rec_user001;
   h_billable := 0;
   h_capital_expense := 0;
   h_rate_adjustment := 10000;
   
   If Coalesce(Length(h_assign_string), 0) > PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 210+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(211) : cplan_fun.sql(7540) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   h_effort := Coalesce(effort001, 0);
   h_bitflag := Coalesce(bitflag001, 0);
   h_work_type := Coalesce(work_type, 0);
   h_consider := Coalesce(consider001, 0);
   h_eunit := Coalesce(eunit, 0);
   h_unit := Coalesce(unit, 0);
   h_work := Coalesce(work001, 0);
   h_contour := Coalesce(contour001, 0);
   h_contour_type := Coalesce(contour_type001, 0);
   h_occurrence := Coalesce(occurrence, 0);
   h_per_day := Coalesce(hrs_per_day, 0);
   
   assign_to_prj := 1;
   h_SD := '';
   h_FD := '';
   
   assign_to_prj := PMO_ORASTD.strcmp(h_project_id, h_task_id);
   
   If Coalesce(h_consider, 0) >= PMO_CNST.SC_BIT_30 Then
      h_assigned := 1;
      -- this is a search assign (type > 0)
      h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove search assign (type > 0)
   Else
      h_assigned := 0;
      -- this is not a search assign (type == 0)
   End If;
   
   --trace_printf("h_assigned: %d, h_bitflag: %d, h_work_type: %d, h_start_date: %d %s, h_finish_date: %d %s\n",h_assigned,h_bitflag,h_work_type,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   --trace_printf("assign_id: %s, h_work: %d, h_consider: %d, h_effort: %d, h_per_day: %d, h_assign_string: %s\n", assign_id,h_work,h_consider,h_effort,h_per_day,h_assign_string);
   --trace_printf("h_task_id: %d:%s, h_res_id: %d:%s\n",h_task_id_Ind,h_task_id,h_res_id_Ind,h_res_id);
   --trace_printf("TT_1:h_prj_assignment_id: %d:%s, h_unit: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_unit);
   --trace_printf("p_reg_sale = %.4f, p_spec_sale = %.4f\n",p_reg_sale, p_spec_sale);
   
   BEGIN
      SELECT PMO_ORASTD.DAYS(h_finish_date) - PMO_ORASTD.DAYS(h_start_date)
      INTO h_total
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 259+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(260) : cplan_fun.sql(7589) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_total, 0) < 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 266+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(267) : cplan_fun.sql(7596) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
   End If;
   
   h_u_etc := 0;
   If Coalesce(h_bitflag, 0) = 4 Then
      h_rollup := 3;
      If Coalesce(h_work_type, 0) = 2 Then
         h_bitflag := 2;
      Else
         h_bitflag := 0;
      End If;
      
      --trace_printf("1_* h_percent_units: %d, h_unit: %d, h_work: %d, h_per_day: %d\n", h_percent_units,h_unit,h_work,h_per_day);
      /*if (h_unit == 101)
      		{
      			h_min_per_day = h_eunit;
      			h_percent_units = h_work;
      			h_work = 0;
      			h_unit = 0;
      			h_task_type = SC_FD_EFFORT_DRIVEN;
      		}
      		else */
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 0) AND (Coalesce(h_per_day, 0) > 0) Then
         h_percent_units := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.499);
      End If;
      --trace_printf("2_* h_percent_units: %d, h_unit: %d, h_work: %d, h_per_day: %d\n", h_percent_units,h_unit,h_work,h_per_day);
      
      h_tsk_assignment_id := assign_id;
      -- TSK_ASSIGNMENT_ID
      h_prj_assignment_id := h_tsk_assignment_id;
   ElsIf (BITAND(Coalesce(h_bitflag, 0), 255)) = 10 Then
      -- else if this leveling do ....... (update our task_assign.level_xxxx  one stmt) also "h_rollup = 2;" to skip rollup
      
      h_tsk_assignment_id := assign_id;
      -- TSK_ASSIGNMENT_ID
      h_lev_start_DT := h_start_date;
      h_rollup := 3;
   Else
      h_rollup := 0;
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
         h_from_prj := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_30)), 29);
         -- set assign from constrained by project (type > 0)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
         -- remove assign from constrained by project (type > 0)
         
         If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
            h_rollup := 3;
            -- set do not rollup
            h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29));
            -- remove do not rollup
         Else
            h_rollup := 0;
            NULL;
         End If;
      Else
         h_from_prj := 0;
         NULL;
      End If;
      
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
         h_rollup := 2;
         -- set do not rollup (import from MSP)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
         -- remove do not rollup
      ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27 Then
         h_rollup := 2;
         -- set do not rollup (import from MSP)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
         -- remove do not rollup
         h_u_etc := 1;
      End If;
      --trace_printf("==== h_u_etc: %d, h_bitflag: %d, h_rollup: %d,h_task_id: %d %s, h_res_type: %s\n",h_u_etc,h_bitflag,h_rollup,h_task_id_Ind,h_task_id,h_res_type);
      
      If (Coalesce(h_rollup, 0) = 3) OR (Coalesce(h_assigned, 0) = 1) Then
         h_tsk_assignment_id := assign_id;
         -- TSK_ASSIGNMENT_ID
         If Coalesce(h_assigned, 0) = 1 Then
            If SubStr(h_res_type, 1, 1) = '0' Then
               BEGIN
                  SELECT RES_ASSIGNMENT_ID, ELEMENT_TYPE, TABLE_TYPE, PARTICIPANT, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
                  INTO h_assign_id,h_type_id, h_table_type, h_participant, h_security_id, h_transfered_cri
                  FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 356+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(357) : cplan_fun.sql(7686) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT RES_ASSIGNMENT_ID, ELEMENT_TYPE, TABLE_TYPE, PARTICIPANT, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
                  INTO h_assign_id,h_type_id, h_table_type, h_participant, h_security_id, h_transfered_cri
                  FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_ID = h_task_id AND PROFILE_ID = h_res_id AND RESOURCE_TYPE = '1';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 371+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(372) : cplan_fun.sql(7701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("SC_STATE: %d, h_security_id: %s, h_type_id: %d, h_table_type: %d:%d, h_participant: %d, h_transfered_cri: %d\n",SC_STATE,h_security_id,h_type_id,h_table_type_Ind,h_table_type,h_participant,h_transfered_cri);
            --trace_printf("h_task_id: %d:%s, h_res_id: %d:%s\n",h_task_id_Ind,h_task_id,h_res_id_Ind,h_res_id);
            
            If ((h_table_type IS NULL)) OR (Coalesce(h_table_type, 0) = 0) Then
               h_table_type := GetTable(Coalesce(h_type_id, 0));
            End If;
         End If;
      Else
         h_assign_id := assign_id;
         -- RES_ASSIGNMENT_ID
         
         If Coalesce(h_update, 0) = 0 Then
            --Get a unique ID
            h_tsk_assignment_id := SYS_GUID();
            h_prj_assignment_id := h_tsk_assignment_id;
            
            BEGIN
               SELECT ELEMENT_TYPE, TABLE_TYPE, PARTICIPANT, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
               INTO h_type_id, h_table_type, h_participant, h_security_id, h_transfered_cri
               FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 402+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(403) : cplan_fun.sql(7732) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_security_id: %s, h_type_id: %d, h_table_type: %d:%d, h_participant: %d, h_transfered_cri: %d\n",SC_STATE,h_security_id,h_type_id,h_table_type_Ind,h_table_type,h_participant,h_transfered_cri);
            
            If ((h_table_type IS NULL)) OR (Coalesce(h_table_type, 0) = 0) Then
               h_table_type := GetTable(Coalesce(h_type_id, 0));
            End If;
         End If;
         --trace_printf("h_update: %d, h_type_id: %d, h_table_type: %d:%d, h_participant: %d\n",h_update,h_type_id,h_table_type_Ind,h_table_type,h_participant);
      End If;
      
      --trace_printf("==== h_assign_id: %s\n",h_assign_id);
      --trace_printf("assign_to_prj: %d, h_update: %d, h_type_id: %d, h_table_type: %d, h_task_type: %d\n",assign_to_prj,h_update,h_type_id,h_table_type,h_task_type);
      
      If isChangeToTask(Coalesce(h_type_id, 0)) <> 0 Then
         If isPureCRIDeliv(Coalesce(h_type_id, 0)) <> 0 Then
            h_id := NULL;
            BEGIN
               SELECT ELEMENT_ID INTO h_id
               FROM PMO_DEFAULTS
               WHERE PROJECT_ID = h_project_id AND TYPE_ID = h_type_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 431+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(432) : cplan_fun.sql(7761) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_id: %d %s\n", SC_STATE,h_id_Ind,h_id);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(
                  h_task_id, h_id) = 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 440+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(441) : cplan_fun.sql(7770) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI;
            End If;
         End If;
         
         h_temp_sc_state := FC_CHANGE_TYPE_BY_ASSIGN(h_type_id, h_task_id, h_rec_user);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 450+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(451) : cplan_fun.sql(7780) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         If BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_PARTICIPANT) <> 0 Then
            h_participant := Coalesce(h_participant, 0) - PMO_CNST.SC_PARTICIPANT;
         End If;
         
         h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
         
         --          h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
         If Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
            If Coalesce(h_transfered_cri, 0) = 0 Then
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
               
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 480+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(481) : cplan_fun.sql(7810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
               
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 503+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(504) : cplan_fun.sql(7833) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      ElsIf (Coalesce(assign_to_prj, 0) <> 0) AND (Coalesce(h_update, 0) = 0) AND (isTask(Coalesce(
                h_type_id, 0))) <> 0 AND (SubStr(h_res_type, 1, 1) = '0') Then
         --timeLog (file,"1_ update created_by:");
         
         If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
            h_created_by := '';
            h_creator_name := '';
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, ra.RESOURCE_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')), tw.TIMECODE_1_ID, tw.TIMECODE_2_ID
               INTO h_id,h_created_by,h_creator_name, h_timecode_1_id, h_timecode_2_id
               FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.PROJECT_ID = h_project_id AND tw.ELEMENT_ID = h_task_id
               AND tw.ELEMENT_ID = ra.ELEMENT_ID AND ra.RESOURCE_ID = tw.CREATED_BY AND ra.RESOURCE_ID <> h_res_id AND ra.PARTICIPANT = 32
               AND ASSIGN_COUNT = 0;--NOT EXISTS(SELECT ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 529+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(530) : cplan_fun.sql(7859) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("WBS: SC_STATE: %d, h_id: %d:%s, h_assign_id: %s\nh_task_type: %d, h_task_id: %s, h_created_by: %s, h_full_name: %s\n",SC_STATE,h_id_Ind,h_id,h_assign_id,h_task_type,h_task_id,h_created_by,h_full_name);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_id IS NOT NULL)) Then
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 548+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(549) : cplan_fun.sql(7878) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DELETE: SC_STATE = %d, h_id = %s\n",SC_STATE,h_id);
               
               BEGIN
                  SELECT NICK_NAME || ';' INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 562+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(563) : cplan_fun.sql(7892) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_full_name,'')
                  WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 580+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(581) : cplan_fun.sql(7910) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_creator_name) > 0
                  THEN REPLACE (ASSIGN_NAMES, h_creator_name, h_full_name)
                  ELSE h_full_name || ASSIGN_NAMES
                  END
                  WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 603+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(604) : cplan_fun.sql(7933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_transfered_cri, 0) = 0 Then
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_RESPONSIBLE_GROUP, SC_PARAM(h_security_id));
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                  End If;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = 48, SECURITY_ID= h_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 632+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(633) : cplan_fun.sql(7962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, FC_MIRROR_TYPE(SC_PARAM(h_type_id)), SC_RESPONSIBLE_GROUP, SC_PARAM(h_security_id));
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  End If;
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = 48, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 664+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(665) : cplan_fun.sql(7994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("2_* SC_STATE: %d,h_security_id: %s, h_temp_security_id: %s\n", SC_STATE, h_security_id,h_temp_security_id);
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_created_by, h_temp_res_type);
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_res_id, h_temp_res_type);
            Else
               BEGIN
                  SELECT  tw.TIMECODE_1_ID, tw.TIMECODE_2_ID
                  INTO h_timecode_1_id, h_timecode_2_id
                  FROM TMT_WBS tw
                  WHERE tw.PROJECT_ID = h_project_id AND tw.ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 688+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(689) : cplan_fun.sql(8018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_PARTICIPANT)) > 0 Then
                  h_participant := PMO_CNST.SC_TEAMMEMBER;
               Else
                  h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
               End If;
               
               If Coalesce(h_transfered_cri, 0) = 0 Then
                  If Coalesce(h_participant, 0) = PMO_CNST.SC_TEAMMEMBER Then
                     --                          h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
                     If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                           AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                        h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     Else
                        h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     End If;
                     
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 723+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(724) : cplan_fun.sql(8053) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 740+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(741) : cplan_fun.sql(8070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  End If;
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
                  If Coalesce(h_participant, 0) = PMO_CNST.SC_TEAMMEMBER Then
                     --                          h_temp_sc_state = GetSecurityID(h_table_type, FC_MIRROR_TYPE(SC_PARAM(h_type_id)), SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
                     
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
                        WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 775+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(776) : cplan_fun.sql(8105) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant
                        WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 793+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(794) : cplan_fun.sql(8123) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               --trace_printf("3.2_* SC_STATE: %d, h_participant: %d, h_security_id: %s, h_temp_security_id: %s\n", SC_STATE, h_participant,h_security_id,h_temp_security_id);
            End If;
            
            If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 816+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(817) : cplan_fun.sql(8146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT ra.RES_ASSIGNMENT_ID, ra.RESOURCE_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';'))
               INTO h_id,h_created_by,h_creator_name FROM TMT_CRI tw, RESOURCE_ASSIGNMENTS ra
               WHERE tw.PROJECT_ID = h_project_id AND tw.ELEMENT_ID = h_task_id
               AND tw.ELEMENT_ID = ra.ELEMENT_ID AND ra.RESOURCE_ID = tw.CREATED_BY AND ra.PARTICIPANT = 32
               AND NOT EXISTS(SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 834+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(835) : cplan_fun.sql(8164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("CRI: SC_STATE = %d, h_id_Ind = %d, h_id = %s, h_assign_id = %s\nh_task_id = %s, h_created_by = %s, h_full_name = %s\n",SC_STATE,h_id_Ind,h_id,h_assign_id,h_task_id,h_created_by,h_full_name);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_id IS NOT NULL)) Then
               BEGIN
                  DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 853+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(854) : cplan_fun.sql(8183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DELETE: SC_STATE = %d, h_id = %s\n",SC_STATE,h_id);
               
               BEGIN
                  SELECT NICK_NAME || ';' INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 867+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(868) : cplan_fun.sql(8197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_CRI SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_full_name,'') WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 884+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(885) : cplan_fun.sql(8214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_CRI SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_creator_name) > 0
                  THEN REPLACE (ASSIGN_NAMES, h_creator_name, h_full_name)
                  ELSE h_full_name || ASSIGN_NAMES
                  END
                  WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 907+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(908) : cplan_fun.sql(8237) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_transfered_cri, 0) = 0 Then
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_RESPONSIBLE_GROUP, SC_PARAM(h_security_id));
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                  End If;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = 48, SECURITY_ID= h_security_id WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 935+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(936) : cplan_fun.sql(8265) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, FC_MIRROR_TYPE(SC_PARAM(h_type_id)), SC_RESPONSIBLE_GROUP, SC_PARAM(h_security_id));
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  End If;
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = 48, CRI_SECURITY_ID= h_security_id, SECURITY_ID= h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 967+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(968) : cplan_fun.sql(8297) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_created_by, h_temp_res_type);
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_res_id, h_temp_res_type);
            Else
               If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_PARTICIPANT)) > 0 Then
                  h_participant := PMO_CNST.SC_TEAMMEMBER;
               Else
                  h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
               End If;
               
               If Coalesce(h_transfered_cri, 0) = 0 Then
                  If Coalesce(h_participant, 0) = PMO_CNST.SC_TEAMMEMBER Then
                     --                          h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
                     If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                           AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                        h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     Else
                        h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     End If;
                     
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1009+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1010) : cplan_fun.sql(8339) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1026+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1027) : cplan_fun.sql(8356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                        AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  Else
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
                     h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  End If;
                  --                      h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
                  If Coalesce(h_participant, 0) = PMO_CNST.SC_TEAMMEMBER Then
                     --                          h_temp_sc_state = GetSecurityID(h_table_type, FC_MIRROR_TYPE(SC_PARAM(h_type_id)), SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
                     
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
                        WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1061+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1062) : cplan_fun.sql(8391) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant
                        WHERE RES_ASSIGNMENT_ID = h_assign_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1079+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1080) : cplan_fun.sql(8409) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               --trace_printf("3.2_* SC_STATE: %d, h_participant: %d, h_security_id: %s, h_temp_security_id: %s\n", SC_STATE, h_participant,h_security_id,h_temp_security_id);
            End If;
            --trace_printf("2_* SC_STATE: %d, h_transfered_cri: %d, h_security_id: %s, h_temp_security_id: %s\n", SC_STATE,h_transfered_cri,h_security_id,h_temp_security_id);
         End If;
         --timeLog (file,"2_ update created_by:");
      ElsIf (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_AFFECTED)) <> PMO_CNST.SC_AFFECTED Then
         --trace_printf("PA.1_* h_participant: %d, h_security_id: %d:%s\n", h_participant,h_security_id_Ind,h_security_id);
         If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_PARTICIPANT)) > 0 Then
            If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
               h_participant := PMO_CNST.SC_AFFECTED;
            Else
               h_participant := PMO_CNST.SC_TEAMMEMBER;
            End If;
            
            --              h_temp_sc_state = GetSecurityID(h_table_type, h_type_id, SC_TEAMMEMBER_GROUP, SC_PARAM(h_security_id));
            If (isCRITask(Coalesce(h_type_id, 0)) <> 0 OR isCRIDeliv(Coalesce(h_type_id, 0)) <> 0) 
                  AND (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) Then
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
            Else
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
            End If;
         Else
            If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
               h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_AFFECTED);
            Else
               h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
            End If;
         End If;
         
         If Coalesce(h_transfered_cri, 0) = 0 Then
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1129+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1130) : cplan_fun.sql(8459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_task_id, h_type_id, h_security_id, h_temp_security_id);
            --h_temp_sc_state = GetSecurityID(h_table_type, FC_MIRROR_TYPE(SC_PARAM(h_type_id)), SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
               WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1149+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1150) : cplan_fun.sql(8479) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("* h_temp_security_id: %s\n", h_temp_security_id);
         End If;
         --trace_printf("3_* SC_STATE: %d, h_participant: %d, h_security_id: %s\n", SC_STATE, h_participant,h_security_id);
         
         If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1171+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1172) : cplan_fun.sql(8501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      --trace_printf("1_** h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id, h_tsk_assignment_id_Ind,h_tsk_assignment_id);
      
      If Coalesce(h_bitflag, 0) = 3 Then
         h_bitflag := 2;
      End If;
      
      --      if ( (h_rollup != 2)  (h_work > 0) )
      --          h_unit = int(h_work * 4.8 + .5);
      
      --trace_printf("h_bitflag = %d, h_effort = %d, h_work = %d, h_rollup = %d, h_unit = %d\n",h_bitflag,h_effort,h_work,h_rollup,h_unit);
      --trace_printf("h_from_prj = %d, assign_to_prj = %d, h_rollup = %d, h_work_type = %d\n",h_from_prj,assign_to_prj,h_rollup,h_work_type);
      
      If (Coalesce(h_from_prj, 0) = 0) AND (Coalesce(assign_to_prj, 0) = 0) Then
         OPEN C43;
         
         FETCH C43 INTO h_id;
         IF C43%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               CLOSE C43 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1209+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1210) : cplan_fun.sql(8539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 1215+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(1216) : cplan_fun.sql(8545) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_;
         Else
            BEGIN
               CLOSE C43 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1227+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1228) : cplan_fun.sql(8557) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         h_prj_assignment_id := NULL;
         NULL;
      End If;
      --h_prj_assignment_id_Ind = -1;
      
      h_total := 0;
      h_cri := 0;
      h_priority := 0;
      --trace_printf("2_** h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id, h_tsk_assignment_id_Ind,h_tsk_assignment_id);
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         BEGIN
            SELECT CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), tw.PRIORITY, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4), tw.CURRENCY_ID
            INTO h_fs_project, h_min_per_day, h_priority, h_rate_echange, h_currency_id
            FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
            WHERE tw.ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1255+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1256) : cplan_fun.sql(8585) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT CURRENCY_ID INTO h_currency_id
            FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1269+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1270) : cplan_fun.sql(8599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), tw.PRIORITY,
            PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4), CASE WHEN (h_currency_id = tw.CURRENCY_ID) OR (h_currency_id * tw.CURRENCY_ID = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = tw.CURRENCY_ID) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) END
            INTO h_fs_project, h_min_per_day, h_priority,
            h_prj_rate_echange, h_rate_echange
            FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
            WHERE tw.ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1287+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1288) : cplan_fun.sql(8617) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("SC_STATE: %d, h_currency_id: %d, h_rate_echange = %.4f\n",SC_STATE,h_currency_id,h_rate_echange);
      --trace_printf("SC_STATE: %d, h_from_prj: %d, assign_to_prj: %d, h_work_type: %d\n",SC_STATE,h_from_prj,assign_to_prj,h_work_type);
      
      -- get the project assignment id for project assignment
      If Coalesce(h_from_prj, 0) > 0 Then
         If Coalesce(h_work_type, 0) = 2 Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), CAPITAL_EXPENSE, BILLABLE,
               REGULAR_RATE, SPECIAL_RATE, REGULAR_COST, SPECIAL_COST, LEVEL_START, RATE_ADJUSTMENT
               INTO h_prj_assignment_id, h_PA_SD, h_PA_FD, h_capital_expense, h_billable,
               h_reg_sale , h_spec_sale, h_reg_cost , h_spec_cost, h_lev_start_DT, h_rate_adjustment
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '2' AND
               TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1312+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1313) : cplan_fun.sql(8642) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_work_type, 0) = 0 Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), CAPITAL_EXPENSE, BILLABLE,
               REGULAR_RATE, SPECIAL_RATE, REGULAR_COST, SPECIAL_COST, LEVEL_START, RATE_ADJUSTMENT
               INTO h_prj_assignment_id, h_PA_SD, h_PA_FD, h_capital_expense, h_billable,
               h_reg_sale , h_spec_sale, h_reg_cost , h_spec_cost, h_lev_start_DT, h_rate_adjustment
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '0' AND
               TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1331+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1332) : cplan_fun.sql(8661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("h_work_type = %d, h_reg_sale = %.2f, SC_STATE = %d, h_prj_assignment_id = %s\nh_PA_SD = %s, h_PA_FD = %s,h_billable: %d\n", h_work_type,h_reg_sale,SC_STATE,h_prj_assignment_id,h_PA_SD,h_PA_FD,h_billable);
      ElsIf Coalesce(assign_to_prj, 0) = 0 Then
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, LEVEL_START, RATE_ADJUSTMENT
            INTO h_prj_assignment_id, h_lev_start_DT, h_rate_adjustment
            FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_project_id AND TASK_ID=h_project_id
            AND RESOURCE_ID=h_res_id AND BIT_FLAG = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1349+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1350) : cplan_fun.sql(8679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_prj_assignment_id IS NOT NULL)) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = CASE WHEN h_start_date < PLANNED_START THEN TO_TIMESTAMP(h_start_date) ELSE PLANNED_START END,
               PLANNED_FINISH = CASE WHEN h_finish_date > PLANNED_FINISH THEN TO_TIMESTAMP(h_finish_date) ELSE PLANNED_FINISH END,
               PLANNED_WORK=PLANNED_WORK + h_effort,
               BAK_START_DT=CASE WHEN h_start_date < PLANNED_START THEN TO_TIMESTAMP(h_start_date) ELSE PLANNED_START END,
               BAK_FINISH_DT=CASE WHEN h_finish_date > PLANNED_FINISH THEN TO_TIMESTAMP(h_finish_date) ELSE PLANNED_FINISH END,
               BAK_WORK=PLANNED_WORK + h_effort,
               C_EFFORT = h_work, C_UNIT = h_unit, C_CONTOUR = h_contour, CONTOUR_TYPE = h_contour_type
               WHERE TSK_ASSIGNMENT_ID=h_prj_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1375+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1376) : cplan_fun.sql(8705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_tsk_assignment_id := h_prj_assignment_id;
            h_total := 1;
         Else
            h_prj_assignment_id := h_tsk_assignment_id;
            --h_prj_assignment_id_Ind = 0;
         End If;
      Else
         If Coalesce(h_work_type, 0) = 2 Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND
               TO_DATE(SubStr(h_finish_date, 1, 10)) <= TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)) THEN 0 ELSE 1 END,
               REGULAR_RATE, SPECIAL_RATE, REGULAR_COST, SPECIAL_COST, LEVEL_START, RATE_ADJUSTMENT
               INTO h_prj_assignment_id, h_PA_SD, h_PA_FD, h_billable, h_capital_expense,
               h_total, h_reg_sale , h_spec_sale, h_reg_cost , h_spec_cost, h_lev_start_DT, h_rate_adjustment
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1403+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1404) : cplan_fun.sql(8733) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND
               TO_DATE(SubStr(h_finish_date, 1, 10)) <= TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)) THEN 0 ELSE 1 END,
               REGULAR_RATE, SPECIAL_RATE, REGULAR_COST, SPECIAL_COST, LEVEL_START, RATE_ADJUSTMENT
               INTO h_prj_assignment_id, h_PA_SD, h_PA_FD, h_billable, h_capital_expense,
               h_total, h_reg_sale , h_spec_sale, h_reg_cost , h_spec_cost, h_lev_start_DT, h_rate_adjustment
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1423+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1424) : cplan_fun.sql(8753) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("SC_STATE = %d, h_prj_assignment_id = %d:%s, h_reg_sale = %.2,h_billable: %d\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id,h_reg_sale,h_billable);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_prj_assignment_id IS NOT NULL)) 
               AND ((Coalesce(h_total, 0) = 0) OR (Coalesce(h_rollup, 0) = 2)) Then
            h_from_prj := 1;
         Else
            h_from_prj := 0;
            h_prj_assignment_id := NULL;
         End If;
         
         If Coalesce(assign_to_prj, 0) = 0 Then
            h_total := 1;
         Else
            h_total := 0;
         End If;
      End If;
      
      h_labor_code := '';
      -- Labor Code
      
      --trace_printf("h_rollup = %d, assign_to_prj = %d, h_from_prj: %d\n",h_rollup,assign_to_prj,h_from_prj);
      --trace_printf("3_** h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id, h_tsk_assignment_id_Ind,h_tsk_assignment_id);
      
      If (Coalesce(assign_to_prj, 0) <> 0) AND (SubStr(h_res_type, 1, 1) = '0') Then
         If Coalesce(h_fs_project, 0) = 1 Then
            BEGIN
               SELECT FS_LABOR_CODE INTO h_labor_code FROM RESOURCES WHERE RESOURCE_ID=h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1461+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1462) : cplan_fun.sql(8791) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE = %d, h_labor_code = %d, %s, h_res_id = %s\n", SC_STATE,h_labor_code_Ind,h_labor_code,h_res_id);
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_labor_code IS NOT NULL)) Then
               BEGIN
                  SELECT CASE WHEN FS_LABOR_CODE IS NuLL THEN 0
                  WHEN FS_LABOR_CODE = h_labor_code THEN 1 ELSE 2 END
                  INTO h_same_labor FROM TMT_WBS WHERE ELEMENT_ID=h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1477+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1478) : cplan_fun.sql(8807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE = %d, h_task_id = %s, h_same_labor = %d\n", SC_STATE,h_task_id,h_same_labor);
               
               If (Coalesce(h_rollup, 0) = 2) OR (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(
                     h_same_labor, 0) = 0)) Then
                  BEGIN
                     UPDATE TMT_WBS SET FS_LABOR_CODE = h_labor_code WHERE ELEMENT_ID=h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1497+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1498) : cplan_fun.sql(8827) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_same_labor, 0) = 2) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 1503+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(1504) : cplan_fun.sql(8833) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CANNOT_ASSIGN_LABOR_CODE;
               End If;
               --trace_printf("SC_STATE = %d, h_same_labor = %d\n", SC_STATE,h_same_labor);
            End If;
         End If;
      End If;
      --trace_printf("h_bitflag = %d, h_work_type = %d, h_effort = %d, h_work: %d, h_unit: %d\n", h_bitflag,h_work_type,h_effort,h_work,h_unit);
      --trace_printf("***** h_res_id = %s, h_prj_assignment_id: %d:%s\n", h_res_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
      --trace_printf("A_* h_bitflag: %d, h_work_type: %d, h_start_date: %s, h_finish_date: %s\n",h_bitflag,h_work_type,h_start_date,h_finish_date);
      
      If (Coalesce(h_bitflag, 0) = 0) OR (Coalesce(h_bitflag, 0) = 2) Then
         If Coalesce(h_from_prj, 0) = 0 Then
            BEGIN
               SELECT ASSIGNMENT_BILLABLE_FLAG, TIMEPHASE_PROJECT_ASSIGNMENT INTO h_billable, h_pa_timephase
               FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1526+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1527) : cplan_fun.sql(8856) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase
               FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1540+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1541) : cplan_fun.sql(8870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("SC_STATE: %d, h_billable: %d, h_pa_timephase: %s\n", SC_STATE,h_billable,h_pa_timephase);
         
         --InitializeHost(SC_PARAM(h_reg_cost), 0.00);
         --InitializeHost(SC_PARAM(h_reg_sale), 0.00);
         
         h_update := 0;
         --          h_unit = h_unit  ( h_bitflag << 8);
         
         --trace_printf("A_** h_percent_units = %d, h_work: %d, h_unit: %d\n", h_percent_units,h_work,h_unit);
         /*if (h_unit == 101)
         			{
         				h_min_per_day = h_eunit;
         				h_percent_units = h_work;
         				h_work = 0;
         				h_unit = 0;
         				h_task_type = SC_FD_EFFORT_DRIVEN;
         			}
         			else */
         If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 100) Then
            h_percent_units := Coalesce(h_work, 0);
            h_work := 0;
            h_unit := 0;
         ElsIf (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 0) AND (Coalesce(h_per_day, 0) > 0) Then
            h_percent_units := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.499);
         Else
            h_percent_units := 10000;
         End If;
         
         --trace_printf("B_** h_percent_units = %d, h_work: %d, h_unit: %d\n", h_percent_units,h_work,h_unit);
         
         l_data_flag := 0;
         
         h_temp_sc_state := 0;
         If Coalesce(h_reg_cost, 0) + Coalesce(h_spec_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(
               h_spec_sale, 0) = 0.0 Then
            h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, and_roles, and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
         End If;
         
         If p_reg_sale + p_spec_sale > 0.0 Then
            h_reg_sale := p_reg_sale;
            h_spec_sale := p_spec_sale;
         End If;
         
         -- if user assigned from project_assignments and the cost and rate are not zero (it menas user changed them)
         -- then set RATES_EDITED field in TASK_ASSIGNMENTS to 3
         h_rate_edited := 0;
         If (l_data_flag = 1) AND (Coalesce(h_reg_cost, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
               h_reg_sale, 0) + Coalesce(h_spec_sale, 0) <> 0.0) Then
            h_rate_edited := 3;
         End If;
         
         If (Coalesce(h_from_prj, 0) = 0) AND (Coalesce(h_temp_sc_state, 0) = 0) Then
            If l_data_flag = 0 Then
               h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
               h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
               h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
               h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            ElsIf l_data_flag = 2 Then
               h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_prj_rate_echange, 0);
               h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_prj_rate_echange, 0);
               h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_prj_rate_echange, 0);
               h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_prj_rate_echange, 0);
            End If;
         End If;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_labor_code := NULL;
         End If;
         
         --trace_printf("h_update = %d, h_from_prj = %d, h_prj_assignment_id_Ind = %d\n", h_update,h_from_prj,h_prj_assignment_id_Ind);
         --trace_printf("h_rate_echange: %.4f, h_prj_rate_echange: %.4f, h_assign_string: %d:%s\n",h_rate_echange,h_prj_rate_echange,h_assign_string_Ind,h_assign_string);
         --trace_printf("h_reg_cost = %.4f, h_reg_sale = %.4f, h_spec_cost = %.4f, h_spec_sale = %.4f\n",h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale);
         --trace_printf("h_reg_cost_Ind = %d, h_reg_sale_Ind = %d, h_spec_cost_Ind = %d, h_spec_sale_Ind = %d\n",h_reg_cost_Ind, h_reg_sale_Ind, h_spec_cost_Ind, h_spec_sale_Ind);//trace_flush();
         
         If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(h_from_prj, 0) > 0) Then
            --timeLog (file,"START update/insert project assignment:");
            If (h_prj_assignment_id IS NULL) Then
               BEGIN
                  SELECT RES_ASSIGNMENT_ID INTO h_id
                  FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_ID=h_project_id AND RESOURCE_ID=h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1632+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1633) : cplan_fun.sql(8962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  -- UPDATE assign string function
                  BEGIN
                     SELECT PMO_ORASTD.JULIAN_DAY(h_finish_date) - PMO_ORASTD.JULIAN_DAY(h_start_date)
                     INTO h_count FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1648+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1649) : cplan_fun.sql(8978) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  If Coalesce(h_count, 0) > 1 Then
                     tempStr := TO_CHAR(Coalesce(h_count, 0));
                     tempStr := tempStr || '*0';
                  Else
                     tempStr := '0';
                  End If;
                  
                  --Get a unique ID
                  h_prj_assignment_id := '';
                  h_prj_assignment_id := SYS_GUID();
                  
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, BEST, BIT_FLAG, ETC, CONSIDER,
                     BAK_START_DT, BAK_FINISH_DT, BAK_WORK, RESOURCE_ID, RESOURCE_TYPE,
                     T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                     PROJECT_ASSIGNMENT_ID, ASSIGN_STRING, BAK_ASSIGN_STRING, TABLE_TYPE, RATES_EDITED, TIMECODE_1_ID, TIMECODE_2_ID, RATE_ADJUSTMENT )
                     VALUES(h_prj_assignment_id, h_id, h_project_id, h_project_id,
                     h_start_date, h_finish_date, h_effort, h_effort, '0', h_effort, h_consider,
                     h_start_date, h_finish_date, h_effort, h_res_id,h_res_type,
                     h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                     ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                     h_prj_assignment_id, tempStr, tempStr, h_table_type, h_rate_edited,h_timecode_1_id, h_timecode_2_id, h_rate_adjustment);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1686+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1687) : cplan_fun.sql(9016) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1__ SC_STATE = %d, h_effort = %d, h_reg_sale = %.4f\n", SC_STATE,h_effort,h_reg_sale);
               End If;
            ElsIf Coalesce(h_work_type, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  PLANNED_START   = CASE WHEN h_start_date < PLANNED_START THEN TO_TIMESTAMP(h_start_date) ELSE PLANNED_START END,
                  PLANNED_FINISH  = CASE WHEN h_finish_date > PLANNED_FINISH THEN TO_TIMESTAMP(h_finish_date) ELSE PLANNED_FINISH END,
                  PLANNED_WORK    = PLANNED_WORK + h_effort,
                  BAK_START_DT    = CASE WHEN h_start_date < PLANNED_START THEN TO_TIMESTAMP(h_start_date) ELSE PLANNED_START END,
                  BAK_FINISH_DT   = CASE WHEN h_finish_date > PLANNED_FINISH THEN TO_TIMESTAMP(h_finish_date) ELSE PLANNED_FINISH END,
                  BAK_WORK        = PLANNED_WORK + h_effort,
                  C_EFFORT = h_work, C_UNIT = h_unit, C_CONTOUR = h_contour, CONTOUR_TYPE = h_contour_type
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1713+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1714) : cplan_fun.sql(9043) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("PLAN__ SC_STATE = %d, h_effort = %d\n", SC_STATE,h_effort);
            End If;
            --timeLog (file,"END update/insert project assignment:");
         End If;
         
         --trace_printf("h_total = %d, h_rollup = %d, h_work_type = %d, h_bitflag = %d, h_billable: %d\n", h_total,h_rollup, h_work_type,h_bitflag,h_billable);
         --timeLog (file,"START update/insert task assignment:");
         If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_rollup, 0) <> 3) Then
            --if ( (h_work_type == 2) || (h_bitflag == 2))
            If Coalesce(h_work_type, 0) = 2 Then
               BEGIN
                  INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                  SCHED_START, SCHED_FINISH, SCHED_WORK, BIT_FLAG, ETC, BAK_START_DT, BAK_FINISH_DT, BAK_WORK,
                  RESOURCE_ID, RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                  T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                  REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                  PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE, RATES_EDITED,
                  TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT )
                  VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                  h_start_date, h_finish_date, h_effort, '2', h_effort,
                  h_start_date, h_finish_date, h_effort, h_res_id,h_res_type, h_assign_string, h_assign_string,
                  h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                  ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                  h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                  h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1752+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1753) : cplan_fun.sql(9082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d, h_reg_sale = %.4f\n", SC_STATE,h_start_date,h_finish_date,h_effort,h_reg_sale);
            Else
               If SubStr(h_res_type, 1, 1) = '1' Then
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, BEST, BIT_FLAG, ETC,
                     BAK_START_DT, BAK_FINISH_DT, BAK_WORK, RESOURCE_ID,RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                     T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                     PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE , RATES_EDITED,
                     TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT )
                     VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                     h_start_date, h_finish_date, h_effort, h_effort, '0', h_effort,
                     h_start_date, h_finish_date, h_effort, h_res_id, h_res_type, h_assign_string, h_assign_string,
                     h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                     ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                     h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                     h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1784+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1785) : cplan_fun.sql(9114) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, BEST, BIT_FLAG, ETC, BAK_START_DT, BAK_FINISH_DT, BAK_WORK,
                     RESOURCE_ID,RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                     T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                     PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE, RATES_EDITED,
                     TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT )
                     VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                     h_start_date, h_finish_date, h_effort, h_effort, '0', h_effort,
                     h_start_date, h_finish_date, h_effort, h_res_id,h_res_type, h_assign_string, h_assign_string,
                     h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                     ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                     h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                     h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1814+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1815) : cplan_fun.sql(9144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("3__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d, h_reg_sale = %.4f\n", SC_STATE,h_start_date,h_finish_date,h_effort, h_reg_sale);
            End If;
         End If;
      ElsIf Coalesce(h_work_type, 0) = 0 Then
         h_tsk_assignment_id := assign_id;
         -- TSK_ASSIGNMENT_ID
         h_update := 1;
         -- UPDATE
         
         h_planned_work := 0;
         BEGIN
            SELECT PROJECT_ASSIGNMENT_ID, PLANNED_WORK, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_planned_work, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1840+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1841) : cplan_fun.sql(9170) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_SD: %s, h_tsk_assignment_id: %s, h_planned_work: %d\nh_billable: %d, h_prj_assignment_id: %d:%s, h_reg_sale = %.4f\n",SC_STATE,h_SD,h_tsk_assignment_id,h_planned_work,h_billable,h_prj_assignment_id_Ind,h_prj_assignment_id, h_reg_sale);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_SD IS NOT NULL)) Then
            If (h_prj_assignment_id IS NULL) Then
               h_from_prj := 0;
            Else
               h_from_prj := 1;
            End If;
            
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
         
         If (Coalesce(h_rollup, 0) = 2) AND (Coalesce(h_work, 0) > 0) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START       = h_start_date,
               PLANNED_FINISH      = h_finish_date,
               PLANNED_WORK        = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR = h_contour, C_UNIT = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '0'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1883+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1884) : cplan_fun.sql(9213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START       = h_start_date,
               PLANNED_FINISH      = h_finish_date,
               PLANNED_WORK        = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR = h_contour, C_UNIT = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '0'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1911+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1912) : cplan_fun.sql(9241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("4__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d, h_reg_sale = %.4f\n", SC_STATE,h_effort,h_start_date,h_finish_date, h_reg_sale);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            If Coalesce(h_from_prj, 0) = 1 Then
               OPEN C41;
               LOOP
                  FETCH C41 INTO h_id, h_dayofyear, h_minutes, h_bitID;
                  IF C41%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1945+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1946) : cplan_fun.sql(9275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1.1_TDY_SC_STATE = %d, h_id = %s, h_dayofyear = %s, h_minutes = %d\n", SC_STATE,h_id,h_dayofyear,h_minutes);//trace_flush();
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '0';
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1964+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1965) : cplan_fun.sql(9294) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               END LOOP;
               BEGIN
                  CLOSE C41 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1978+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1979) : cplan_fun.sql(9308) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '0';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1996+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1997) : cplan_fun.sql(9326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1__DELETE FROM TASK_DAY_OF_YEAR: h_tsk_assignment_id: %s\n",h_tsk_assignment_id);//trace_flush();
         End If;
      ElsIf Coalesce(h_work_type, 0) = 2 Then
         h_tsk_assignment_id := assign_id;
         -- TSK_ASSIGNMENT_ID
         
         BEGIN
            SELECT PROJECT_ASSIGNMENT_ID, SCHED_WORK, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_sched_work, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2017+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2018) : cplan_fun.sql(9347) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_assignment_id = %s, h_sched_work: %d\nh_prj_assignment_id: %d:%s\n", SC_STATE,h_tsk_assignment_id,h_sched_work,h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS OR ((h_SD IS NOT NULL)) Then
            If (h_prj_assignment_id IS NULL) Then
               h_from_prj := 0;
            Else
               h_from_prj := 1;
            End If;
            
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
         
         If Coalesce(h_from_prj, 0) = 0 Then
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2049+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2050) : cplan_fun.sql(9379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("2__DELETE FROM TASK_DAY_OF_YEAR: h_tsk_assignment_id: %s\n",h_tsk_assignment_id);//trace_flush();
         Else
            OPEN C40;
            LOOP
               FETCH C40 INTO h_id, h_dayofyear, h_minutes, h_bitID;
               IF C40%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2080+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2081) : cplan_fun.sql(9410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2_TDY_SC_STATE = %d, h_id = %s, h_dayofyear = %s, h_minutes = %d\n", SC_STATE,h_id,h_dayofyear,h_minutes);
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2100+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2101) : cplan_fun.sql(9430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C40 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2114+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2115) : cplan_fun.sql(9444) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2131+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2132) : cplan_fun.sql(9461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("3__DELETE FROM TASK_DAY_OF_YEAR: h_tsk_assignment_id: %s\n",h_tsk_assignment_id);//trace_flush();
            
            -- Check if there is enough effort @ project
            h_total := 0;
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_total
               FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND BIT_FLAG = '2'
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2149+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2150) : cplan_fun.sql(9479) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_rollup, 0) = 2) AND (Coalesce(h_work, 0) > 0) Then
            If Coalesce(h_total, 0) < Coalesce(h_effort, 0) Then
               h_total := Coalesce(h_effort, 0) - Coalesce(h_total, 0);
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET ETC = h_total
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2171+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2172) : cplan_fun.sql(9501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_START         = h_start_date,
                  SCHED_FINISH        = h_finish_date,
                  SCHED_WORK          = h_effort,
                  BAK_START_DT        = h_start_date,
                  BAK_FINISH_DT       = h_finish_date,
                  BAK_WORK            = h_effort,
                  ETC                 = CASE WHEN h_u_etc = 0 THEN h_effort ELSE ETC END,
                  C_EFFORT = h_work, C_CONTOUR    = h_contour,
                  C_UNIT   = h_unit, CONTOUR_TYPE = h_contour_type,
                  ASSIGN_STRING       = h_assign_string,
                  BAK_ASSIGN_STRING   = h_assign_string,
                  BIT_FLAG            = '2'
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2201+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2202) : cplan_fun.sql(9531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("U1_* SC_STATE = %d, h_total: %d, h_effort: %d\n", SC_STATE,h_total,h_effort);//trace_flush();
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START         = h_start_date,
               SCHED_FINISH        = h_finish_date,
               SCHED_WORK          = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ETC                 = CASE WHEN h_u_etc = 0 THEN h_effort ELSE ETC END,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR    = h_contour,
               C_UNIT   = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '2'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2233+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2234) : cplan_fun.sql(9563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            NULL;
         End If;
         --trace_printf("U2_* SC_STATE = %d, h_from_prj = %d\n", SC_STATE,h_from_prj);//trace_flush();
      End If;
      --timeLog (file,"END update/insert task assignment:");
      
      h_cri := (PMO_FUNC.BIT_OR(Coalesce(h_cri, 0), PMO_CNST.SC_BIT_30));
      --      InitializeHost(SC_PARAM(h_prj_assignment_id), h_prj_assignment_id);     //h_prj_assignment_id_Ind = 0;
      --trace_printf("AA_1* h_prj_assignment_id: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
   End If;
   --trace_printf("h_task_type: %d, h_percent_units: %d, h_min_per_day: %d\n",h_task_type,h_percent_units,h_min_per_day);
   
   --//timeLog (file,"1_ ASSIGN STRING:");
   --trace_printf("h_task_id: %d:%s, h_res_id: %d:%s\n",h_task_id_Ind,h_task_id,h_res_id_Ind,h_res_id);
   --trace_printf("h_assigned: %d, h_assign_string: %s\n",h_assigned,h_assign_string);
   
   If (Coalesce(h_assigned, 0) = 0) AND (Coalesce(Length(h_assign_string), 0) > 0) Then
      --trace_printf("search + assign = %d, h_bitflag: %d, h_consider: %d, h_from_prj: %d, h_update: %d, h_work_type: %d\n",h_assigned,h_bitflag,h_consider,h_from_prj,h_update,h_work_type);
      -- if not search assign load temp table Task_Day_Of_Year
      If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) AND ((BITAND(Coalesce(
             h_bitflag, 0), 255)) <> 10) Then
         If (Coalesce(h_bitflag, 0) = 1) AND (Coalesce(h_consider, 0) = 0) AND (Coalesce(
               h_from_prj, 0) = 1) Then
            If Coalesce(h_work_type, 0) = 2 Then
               h_total := 0;
            Else
               h_total := 3;
            End If;
            
            h_minutes := 10;
            h_temp_sc_state := FC_CREATE_TDY(h_project_id, h_res_id, h_start_date, h_minutes, h_cri, h_priority
                                 , h_total);
         Else
            h_total := 0;
            h_temp_sc_state := FC_CREATE_TDY(h_project_id, h_res_id, h_start_date, h_consider, h_cri, h_priority
                                 , h_total);
         End If;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLNF_0001 SELECT DAY_YEAR, SUM(SCHED_MIN)
            FROM PMO_TMP_TDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            GROUP BY DAY_YEAR;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2289+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2290) : cplan_fun.sql(9619) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDOY';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         SELECT TO_DATE(SubStr(h_start_date, 1, 10)), PMO_ORASTD.TIME(h_start_date) INTO h_dayofyear, h_first_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2308+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2309) : cplan_fun.sql(9638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_first_time: %d:%s\n",h_first_time_Ind,h_first_time);
      
      token := h_assign_string;
      pos := h_assign_string;
      
      tempStr := '';
      
      h_minutes := 0;
      token := PMO_STPU.REstrtok(token, PMO_CNST.SC_SEP, pos);
      
      result := InStr(token, PMO_CNST.SC_STAR_CHAR) - 1;
      
      --trace_printf("result: %d\n",result); trace_flush();
      If result > 0 Then
         h_uniqueID := h_prj_assignment_id;
         -- Get next assignment
         h_count := Length(token) - result;
         
         --          pmo_substr(tempStr, token, 0, result);
         --          NullifyStrPos(tempStr, result);
         --          pmo_substr(h_min, token, result + 1, h_count);
         tempStr := SubStr(token, 1, result);
         h_min := SubStr(token, result + 1 + 1, Coalesce(h_count, 0));
         
         BEGIN
            SELECT Trunc(tempStr), Trunc(h_min)
            INTO h_count, h_minutes
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2346+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2347) : cplan_fun.sql(9676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         WHILE Coalesce(h_count, 0) > 0 LOOP
            If Coalesce(h_minutes, 0) > 0 Then
               h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_minutes, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_bitID);
               h_first_time := NULL;
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2369+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2370) : cplan_fun.sql(9699) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_count := Coalesce(h_count, 0) - 1;
         END LOOP;
      Else
         h_min := token;
         
         BEGIN
            SELECT Trunc(h_min) INTO h_minutes
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2388+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2389) : cplan_fun.sql(9718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_minutes, 0) > 0 Then
            h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_minutes, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_bitID);
            h_first_time := NULL;
         End If;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2410+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2411) : cplan_fun.sql(9740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      tempStr := '';
      
      h_minutes := 0;
      
      LOOP
         token := PMO_STPU.REstrtok(NULL, PMO_CNST.SC_SEP, pos);
         If (token IS NULL) Then
            EXIT;
         End If;
         
         result := InStr(token, PMO_CNST.SC_STAR_CHAR) - 1;
         
         If result > 0 Then
            -- Get next assignment
            --              pmo_substr(tempStr, token, 0, result);
            --              NullifyStrPos(tempStr, result);
            tempStr := SubStr(token, 1, result);
            
            h_count := Length(token) - result;
            --              pmo_substr(h_min, token, result + 1, h_count);
            --              NullifyStrPos(h_min, h_count);
            h_min := SubStr(token, result + 1 + 1, Coalesce(h_count, 0));
            
            BEGIN
               SELECT Trunc(tempStr), Trunc(h_min) INTO h_count, h_minutes
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2448+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2449) : cplan_fun.sql(9778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            WHILE Coalesce(h_count, 0) > 0 LOOP
               If Coalesce(h_minutes, 0) > 0 Then
                  h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_minutes, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_bitID);
                  h_first_time := NULL;
               End If;
               
               BEGIN
                  SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2471+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2472) : cplan_fun.sql(9801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_count := Coalesce(h_count, 0) - 1;
            END LOOP;
         Else
            h_min := token;
            BEGIN
               SELECT Trunc(h_min) INTO h_minutes FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2488+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2489) : cplan_fun.sql(9818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_minutes, 0) > 0 Then
               h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_minutes, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_bitID);
               h_first_time := NULL;
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2510+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2511) : cplan_fun.sql(9840) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      --trace_printf("1_***__h_bitflag: %d, h_update: %d\n",h_bitflag,h_update);
      
      -- AEL: if leveling insert into out new task_day_of_years
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         BEGIN
            INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2532+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2533) : cplan_fun.sql(9862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (BITAND(Coalesce(h_update, 0), 256)) <> 256 Then
            If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
               h_proj_assign_flag := 0;
            Else
               h_proj_assign_flag := 1;
            End If;
            
            BEGIN
               INSERT INTO PMO_TMP_TDY_LEVEL
               SELECT
               PROJECT_ID,
               RESOURCE_ID,
               h_priority,
               0,
               TSK_ASSIGNMENT_ID,
               DAY_YEAR,
               SCHED_MIN,
               BIT_FLAG,
               BIT_SCHED,
               h_proj_assign_flag
               FROM PMO_TMP_TDOY;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2568+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2569) : cplan_fun.sql(9898) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("2_a__SC_STATE: %d\n",SC_STATE);
         End If;
      Else
         BEGIN
            INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2587+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2588) : cplan_fun.sql(9917) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_lev_start_DT IS NOT NULL) Then
            --trace_printf("U_TSKRES2: nullify level dates\n");
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET LEVEL_START = NULL, LEVEL_FINISH = NULL, LEVEL_EFFORT = 0, LEVEL_ASSIGN_STRING = NULL
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2607+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2608) : cplan_fun.sql(9937) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM LEVEL_TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2625+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2626) : cplan_fun.sql(9955) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   --timeLog (file,"2_ ASSIGN STRING:");
   --trace_printf("h_rollup = %d, h_update = %d, assign_to_prj: %d, h_from_prj = %d,h_tsk_assignment_id = %s\nh_table_type: %d, h_total: %d, h_prj_assignment_id: %d:%s\n",h_rollup,h_update,assign_to_prj,h_from_prj,h_tsk_assignment_id,h_table_type,h_total,h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   -- AEL: if this is leveling skip the next if for rollup
   
   If Coalesce(h_rollup, 0) <> 3 Then
      h_temp_sc_state := FC_UPDATE_LABOR_BUDGET_BITS(h_project_id, h_task_id, h_capital_expense
                                               , h_billable, h_rec_user);
   End If;
   
   If (BITAND(Coalesce(h_bitflag, 0), 255)) <> 10 Then
      If Coalesce(assign_to_prj, 0) = 0 Then
         BEGIN
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2652+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2653) : cplan_fun.sql(9982) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_pa_timephase: %s\n", SC_STATE,h_pa_timephase);
         
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            h_minutes := 0;
            BEGIN
               SELECT SUM(ESTIMATED_PLANNED_EFFORT + ESTIMATED_SCHED_EFFORT) INTO h_minutes
               FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2669+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2670) : cplan_fun.sql(9999) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_minutes, 0) = 0 Then
               BEGIN
                  DELETE FROM TIMEPHASED_DELTA WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id
                  AND BUDGET_TYPE IN (PMO_CNST.SC_ESTIMATED_BUDGET,PMO_CNST.SC_ESTIMATED_ACTUAL_BUDGET)
                  AND CHARGECODE_ID IN ( 'CAPITAL_LABOR',
                  'EXPENSE_LABOR',
                  'BILLABLE_BENEFIT_LABOR',
                  'EXPENSE_HOURS',
                  'EXPENSE_AVERAGE_COST',
                  'EXPENSE_FULL_TIME_EQUIVALENT',
                  'CAPITAL_HOURS',
                  'CAPITAL_AVERAGE_COST',
                  'CAPITAL_FULL_TIME_EQUIVALENT',
                  'BENEFIT_HOURS',
                  'BENEFIT_AVERAGE_RATE',
                  'BENEFIT_FULL_TIME_EQUIVALENT' );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2700+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2701) : cplan_fun.sql(10030) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  DELETE FROM PROJECT_BUDGET_BREAKDOWN WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id
                  AND BUDGET_TYPE IN (PMO_CNST.SC_ESTIMATED_BUDGET,PMO_CNST.SC_ESTIMATED_ACTUAL_BUDGET)
                  AND CHARGECODE_ID IN ( 'CAPITAL_LABOR',
                  'EXPENSE_LABOR',
                  'BILLABLE_BENEFIT_LABOR',
                  'EXPENSE_HOURS',
                  'EXPENSE_AVERAGE_COST',
                  'EXPENSE_FULL_TIME_EQUIVALENT',
                  'CAPITAL_HOURS',
                  'CAPITAL_AVERAGE_COST',
                  'CAPITAL_FULL_TIME_EQUIVALENT',
                  'BENEFIT_HOURS',
                  'BENEFIT_AVERAGE_RATE',
                  'BENEFIT_FULL_TIME_EQUIVALENT' );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2730+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2731) : cplan_fun.sql(10060) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_tph_delete := 0;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_prj_assignment_id, h_start_date
                                          , h_finish_date, h_reg_cost, h_reg_sale, h_tph_delete
                                          , h_capital_expense, h_billable);
         End If;
         
         If Coalesce(h_total, 0) = 1 Then
            BEGIN
               SELECT PLANNED_WORK, PLANNED_START, PLANNED_FINISH, BILLABLE, REGULAR_COST, REGULAR_RATE
               INTO h_effort, h_start_date, h_finish_date, h_billable, h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2753+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2754) : cplan_fun.sql(10083) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s, h_effort: %d, h_start_date: %s, h_finish_date: %s, h_reg_sale: %.4f\n", SC_STATE,h_tsk_assignment_id,h_effort,h_start_date,h_finish_date, h_reg_sale); trace_flush();
         End If;
         
         If Coalesce(h_billable, 0) = 0 Then
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START = h_start_date,
               ESTIMATED_PLANNED_FINISH = h_finish_date,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = 0.00
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2779+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2780) : cplan_fun.sql(10109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START = h_start_date,
               ESTIMATED_PLANNED_FINISH = h_finish_date,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_reg_sale
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2802+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2803) : cplan_fun.sql(10132) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("SC_STATE: %d\n", SC_STATE);
      Else
         If Coalesce(h_rollup, 0) = 3 Then
            -- CALL UPDATE assign string function
            /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
            				{
            					if (h_work_type == 0)
            					{
            						BEGIN_SQL
            							UPDATE TASK_ASSIGNMENTS SET
            								PLANNED_START       = HOST(h_start_date),
            								PLANNED_FINISH      = HOST(h_finish_date)
            							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
            						END_SQL;
            					}
            					else
            					{
            						BEGIN_SQL
            							UPDATE TASK_ASSIGNMENTS SET
            								SCHED_START         = HOST(h_start_date),
            								SCHED_FINISH        = HOST(h_finish_date)
            							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
            						END_SQL;
            					}
            
            					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
            					h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_min_per_day);   // fix ACTUAL_START = SCHED_START  ACTUAL_FINISH = SCHED_FINISH if ETC = 0
            				}
            				else
            				{*/
            If (Coalesce(h_percent_units, 0) > 0) AND (Coalesce(h_percent_units, 0) < 10000) Then
               h_temp_var := (PMO_FUNC.ShiftLeft(Coalesce(h_percent_units, 0), 15)) + 1;
            Else
               h_temp_var := 1;
            End If;
            
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            -- fix ACTUAL_START = SCHED_START  ACTUAL_FINISH = SCHED_FINISH if ETC = 0
            --}
            
            BEGIN
               SELECT TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
               INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2858+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2859) : cplan_fun.sql(10188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_tsk_assignment_id = %s, h_reg_cost: %.2f, h_reg_sale: %.2f, h_SD: %s, h_FD: %s\n", SC_STATE,h_tsk_assignment_id,h_reg_cost,h_reg_sale,h_SD,h_FD);
         End If;
         
         --trace_printf("h_bitflag: %d, h_task_type: %d\n",h_bitflag,h_task_type);
         /*if ( (h_bitflag == 0)  (h_task_type == SC_FD_EFFORT_DRIVEN) )
         			{
         				InitializeHost(SC_PARAM(h_tph_delete),0);
         				h_temp_var = FC_TPHASE_FD_SCHED(
         					SC_PARAM(h_project_id),
         					SC_PARAM(h_task_id),
         					SC_PARAM(h_tph_delete));
         			}
         			else
         			{*/
         h_SD := SubStr(h_start_date, 1, 10);
         h_FD := SubStr(h_finish_date, 1, 10);
         --trace_printf("h_SD: %s, h_FD: %s, h_reg_sale: %.2f\n",h_SD,h_FD, h_reg_sale);
         --trace_printf("h_start_date: %s, h_finish_date: %s\n",h_start_date,h_finish_date);
         
         h_tph_delete := 0;
         h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
         --}
      End If;
   End If;
   
   If Coalesce(h_rollup, 0) <> 3 Then
      /*      if ( assign_to_prj != 0 )
      		{
      			h_temp_sc_state = FC_GENERATE_EW(SC_PARAM(h_tsk_assignment_id));
      		}*/
      
      -- Update the contour
      --trace_printf("*** h_prj_assignment_id = %d %s, h_rollup = %d, h_from_prj = %d, h_type_id: %d, h_task_type: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_rollup,h_from_prj,h_type_id,h_task_type);
      
      If (Coalesce(h_rollup, 0) = 0) OR ((Coalesce(h_rollup, 0) = 2) AND (Coalesce(
             h_from_prj, 0) > 0)) Then
         --trace_printf("*** h_tsk_assignment_id = %d %s\n",h_tsk_assignment_id_Ind,h_tsk_assignment_id);
         -- CALL UPDATE assign string function
         /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
         			{
         				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
         				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_min_per_day,h_effort);
         			}
         			else //if ( strcmp(h_prj_assignment_id, h_tsk_assignment_id) == 0 )
         			{*/
         If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
            h_temp_var := 3;
         Else
            h_temp_var := 0;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(
                                   h_per_day, 0), 0);
         --}
         --trace_printf("TA__h_temp_sc_state: %d, h_tsk_assignment_id = %s\n",h_temp_sc_state,h_tsk_assignment_id);
      End If;
      
      If ((h_prj_assignment_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_prj_assignment_id
            , h_tsk_assignment_id) <> 0) Then
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, Coalesce(
                                   h_per_day, 0), 0);
         --trace_printf("PA__h_temp_sc_state: %d, h_prj_assignment_id = %s\n",h_temp_sc_state,h_prj_assignment_id);
      End If;
      
      If Coalesce(h_rollup, 0) = 0 Then
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
         --trace_printf("FC_Rollup__ h_temp_sc_state = %d, h_project_id: %s\n",h_temp_sc_state,h_project_id);
         
         If Coalesce(h_work_type, 0) = 2 Then
            BEGIN
               SELECT CASE WHEN (SD <> PMO_CNST.SC_NS) AND (FD <> PMO_CNST.SC_NF) AND (TYPE_ID = PMO_CNST.SC_PROPOSAL) THEN 1 ELSE 0 END
               INTO h_change_flag
               FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2945+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2946) : cplan_fun.sql(10275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_change_flag, 0) = 1 Then
               h_old_type_id := PMO_CNST.SC_PROPOSAL;
               PMO_DB2M.SC_STATE := FC_CHANGE_PROJ_TYPE(h_project_id, h_change_flag, h_old_type_id);
            End If;
         End If;
      End If;
      
      If Coalesce(h_bitflag, 0) = 0 Then
         h_and_roles := and_roles;
         If Coalesce(Length(h_and_roles), 0) > 4096 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 2961+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(2962) : cplan_fun.sql(10291) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
         End If;
         
         If (Coalesce(assign_to_prj, 0) = 0) AND ((h_prj_assignment_id IS NULL)) Then
            If Coalesce(Length(h_and_roles), 0) = 0 Then
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT PRIMARY_ROLE INTO h_id FROM RESOURCES WHERE RESOURCE_ID=h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2977+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2978) : cplan_fun.sql(10307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_id IS NOT NULL)) Then
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, h_id, 0, 0, COALESCE(EXPERIENCE_ID,0), LAST_USED_DATETIME,'0'
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2996+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2997) : cplan_fun.sql(10326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf SubStr(h_res_type, 1, 1) = '0' Then
               --Parse Roles list
               If Coalesce(Length(h_and_roles), 0) < 32 Then
                  numItems := 0;
               Else
                  numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
               End If;
               
               If numItems > 0 Then
                  h_role_id := SubStr(h_role_id, 1, 32);
                  
                  h_count := 0;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next role_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
                     h_role_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 32 + 1, 32);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t2.ELEMENT_ID=t1.PARENT_ID AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8)
                        AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK)end) = t3.RANK
                        );
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3039+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3040) : cplan_fun.sql(10369) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                  END LOOP;
               End If;
            End If;
         End If;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID=h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3059+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3060) : cplan_fun.sql(10389) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID=h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3073+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3074) : cplan_fun.sql(10403) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_count, 0) = 0 Then
               h_tsk_assignment_id := h_res_id;
            End If;
         Else
            h_count := 0;
            NULL;
         End If;
         
         If Coalesce(h_count, 0) = 0 Then
            --Parse Roles list
            If Coalesce(Length(h_and_roles), 0) < 32 Then
               numItems := 0;
            Else
               numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
            End If;
            
            If numItems > 0 Then
               h_role_id := SubStr(h_role_id, 1, 32);
               
               If SubStr(h_res_type, 1, 1) = '0' Then
                  h_count := 0;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next skill_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
                     h_role_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 32 + 1, 32);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t2.ELEMENT_ID=t1.PARENT_ID AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK
                        ) AND
                        NOT EXISTS(SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_ROLES
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (ROLE_ID = t3.PARENT_ID OR PRIMARY_ROLE = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3126+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3127) : cplan_fun.sql(10456) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                  END LOOP;
               ElsIf SubStr(h_res_type, 1, 1) = '1' Then
                  h_count := 1;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next role_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);   //strncpy( h_role_id, h_and_roles[h_count*32], 32);
                     h_role_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 32 + 1, 32);
                     
                     --trace_printf("h_count = %d, h_and_roles = %s\n",h_count,h_and_roles);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, h_role_id, PROFICIENCY_LEVEL_ID,
                        PREFERENCE_LEVEL_ID, EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID=h_role_id AND NOT EXISTS(
                        SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_ROLES
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (ROLE_ID = h_role_id OR PRIMARY_ROLE = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3158+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3159) : cplan_fun.sql(10488) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                     --trace_printf("h_count = %d, SC_STATE = %d, h_role_id = %s, h_role_id_Ind = %d\n",h_count,SC_STATE,h_role_id,h_role_id_Ind);
                  END LOOP;
               End If;
            End If;
            
            --Parse Skills list
            h_and_skills := and_skills;
            If Coalesce(Length(h_and_skills), 0) > 4096 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 3173+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(3174) : cplan_fun.sql(10503) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
            End If;
            
            If Coalesce(Length(h_and_skills), 0) < 32 Then
               numItems := 0;
            Else
               numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
            End If;
            
            If numItems > 0 Then
               h_role_id := SubStr(h_role_id, 1, 32);
               
               h_count := 0;
               WHILE Coalesce(h_count, 0) < numItems LOOP
                  -- Get next skill_id
                  --                      pmo_substr(h_role_id, h_and_skills, h_count*32, 32);
                  h_role_id := SubStr(h_and_skills, Coalesce(h_count, 0) * 32 + 1, 32);
                  
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_SKILLS SELECT h_tsk_assignment_id, h_role_id, PROFICIENCY_LEVEL_ID,
                        PREFERENCE_LEVEL_ID, EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID=h_role_id AND NOT EXISTS(
                        SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_SKILLS
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (SKILL_ID = h_role_id OR PRIMARY_SKILL = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3210+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3211) : cplan_fun.sql(10540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_SKILLS SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK
                        )
                        AND t2.ELEMENT_ID=t1.PARENT_ID AND NOT EXISTS(SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_SKILLS
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (SKILL_ID = t3.PARENT_ID OR PRIMARY_SKILL = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 3236+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(3237) : cplan_fun.sql(10566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  h_count := Coalesce(h_count, 0) + 1;
               END LOOP;
            End If;
         End If;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 3252+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(3253) : cplan_fun.sql(10582) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_TSKRES2;
ALTER FUNCTION U_TSKRES2 COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- search function

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_SEARCH_2 ==
CREATE OR REPLACE FUNCTION FC_SEARCH_2
(
     assign_id IN OUT NOCOPY VARCHAR2
   , project_id001 IN OUT NOCOPY VARCHAR2
   , task_id001 IN OUT NOCOPY VARCHAR2
   , res_id001 IN OUT NOCOPY VARCHAR2
   , sh_res_cost NUMBER
   , sh_type_id001 PLS_INTEGER
   , and_roles IN OUT NOCOPY VARCHAR2
   , and_skills IN OUT NOCOPY VARCHAR2
   , shw_roles IN OUT NOCOPY VARCHAR2
   , shw_skills IN OUT NOCOPY VARCHAR2
   , start_date001 IN OUT NOCOPY VARCHAR2
   , finish_date001 IN OUT NOCOPY VARCHAR2
   , sh_duration001 PLS_INTEGER
   , sh_effort001 PLS_INTEGER
   , sh_work001 PLS_INTEGER
   , sh_eunit PLS_INTEGER
   , sh_unit PLS_INTEGER
   , sh_cri PLS_INTEGER
   , sh_consider001 PLS_INTEGER
   , res_type IN OUT NOCOPY VARCHAR2
   , sh_work_type PLS_INTEGER
   , sh_bitflag001 PLS_INTEGER
   , sh_contour001 PLS_INTEGER
   , sh_occurrence PLS_INTEGER
   , sh_p_update PLS_INTEGER
   , sh_hrs_per_day PLS_INTEGER
   , sh_priority001 PLS_INTEGER
   , rec_user001 IN OUT NOCOPY VARCHAR2
   , sh_calculate_flag PLS_INTEGER := 0
   , sh_delay PLS_INTEGER := 0
   , sh_p_reg_sale NUMBER := 0.00
   , sh_p_spec_sale NUMBER := 0.00
) RETURN PLS_INTEGER
AS
   res_cost NUMBER := sh_res_cost;
   type_id001 PLS_INTEGER := sh_type_id001;
   duration001 PLS_INTEGER := sh_duration001;
   effort001 PLS_INTEGER := sh_effort001;
   work001 PLS_INTEGER := sh_work001;
   eunit PLS_INTEGER := sh_eunit;
   unit PLS_INTEGER := sh_unit;
   cri PLS_INTEGER := sh_cri;
   consider001 PLS_INTEGER := sh_consider001;
   work_type PLS_INTEGER := sh_work_type;
   bitflag001 PLS_INTEGER := sh_bitflag001;
   contour001 PLS_INTEGER := sh_contour001;
   occurrence PLS_INTEGER := sh_occurrence;
   p_update PLS_INTEGER := sh_p_update;
   hrs_per_day PLS_INTEGER := sh_hrs_per_day;
   priority001 PLS_INTEGER := sh_priority001;
   calculate_flag PLS_INTEGER := Coalesce(sh_calculate_flag, 0);
   delay PLS_INTEGER := Coalesce(sh_delay, 0);
   p_reg_sale NUMBER := Coalesce(sh_p_reg_sale, 0);
   p_spec_sale NUMBER := Coalesce(sh_p_spec_sale, 0);
   t_start PLS_INTEGER := 0;
   t_finish PLS_INTEGER := 0;
   t_total NUMBER := 0.00;
   t_isLevel PLS_INTEGER := 0;
   h_min_work_per_day PLS_INTEGER := 0;
   s_time_1 PLS_INTEGER := 0;
   s_time_2 PLS_INTEGER := 0;
   s_time_3 PLS_INTEGER := 0;
   s_time_4 PLS_INTEGER := 0;
   h_res_cost NUMBER := 0.00;
   h_uniqueID VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_min VARCHAR2(19);
   h_and_roles VARCHAR2(4000);
   h_and_skills VARCHAR2(4000);
   h_shw_roles VARCHAR2(4000);
   h_shw_skills VARCHAR2(4000);
   h_res_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_start_time VARCHAR2(8);
   h_first_time VARCHAR2(8);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_finish_time VARCHAR2(8);
   h_work_loc VARCHAR2(32);
   h_dayofyear VARCHAR2(10);
   h_finish_D VARCHAR2(10);
   h_res_type VARCHAR2(1);
   h_active VARCHAR2(1);
   h_assign_string VARCHAR2(8192);
   h_lev_assign_string VARCHAR2(8192);
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_ta_start_DT VARCHAR2(26);
   h_ta_finish_DT VARCHAR2(26);
   h_lev_start_DT VARCHAR2(26);
   h_lev_finish_DT VARCHAR2(26);
   h_act_start_DT VARCHAR2(26);
   h_start_D DATE;
   h_lev_effort PLS_INTEGER := 0;
   h_lev_consider PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_prev_min PLS_INTEGER := 0;
   h_prev_week PLS_INTEGER := 0;
   h_p2_week PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_total_min PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_prev_day PLS_INTEGER := 0;
   h_planned_work PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_spec_cost NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_rate_echange NUMBER := 1.00;
   h_currency_id PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_bit_task PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_save PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_prj_percent_units PLS_INTEGER := 0;
   h_tmp_percent_units PLS_INTEGER := 0;
   h_work_per_day PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_cnt PLS_INTEGER := 0;
   assign_to_prj PLS_INTEGER := 0;
   h_proj_assign_flag PLS_INTEGER := 0;
   tempStr VARCHAR2(255);
   h_temp_sc_state PLS_INTEGER := 0;
   h_continue_01 PLS_INTEGER := 0;
   h_header VARCHAR2(7999);
   h_tail VARCHAR2(999);
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
   h_pmo_tmp_res_ins_stmt VARCHAR2(3999);
   h_bitflag_condition VARCHAR2(199);
   h_pmo_tmp_every_ins_stmt VARCHAR2(3999);
   h_pmo_tmp_sum_ins_stmt VARCHAR2(3999);
   h_pmo_tmp_md_ins_stmt VARCHAR2(3999);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   i_lev_etc PLS_INTEGER NOT NULL := 0;
   i_has_actual PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER NOT NULL := 0;
   ii PLS_INTEGER NOT NULL := 9;
   iii PLS_INTEGER NOT NULL := 0;
   h_amnt PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   wBits PLS_INTEGER NOT NULL := 0;
   wHour PLS_INTEGER NOT NULL := 0;
   wMin PLS_INTEGER NOT NULL := 0;
   i_pa_replace PLS_INTEGER NOT NULL := 0;
   token VARCHAR2(32);
   i_noskill PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
   iSD PLS_INTEGER;
   i_break PLS_INTEGER;
   i_work PLS_INTEGER;
   l_data_flag PLS_INTEGER;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_DELAY1 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_DELAY0 PMO_DB2M.REF_CURSOR;

      CURSOR C_LS IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_res_id AND DAY_YEAR > h_start_D
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_TA IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id AND DAY_YEAR > h_start_D
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      -- Dynamic Cursor Declaration
      C332 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEARCH_2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEARCH_2'), '-- ' || SYSTIMESTAMP, ':r := FC_SEARCH_2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || project_id001 || ''''
          || ', ' || '''' || task_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_res_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_type_id001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || shw_roles || ''''
          || ', ' || '''' || shw_skills || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_duration001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_consider001), 'NULL')
          || ', ' || '''' || res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_priority001), 'NULL')
          || ', ' || '''' || rec_user001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_calculate_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 259+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEARCH_2(260) : cplan_fun.sql(10856) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --temporary var. for debug.
   
   
   /**NOTE** Dynamic Insert statement for PMO_TMP_RES temporary table*/
   /**NOTE** temporary variable to hold the condition applied for bit_flag field */
   /**NOTE** Dynamic Insert statement for PMO_TMP_EVERY temporary table*/
   /**NOTE** Dynamic Insert statement for PMO_TMP_SUM temporary table*/
   /**NOTE** Dynamic Insert statement for PMO_TMP_SUM temporary table*/
   
   h_assign_id := assign_id;
   h_project_id := project_id001;
   h_task_id := task_id001;
   h_res_id := res_id001;
   h_res_type := res_type;
   h_and_roles := and_roles;
   h_and_skills := and_skills;
   h_shw_roles := shw_roles;
   h_shw_skills := shw_skills;
   h_rec_user := rec_user001;
   h_delay := delay;
   h_task_type := 0;
   assign_to_prj := 1;
   h_work_todate := 0;
   
   If (Coalesce(Length(h_and_roles), 0) > 4096) OR (Coalesce(Length(h_and_skills), 0) > 4096) 
         OR (Coalesce(Length(h_shw_roles), 0) > 4096) 
         OR (Coalesce(Length(h_shw_skills), 0) > 4096) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 292+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(293) : cplan_fun.sql(10889) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   h_duration := Coalesce(duration001, 0);
   h_effort := Coalesce(effort001, 0);
   h_bitflag := Coalesce(bitflag001, 0);
   h_work_type := Coalesce(work_type, 0);
   h_eunit := Coalesce(eunit, 0);
   h_unit := Coalesce(unit, 0);
   h_work := Coalesce(work001, 0);
   h_contour := Coalesce(contour001, 0);
   h_occurrence := Coalesce(occurrence, 0);
   h_res_cost := Coalesce(res_cost, 0);
   h_type := Coalesce(type_id001, 0);
   h_cri := Coalesce(cri, 0);
   h_consider := Coalesce(consider001, 0);
   h_update := Coalesce(p_update, 0);
   h_per_day := Coalesce(hrs_per_day, 0);
   h_priority := Coalesce(priority001, 0);
   h_j_disconti_date := 0;
   
   If (Coalesce(h_effort, 0) = 0) AND ((Coalesce(Length(finish_date001), 0) < 10)) Then
      finish_date001 := start_date001;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 318+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(319) : cplan_fun.sql(10915) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_start_time := '08:00';
   --strcpy(h_start_time[5], "\0");
   
   h_finish_time := '17:00';
   --strcpy(h_finish_time[5], "\0");
   
   h_prj_assignment_id := NULL;
   h_billable := 0;
   
   If Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30 Then
      h_skip := 1;
      h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
   End If;
   h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0001F000)), 12);
   
   --iSF == 0 when start_date or finish_date is null
   
   --trace_printf("h_assign_id: %s, h_res_id = %s, h_res_type = %s, h_cri: %d\n", h_assign_id,h_res_id,h_res_type,h_cri);
   --trace_printf("p_reg_sale = %.4f, p_spec_sale = %.4f\n",p_reg_sale, p_spec_sale);
   
   If (Coalesce(h_type, 0) > 0) AND (Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30) Then
      h_rollup := 1;
      -- set assign from constrained by project (type > 0)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove assign from constrained by project (type > 0)
      
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
         h_rollup := 3;
         -- set do not rollup
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29));
         -- remove do not rollup
      End If;
   ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
      h_rollup := 2;
      -- set do not rollup (import from MSP)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
      -- remove do not rollup
   ElsIf (Coalesce(h_type, 0) > 0) AND (Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27) Then
      h_rollup := 4;
      -- re_etc schedule
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
   Else
      h_rollup := 0;
      NULL;
   End If;
   
   --trace_printf("*** h_update: %d, start_date = %s, finish_date = %s, h_duration: %d\n", h_update,start_date,finish_date,h_duration);
   --trace_printf("h_every = %d, h_contour = %d, h_type = %d, h_bitflag = %d, h_consider = %d, h_effort = %d, h_unit = %d, h_work: %d\n", h_every,h_contour,h_type,h_bitflag,h_consider,h_effort,h_unit,h_work);
   
   If Coalesce(h_work, 0) >= PMO_CNST.SC_BIT_30 Then
      i_pa_replace := 1;
      -- replace profile, assign remaining @ the project level
      h_work := (BITAND(Coalesce(h_work, 0), PMO_CNST.SC_BIT_NOT_30));
   End If;
   
   h_percent_units := 0;
   If Coalesce(h_unit, 0) = 0 Then
      h_amnt := 1;
   ElsIf (Coalesce(h_unit, 0) = 100) AND (Coalesce(h_work, 0) > 0) Then
      h_percent_units := Coalesce(h_work, 0);
      If (Coalesce(h_duration, 0) > 0) AND (Coalesce(h_work, 0) < 100) Then
         h_duration := Trunc(Coalesce(h_duration, 0) * 10000 / Coalesce(h_work, 0));
      End If;
      h_work := 0;
      h_occurrence := 0;
   End If;
   
   /*  if ( h_bitflag == 0 )   // ASAP
   	{
   		h_work = (h_work  SC_BIT_NOT_30);
   	}*/
   
   --trace_printf("*** h_effort = %d, h_work: %d, h_unit: %d, h_percent_units: %d, h_contour: %d, h_every: %d, calculate_flag: %d\n", h_effort,h_work,h_unit,h_percent_units,h_contour,h_every,calculate_flag);
   --trace_printf("*** h_assign_id: %s, h_duration = %d, h_consider = %d, work_type: %d\n", h_assign_id,h_duration,h_consider,work_type);
   
   If Coalesce(h_consider, 0) = 0 Then
      h_consider := 7;
   End If;
   
   If (Coalesce(h_duration, 0) = 0) AND (Coalesce(Length(start_date001), 0) >= 10) 
             AND (Coalesce(Length(finish_date001), 0) >= 10) Then
      iSF := 1;
   Else
      iSF := 0;
      If Coalesce(h_duration, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 409+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(410) : cplan_fun.sql(11006) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 220;
      End If;
      
      If (Coalesce(h_percent_units, 0) > 0) AND (Coalesce(h_percent_units, 0) < 10000) Then
         h_duration := Trunc(Coalesce(h_duration, 0) * 10000.00 / Coalesce(h_percent_units, 0));
      End If;
   End If;
   
   h_start_DT := start_date001;
   h_finish_DT := finish_date001;
   If Coalesce(Length(h_start_DT), 0) = 10 Then
      h_start_DT := h_start_DT || ' 00:00:00';
   ElsIf Coalesce(h_consider, 0) = 7 AND (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)) , PMO_ORASTD.TIME('00:00:00'))
         INTO h_start_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 434+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(435) : cplan_fun.sql(11031) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("3_ iSF: %d, h_type: %d, h_res_type: %s, h_delay: %d, h_start_DT: %s, h_finish_DT: %d %s, h_duration: %d\n", iSF,h_type,h_res_type,h_delay,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration);
   If Coalesce(h_type, 0) = 0 Then
      h_delay := 0;
   Else
      /*        BEGIN_SQL
      			SELECT TASK_TYPE, UNRESOLVED_START, UNRESOLVED_FINISH, ETC, WORK_TODATE, START_DT
      			INTO HOST(h_task_type), HOST(h_task_start_DT), HOST(h_task_finish_DT), HOST(h_etc), HOST(h_work_todate), HOST(h_act_start_DT)
      			FROM TMT_WBS
      			WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id) ;
      		END_SQL;*/
      --trace_printf("SC_STATE: %d, h_task_type: %d, h_etc: %d, h_work_todate: %d\n", SC_STATE,h_task_type,h_etc,h_work_todate);//trace_flush();
      --trace_printf("h_task_start_DT: %d %s, h_task_finish_DT: %d %s\n", h_task_start_DT_Ind,h_task_start_DT,h_task_finish_DT_Ind,h_task_finish_DT);//trace_flush();
      
      If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
         h_task_type := 0;
      End If;
      
      If Coalesce(h_delay, 0) > 0 Then
         h_dayofyear := NULL;
         stmt := '';
         
         If (SubStr(h_res_type, 1, 1) = '0') Then
            stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                   || 'WHERE ELEMENT_ID = ''' || h_res_id || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_start_DT 
                        || ''', 1, 10)) '
                   || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
            
            OPEN C_DELAY1 FOR stmt;
            
            LOOP
               FETCH C_DELAY1 INTO h_dayofyear;
               IF C_DELAY1%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C_DELAY1;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 489+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(490) : cplan_fun.sql(11086) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(''' 
                  || h_start_DT || ''', 1, 10)) '
                   || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
            
            OPEN C_DELAY0 FOR stmt;
            
            LOOP
               FETCH C_DELAY0 INTO h_dayofyear;
               IF C_DELAY0%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C_DELAY0;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 520+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(521) : cplan_fun.sql(11117) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (h_dayofyear IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 528+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(529) : cplan_fun.sql(11125) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         Else
            If Coalesce(Length(h_start_DT), 0) > 10 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT))
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 543+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(544) : cplan_fun.sql(11140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (SubStr(h_res_type, 1, 1) = '1') Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
                  INTO h_start_DT
                  FROM PMO_TMP_CDY WHERE DAY_YEAR = h_dayofyear;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 558+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(559) : cplan_fun.sql(11155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('08:00'))
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 573+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(574) : cplan_fun.sql(11170) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_temp_var := 0;
            
            start_date001 := h_start_DT;
            
            --trace_printf("D_1* iSF: %d, start_date: %s, h_start_DT: %s, h_finish_DT: %s\n", iSF,start_date,h_start_DT,h_finish_DT);
            If (iSF = 0) AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
                  h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) Then
               If Coalesce(Length(h_finish_DT), 0) > 10 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(h_finish_DT) + h_delay      INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 595+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(596) : cplan_fun.sql(11192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(Length(h_finish_DT), 0) > 0 Then
                  BEGIN
                     SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)) + h_delay      INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 608+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(609) : cplan_fun.sql(11205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("DELAY_* SC_STATE: %d, h_start_DT: %s, h_finish_DT: %d %s\n", SC_STATE,h_start_DT,h_finish_DT_Ind,h_finish_DT); trace_flush();
            End If;
         End If;
      End If;
   End If;
   --trace_printf("h_update: %d, h_start_DT: %s, start_date: %s\n", h_update,h_start_DT,start_date); trace_flush();
   
   --  InitializeHost(SC_PARAM(h_start_DT), SC_PARAM(start_date));
   If (BITAND(Coalesce(h_update, 0), 256)) = 256 Then
      h_start_D := NULL;
      h_lev_start_DT := NULL;
      h_lev_finish_DT := NULL;
      h_start_date := start_date001;
      h_remaining := h_effort;
      
      If Coalesce(work_type, 0) = 2 Then
         BEGIN
            SELECT LAST_SUBMITTED, LAST_SCHED_MIN, SCHED_START, SCHED_FINISH, SCHED_WORK - WORK_TODATE,
            LEVEL_START, LEVEL_FINISH, LEVEL_EFFORT, CONSIDER, ASSIGN_STRING, PROJECT_ASSIGNMENT_ID
            INTO h_start_D, h_last_day, h_ta_start_DT, h_ta_finish_DT, h_remaining,
            h_lev_start_DT, h_lev_finish_DT, h_lev_effort, h_lev_consider, h_lev_assign_string, h_prj_assignment_id
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 642+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(643) : cplan_fun.sql(11239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_start_D IS NOT NULL) Then
            h_act_start_DT := h_ta_start_DT;
            i_has_actual := 1;
            --trace_printf("h_ta_start_DT: %d:%s, h_start_D: %s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_start_D);
            
            If Coalesce(h_last_day, 0) > 0 Then
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assignment_id
                                   , h_assign_id, h_start_D, h_last_day, 0);
            End If;
            
            -- get start first datetime > last_submitted
            
            OPEN C_LS;
            
            FETCH C_LS INTO h_ta_start_DT;
            IF C_LS%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("h_ta_start_DT: %d:%s, start_date: %s\n", h_ta_start_DT_Ind,h_ta_start_DT,start_date);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (PMO_ORASTD.strncmp(start_date001, h_ta_start_DT
                  , 16) <= 0) Then
               h_start_date := h_ta_start_DT;
            End If;
            
            BEGIN
               CLOSE C_LS ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 682+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(683) : cplan_fun.sql(11279) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- get the real assignment start date
            
            OPEN C_TA;
            
            FETCH C_TA INTO h_ta_start_DT;
            IF C_TA%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("REAL: h_ta_start_DT: %d:%s, start_date: %s\n", h_ta_start_DT_Ind,h_ta_start_DT,start_date);
            
            BEGIN
               CLOSE C_TA ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 707+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(708) : cplan_fun.sql(11304) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH, LEVEL_START, LEVEL_FINISH, LEVEL_EFFORT,
            CONSIDER, PLANNED_WORK, ASSIGN_STRING, PROJECT_ASSIGNMENT_ID
            INTO h_ta_start_DT, h_ta_finish_DT, h_lev_start_DT, h_lev_finish_DT, h_lev_effort,
            h_lev_consider, h_remaining, h_lev_assign_string, h_prj_assignment_id
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 726+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(727) : cplan_fun.sql(11323) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("*** h_lev_consider: %d\n", h_lev_consider);
      
      If (Coalesce(h_lev_consider, 0) = 0) OR (Coalesce(h_lev_consider, 0) = 10) Then
         h_lev_consider := 7;
      End If;
      
      --trace_printf("Calculate_1: SC_STATE: %d, h_start_date: %d:%s, h_effort: %d, h_remaining: %d, calculate_flag: %d\n", SC_STATE,h_start_date_Ind,h_start_date,h_effort,h_remaining,calculate_flag);
      --trace_printf("h_ta_start_DT: %d:%s, h_ta_finish_DT: %d:%s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_ta_finish_DT_Ind,h_ta_finish_DT);
      --trace_printf("h_lev_start_DT: %d:%s, h_lev_finish_DT: %d:%s, h_lev_effort: %d, h_lev_consider: %d\n", h_lev_start_DT_Ind,h_lev_start_DT,h_lev_finish_DT_Ind,h_lev_finish_DT,h_lev_effort,h_lev_consider);
      
      If Coalesce(h_last_day, 0) > 0 Then
         h_start_date := h_ta_start_DT;
      ElsIf ((h_lev_start_DT IS NOT NULL)) AND (Coalesce(h_lev_effort, 0) = Coalesce(h_effort, 0)) 
            AND (PMO_ORASTD.strncmp(h_start_date, h_lev_start_DT, 16) = 0) 
            AND (calculate_flag = 1) Then
         start_date001 := h_lev_start_DT;
         finish_date001 := h_lev_finish_DT;
         
         --trace_printf("RETURN_LEV: start_date: %d %s, finish_date: %d:%s\n", start_date_Ind,start_date,finish_date_Ind,finish_date);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 751+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(752) : cplan_fun.sql(11348) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      ElsIf (PMO_ORASTD.strncmp(h_start_date, h_ta_start_DT, 16) = 0) AND (Coalesce(
            h_lev_consider, 0) = 7) AND (Coalesce(h_remaining, 0) = Coalesce(h_effort, 0)) Then
         start_date001 := h_start_date;
         finish_date001 := h_ta_finish_DT;
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            LEVEL_START         = h_start_date,
            LEVEL_FINISH        = h_ta_finish_DT,
            LEVEL_EFFORT        = h_effort,
            LEVEL_ASSIGN_STRING = ASSIGN_STRING
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 776+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(777) : cplan_fun.sql(11373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM LEVEL_TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 794+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(795) : cplan_fun.sql(11391) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("RETURN_TA: start_date: %d %s, finish_date: %d:%s\n", start_date_Ind,start_date,finish_date_Ind,finish_date);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 801+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(802) : cplan_fun.sql(11398) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      ElsIf (Coalesce(h_effort, 0) > Coalesce(h_remaining, 0)) AND (PMO_ORASTD.strncmp(h_start_date
                                        , h_ta_start_DT, 16) = 0) Then
         BEGIN
            SELECT SCHED_MIN INTO h_minutes
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id
            AND DAY_YEAR = TO_DATE(SubStr(h_ta_finish_DT, 1, 10)) AND BIT_FLAG = '2' AND ACTUAL_FLAG = 0;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 817+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(818) : cplan_fun.sql(11414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            If i_has_actual = 0 Then
               h_act_start_DT := h_start_date;
            End If;
            i_lev_etc := 1;
            h_effort := Coalesce(h_effort, 0) + Coalesce(h_minutes, 0) - Coalesce(h_remaining, 0);
            
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_ta_finish_DT, 1, 10)),PMO_ORASTD.TIME(h_ta_start_DT)) INTO h_start_date FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 837+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(838) : cplan_fun.sql(11434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            start_date001 := h_start_date;
         End If;
         --trace_printf("NEW_EFFORT: h_effort: %d, h_minutes: %d, h_remaining: %d\n", h_effort, h_minutes, h_remaining);
      End If;
      
      start_date001 := h_start_date;
      --trace_printf("Calculate_2: start_date: %s, h_start_date: %d:%s, h_effort: %d, i_lev_etc: %d\n", start_date,h_start_date_Ind,h_start_date,h_effort,i_lev_etc);
      --trace_printf("h_start_DT: %d %s, h_finish_DT: %d:%s, h_duration: %d\n", h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
      --trace_printf("i_has_actual: %d, i_lev_etc: %d, h_act_start_DT: %s\n", i_has_actual,i_lev_etc,h_act_start_DT);
   Else
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         h_lev_start_DT := NULL;
         h_lev_finish_DT := NULL;
         
         BEGIN
            SELECT LAST_SUBMITTED, SCHED_START, LEVEL_START, LEVEL_FINISH
            INTO h_start_D, h_ta_start_DT, h_lev_start_DT, h_lev_finish_DT
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 866+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(867) : cplan_fun.sql(11463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_assign_id: %s, h_lev_start_DT: %d %s, h_lev_finish_DT: %d %s\n", SC_STATE,h_assign_id,h_lev_start_DT_Ind,h_lev_start_DT,h_lev_finish_DT_Ind,h_lev_finish_DT);
         
         If (h_start_D IS NOT NULL) Then
            h_act_start_DT := h_ta_start_DT;
            i_has_actual := 1;
         End If;
         
         If ((h_lev_start_DT IS NOT NULL) OR (h_lev_finish_DT IS NOT NULL)) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET LEVEL_START = NULL, LEVEL_FINISH = NULL, LEVEL_EFFORT = 0, LEVEL_ASSIGN_STRING = NULL
               WHERE TSK_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 891+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(892) : cplan_fun.sql(11488) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 908+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(909) : cplan_fun.sql(11505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      h_start_date := start_date001;
   End If;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      h_finish_date := finish_date001;
      h_assign_string := '';
      h_julian_day := 1;
      --trace_printf("h_start_date: %d %s, finish_date: %d %s\n", h_start_date_Ind,h_start_date,finish_date_Ind,finish_date);
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_date), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
         INTO h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
         FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 933+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(934) : cplan_fun.sql(11530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_active: %s, h_j_disconti_date: %d, h_julian_day: %d\n", SC_STATE,h_active,h_j_disconti_date,h_julian_day);
      
      If (SubStr(h_active, 1, 1) = '0') OR ((Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(
            h_julian_day, 0) > Coalesce(h_j_disconti_date, 0))) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 942+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(943) : cplan_fun.sql(11539) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
      End If;
   Else
      --trace_printf("h_update: %d, h_lev_start_DT: %d %s, h_lev_finish_DT: %d %s\n", h_update,h_lev_start_DT_Ind,h_lev_start_DT,h_lev_finish_DT_Ind,h_lev_finish_DT);
      
      If ((BITAND(Coalesce(h_update, 0), 255)) = 10) AND (((h_lev_start_DT IS NOT NULL)) 
                     OR ((h_lev_finish_DT IS NOT NULL))) Then
         --trace_printf("SEARCH: nullify level dates\n");
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET LEVEL_START = NULL, LEVEL_FINISH = NULL, LEVEL_EFFORT = 0, LEVEL_ASSIGN_STRING = NULL
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 965+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(966) : cplan_fun.sql(11562) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE = %d, h_assign_id = %d:%s\n", SC_STATE,h_assign_id_Ind,h_assign_id);
         
         BEGIN
            DELETE FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 983+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(984) : cplan_fun.sql(11580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("h_type = %d, h_percent_units: %d, h_tsk_assignment_id: %d:%s, h_prj_assignment_id = %d:%s\n", h_type,h_percent_units,h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
      h_day_min := 0;
      If Coalesce(h_type, 0) > 0 Then
         assign_to_prj := PMO_ORASTD.strcmp(h_project_id, h_task_id);
         h_prj_percent_units := 0;
         --trace_printf("assign_to_prj: %d, h_rollup: %d, h_update: %d\nh_project_id: %s, h_task_id = %s\n", assign_to_prj,h_rollup,h_update,h_project_id,h_task_id);
         
         If (Coalesce(h_rollup, 0) = 4) OR ((BITAND(Coalesce(h_update, 0), 255)) = 10) Then
            h_tsk_assignment_id := assign_id;
            
            If ((BITAND(Coalesce(h_update, 0), 255)) = 10) OR (Coalesce(h_percent_units, 0) = 0 AND Coalesce(
                     h_work, 0) = 0 AND Coalesce(h_bitflag, 0) = 1) Then
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, UNITS_MSP
                  INTO h_prj_assignment_id, h_percent_units
                  FROM TASK_ASSIGNMENTS 
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1012+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1013) : cplan_fun.sql(11609) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID
                  INTO h_prj_assignment_id
                  FROM TASK_ASSIGNMENTS 
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1028+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1029) : cplan_fun.sql(11625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_work_type, 0) = 2 Then
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, UNITS_MSP
                  INTO h_prj_assignment_id, h_prj_percent_units
                  FROM TASK_ASSIGNMENTS 
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '2';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1046+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1047) : cplan_fun.sql(11643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_work_type, 0) = 0 Then
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, UNITS_MSP
                  INTO h_prj_assignment_id, h_prj_percent_units
                  FROM TASK_ASSIGNMENTS 
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1062+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1063) : cplan_fun.sql(11659) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
                  h_prj_percent_units, 0) > 0) AND (Coalesce(h_percent_units, 0) > Coalesce(
                  h_prj_percent_units, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1072+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1073) : cplan_fun.sql(11669) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
            End If;
            
            --                if ( (SC_SUCCESS)  ((isProjectType(h_type)) || (h_consider == SC_SRCH_FROM_PROJECT)) )
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND isProjectType(Coalesce(h_type, 0)) <> 0 Then
               h_tsk_assignment_id := h_prj_assignment_id;
            Else
               h_tsk_assignment_id := SYS_GUID();
            End If;
         End If;
         
         If (Coalesce(h_percent_units, 0) = 0) AND (Coalesce(h_prj_percent_units, 0) > 0) Then
            h_percent_units := Coalesce(h_prj_percent_units, 0);
         End If;
         --trace_printf("1_____* h_percent_units: %d\n", h_percent_units);
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDOY';
         PMO_DB2M.SC_STATE := 0;
         
         /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_etc > 0)  ((h_bitflag == 0) || ((h_bitflag == 1)))  (h_rollup == 0) )
         			{
         				iSF = 1;
         				h_work = 0;
         				if (h_consider != SC_SRCH_PROFILE)
         				{
         					h_consider = 7;
         				}
         
         				BEGIN_SQL
         					SELECT COUNT(*) INTO HOST(h_cnt)
         					FROM TASK_ASSIGNMENTS
         					WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id);
         				END_SQL;
         //trace_printf("SC_STATE: %d, h_etc: %d, h_work_todate: %d, h_cnt: %d\n", SC_STATE,h_etc,h_work_todate,h_cnt);//trace_flush();
         
         				if (h_cnt == 0) // this is the first assignment
         				{
         					h_percent_units = 0;
         				}
         				else
         				{
         					h_temp_sc_state = FC_FD_EFFORT_DRIVEN(
         									SC_PARAM(h_project_id),
         									SC_PARAM(h_task_id),
         									SC_PARAM(h_res_id),
         									SC_PARAM(h_res_type),
         									SC_PARAM(h_task_start_DT),
         									SC_PARAM(h_task_finish_DT),
         									SC_PARAM(h_etc),
         									SC_PARAM(h_work_todate),
         									SC_PARAM(h_cnt),
         									SC_PARAM(h_per_day));
         //trace_printf("h_temp_sc_state: %d, h_task_start_DT: %d %s, h_task_finish_DT: %d %s\n", h_temp_sc_state,h_task_start_DT_Ind,h_task_start_DT,h_task_finish_DT_Ind,h_task_finish_DT);//trace_flush();
         
         					if ( h_temp_sc_state != 0 )
         					{
         						return h_temp_sc_state;
         					}
         
         					BEGIN_SQL
         						SELECT RESUME_DT, NEW_REMAINING, NEW_UNITS
         						INTO HOST(h_start_DT), HOST(h_effort), HOST(h_percent_units)
         						FROM session.PMO_TMP_FIXDUR WHERE RESOURCE_ID = HOST(h_res_id);
         					END_SQL;
         //trace_printf("a.** SC_STATE: %d, h_start_DT: %s, h_effort: %d, h_percent_units: %d\n", SC_STATE,h_start_DT,h_effort,h_percent_units);
         
         //                  InitializeHost(SC_PARAM(h_effort), 0);
         					InitializeHost(SC_PARAM(h_finish_DT), h_task_finish_DT);
         				}
         			}*/
         
         h_uniqueID := h_prj_assignment_id;
         --trace_printf("2_____* h_percent_units: %d, h_update: %d\n", h_percent_units,h_update);
         
         If ((BITAND(Coalesce(h_update, 0), 255)) <> 10) AND (Coalesce(h_unit, 0) <> 100) AND (Coalesce(
                     h_work, 0) > 0) Then
            h_percent_units := 0;
            h_day_min := Coalesce(h_work, 0);
         End If;
      Else
         If (Coalesce(h_duration, 0) = 0) AND (Coalesce(Length(start_date001), 0) >= 10) 
               AND (Coalesce(Length(finish_date001), 0) >= 10) Then
            iSF := 1;
            h_finish_DT := finish_date001;
         Else
            iSF := 0;
         End If;
         
         h_start_DT := start_date001;
      End If;
      --trace_printf("iSF: %d, start_date = %d:%s, finish_date = %d:%s\nh_start_DT = %d:%s, h_finish_DT = %d:%s, h_duration = %d\n", iSF,start_date_Ind,start_date,finish_date_Ind,finish_date,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration);
      --trace_printf("h_res_id: %s, h_bitflag = %d, h_consider = %d, h_rollup = %d, h_effort: %d, h_work: %d\nh_start_DT = %s, h_finish_DT = %s, h_duration = %d\n", h_res_id,h_bitflag,h_consider,h_rollup,h_effort,h_work,h_start_DT,h_finish_DT,h_duration); trace_flush();
      If (Coalesce(h_effort, 0) > 0) AND (iSF = 1) Then
         h_total := 0;
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_count 
               FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1181) : cplan_fun.sql(11777) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_count
               FROM TASK_DAY_OF_YEAR
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1196+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1197) : cplan_fun.sql(11793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_count
               FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_res_id 
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1212+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1213) : cplan_fun.sql(11809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_count, 0) > 0 Then
            --h_tmp_percent_units = ((double) h_effort / (double) h_total) * 10000.00;
            h_work_per_day := Trunc((Coalesce(h_effort, 0) / Coalesce(h_count, 0)) + 0.49);
            If (Coalesce(h_per_day, 0) > 0) AND (Coalesce(h_work_per_day, 0) > 0) Then
               If Coalesce(h_work_per_day, 0) = Coalesce(h_per_day, 0) Then
                  h_tmp_percent_units := 10000;
               Else
                  h_tmp_percent_units := Trunc((Coalesce(h_work_per_day, 0) / Coalesce(h_per_day, 0)) * 10000.00);
               End If;
            End If;
            h_work := 0;
            
            If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
                  h_prj_percent_units, 0) > 0) AND (Coalesce(h_tmp_percent_units, 0) > Coalesce(
                  h_prj_percent_units, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1234+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1235) : cplan_fun.sql(11831) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
            Else
               h_percent_units := Coalesce(h_tmp_percent_units, 0);
               --h_work_units = h_tmp_percent_units;
            End If;
            
            If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) AND (Coalesce(
                  h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
               h_consider := PMO_CNST.SC_SRCH_OVERALLOCATE;
            End If;
         End If;
         
         --   if ( h_total > 0 )
         --   {
         --    h_temp_var = ((double) h_effort / (double) h_total) * 10000.00;
         --    h_work = 0;
         --
         --    if ((h_consider == SC_SRCH_FROM_PROJECT)  (h_temp_var > h_percent_units))
         --    {
         --     return CANNOT_ASSIGN_PROJ_AFFECT_DATES;
         --    }
         --    else
         --    {
         --     h_percent_units = h_temp_var;
         --    }
         --
         --    if (h_consider != SC_SRCH_PROFILE)
         --    {
         --     h_consider = 7;
         --    }
         --trace_printf("UNITS_** h_percent_units: %d, h_consider = %d\n", h_percent_units,h_consider); trace_flush();
         --   }
      ElsIf ((BITAND(Coalesce(h_update, 0), 255)) <> 10) AND (Coalesce(h_rollup, 0) <> 4) AND ((Coalesce(
                        h_bitflag, 0) = 0) OR (Coalesce(h_percent_units, 0) = 10000)) Then
         -- h_update  0x000FF)  == 10 --> Leveling
         -- h_rollup             == 4  --> reschedule
         -- h_bitflag            == 0  --> ASAP
         h_percent_units := 0;
      End If;
      
      If Coalesce(h_percent_units, 0) > 0 Then
         If Coalesce(h_per_day, 0) = 0 Then
            h_per_day := 480;
         End If;
         
         If Coalesce(h_percent_units, 0) = 10000 Then
            h_day_min := Coalesce(h_per_day, 0);
         Else
            h_day_min := Trunc(((Coalesce(h_per_day, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.5);
         End If;
         
         If Coalesce(h_day_min, 0) < 5 Then
            h_day_min := 5;
         End If;
      End If;
      --trace_printf("h_task_type = %d, h_percent_units: %d, h_day_min: %d, h_tsk_assignment_id: %d:%s, h_prj_assignment_id = %d:%s\n", h_task_type,h_percent_units,h_day_min,h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      --trace_printf("1_** h_consider = %d\n", h_consider); trace_flush();
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         If (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
            If Coalesce(h_duration, 0) = 0 Then
               BEGIN
                  SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
                  INTO h_julian_day, h_julian_finish, h_active, h_j_efective_date, h_j_disconti_date
                  FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1307+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1308) : cplan_fun.sql(11904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_julian_finish, 0) < Coalesce(h_j_efective_date, 0) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1314+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1315) : cplan_fun.sql(11911) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
               End If;
            Else
               BEGIN
                  SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
                  INTO h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
                  FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1329+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1330) : cplan_fun.sql(11926) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_julian_finish := 0;
            --trace_printf("1_h_active = %s, h_j_disconti_date = %d, h_julian_day = %d, h_j_efective_date = %d\n", h_active,h_j_disconti_date,h_julian_day,h_j_efective_date); trace_flush();
            
            If (SubStr(h_active, 1, 1) = '0') OR ((Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(
                  h_julian_day, 0) > Coalesce(h_j_disconti_date, 0))) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1341+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1342) : cplan_fun.sql(11938) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
            End If;
         End If;
      Else
         h_j_efective_date := 0;
         h_j_disconti_date := 0;
         h_active := '';
      End If;
      --trace_printf("2_h_active = %s, h_j_disconti_date = %d, h_julian_day = %d, h_j_efective_date = %d\n", h_active,h_j_disconti_date,h_julian_day,h_j_efective_date); trace_flush();
      
      --trace_printf("0.2_ h_start_DT: %s, h_finish_DT: %s, h_duration: %d\n", h_start_DT,h_finish_DT,h_duration);
      If Coalesce(h_julian_day, 0) < Coalesce(h_j_efective_date, 0) Then
         If Coalesce(Length(h_start_DT), 0) > 10 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(h_start_DT) + (h_j_efective_date - h_julian_day)     
               INTO h_start_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1366+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1367) : cplan_fun.sql(11963) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT TO_DATE(SubStr(h_start_DT, 1, 10)) + (h_j_efective_date - h_julian_day)     
               INTO h_start_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1381+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1382) : cplan_fun.sql(11978) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_skip := 0;
      End If;
      
      --AEL: Loading the data to the tmp table
      --trace_printf("1_ h_start_DT: %s, h_finish_DT: %s, h_duration: %d,h_priority: %d\n", h_start_DT,h_finish_DT,h_duration,h_priority);
      --trace_printf("h_update: %d, assign_to_prj: %d, h_consider: %d, h_work: %d, h_min_work_per_day: %d\n",h_update,assign_to_prj,h_consider,h_work, h_min_work_per_day);
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         --t_isLevel = 0;
         If Coalesce(h_work, 0) >= PMO_CNST.SC_BIT_30 Then
            h_min_work_per_day := (BITAND(Coalesce(h_work, 0), PMO_CNST.SC_BIT_NOT_30));
            h_work := 0;
         Else
            h_min_work_per_day := 0;
         End If;
         
         h_temp_sc_state := FC_CREATE_TDY_LEVEL(h_project_id, h_res_id, h_start_DT, h_consider, h_cri
                                       , h_priority, h_update);
         
         If Coalesce(assign_to_prj, 0) = 0 Then
            BEGIN
               SELECT COUNT(RESOURCE_ID) INTO h_temp_var
               FROM PMO_TMP_TDY_LEVEL
               WHERE RESOURCE_ID = h_res_id AND PROJECT_ID = h_project_id AND PROJ_ASSIGN_FLAG = 0;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1416+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1417) : cplan_fun.sql(12013) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) = 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_TDY
                  SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN END,BIT_FLAG, BIT_SCHED
                  FROM LEVEL_TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND TASK_ID <> PROJECT_ID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1437+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1438) : cplan_fun.sql(12034) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2.0 ==> SC_STATE: %d, h_project_id: %s, h_res_id: %s\n", SC_STATE,h_project_id,h_res_id);
            End If;
         End If;
      Else
         h_temp_sc_state := FC_CREATE_TDY(h_project_id, h_res_id, h_start_DT, h_consider, h_cri, h_priority
                                 , h_update);
      End If;
      --trace_printf("2_ h_consider: %d, h_start_DT: %s, h_finish_DT: %d %s, h_duration: %d\n", h_consider,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration);
      --trace_printf("h_work: %d, h_min_work_per_day: %d\n",h_work, h_min_work_per_day);
      --t_start = clock();
      --s_time_2 = clock();
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) AND (Coalesce(
               h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) Then
            -- Extend resource calendar
            If EXTEND_PRCAL(h_res_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_RESOURCE) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1458+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1459) : cplan_fun.sql(12055) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            ElsIf Coalesce(Length(h_finish_DT), 0) > 0 Then
               h_finish_DT := h_finish_DT;
               -- h_finish_DT_Ind = 0;
            End If;
         End If;
         
         If Coalesce(h_duration, 0) > 0 Then
            If Coalesce(h_bitflag, 0) > 1 Then
               h_temp_var := 0;
            Else
               h_temp_var := h_work;
            End If;
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                  h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
                  If Coalesce(h_min_work_per_day, 0) > 0 Then
                     h_temp_var := Coalesce(h_min_work_per_day, 0);
                  ElsIf (Coalesce(h_temp_var, 0) > 0) AND (Coalesce(assign_to_prj, 0) = 0) Then
                     BEGIN
                        SELECT MAX(SCHED_MIN) INTO h_min_work_per_day FROM PMO_TMP_PA_SUM;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1488+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1489) : cplan_fun.sql(12085) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     If (Coalesce(h_min_work_per_day, 0) > 0) AND (Coalesce(h_min_work_per_day, 0) <= Coalesce(
                           h_temp_var, 0)) Then
                        h_temp_var := Coalesce(h_temp_var, 0) - Coalesce(h_min_work_per_day, 0);
                     End If;
                  End If;
                  
                  If CHECK_ASSIGN_PROF(h_project_id, h_res_id, h_start_DT, h_finish_DT, h_duration, h_temp_var
                        , h_effort, h_consider) = -1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1501+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1502) : cplan_fun.sql(12098) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
                  End If;
                  
                  If Coalesce(h_min_work_per_day, 0) > 0 Then
                     h_temp_var := 0;
                  End If;
                  h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
               Else
                  If CHECK_ASSIGN_PROF(h_project_id, h_res_id, h_start_DT, h_finish_DT, h_duration, h_temp_var
                        , h_effort, h_consider) = -1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1514+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1515) : cplan_fun.sql(12111) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
                  End If;
               End If;
               --trace_printf("CHECK_ASSIGN_PROF: h_start_DT:%s, h_finish_DT:%s\n", h_start_DT,h_finish_DT);
            ElsIf NOT ((Coalesce(h_consider, 0) = 7) OR (Coalesce(h_bitflag, 0) = 4)) Then
               --t_start = clock();
               If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                  h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
                  If Coalesce(h_min_work_per_day, 0) > 0 Then
                     h_temp_var := Coalesce(h_min_work_per_day, 0);
                  End If;
                  
                  If CHECK_ASSIGN(h_project_id, h_res_id, h_start_DT, h_finish_DT, h_duration, h_temp_var, h_effort
                        , h_consider) = -1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1531+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1532) : cplan_fun.sql(12128) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
                  End If;
                  
                  If Coalesce(h_min_work_per_day, 0) > 0 Then
                     h_temp_var := 0;
                  End If;
                  h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
                  --t_finish = clock();
               Else
                  If CHECK_ASSIGN(h_project_id, h_res_id, h_start_DT, h_finish_DT, h_duration, h_temp_var, h_effort
                        , h_consider) = -1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1545+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1546) : cplan_fun.sql(12142) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
                  End If;
               End If;
               --trace_printf("CHECK_ASSIGN: h_start_DT:%s, h_finish_DT:%s\n", h_start_DT,h_finish_DT);
            End If;
         Else
            BEGIN
               INSERT INTO PMO_TMP_CPLNF_0001 SELECT DAY_YEAR, SUM(SCHED_MIN)
               FROM PMO_TMP_TDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
               GROUP BY DAY_YEAR;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1567+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1568) : cplan_fun.sql(12164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("** INSERT: SC_STATE: %d, h_start_DT = %s\n",SC_STATE,h_start_DT);
         End If;
      End If;
      
      --trace_printf("3_ h_skip: %d, h_start_DT: %s, h_finish_DT: %d %s\n",h_skip,h_start_DT,h_finish_DT_Ind,h_finish_DT);
      If (Coalesce(Length(h_start_DT), 0) > 10) AND (Coalesce(Length(h_finish_DT), 0) > 10) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_DT)
            INTO h_start_date, h_start_time
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1587+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1588) : cplan_fun.sql(12184) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)
            INTO h_finish_date, h_finish_time
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1602+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1603) : cplan_fun.sql(12199) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_dayofyear := SubStr(h_finish_date, 1, 10);
      Else
         If Coalesce(Length(h_finish_DT), 0) > 10 Then
            BEGIN
               SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)
               INTO h_finish_date, h_finish_time
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1620+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1621) : cplan_fun.sql(12217) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_dayofyear := SubStr(h_finish_date, 1, 10);
         ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
            BEGIN
               SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_DT)
               INTO h_start_date, h_start_time
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1637+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1638) : cplan_fun.sql(12234) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(Length(h_start_DT), 0) > 0) AND (Coalesce(Length(h_finish_DT), 0) > 0) Then
            h_start_date := h_start_DT;
            h_finish_date := h_finish_DT;
            h_dayofyear := SubStr(h_start_date, 1, 10);
         ElsIf Coalesce(Length(h_start_DT), 0) > 0 Then
            h_start_date := h_start_DT;
            h_dayofyear := SubStr(h_start_date, 1, 10);
         ElsIf Coalesce(Length(h_finish_DT), 0) > 0 Then
            h_finish_date := h_finish_DT;
            h_dayofyear := SubStr(h_start_date, 1, 10);
         End If;
      End If;
      --trace_printf("3__*** h_start_time = %s, h_finish_time = %s\n",h_start_time,h_finish_time);
      --trace_printf("1_* h_work: %d, h_min_work_per_day: %d\n", h_work,h_min_work_per_day);
      --trace_printf("h_start_date = %s, h_finish_date = %s\n", h_start_date,h_finish_date);
      --trace_printf("h_start_DT = %s, h_finish_DT = %s\n", h_start_DT,h_finish_DT);
      
      --if (t_isLevel == 1){
      --BEGIN_SQL
      --  UPDATE session.PMO_TMP_TIMEING SET DURATION = DAYS(DATE(HOST(h_start_DT)))  WHERE NAME = 'A_START_DATE';
      --END_SQL;}
      
      --trace_printf("\n 000=====>h_bitflag = %d\n",h_bitflag);
      
      --trace_printf("\n 001=====> %s \n",(char *) h_pmo_tmp_res_ins_stmt);
      /**NOTE** Apply Temporary Tables Here */
      
      If Coalesce(h_bitflag, 0) = 0 Then
         h_work := 0;
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            --no need to temporary tables
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.SCHED_MIN, 0 AS WEEK_OF, 0 AS CNT FROM PMO_TMP_TDY p WHERE p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_start_DT;
            stmt := stmt || ''')';
            stmt := stmt || ' AND p.SCHED_MIN > 0 ORDER BY DAY_YEAR';
         ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            -- no need to temporary tables
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT FROM PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_start_DT;
            stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_finish_DT;
            stmt := stmt || ''') AND AVAILABLE_MIN > 0 ORDER BY DAY_YEAR';
         ElsIf ((BITAND(Coalesce(h_update, 0), 255)) = 10) AND (Coalesce(h_min_work_per_day, 0) > 0) Then
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT FROM CALENDAR_DAY_OF_YEAR p ';
            stmt := stmt || 'WHERE p.ELEMENT_ID=''';
            stmt := stmt || h_res_id;
            stmt := stmt || ''' AND p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_start_DT;
            stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_finish_DT;
            stmt := stmt || ''') AND p.AVAILABLE_MIN > ';
            stmt := stmt || TO_CHAR(h_min_work_per_day);
            stmt := stmt || ' ORDER BY DAY_YEAR';
         Else
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT FROM CALENDAR_DAY_OF_YEAR p ';
            stmt := stmt || 'WHERE p.ELEMENT_ID=''';
            stmt := stmt || h_res_id;
            stmt := stmt || ''' AND p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_start_DT;
            stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
            stmt := stmt || h_finish_DT;
            stmt := stmt || ''') AND p.AVAILABLE_MIN > 0 ORDER BY DAY_YEAR';
         End If;
      Else
         --trace_printf("\n 002=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
               -- no need to temporary tables
               --stmt.strcpy ("SELECT JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, COUNTER() AS CNT FROM session.PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN DATE('");
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                     || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR 
                     || ' AS CNT FROM PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
               
               stwhere := stwhere || h_start_DT;
               stwhere := stwhere || ''') AND PMO_ORASTD.DB2DATE(''';
               stwhere := stwhere || h_finish_DT;
               stwhere := stwhere || ''')';
               stmt := stmt || stwhere;
            Else
               --trace_printf("\n 003=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
               -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
               
               -- create PMO_TMP_RES temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
               PMO_DB2M.SC_STATE := 0;
               
               -- Prepare Dynamic Insert statement to insert records in PMO_TMP_RES
               -- Use SESSION.PMO_TMP_CDY (p) as a source table
               h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES(DAY_YEAR, DAY_WEEK, WEEK_OF, MONTH_OF, YEAR_OF, AVAIL_MIN) ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR AS DATE, p.DAY_WEEK AS DAY_WEEK, '
                      || ' p.WEEK_OF AS WEEK_OF, p.MONTH_OF AS MONTH_OF, '
                      || ' p.YEAR_OF AS YEAR_OF, p.AVAILABLE_MIN AS AVAIL_MIN '
                      || ' FROM PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
               
               --trace_printf("\n 004=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
               -- stmt statement will be prepared with using PMO_TMP_RES in the rest of lines
               
               /**NOTE** Commented by Amir September 08, 2002
               					stmt.strcpy("WITH pd AS (SELECT p.DAY_YEAR, p.DAY_WEEK, p.YEAR_OF, p.MONTH_OF, p.WEEK_OF, p.AVAILABLE_MIN AS AVAIL_MIN FROM session.PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN DATE('");
               					stwhere.strcat(h_start_DT);
               					stwhere.strcat("') AND DATE('");
               					stwhere.strcat(h_finish_DT);
               					stwhere.strcat("')");
               					stmt.strcat(stwhere);*/
            End If;
         ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
               -- no need to temporary tables
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.SCHED_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                     || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR 
                     || ' AS CNT FROM PMO_TMP_TDY p WHERE p.DAY_YEAR >= TO_DATE(SubStr(''' 
                     || h_start_DT || ''', 1, 10))';
            Else
               -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
               
               -- create PMO_TMP_RES temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
               PMO_DB2M.SC_STATE := 0;
               
               If Coalesce(h_bitflag, 0) = 2 Then
                  -- Prepare dynamic Insert Statement to insert records in PMO_TMP_RES
                  -- this dynamic statement will be executed in the rest of statement
                  -- for this case there is no need to MONTH_OF field, it will be null in all records
                  
                  -- Use TASK_ASSIGNMENTS(d) and TASK_DAY_OF_YEAR (p) as source tables
                  
                  h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, YEAR_OF, WEEK_OF, AVAIL_MIN) ';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR AS DAY_YEAR, PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) AS DAY_WEEK, '
                         || ' PMO_ORASTD.YEAR(p.DAY_YEAR) + CASE WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 1 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 12 THEN 1 '
                         || ' WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 53 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 1 THEN -1 ELSE 0 END AS YEAR_OF, '
                         || ' PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) AS WEEK_OF, p.SCHED_MIN AS AVAIL_MIN '
                         || ' FROM TASK_ASSIGNMENTS d JOIN TASK_DAY_OF_YEAR p ON d.TSK_ASSIGNMENT_ID = p.TSK_ASSIGNMENT_ID '
                         || ' WHERE d.RESOURCE_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND d.TASK_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_project_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') ';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_bitflag_condition;
                  -- condition for BIT_FLAG field
                  
                  --trace_printf("\n 005=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
               Else
                  -- Use TASK_ASSIGNMENTS(d) and TASK_DAY_OF_YEAR (p) as source tables
                  
                  h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, WEEK_OF, MONTH_OF, DAY_WEEK, YEAR_OF, AVAIL_MIN) ';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR, PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) AS WEEK_OF, PMO_ORASTD.MONTH(p.DAY_YEAR) AS MONTH_OF, '
                         || ' PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) AS DAY_WEEK, PMO_ORASTD.YEAR(p.DAY_YEAR) + CASE WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 1 AND '
                         || ' PMO_ORASTD.MONTH(p.DAY_YEAR) = 12 THEN 1 WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 53 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 1 THEN -1 '
                         || ' ELSE 0 END AS YEAR_OF, p.SCHED_MIN AS AVAIL_MIN '
                         || ' FROM TASK_ASSIGNMENTS d JOIN TASK_DAY_OF_YEAR p ON d.TSK_ASSIGNMENT_ID = p.TSK_ASSIGNMENT_ID '
                         || ' WHERE d.RESOURCE_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND d.TASK_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_project_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') ';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_bitflag_condition;
                  -- condition for BIT_FLAG field
               End If;
               --trace_printf("\n 006=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
            End If;
         Else
            If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
               -- no need to temporary table
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                     || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR 
                     || ' AS CNT FROM CALENDAR_DAY_OF_YEAR p WHERE p.ELEMENT_ID=''';
               stmt := stmt || h_res_id;
               stwhere := ''' AND p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
               stwhere := stwhere || h_start_DT;
               stwhere := stwhere || ''') AND PMO_ORASTD.DB2DATE(''';
               stwhere := stwhere || h_finish_DT;
               stwhere := stwhere || ''')';
               stmt := stmt || stwhere;
            Else
               -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
               
               -- create PMO_TMP_RES temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
               PMO_DB2M.SC_STATE := 0;
               
               If Coalesce(h_bitflag, 0) = 2 Then
                  -- Prepare Dynamic Insert statement for inserting necessary records in PMO_TMP_RES
                  -- for this case there is no need to MONTH_OF field
                  
                  -- Use CALENDAR_DAY_OF_YEAR(d),  session.PMO_TMP_CDY (p)  as Source Tables
                  h_pmo_tmp_res_ins_stmt := 'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, WEEK_OF,YEAR_OF, AVAIL_MIN)';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' SELECT p.DAY_YEAR, p.DAY_WEEK, p.WEEK_OF, p.YEAR_OF,'
                         || ' p.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)'
                         || ' FROM CALENDAR_DAY_OF_YEAR p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR'
                         || ' WHERE p.ELEMENT_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ''' AND p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
                  --trace_printf("\n 007=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
               Else
                  -- Use CALENDAR_DAY_OF_YEAR(d), session.PMO_TMP_CDY (p) as Source Tables
                  h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, WEEK_OF, MONTH_OF, YEAR_OF, AVAIL_MIN)';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' SELECT p.DAY_YEAR, p.DAY_WEEK, p.WEEK_OF, p.MONTH_OF, p.YEAR_OF,'
                         || ' p.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)'
                         || ' FROM CALENDAR_DAY_OF_YEAR p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR'
                         || ' WHERE p.ELEMENT_ID = ''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ''' AND p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
                  --trace_printf("\n 008=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
               End If;
            End If;
         End If;
         
         --trace_printf("\n 009=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
         --trace_printf("\n 010=====> %s \n", (char *)stmt);
         
         CASE h_bitflag
            
            WHEN 1 THEN
               -- Daily, every X workdays
               /**NOTE** THIS CASE DOSE NOT NEED TO ANY TEMPORARY TABLES */
               --trace_printf("\n 011=====> case 1");
               h_min := TO_CHAR(Coalesce(h_work, 0));
               
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                  stmt := stmt || ' AND AVAILABLE_MIN >= ';
               ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  stmt := stmt || ' AND p.SCHED_MIN >= ';
               Else
                  stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
               End If;
               stmt := stmt || h_min;
               
               If Coalesce(h_every, 0) > 1 Then
                  h_header := 'SELECT J_DAY, DAY_YEAR, AVAIL_MIN, WEEK_OF, CNT FROM ( ';
                  h_header := h_header || stmt;
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  
                  h_tail := ' ' || PMODEFINES.PMO_ORDER_BY_DAY_YEAR || ' )  WHERE MOD(CNT-1,' || h_min || ' )=0 ';
                  
                  h_header := h_header || h_tail;
                  
                  stmt := h_header;
                  stmt := stmt || ' ORDER BY DAY_YEAR';
               Else
                  stmt := stmt || ' ORDER BY p.DAY_YEAR';
               End If;
               
               --trace_printf("\n\n =====> Daily stmt : %s\n", (char *) stmt);
               
            
            WHEN 2 THEN
               -- Weekly, every X week on Mo, We
               /**NOTE** THIS CASE USES PMO_TMP_RES, PMO_TMP_SUM, AND PMO_TMP_EVERY */
               --         TEMPORARY TABLES
               --trace_printf("\n 012=====> case 2 h_contour: %d\n", h_contour);
               
               h_min := TO_CHAR(Coalesce(h_work, 0));
               h_work_loc := '';
               
               -- This case needs to use PMO_TMP_SUM temporary table
               -- create PMO_TMP_SUM temporary table instead of w WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
               PMO_DB2M.SC_STATE := 0;
               
               If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) = 0 Then
                  -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
                  -- first execute dynamic insert statement to insert records in PMO_TMP_RES
                  h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
                  -- execute dynamic Insert statement for PMO_TMP_RES
                  --trace_printf("\n 12.0=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
                  
                  -- Insert records in PMO_TMP_SUM with using PMO_TMP_RES temporary table
                  -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
                  BEGIN
                     INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF, AVAIL_MIN)
                     SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS AVAIL_MIN /*WEEK_MIN */
                     FROM PMO_TMP_RES
                     GROUP BY YEAR_OF,WEEK_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1962+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1963) : cplan_fun.sql(12559) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  /**NOTE** Commented by Amir
                  						stmt.strcat("), w AS (SELECT YEAR_OF,WEEK_OF,SUM(AVAIL_MIN) AS WEEK_MIN FROM pd GROUP BY YEAR_OF,WEEK_OF)");
                  						*/
               Else
                  i := 1;
                  ii := 1;
                  WHILE ii <= 7 LOOP
                     If (BITAND(Coalesce(h_contour, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  
                  -- compelete h_pmo_tmp_res_ins_stmt (Dynamic Insert Statement for PMO_TMP_RES
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) IN (';
                  Else
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
                  End If;
                  
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
                  
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  If h_amnt = 1 Then
                     If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                        h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
                     Else
                        h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
                     End If;
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                     
                     -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
                     -- first execute dynamic insert statement to insert records in PMO_TMP_RES
                     h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
                     -- execute dynamic Insert statement for PMO_TMP_RES
                     --trace_printf("\n 12.1=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
                     
                     -- Insert necessary records in PMO_TMP_SUM temporary table
                     -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
                     BEGIN
                        INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF)
                        SELECT YEAR_OF, WEEK_OF
                        FROM PMO_TMP_RES
                        GROUP BY YEAR_OF,WEEK_OF;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2030+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2031) : cplan_fun.sql(12627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     /**NOTE** stmt.strcat("), w AS (SELECT YEAR_OF,WEEK_OF FROM pd");*/
                     /**NOTE** stmt.strcat(" GROUP BY YEAR_OF,WEEK_OF)"); */
                  Else
                     -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
                     -- first execute dynamic insert statement to insert records in PMO_TMP_RES
                     h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
                     -- execute dynamic Insert statement for PMO_TMP_RES
                     --trace_printf("\n 12.2=====> %s \n", (char *)h_pmo_tmp_res_ins_stmt);
                     
                     -- Insert necessary records in PMO_TMP_SUM temporary table
                     -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
                     BEGIN
                        INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF, AVAIL_MIN)
                        SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS AVAIL_MIN  /*WEEK_MIN */
                        FROM PMO_TMP_RES
                        GROUP BY YEAR_OF,WEEK_OF;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2061+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2062) : cplan_fun.sql(12658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     /**NOTE** stmt.strcat("), w AS (SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS WEEK_MIN FROM pd"); */
                     /**NOTE** stmt.strcat(" GROUP BY YEAR_OF,WEEK_OF)"); */
                  End If;
               End If;
               
               -- create PMO_TMP_EVERY temporary table,  this temporary table will be used instead of every WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
               PMO_DB2M.SC_STATE := 0;
               
               -- Preparing Dynamic Insert Statement for PMO_TMP_EVERY
               --trace_printf("h_every: %d\n", h_every);
               If Coalesce(h_every, 0) > 1 Then
                  --h_pmo_tmp_every_ins_stmt.strcpy("SELECT YEAR_OF, WEEK_OF, COUNTER() AS CNT FROM SESSION.PMO_TMP_SUM w ");
                  h_pmo_tmp_every_ins_stmt := 'SELECT YEAR_OF, WEEK_OF, ' 
                            || PMODEFINES.PMO_COUNTER_OVER_YEAROF_WEEKOF 
                            || ' AS CNT FROM PMO_TMP_SUM w ';
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE w.AVAIL_MIN >= ';
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
                  
                  h_header := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, WEEK_OF) SELECT YEAR_OF, WEEK_OF FROM ( ';
                  h_header := h_header || h_pmo_tmp_every_ins_stmt;
                  
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  
                  h_tail := ' )  WHERE MOD(CNT-1,' || h_min || ' )=0 ';
                  --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1,%s )=0 ",  PMO_ORDER_BY_YEAROF_WEEKOF, h_min);
                  h_header := h_header || h_tail;
               Else
                  h_pmo_tmp_every_ins_stmt := 'SELECT YEAR_OF,WEEK_OF FROM PMO_TMP_SUM w ';
                  
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE w.AVAIL_MIN >= ';
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
                  
                  h_header := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, WEEK_OF) ';
                  h_header := h_header || h_pmo_tmp_every_ins_stmt;
               End If;
               
               --h_pmo_tmp_every_ins_stmt.Clear();
               h_pmo_tmp_every_ins_stmt := h_header;
               
               --trace_printf("\n\n\n ======> Monthly stmt(0003): %s\n", (char *) h_pmo_tmp_every_ins_stmt);
               
               -- execute h_pmo_tmp_every_ins_stmt to insert necessary data in PMO_TMP_EVERY
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
               
               -- rewrite the last select statement with using previouse temporary tables
               If Coalesce(Length(h_work_loc), 0) = 0 Then
                  /**IMPORTANT NOTE** the following select statement can be rewritten as static SQL statement*/
                  stmt := 
                        'SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, every.WEEK_OF, 0 AS CNT '
                         || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                         || ' WHERE pd.YEAR_OF = every.YEAR_OF AND '
                         || ' pd.WEEK_OF = every.WEEK_OF '
                         || ' ORDER BY pd.DAY_YEAR ';
               Else
                  /**IMPORTANT NOTE** the following select statement can be rewritten as static SQL statement*/
                  stmt := 
                        ' SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, every.WEEK_OF, 0 AS CNT '
                         || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                         || ' WHERE pd.YEAR_OF = every.YEAR_OF AND '
                         || ' pd.WEEK_OF = every.WEEK_OF AND pd.DAY_WEEK IN (';
                  stmt := stmt || h_work_loc;
                  stmt := stmt || ') ORDER BY pd.DAY_YEAR';
               End If;
               
               
            
            WHEN 3 THEN
               -- Monthly, The Day 6 every X months
               /**NOTE** THIS CASE USE PMO_TMP_RES TEMPORARY TABLE*/
               --trace_printf("\n 013=====> case 3");
               h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
               
               h_min := TO_CHAR(Coalesce(h_week, 0));
               
               -- Compelete the rest of Dynamic Insert Statement for PMO_TMP_RES
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) = ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               
               -- rewrite the last select statement with using PMO_TMP_RES
               -- first execute the dynamic Insert statement to insert records in temporary table
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
               --trace_printf("\n 013=====> h_week = %d, h_min = %s, doUpdate\n%s",h_week,h_min,(char *)h_pmo_tmp_res_ins_stmt);
               
               -- Prepare last select statement
               
               If Coalesce(h_every, 0) > 1 Then
                  --stmt.strcpy("SELECT JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, WEEK_OF, 0 AS CNT FROM ( "
                  --          " SELECT JULIAN_DAY(DAY_YEAR) AS JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, 0  AS WEEK_OF, COUNTER() AS CNT "
                  --          " FROM SESSION.PMO_TMP_RES pd WHERE AVAIL_MIN >= ");
                  stmt := 'SELECT JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, WEEK_OF, 0 AS CNT FROM ( '
                         || ' SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, 0  AS WEEK_OF, ' 
                              || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR || ' AS CNT '
                         || ' FROM PMO_TMP_RES p WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  stmt := stmt || h_min;
                  
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  
                  h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ORDER BY DAY_YEAR ';
                  --h_tail.sprintf(" %s  ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ORDER BY DAY_YEAR ", PMO_ORDER_BY_DAY_YEAR ,h_min);
                  
                  stmt := stmt || h_tail;
               Else
                  stmt := 'SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), DAY_YEAR, AVAIL_MIN, 0 AS WEEK_OF, 0 AS CNT '
                         || ' FROM PMO_TMP_RES pd WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  stmt := stmt || h_min;
                  
                  stmt := stmt || ' ORDER BY DAY_YEAR';
               End If;
               
               --trace_printf("\n\n\n =======> (Monthly, The Day 6 every X months) %s \n", (char *) stmt);
               
            
            WHEN 4 THEN
               -- Monthly, First/Second/Third/Forth/Last Monday every X months
               -- This case uses PMO_TMP_RES, PMO_TMP_SUM
               --trace_printf("\n 014=====> case 4");
               h_days := BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F);
               -- Monday
               i := 1;
               ii := 1;
               WHILE ii <= 7 LOOP
                  If (BITAND(Coalesce(h_days, 0), i)) = i Then
                     h_days := ii;
                     
                     EXIT;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
               -- First/Second/Third/Forth/Last
               i := 1;
               ii := 1;
               WHILE ii <= 5 LOOP
                  If (BITAND(Coalesce(h_week, 0), i)) = i Then
                     h_week := ii;
                     
                     EXIT;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               h_min := TO_CHAR(7 * (Coalesce(h_week, 0) - 1) + 1);
               
               -- Prepare the rest of PMO_TMP_RES Dynamic Insert statement
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) BETWEEN ';
               --stmt.strcat(" AND DAY(p.DAY_YEAR) BETWEEN ");
               
               If Coalesce(h_week, 0) = 5 Then
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' CASE WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) = 2 AND MOD(PMO_ORASTD.YEAR(p.DAY_YEAR),4)=0 THEN 23 WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) = 2 AND MOD(PMO_ORASTD.YEAR(p.DAY_YEAR),4)>0 THEN 22 WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
                     --stmt.strcat(" CASE WHEN MONTH(p.DAY_YEAR) = 2 AND MOD(YEAR(p.DAY_YEAR),4)=0 THEN 23 WHEN MONTH(p.DAY_YEAR) = 2 AND MOD(YEAR(p.DAY_YEAR),4)>0 THEN 22 WHEN MONTH(p.DAY_YEAR) IN (4,6,9,11) THEN 24 ELSE 25 END AND 31");
                  Else
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
                     --stmt.strcat(" CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31");
                  End If;
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                  --stmt.strcat(h_min);
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND ';
                  --stmt.strcat(" AND ");
                  
                  h_min := TO_CHAR(7 * Coalesce(h_week, 0));
                  
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                  --stmt.strcat(h_min);
               End If;
               
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR)=';
                  --stmt.strcat(" AND DAYOFWEEK(p.DAY_YEAR)=");
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK=';
                  --stmt.strcat(" AND p.DAY_WEEK=");
               End If;
               h_min := TO_CHAR(Coalesce(h_days, 0));
               -- Monday
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               --stmt.strcat(h_min);
               
               -- Dynamic insert statement for PMO_TMP_RES  is ready to execute
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
               --trace_printf("doUpdate.1__* h_temp_sc_state: %d\n%s \n",h_temp_sc_state,(char *)h_pmo_tmp_res_ins_stmt);
               
               -- create PMO_TMP_SUM temporary table instead of w WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
               PMO_DB2M.SC_STATE := 0;
               
               -- Insert statement to insert records in PMO_TMP_SUM temporary table, with using PMO_TMP_RES
               -- no need to use dynamic insert statement
               BEGIN
                  INSERT INTO PMO_TMP_SUM (YEAR_OF, MONTH_OF, AVAIL_MIN)
                  SELECT YEAR_OF AS YEAR_OF,MONTH_OF AS MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN /*WEEK_MIN*/
                  FROM PMO_TMP_RES pd
                  GROUP BY YEAR_OF,MONTH_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2290+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2291) : cplan_fun.sql(12887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- Prepare PMO_TMP_EVERY temporary table for the rest of algorithm
               -- create PMO_TMP_EVERY temporary table,  this temporary table will be used instead of every WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
               PMO_DB2M.SC_STATE := 0;
               
               If Coalesce(h_every, 0) > 1 Then
                  -- Prepare dynamic insert statement to insert records in PMO_TMP_EVERY
                  --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
                  --                              "SELECT  YEAR_OF, MONTH_OF FROM ( "
                  --                              " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
                  --                              " FROM SESSION.PMO_TMP_SUM w WHERE AVAIL_MIN >= ");
                  
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                         || 'SELECT  YEAR_OF, MONTH_OF FROM ( '
                         || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                         || ' FROM PMO_TMP_SUM w WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  
                  h_tail := ' )   WHERE MOD(CNT-1, ' || h_min || ' )=0 ';
                  --h_tail.sprintf(" %s ) AS PMO_ALIAS  WHERE MOD(CNT-1, %s )=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
               Else
                  -- Prepare dynamic insert statement to insert records in PMO_TMP_EVERY
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                         || ' SELECT YEAR_OF, MONTH_OF '
                         || ' FROM PMO_TMP_SUM w WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
               
               -- execute h_pmo_tmp_every_ins_stmt to prepare records for the last select statement
               --trace_printf("\n\n\n ======> Monthly stmt(0001): %s", (char *) h_pmo_tmp_every_ins_stmt);
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
               
               -- all the temporary tables are ready to use for creating the last select statement
               stmt := stmt || 'SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, 0  '
                      || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                      || ' WHERE pd.YEAR_OF = every.YEAR_OF AND pd.MONTH_OF = every.MONTH_OF '
                      || ' ORDER BY pd.DAY_YEAR ';
               
               
            
            WHEN 5 THEN
               -- Monthly, Mo, We, Fr  on Week1, Week4 every X months
               -- This case uses PMO_TMP_RES, PMO_TMP_SUM, PMO_TMP_EVERY, PMO_TMP_MD
               --trace_printf("\n 015=====> case 5");
               i := 1;
               ii := 1;
               iii := 0;
               If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) > 0 Then
                  WHILE ii <= 7 LOOP
                     If (BITAND(Coalesce(h_contour, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  -- Prepare the rest of h_pmo_tmp_res_ins_stmt dynamic insert statement
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
                  
                  --stmt.strcat(" AND p.DAY_WEEK IN (");
                  --stmt.strcat(h_work_loc);
                  --stmt.strcat(")");
               End If;
               
               If h_amnt = 1 Then
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
                  ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
                  Else
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.AVAILABLE_MIN >= ';
                  End If;
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                  --stmt.strcat(h_min);
               End If;
               
               -- the end of dynamic insert statement for PMO_TMP_RES temporary table
               -- execute the h_pmo_tmp_res_ins_stmt to insert data in PMO_TMP_RES
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
               
               -- Use PMO_TMP_RES temporary table to prepare necessary data for PMO_TMP_MD
               -- Create PMO_TMP_MD temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MD';
               PMO_DB2M.SC_STATE := 0;
               
               -- Prepare dynamic insert statement to insert records in PMO_TMP_MD
               h_pmo_tmp_md_ins_stmt := 'INSERT INTO PMO_TMP_MD(YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN) ';
               h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || 
                                                  'SELECT YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN '
                      || ' FROM PMO_TMP_RES pd ';
               
               --stmt.strcat("), md AS (SELECT YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN FROM pd");
               
               h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
               -- Week1, Week2, Week3, Week4
               --trace_printf("**** Week1, Week2, Week3, Week4:  h_week: %d\n",h_week);
               If (Coalesce(h_week, 0) > 0) AND (Coalesce(h_week, 0) <> 15) Then
                  i := 1;
                  ii := 1;
                  WHILE ii <= 4 LOOP
                     If (BITAND(Coalesce(h_week, 0), i)) = i Then
                        If iii = 0 Then
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' WHERE PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                           --stmt.strcat(" WHERE DAY(DAY_YEAR) BETWEEN ");
                           iii := 2;
                        Else
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' OR PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                           --stmt.strcat(" OR DAY(DAY_YEAR) BETWEEN ");
                        End If;
                        h_min := TO_CHAR(7 * (ii - 1) + 1);
                        
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND ';
                        --stmt.strcat(h_min);
                        --stmt.strcat(" AND ");
                        
                        i := i * 2;
                        ii := ii + 1;
                        WHILE ii <= 5 LOOP
                           If (BITAND(Coalesce(h_week, 0), i)) = 0 Then
                              h_min := TO_CHAR(7 * (ii - 1));
                              
                              h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                              --stmt.strcat(h_min);
                              
                              EXIT;
                           End If;
                           i := i * 2;
                           ii := ii + 1;
                        END LOOP;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
               End If;
               
               --trace_printf("\n 5.1__* %s \n", (char *)h_pmo_tmp_md_ins_stmt);
               -- end of dynamic insert statemnt for h_pmo_tmp_md_ins_stmt
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
               
               -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
               -- create PMO_TMP_SUM temporary table instead of w WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
               PMO_DB2M.SC_STATE := 0;
               
               -- Insert necessary records in PMO_TMP_SUM, no need to create dynamic insert statement
               If h_amnt = 0 Then
                  BEGIN
                     INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                     SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                     FROM PMO_TMP_MD md
                     GROUP BY YEAR_OF,MONTH_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2477+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2478) : cplan_fun.sql(13074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --stmt.strcat("), w AS (SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS WEEK_MIN FROM md GROUP BY YEAR_OF,MONTH_OF");
               Else
                  BEGIN
                     INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF)
                     SELECT YEAR_OF, MONTH_OF
                     FROM PMO_TMP_MD md
                     GROUP BY YEAR_OF,MONTH_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2498+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2499) : cplan_fun.sql(13095) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --stmt.strcat("), w AS (SELECT YEAR_OF, MONTH_OF FROM md GROUP BY YEAR_OF,MONTH_OF");
               End If;
               
               -- Create PMO_TMP_EVERY temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
               PMO_DB2M.SC_STATE := 0;
               
               If Coalesce(h_every, 0) > 1 Then
                  -- Prepare h_pmo_tmp_every_ins_stmt dynamic insert statement to insert records in PMO_TMP_EVERY
                  --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
                  --                              " SELECT YEAR_OF, MONTH_OF FROM ( "
                  --                              " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
                  --                              " FROM SESSION.PMO_TMP_SUM w ");
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                         || ' SELECT YEAR_OF, MONTH_OF FROM ( '
                         || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                         || ' FROM PMO_TMP_SUM w ';
                  
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                     
                     h_min := TO_CHAR(Coalesce(h_work, 0));
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
                  
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ';
                  --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
               Else
                  -- Prepare h_pmo_tmp_every_ins_stmt dynamic insert statement to insert records in PMO_TMP_EVERY
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) ';
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' SELECT YEAR_OF, MONTH_OF '
                         || ' FROM PMO_TMP_SUM w';
                  
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                     
                     h_min := TO_CHAR(Coalesce(h_work, 0));
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
               End If;
               
               --trace_printf("\n\n\n Monthly, Mo, We, Fr  on Week1, Week4 every X months %s \n", (char *)h_pmo_tmp_every_ins_stmt);
               -- execute h_pmo_tmp_every_ins_stmt
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
               
               -- ready to prepare the last select statement with using temporary tables
               If Coalesce(Length(h_work_loc), 0) = 0 Then
                  stmt := 
                        'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, PMO_ORASTD.MONTH(md.DAY_YEAR), 0 AS CNT '
                         || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                         || ' WHERE md.YEAR_OF = every.YEAR_OF AND '
                         || '       md.MONTH_OF = every.MONTH_OF '
                         || ' ORDER BY md.DAY_YEAR ';
                  
                  --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, MONTH(md.DAY_YEAR) FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF ORDER BY md.DAY_YEAR");
               Else
                  stmt := 
                        'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, every.MONTH_OF, 0 AS CNT '
                         || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                         || ' WHERE md.YEAR_OF = every.YEAR_OF AND '
                         || ' md.MONTH_OF = every.MONTH_OF AND '
                         || ' md.DAY_WEEK IN (';
                  
                  --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, every.MONTH_OF FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF AND md.DAY_WEEK IN (");
                  stmt := stmt || h_work_loc;
                  stmt := stmt || ') ORDER BY md.DAY_YEAR';
               End If;
               
               
            WHEN 6 THEN
               -- Monthly, The Day 6 of Jan, Mar, Apr, Nov  Dec
               -- This case never use of temporary tables
               --trace_printf("\n 016=====> case 6");
               h_days := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_1FFE0000)), 17);
               -- Jan, Mar, Apr, Nov  Dec
               
               If Coalesce(h_days, 0) > 0 Then
                  i := 1;
                  ii := 1;
                  WHILE ii <= 12 LOOP
                     If (BITAND(Coalesce(h_days, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  
                  h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
                  -- The Day X of
                  
                  h_min := TO_CHAR(Coalesce(h_week, 0));
                  
                  stmt := stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR)=';
                  stmt := stmt || h_min;
                  
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                     stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
                  ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     stmt := stmt || ' AND p.SCHED_MIN >= ';
                  Else
                     stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
                  End If;
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  stmt := stmt || h_min;
                  stmt := stmt || ' AND PMO_ORASTD.MONTH(p.DAY_YEAR) IN (';
                  --                      stmt.strcat(" AND p.MONTH_OF IN (");
                  stmt := stmt || h_work_loc;
                  stmt := stmt || ') ORDER BY p.DAY_YEAR';
               End If;
               
               
            
            WHEN 7 THEN
               -- Yearly, The First/Second/Third/Forth/Last Mondy of July, August, Decembre
               -- this case never uses the temporary tables
               --trace_printf("\n 017=====> case 7");
               h_days := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_1FFE0000)), 17);
               -- July, August, Decembre
               
               If Coalesce(h_days, 0) > 0 Then
                  i := 1;
                  ii := 1;
                  WHILE ii <= 12 LOOP
                     If (BITAND(Coalesce(h_days, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  
                  h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
                  -- First/Second/Third/Forth/Last
                  i := 1;
                  ii := 1;
                  WHILE ii <= 5 LOOP
                     If (BITAND(Coalesce(h_week, 0), i)) = i Then
                        h_week := ii;
                        
                        EXIT;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  
                  h_min := TO_CHAR(7 * (Coalesce(h_week, 0) - 1) + 1);
                  h_days := BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F);
                  -- Monday
                  
                  i := 1;
                  ii := 1;
                  WHILE ii <= 7 LOOP
                     If (BITAND(Coalesce(h_days, 0), i)) = i Then
                        h_days := ii;
                        
                        EXIT;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  
                  stmt := stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) BETWEEN ';
                  If Coalesce(h_week, 0) = 5 Then
                     stmt := stmt || 
                           ' CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
                  Else
                     stmt := stmt || h_min;
                     stmt := stmt || ' AND ';
                     h_min := TO_CHAR(7 * Coalesce(h_week, 0));
                     stmt := stmt || h_min;
                  End If;
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     stmt := stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR)=';
                  Else
                     stmt := stmt || ' AND p.DAY_WEEK=';
                  End If;
                  h_min := TO_CHAR(Coalesce(h_days, 0));
                  stmt := stmt || h_min;
                  stmt := stmt || ' AND PMO_ORASTD.MONTH(p.DAY_YEAR) IN (';
                  stmt := stmt || h_work_loc;
                  stmt := stmt || ') ORDER BY p.DAY_YEAR';
               End If;
               
               
            
            WHEN 8 THEN
               -- Yearly, Mo, We, Fr  on Week1, Week4 on Jan, June, July  Dec
               -- This case uses PMO_TMP_RES, PMO_TMP_SUM, PMO_TMP_EVERY, PMO_TMP_MD
               --trace_printf("\n 018=====> case 8");
               If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) > 0 Then
                  i := 1;
                  ii := 1;
                  iii := 0;
                  WHILE ii <= 7 LOOP
                     If (BITAND(Coalesce(h_contour, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  -- Prepare the rest of h_pmo_tmp_res_ins_stmt
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
                  
                  --stmt.strcat(" AND p.DAY_WEEK IN (");
                  --stmt.strcat(h_work_loc);
                  --stmt.strcat(")");
               End If;
               
               If h_amnt = 1 Then
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
                  Else
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
                  End If;
                  h_min := TO_CHAR(Coalesce(h_work, 0));
                  
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                  --stmt.strcat(h_min);
               End If;
               
               -- end of h_pmo_tmp_res_ins_stmt statement, execute it to prepare data for other temporary tables
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
               
               -- Use PMO_TMP_RES temporary table to prepare necessary data for PMO_TMP_MD
               -- Create PMO_TMP_MD temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MD';
               PMO_DB2M.SC_STATE := 0;
               
               -- Prepare dynamic insert statement to insert records in PMO_TMP_MD
               h_pmo_tmp_md_ins_stmt := 'INSERT INTO PMO_TMP_MD(YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN) ';
               h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' SELECT YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN '
                      || ' FROM PMO_TMP_RES pd WHERE ';
               
               --stmt.strcat("), md AS (SELECT YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN FROM pd WHERE ");
               
               h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
               -- Week1, Week2, Week3, Week4
               If Coalesce(h_week, 0) > 0 Then
                  i := 1;
                  ii := 1;
                  WHILE ii <= 4 LOOP
                     If (BITAND(Coalesce(h_week, 0), i)) = i Then
                        If iii = 0 Then
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || '(PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                           --stmt.strcat("(DAY(DAY_YEAR) BETWEEN ");
                           iii := 2;
                        ElsIf iii = 1 Then
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND (PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                           --stmt.strcat(" AND (DAY(DAY_YEAR) BETWEEN ");
                           iii := 2;
                        Else
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' OR PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                           --stmt.strcat(" OR DAY(DAY_YEAR) BETWEEN ");
                        End If;
                        
                        h_min := TO_CHAR(7 * (ii - 1) + 1);
                        
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND ';
                        --stmt.strcat(h_min);
                        --stmt.strcat(" AND ");
                        
                        i := i * 2;
                        ii := ii + 1;
                        WHILE ii <= 5 LOOP
                           If (BITAND(Coalesce(h_week, 0), i)) = 0 Then
                              h_min := TO_CHAR(7 * (ii - 1));
                              
                              h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                              --stmt.strcat(h_min);
                              
                              EXIT;
                           End If;
                           i := i * 2;
                           ii := ii + 1;
                        END LOOP;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ')';
                  --stmt.strcat(")");
               End If;
               
               h_days := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_1FFE0000)), 17);
               -- July, August, Decembre
               
               If Coalesce(h_days, 0) > 0 Then
                  i := 1;
                  ii := 1;
                  WHILE ii <= 12 LOOP
                     If (BITAND(Coalesce(h_days, 0), i)) = i Then
                        If Coalesce(Length(h_work_loc), 0) = 0 Then
                           h_min := TO_CHAR(ii);
                        Else
                           h_min := ',' || TO_CHAR(ii);
                        End If;
                        
                        h_work_loc := h_work_loc || h_min;
                     End If;
                     i := i * 2;
                     ii := ii + 1;
                  END LOOP;
                  If iii = 0 Then
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' PMO_ORASTD.MONTH(DAY_YEAR) IN(';
                     --stmt.strcat(" MONTH(DAY_YEAR) IN(");
                  Else
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND PMO_ORASTD.MONTH(DAY_YEAR) IN(';
                     --stmt.strcat(" AND MONTH(DAY_YEAR) IN(");
                  End If;
                  
                  h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_work_loc;
                  --stmt.strcat(h_work_loc);
                  h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ')';
                  --trace_printf("\n doUpdate_1 =====> %s \n", (char *)h_pmo_tmp_md_ins_stmt); trace_flush();
                  -- the end of h_pmo_tmp_md_ins_stmt dynamic insert statement for PMO_TMP_MD
                  h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
                  
                  -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
                  -- create PMO_TMP_SUM temporary table instead of w WITH Clause
                  -- Temporary Table Declaration has been changed to Truncate
                  EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
                  PMO_DB2M.SC_STATE := 0;
                  
                  -- Insert records in PMO_TMP_SUM, no need to dynamic insert
                  BEGIN
                     INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                     SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                     FROM PMO_TMP_MD md
                     GROUP BY YEAR_OF, MONTH_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2866+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2867) : cplan_fun.sql(13463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  -- Create PMO_TMP_EVERY temporary table
                  -- Temporary Table Declaration has been changed to Truncate
                  EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
                  PMO_DB2M.SC_STATE := 0;
               Else
                  --trace_printf("\n doUpdate_2 =====> %s \n", (char *)h_pmo_tmp_md_ins_stmt); trace_flush();
                  -- the end of h_pmo_tmp_md_ins_stmt dynamic insert statement for PMO_TMP_MD
                  h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
                  
                  -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
                  -- create PMO_TMP_SUM temporary table instead of w WITH Clause
                  -- Temporary Table Declaration has been changed to Truncate
                  EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
                  PMO_DB2M.SC_STATE := 0;
                  
                  -- Insert records in PMO_TMP_SUM, no need to dynamic insert
                  BEGIN
                     INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                     SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                     FROM PMO_TMP_MD md
                     GROUP BY YEAR_OF, MONTH_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2902+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2903) : cplan_fun.sql(13499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  -- Create PMO_TMP_EVERY temporary table
                  -- Temporary Table Declaration has been changed to Truncate
                  EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
                  PMO_DB2M.SC_STATE := 0;
               End If;
               
               If Coalesce(h_every, 0) > 1 Then
                  -- Prepare a dynamic insert statement to insert records in PMO_TMP_EVERY
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                         || ' SELECT YEAR_OF, MONTH_OF FROM ( '
                         || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                         || ' FROM PMO_TMP_SUM w ';
                  
                  --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
                  --                              " SELECT YEAR_OF, MONTH_OF FROM ( "
                  --                                " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
                  --                              " FROM SESSION.PMO_TMP_SUM w ");
                  
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                     
                     h_min := TO_CHAR(Coalesce(h_work, 0));
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
                  
                  h_min := TO_CHAR(Coalesce(h_every, 0));
                  h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ';
                  --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
               Else
                  -- Prepare a dynamic insert statement to insert records in PMO_TMP_EVERY
                  h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) ';
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' SELECT YEAR_OF, MONTH_OF '
                         || ' FROM PMO_TMP_SUM w ';
                  
                  -- continue concating the rest of h_pmo_tmp_every_ins_stmt
                  
                  If h_amnt = 0 Then
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                     
                     h_min := TO_CHAR(Coalesce(h_work, 0));
                     
                     h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
                  End If;
               End If;
               
               --trace_printf("\n doUpdate_3 =====> %s \n", (char *)h_pmo_tmp_every_ins_stmt); trace_flush();
               
               -- end of h_pmo_tmp_every_ins_stmt, execute it to prepare necessary date
               --trace_printf("\n\n\n ======> Monthly stmt(0002): %s", (char *) h_pmo_tmp_every_ins_stmt);
               
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
               
               -- ready to prepare the last select statement, all temporary tables are ready
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, PMO_ORASTD.MONTH(md.DAY_YEAR), 0 AS CNT '
                      || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                      || ' WHERE md.YEAR_OF = every.YEAR_OF AND md.MONTH_OF = every.MONTH_OF '
                      || ' ORDER BY md.DAY_YEAR ';
               
               --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, MONTH(md.DAY_YEAR) FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF ORDER BY md.DAY_YEAR");
               
            ELSE
               NULL;
         END CASE;
      End If;
      -- end of (h_bitflag == 0) or ASAP, No contour
      
      stmt := stmt || '  ';
      
      If iSF = 1 Then
         BEGIN
            SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT)
            INTO h_julian_start, h_julian_finish
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2988+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2989) : cplan_fun.sql(13585) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT)
            INTO h_julian_start
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3003+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3004) : cplan_fun.sql(13600) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_julian_finish := 0;
      End If;
      
      h_dayofyear := '';
      
      h_julian_day := 0;
      h_total_min := 0;
      h_total := 0;
      h_totpr := 0;
      h_bit_task := 0;
      h_prev_min := 0;
      h_prev_day := 0;
      h_prev_week := 0;
      h_available := 0;
      h_planned_work := 0;
      h_sched_work := 0;
      h_bit_minutes := 0;
      h_bit_task := -1;
      PMO_DB2M.SC_STATE := 0;
      ii := 1;
      iSD := 1;
      i_break := 0;
      i_work := Coalesce(h_work, 0);
      
      --trace_printf("h_effort: %d, h_julian_start: %d, h_julian_finish: %d, i_work: %d, h_work: %d\n",h_effort,h_julian_start,h_julian_finish,i_work,h_work);
      --trace_printf("h_start_DT: %s, h_finish_DT: %s\n",h_start_DT,h_finish_DT);
      -- contourObject cntr;
      
      --trace_printf("\n\n =====> before FC_INI_ASSIGN_STR");
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_INI_ASSIGN_STR(1);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
      
      h_assign_string := '';
      --h_temp_sc_state = FC_INI_ASSIGN_STR(SC_PARAM(h_assign_string));
      --trace_printf("\n\n =====> after FC_INI_ASSIGN_STR");
      
      --trace_printf("\n%s\n", stmt);
      --      if ( (h_type > 0)  (h_consider != SC_SRCH_FROM_PROJECT) )
      
      If PMO_DB2M.SC_STATE < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3049+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3050) : cplan_fun.sql(13646) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
      
      --timeLog (file,"Before PREPARE S342:");
      
      OPEN C332 FOR stmt;
      
      LOOP
         h_continue_01 := 1;
         
         FETCH C332 INTO h_julian_day, h_dayofyear, h_minutes, h_week, h_cnt;
         IF C332%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         --trace_printf("SC_STATE = %d, h_minutes: %d, h_skip: %d, h_dayofyear: %s, start_date: %s\n", SC_STATE, h_minutes, h_skip, h_dayofyear,start_date);
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_minutes, 0) > 0) Then
            iSD := PMO_ORASTD.strncmp(start_date001, h_dayofyear, 10);
            
            If iSD <> 0 Then
               h_skip := 0;
            End If;
         End If;
         --if ( (h_skip == 1)  (h_consider != SC_SRCH_FROM_PROJECT) )
         If Coalesce(h_skip, 0) = 1 Then
            h_skip := 0;
            FETCH C332 INTO h_julian_day, h_dayofyear, h_minutes, h_week, h_cnt;
            IF C332%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               BEGIN
                  CLOSE C332 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3099+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3100) : cplan_fun.sql(13696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- we need to return an error message, in case it's from project assignment
               If (h_prj_assignment_id IS NOT NULL) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3107+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3108) : cplan_fun.sql(13704) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
               Else
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3112+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3113) : cplan_fun.sql(13709) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.RECORD_NOT_FOUND;
               End If;
            End If;
            
            If h_amnt = 0 Then
               If Coalesce(h_unit, 0) = 3 Then
                  BEGIN
                     SELECT PMO_ORASTD.WEEK(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                     INTO h_prev_week
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3130+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3131) : cplan_fun.sql(13727) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     SELECT SUM(SCHED_MIN) INTO h_total_min
                     FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id
                     AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND PMO_ORASTD.YEAR(DAY_YEAR) = PMO_ORASTD.YEAR(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                     AND PMO_ORASTD.WEEK(DAY_YEAR) = h_prev_week;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3148+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3149) : cplan_fun.sql(13745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               If Coalesce(h_unit, 0) = 4 Then
                  BEGIN
                     SELECT PMO_ORASTD.MONTH(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                     INTO h_prev_week
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3164+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3165) : cplan_fun.sql(13761) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     SELECT SUM(SCHED_MIN) INTO h_total_min
                     FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id
                     AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND PMO_ORASTD.YEAR(DAY_YEAR) = PMO_ORASTD.YEAR(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                     AND PMO_ORASTD.MONTH(DAY_YEAR) = h_prev_week;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3182+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3183) : cplan_fun.sql(13779) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("*** SKIP h_amnt: %d, h_minutes: %d, h_skip: %d, h_dayofyear: %s\n", h_amnt,h_minutes, h_skip, h_dayofyear);
            End If;
         End If;
         
         If (Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(h_julian_day, 0) > Coalesce(
               h_j_disconti_date, 0)) AND ((BITAND(Coalesce(h_update, 0), 255)) <> 10) Then
            BEGIN
               CLOSE C332 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3202) : cplan_fun.sql(13798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3207+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3208) : cplan_fun.sql(13804) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
         End If;
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_available, h_bitID
               FROM PMO_TMP_TDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3222+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3223) : cplan_fun.sql(13819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            BEGIN
               SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available, h_bitID
               FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3236+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3237) : cplan_fun.sql(13833) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available,h_bitID
               FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID=h_res_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3251+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3252) : cplan_fun.sql(13848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If iSD = 0 Then
            wBits := PMO_STPU.getFirstBit(h_bitID);
            
            wHour := Trunc(wBits / 12);
            wMin := (wBits MOD 12) * 5;
            
            h_first_time := LPad(wHour, 2,'0') || ':' || LPad(wMin, 2,'0') || ':00';
            --sprintf(h_first_time, "%02d.%02d.00", wHour, wMin);
         End If;
         /*
         strcpy(tempStr,"");
         printTimes((unsigned char *)h_bitID.h_value,tempStr);
         */
         --trace_printf("1_* h_dayofyear: %s, h_first_time: %s, h_start_time: %s, iSD: %d, tempStr: %s\n",h_dayofyear,h_first_time,h_start_time,iSD,tempStr);//trace_flush();
         --t_finish = clock();
         
         If (iSD = 0) AND (PMO_ORASTD.strncmp(h_first_time, h_start_time, 5) < 0) 
               AND (Coalesce(Length(start_date001), 0) > 10) Then
            iSD := PMO_STPU.getNumberOfMinutesFinishingAt(h_bitID, h_start_time);
            PMO_STPU.cleardayStruct(h_first_time, h_start_time, h_bitID);
         ElsIf iSD <> 1 Then
            iSD := 1;
         End If;
         --trace_printf("1_1* h_consider: %d, h_min_work_per_day: %d, h_first_time: %s, h_start_time: %s, iSD: %d\n",h_consider,h_min_work_per_day,h_first_time,h_start_time,iSD);
         
         If NOT ((Coalesce(h_consider, 0) = 7) OR (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) 
                  OR (Coalesce(h_bitflag, 0) = 2)) Then
            If ((BITAND(Coalesce(h_update, 0), 255)) = 10) AND (Coalesce(h_min_work_per_day, 0) > 0) 
                     AND (Coalesce(h_min_work_per_day, 0) <= Coalesce(h_available, 0)) Then
               h_bit_minutes := FC_CONSIDER_DAY(h_dayofyear, Coalesce(h_available, 0), Coalesce(h_type, 0)
                                 , Coalesce(h_min_work_per_day, 0), h_bitID);
            ElsIf Coalesce(h_total, 0) = 0 Then
               h_bit_minutes := FC_CONSIDER_DAY(h_dayofyear, Coalesce(h_available, 0), 3, 0, h_bitID);
            Else
               h_bit_minutes := FC_CONSIDER_DAY(h_dayofyear, Coalesce(h_available, 0), Coalesce(h_type, 0), 0
                                 , h_bitID);
            End If;
         Else
            h_bit_minutes := 0;
            --              iSD = 1;
         End If;
         
         -- h_minutes = daily available minutes, h_bit_minutes = daily assigned minutes
         --trace_printf("22_* iSD: %d,h_bit_minutes: %d, h_minutes: %d, h_consider: %d\n",iSD,h_bit_minutes,h_minutes,h_consider);
         --trace_printf("h_consider: %d, assign_to_prj: %d, h_bitflag: %d, iSF: %d\n",h_consider,assign_to_prj,h_bitflag,iSF);
         
         If (Coalesce(assign_to_prj, 0) = 0) AND (Coalesce(h_bitflag, 0) = 1) AND (iSF = 1) Then
            h_bit_minutes := 0;
         ElsIf iSD > 1 Then
            h_minutes := PMO_STPU.getNumberOfBits(h_bitID) * 5;
            --              h_bit_minutes += iSD;
            --              h_minutes -= h_bit_minutes;
            iSD := 1;
         Else
            h_minutes := Coalesce(h_minutes, 0) - Coalesce(h_bit_minutes, 0);
         End If;
         
         --trace_printf("2_* iSD: %d, h_dayofyear: %s, h_julian_day: %d, h_bit_minutes: %d, h_minutes: %d\n",iSD,h_dayofyear,h_julian_day,h_bit_minutes,h_minutes);
         --trace_printf("h_bitflag = %d, h_update: %d, i_pa_replace: %d, i_work = %d, h_work = %d, h_amnt = %d, assign_to_prj: %d\n",h_bitflag,h_update,i_pa_replace,i_work,h_work,h_amnt,assign_to_prj);
         --strcpy(tempStr,"");
         --printTimes((unsigned char *)h_bitID.h_value,tempStr);
         --trace_printf("tempStr = %s\n", tempStr);//trace_flush();
         
         If ((i_pa_replace = 1) OR ((BITAND(Coalesce(h_update, 0), 255)) = 10)) AND (Coalesce(
               assign_to_prj, 0) = 0) AND (Coalesce(h_work, 0) > 0) Then
            h_temp_var := 0;
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_temp_var
               FROM PMO_TMP_PA_SUM
               WHERE DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3333+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3334) : cplan_fun.sql(13930) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_dayofyear: %s, h_temp_var: %d\n",SC_STATE,h_dayofyear,h_temp_var);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_temp_var, 0) > 0) Then
               i_work := Coalesce(h_work, 0) - Coalesce(h_temp_var, 0);
               
               If i_work <= 0 Then
                  h_minutes := 0;
               End If;
            Else
               i_work := Coalesce(h_work, 0);
            End If;
            --trace_printf("######### i_work = %d, h_minutes: %d\n",i_work,h_minutes);
         End If;
         
         If i_work > 0 Then
            If h_amnt = 1 Then
               If Coalesce(h_minutes, 0) > i_work Then
                  h_minutes := i_work;
               ElsIf Coalesce(h_minutes, 0) < i_work Then
                  h_minutes := 0;
               End If;
            Else
               If Coalesce(h_minutes, 0) > 0 Then
                  --trace_printf("1__ h_week: %d, h_prev_week: %d, h_total_min: %d\n",h_week,h_prev_week,h_total_min);
                  If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 5) OR (Coalesce(h_bitflag, 0) = 8) Then
                     If Coalesce(h_week, 0) = Coalesce(h_prev_week, 0) Then
                        If (Coalesce(h_total_min, 0) + Coalesce(h_minutes, 0)) > i_work Then
                           h_minutes := i_work - Coalesce(h_total_min, 0);
                        End If;
                        
                        h_total_min := Coalesce(h_total_min, 0) + Coalesce(h_minutes, 0);
                     Else
                        If Coalesce(h_minutes, 0) > i_work Then
                           h_minutes := i_work;
                        End If;
                        
                        h_total_min := Coalesce(h_minutes, 0);
                        h_prev_week := Coalesce(h_week, 0);
                     End If;
                     --trace_printf("2__ h_week: %d, h_prev_week: %d, h_total_min: %d\n",h_week,h_prev_week,h_total_min);
                  Else
                     If Coalesce(h_minutes, 0) > i_work Then
                        h_minutes := i_work;
                     End If;
                  End If;
               End If;
            End If;
         End If;
         
         If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_minutes, 0) > 4) Then
            --trace_printf("h_start_date: %s, h_julian_day: %d, h_j_disconti_date: %d\n",h_start_date,h_julian_day,h_j_disconti_date);
            If (Coalesce(h_j_disconti_date, 0) = 0) OR (Coalesce(h_julian_day, 0) <= Coalesce(
                  h_j_disconti_date, 0)) Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(h_bitID)))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3400+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3401) : cplan_fun.sql(13997) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_bit_task := Coalesce(h_bit_minutes, 0);
            Else
               h_minutes := 0;
            End If;
            --trace_printf("*** h_start_date: %s, h_bit_task: %d, h_minutes: %d, h_bit_minutes: %d\n",h_start_date,h_bit_task,h_minutes,h_bit_minutes);
         End If;
         
         --trace_printf("*** h_dayofyear: %s, iSF: %d, h_skip: %d, h_minutes: %d, h_total: %d, h_percent_units: %d\n", h_dayofyear,iSF,h_skip,h_minutes,h_total,h_percent_units);
         If Coalesce(h_minutes, 0) > 4 Then
            --t_finish = clock();
            If iSF = 1 Then
               --trace_printf("h_total: %d, h_effort = %d, h_minutes: %d, h_julian_day: %d, h_julian_finish: %d\n",h_total,h_effort,h_minutes,h_julian_day,h_julian_finish);
               If Coalesce(h_julian_day, 0) > Coalesce(h_julian_finish, 0) Then
                  h_dayofyear := h_finish_D;
                  EXIT;
               End If;
               
               If Coalesce(h_percent_units, 0) = 0 Then
                  h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                  h_sched_work := Coalesce(h_minutes, 0);
               ElsIf Coalesce(h_percent_units, 0) = 10000 Then
                  If Coalesce(h_minutes, 0) > Coalesce(h_per_day, 0) Then
                     h_total := Coalesce(h_total, 0) + Coalesce(h_per_day, 0);
                     h_sched_work := Coalesce(h_per_day, 0);
                  Else
                     h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                     h_sched_work := Coalesce(h_minutes, 0);
                  End If;
               ElsIf Coalesce(h_day_min, 0) <> 0 Then
                  If (Coalesce(h_day_min, 0) > Coalesce(h_minutes, 0)) AND (Coalesce(
                                      h_percent_units, 0) <= 10000) Then
                     h_sched_work := Coalesce(h_minutes, 0);
                  Else
                     h_sched_work := Coalesce(h_day_min, 0);
                  End If;
                  
                  h_total := Coalesce(h_total, 0) + Coalesce(h_sched_work, 0);
               Else
                  h_sched_work := Trunc(((Coalesce(h_minutes, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.499);
                  If Coalesce(h_sched_work, 0) < 5 Then
                     h_sched_work := 5;
                  End If;
                  
                  h_total := Coalesce(h_total, 0) + Coalesce(h_sched_work, 0);
               End If;
               
               h_finish_D := h_dayofyear;
               
               --cntr.add(h_julian_day, h_minutes);
               
               /*if (h_sched_work > i_max_min)
               					{
               						i_max_min = h_sched_work;
               					}*/
               
               --trace_printf("1_ h_julian_day: %d, h_sched_work: %d, h_assign_string: %s\n",h_julian_day,h_sched_work,h_assign_string);
               If (Coalesce(h_julian_day, 0) = Coalesce(h_julian_finish, 0)) OR ((Coalesce(h_effort, 0) > 0) 
                                         AND (Coalesce(h_total, 0) > Coalesce(h_effort, 0))) Then
                  If (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_total, 0) > Coalesce(h_effort, 0)) Then
                     If Coalesce(h_totpr, 0) = 0 Then
                        h_totpr := Coalesce(h_effort, 0);
                     Else
                        h_totpr := Coalesce(h_effort, 0) - Coalesce(h_totpr, 0);
                     End If;
                     h_sched_work := Coalesce(h_totpr, 0);
                  End If;
                  
                  ora_temp_sc_state := PMO_DB2M.SC_STATE;
                  null_return := FC_ASSIGN_STRING(h_julian_day, h_sched_work);
                  PMO_DB2M.SC_STATE := ora_temp_sc_state;
                  
                  If (Coalesce(h_type, 0) > 0) AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
                     If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                        h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_update
                                         , h_consider, h_bitID);
                     Else
                        h_temp_sc_state := 11;
                        h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time
                                         , h_temp_sc_state, h_consider, h_bitID);
                     End If;
                     
                     If Coalesce(h_temp_sc_state, 0) <> 0 Then
                        BEGIN
                           CLOSE C332 ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3499+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3500) : cplan_fun.sql(14096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3505+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3506) : cplan_fun.sql(14102) : ' || SYSTIMESTAMP);
                        END IF; 
                        RETURN Coalesce(h_temp_sc_state, 0);
                     End If;
                  End If;
                  
                  EXIT;
               Else
                  ora_temp_sc_state := PMO_DB2M.SC_STATE;
                  null_return := FC_ASSIGN_STRING(h_julian_day, h_sched_work);
                  PMO_DB2M.SC_STATE := ora_temp_sc_state;
               End If;
            Else
               If Coalesce(h_percent_units, 0) = 0 Then
                  h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                  h_sched_work := Coalesce(h_minutes, 0);
               ElsIf Coalesce(h_percent_units, 0) = 10000 Then
                  If Coalesce(h_minutes, 0) > Coalesce(h_per_day, 0) Then
                     h_total := Coalesce(h_total, 0) + Coalesce(h_per_day, 0);
                     h_sched_work := Coalesce(h_per_day, 0);
                  Else
                     h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                     h_sched_work := Coalesce(h_minutes, 0);
                  End If;
               ElsIf Coalesce(h_day_min, 0) <> 0 Then
                  If (Coalesce(h_day_min, 0) > Coalesce(h_minutes, 0)) AND (Coalesce(
                                      h_percent_units, 0) <= 10000) Then
                     h_sched_work := Coalesce(h_minutes, 0);
                  Else
                     h_sched_work := Coalesce(h_day_min, 0);
                  End If;
                  
                  h_total := Coalesce(h_total, 0) + Coalesce(h_sched_work, 0);
                  
                  --                      h_sched_work = h_day_min;
                  --                      h_total += h_sched_work;
               Else
                  h_sched_work := Trunc(((Coalesce(h_minutes, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.49);
                  If Coalesce(h_sched_work, 0) < 5 Then
                     h_sched_work := 5;
                  End If;
                  h_total := Coalesce(h_total, 0) + Coalesce(h_sched_work, 0);
               End If;
               
               h_temp_sc_state := 1;
            End If;
            
            --trace_printf("h_temp_sc_state: %d, h_total: %d, h_effort: %d, h_totpr: %d, h_sched_work: %d\n",h_temp_sc_state,h_total,h_effort,h_totpr,h_sched_work);
            If Coalesce(h_total, 0) = Coalesce(h_effort, 0) Then
               h_totpr := Coalesce(h_effort, 0) - Coalesce(h_totpr, 0);
               
               --cntr.add(h_julian_day, h_minutes);
               If Coalesce(h_temp_sc_state, 0) = 1 Then
                  h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_totpr);
                  
                  /*if (h_totpr > i_max_min)
                  						{
                  							i_max_min = h_totpr;
                  						}*/
               End If;
               --trace_printf("2_ h_julian_day: %d, h_minutes: %d, h_totpr: %d, h_assign_string: %s\n",h_julian_day,h_minutes,h_totpr,h_assign_string);
               
               i_break := 1;
               h_sched_work := Coalesce(h_totpr, 0);
            ElsIf (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_total, 0) > Coalesce(h_effort, 0)) Then
               --t_finish = clock();
               If Coalesce(h_totpr, 0) = 0 Then
                  h_totpr := Coalesce(h_effort, 0);
                  
                  --cntr.add(h_julian_day, h_effort);
                  If Coalesce(h_temp_sc_state, 0) = 1 Then
                     h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_totpr);
                     
                     /*if (h_totpr > i_max_min)
                     							{
                     								i_max_min = h_totpr;
                     							}*/
                  End If;
                  
                  --h_totpr = h_effort;
                  --trace_printf("3_ h_julian_day: %d, h_minutes: %d, h_totpr: %d, h_assign_string: %s\n",h_julian_day,h_minutes,h_totpr,h_assign_string);
               Else
                  h_totpr := Coalesce(h_effort, 0) - Coalesce(h_totpr, 0);
                  
                  --cntr.add(h_julian_day, h_totpr);
                  If Coalesce(h_temp_sc_state, 0) = 1 Then
                     h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_totpr);
                     
                     /*if (h_totpr > i_max_min)
                     							{
                     								i_max_min = h_totpr;
                     							}*/
                  End If;
                  
                  --h_totpr = h_effort - h_totpr;
                  --trace_printf("4_ h_julian_day: %d, h_minutes: %d, h_totpr: %d, h_assign_string: %s\n",h_julian_day,h_minutes,h_totpr,h_assign_string);
               End If;
               
               i_break := 1;
               h_sched_work := Coalesce(h_totpr, 0);
            Else
               --t_finish = clock();
               --cntr.add(h_julian_day, h_minutes);
               If Coalesce(h_temp_sc_state, 0) = 1 Then
                  /*if ( (h_percent_units != 0) || (h_percent_units 1= 10000) )
                  						{
                  							//h_sched_work = int(((h_minutes * h_percent_units) / 10000.00) + 0.49);
                  							if (h_task_type != SC_FD_EFFORT_DRIVEN)
                  							{
                  								h_temp_var = h_sched_work % 5;
                  								if (h_temp_var != 0 )
                  								{
                  									if (h_temp_var < 3 )
                  									{
                  										h_sched_work += h_temp_var;
                  									}
                  									else
                  									{
                  										h_sched_work -= 5 - h_temp_var;
                  									}
                  								}
                  							}
                  						}*/
                  
                  h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_sched_work);
                  
                  /*if (h_sched_work > i_max_min)
                  						{
                  							i_max_min = h_sched_work;
                  						}*/
               End If;
               --trace_printf("5_ h_julian_day: %d, h_minutes: %d, h_totpr: %d, h_assign_string: %s\n",h_julian_day,h_minutes,h_totpr,h_assign_string);
            End If;
            --trace_printf("i_break: %d, assign_to_prj: %d, h_sched_work: %d, h_totpr: %d, h_consider: %d\n",i_break,assign_to_prj,h_sched_work,h_totpr,h_consider);
            
            If i_break = 1 Then
               --trace_printf("h_type: %d, h_update: %d, h_consider: %d\n",h_type,h_update,h_consider);
               If (Coalesce(h_type, 0) > 0) AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
                  If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                     h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_update
                                         , h_consider, h_bitID);
                  Else
                     h_temp_sc_state := 11;
                     h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time
                                         , h_temp_sc_state, h_consider, h_bitID);
                  End If;
                  
                  --strcpy(tempStr,"");
                  --printTimes((unsigned char *)h_bitID.h_value,tempStr);
                  --trace_printf("*** LAST DAY h_sched_work: %d, h_dayofyear: %s, tempStr = %s\n", h_sched_work,h_dayofyear,tempStr);//trace_flush();
                  
                  If Coalesce(h_temp_sc_state, 0) <> 0 Then
                     BEGIN
                        CLOSE C332 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3669+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3670) : cplan_fun.sql(14266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3675+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3676) : cplan_fun.sql(14272) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN Coalesce(h_temp_sc_state, 0);
                  End If;
               End If;
               
               EXIT;
            Else
               If (Coalesce(h_type, 0) > 0) AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
                  If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                     h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_update
                                         , h_consider, h_bitID);
                  Else
                     h_temp_sc_state := 11;
                     h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(assign_to_prj, h_work_type, h_sched_work, h_update
                                         , h_project_id, h_task_id, h_res_id, h_res_type, h_uniqueID
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time
                                         , h_temp_sc_state, h_consider, h_bitID);
                  End If;
                  /*
                  strcpy(tempStr,"");
                  printTimes((unsigned char *)h_bitID.h_value,tempStr);
                  //trace_printf("h_dayofyear: %s, tempStr = %s\n", h_dayofyear,tempStr);//trace_flush();
                  */
                  If Coalesce(h_temp_sc_state, 0) <> 0 Then
                     BEGIN
                        CLOSE C332 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3710+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3711) : cplan_fun.sql(14307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3716+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3717) : cplan_fun.sql(14313) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN Coalesce(h_temp_sc_state, 0);
                  End If;
               End If;
            End If;
            
            h_totpr := Coalesce(h_total, 0);
            -- save previous value
            h_first_time := NULL;
         End If;
      END LOOP;
      -- end of while
      
      BEGIN
         CLOSE C332 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3738+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3739) : cplan_fun.sql(14335) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("iSF = %d, h_julian_day = %d, h_total = %d, h_effort = %d, h_totpr: %d\n", iSF, h_julian_day, h_total, h_effort, h_totpr);
      
      --t_start = clock();
      If iSF = 1 Then
         If Coalesce(h_total, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3749+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3750) : cplan_fun.sql(14346) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 110;
         Else
            h_totpr := 0;
            /*if ( (h_effort > 0)  (h_task_type == SC_FD_EFFORT_DRIVEN) )
            				{
            					h_total = h_effort;
            				}
            				else
            				{*/
            h_effort := Coalesce(h_total, 0);
            --}
         End If;
      ElsIf Coalesce(h_total, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3765+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3766) : cplan_fun.sql(14362) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 200;
      End If;
      
      If Coalesce(h_total, 0) < Coalesce(h_effort, 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3772+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3773) : cplan_fun.sql(14369) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 120;
      ElsIf Coalesce(h_julian_day, 0) > 0 Then
         --strcpy(stmt, "");
         --cntr.print(stmt);
         
         --trace_printf("\n\n =====> before  FC_GET_ASSIGN_STR");
         h_temp_sc_state := FC_GET_ASSIGN_STR(h_assign_string, 0);
         --trace_printf("\n\n =====> after  FC_GET_ASSIGN_STR");
         
         --trace_printf("assign_str = %s\n", (char *) h_assign_string);
         
         If Coalesce(Length(h_assign_string), 0) > PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
            h_assign_string := SubStr(h_assign_string, 1, PMO_CNST.SC_ASSIGN_STRING_LENGTH);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 3788+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(3789) : cplan_fun.sql(14385) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
         End If;
         --strcpy(h_assign_string, stmt);
         
         h_start_date := h_start_date;
         If iSF = 1 Then
            h_finish_date := h_finish_date;
         Else
            h_finish_date := h_dayofyear;
         End If;
         --h_start_date_Ind = 0;
         --h_finish_date_Ind = 0;
         
         --trace_printf("h_available: %d, h_totpr: %d, h_bit_task: %d\n", h_available,h_totpr,h_bit_task);
         If Coalesce(h_percent_units, 0) > 10000 Then
            If Coalesce(h_totpr, 0) > Coalesce(h_available, 0) Then
               h_totpr := 0;
            End If;
         End If;
         --          else if ( h_type == 0 )
         --          {
         If (PMO_ORASTD.strncmp(h_start_date, h_finish_date, 10) = 0) AND (Coalesce(h_bit_task, 0) > 0) Then
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
               h_totpr := Coalesce(h_totpr, 0) + Coalesce(h_bit_task, 0);
            Else
               h_totpr := Coalesce(h_effort, 0);
            End If;
            h_bit_task := -2;
         Else
            If (Coalesce(h_bit_task, 0) >= 0) AND (Coalesce(h_type, 0) > 0) Then
               h_bit_task := -2;
            End If;
            
            /*if ( h_work + h_bit_minutes > 0 )
            					{
            						h_totpr = h_totpr + h_work + h_bit_minutes;
            					}*/
         End If;
         --          }
         
         --trace_printf("***** h_task_type: %d\n",h_task_type);
         If Coalesce(Length(h_finish_time), 0) < 5 Then
            h_finish_time := '17:00:00';
         End If;
         
         /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
         			{
         //trace_printf("FIXED DURATION_1: h_start_date: %s, h_start_DT: %s, h_finish_DT: %s\n",h_start_date,h_start_DT,h_finish_DT);
         //trace_printf("iSF: %d, h_etc: %d, h_start_date: %s, h_finish_date: %s\n",iSF,h_etc,h_start_date,h_finish_date);
         				if ( ((h_bitflag == 0) || ((h_bitflag == 1)))  (h_rollup == 0)  (h_etc > 0) )
         				{
         					BEGIN_SQL
         						SELECT TIMESTAMP(DATE(HOST(h_start_date)), TIME(HOST(h_start_DT)))
         						INTO HOST(h_start_date)
         						FROM DUAL;
         					END_SQL;
         //                  InitializeHost(SC_PARAM(h_start_date), SC_PARAM(h_task_start_DT));
         					InitializeHost(SC_PARAM(h_finish_date), SC_PARAM(h_task_finish_DT));
         				}
         				else
         				{
         					BEGIN_SQL
         						SELECT FIRST_AVAIL_TIME(BIT_SCHED) INTO HOST(h_start_time) FROM session.PMO_TMP_CDY
         						WHERE DAY_YEAR=DATE(HOST(h_start_date));
         					END_SQL;
         
         					BEGIN_SQL
         						SELECT LAST_AVAIL_TIME(BIT_SCHED) INTO HOST(h_finish_time) FROM session.PMO_TMP_CDY
         						WHERE DAY_YEAR=DATE(HOST(h_dayofyear));
         					END_SQL;
         
         					if (iSF == 1)
         					{
         						BEGIN_SQL
         							SELECT TIMESTAMP(DATE(HOST(h_start_date)), TIME(HOST(h_start_time))), TIMESTAMP(DATE(HOST(h_finish_date)), TIME(HOST(h_finish_time)))
         							INTO HOST(h_start_date), HOST(h_finish_date)
         							FROM DUAL;
         						END_SQL;
         //trace_printf("SC_STATE: %d, h_start_date: %s, h_finish_date: %s\n",SC_STATE,h_start_date,h_finish_date);
         //trace_printf("h_dayofyear: %s, h_start_time: %s, h_finish_time: %s\n",h_dayofyear,h_start_time,h_finish_time);
         					}
         					else
         					{
         						BEGIN_SQL
         							SELECT TIMESTAMP(DATE(HOST(h_start_date)), TIME(HOST(h_start_time))), TIMESTAMP(DATE(HOST(h_dayofyear)), TIME(HOST(h_finish_time)))
         							INTO HOST(h_start_date), HOST(h_finish_date)
         							FROM DUAL;
         						END_SQL;
         					}
         				}
         //trace_printf("FIXED DURATION_2: h_start_date: %s, h_finish_date: %s\n",h_start_date,h_finish_date);
         			}
         			else
         			{*/
         If Coalesce(h_totpr, 0) < 5 Then
            h_totpr := 0;
         End If;
         --trace_printf("h_consider: %d, h_update: %d, h_bit_task: %d, h_bit_minutes: %d, h_totpr: %d, h_type: %d, h_effort: %d, h_total: %d\n",h_consider,h_update,h_bit_task,h_bit_minutes,h_totpr,h_type,h_effort,h_total);
         --trace_printf("h_start_date: %s, h_finish_date: %s, h_dayofyear: %s\n",h_start_date,h_finish_date,h_dayofyear);
         --trace_printf("h_tsk_assignment_id: %d:%s, h_prj_assignment_id: %d:%s\n",h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            If Coalesce(h_update, 0) = 3 Then
               If Coalesce(h_bit_task, 0) <= 0 Then
                  BEGIN
                     SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3903+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3904) : cplan_fun.sql(14500) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_STPU.TIME_AFTER_SCHED(h_bit_task, BIT_SCHED) INTO h_start_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3917+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3918) : cplan_fun.sql(14514) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If Coalesce(h_totpr, 0) > 0 Then
                  BEGIN
                     SELECT PMO_STPU.TIME_AFTER_SCHED(h_totpr, BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_tsk_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10)) AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3933+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3934) : cplan_fun.sql(14530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_tsk_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10)) AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3947+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3948) : cplan_fun.sql(14544) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If Coalesce(h_type, 0) = 0 Then
                  BEGIN
                     SELECT TSK_ASSIGNMENT_ID INTO h_prj_assignment_id FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID =h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3963+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3964) : cplan_fun.sql(14560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If Coalesce(h_bit_task, 0) <= 0 Then
                  BEGIN
                     SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3979+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3980) : cplan_fun.sql(14576) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_STPU.TIME_BEFORE_SCHED(h_bit_task,BIT_SCHED) INTO h_start_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID =h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10)) AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 3993+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(3994) : cplan_fun.sql(14590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If Coalesce(h_effort, 0) < Coalesce(h_total, 0) Then
                  BEGIN
                     SELECT PMO_STPU.TIME_AFTER_SCHED(h_totpr, BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10)) AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4009+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4010) : cplan_fun.sql(14606) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_prj_assignment_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10)) AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4023+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4024) : cplan_fun.sql(14620) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            
            --strcpy(tempStr,"");
            --printTimes((unsigned char *)h_bitID.h_value,tempStr);
            --trace_printf("h_start_date: %s, h_start_time: %s, h_dayofyear: %s, h_finish_time: %s, tempStr: %s\n", h_start_date,h_start_time,h_dayofyear,h_finish_time,tempStr);//trace_flush();
            
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_date, 1, 10)), PMO_ORASTD.TIME(h_start_time)), PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time))
               INTO h_start_date, h_finish_date
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4044+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4045) : cplan_fun.sql(14641) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            If Coalesce(h_bit_task, 0) = -1 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(h_bitID)))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4060+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4061) : cplan_fun.sql(14657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_bit_task, 0) >= 0) AND (Coalesce(Length(start_date001), 0) = 10) Then
               If Coalesce(h_bit_task, 0) = 0 Then
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                     BEGIN
                        SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time
                        FROM PMO_TMP_CDY
                        WHERE DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4077+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4078) : cplan_fun.sql(14674) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time
                        FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4092+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4093) : cplan_fun.sql(14689) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                     BEGIN
                        SELECT PMO_STPU.TIME_AFTER_SCHED(h_bit_task, BIT_SCHED) INTO h_start_time
                        FROM PMO_TMP_CDY
                        WHERE DAY_YEAR=TO_DATE(SubStr(h_start_date, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4109+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4110) : cplan_fun.sql(14706) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_STPU.TIME_AFTER_SCHED(h_bit_task, BIT_SCHED) INTO h_start_time
                        FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4124+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4125) : cplan_fun.sql(14721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_date, 1, 10)), PMO_ORASTD.TIME(h_start_time))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4141+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4142) : cplan_fun.sql(14738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_totpr, 0) > 0 Then
               h_available := PMO_STPU.getNumberOfBits(h_bitID) * 5;
               
               If Coalesce(h_totpr, 0) < Coalesce(h_available, 0) Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.TIME_AFTER_SCHED(h_totpr, h_bitID)))
                     INTO h_finish_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4161+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4162) : cplan_fun.sql(14758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(h_bitID)))
                     INTO h_finish_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4176+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4177) : cplan_fun.sql(14773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(h_bitID)))
                  INTO h_finish_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4192+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4193) : cplan_fun.sql(14789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            --strcpy(tempStr,"");
            --printTimes((unsigned char *)h_bitID.h_value,tempStr);
            --trace_printf("h_start_time: %s, h_finish_date: %s, h_dayofyear: %s, tempStr = %s\n", h_start_time,h_finish_date, h_dayofyear,tempStr);//trace_flush();
         End If;
         --}
         --trace_printf("h_update: %d, h_consider = %d, h_res_type = %s, h_totpr: %d, h_work: %d\n", h_update,h_consider,h_res_type,h_totpr,h_work);
         --trace_printf("h_start_date = %s, h_finish_date = %s\n", h_start_date,h_finish_date);
         
         h_available := 0;
         h_planned_work := 0;
         h_sched_work := 0;
         
         If (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
            h_start_date := SubStr(h_start_date, 1, 19);
            h_finish_date := SubStr(h_finish_date, 1, 19);
            --strcpy(h_finish_date[19], "\0");
            
            If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) OR (SubStr(h_res_type, 1, 1) = '1') Then
               BEGIN
                  SELECT SUM(AVAILABLE_MIN) INTO h_available FROM PMO_TMP_CDY
                  WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4225+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4226) : cplan_fun.sql(14822) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT SUM(AVAILABLE_MIN) INTO h_available FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID=h_res_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4239+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4240) : cplan_fun.sql(14836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_planned_work FROM TASK_DAY_OF_YEAR --PMO_TMP_TDY
               WHERE RESOURCE_ID=h_res_id AND BIT_FLAG='0' AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4254+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4255) : cplan_fun.sql(14851) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_sched_work FROM TASK_DAY_OF_YEAR --PMO_TMP_TDY
               WHERE RESOURCE_ID=h_res_id AND BIT_FLAG='2' AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4268+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4269) : cplan_fun.sql(14865) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4275+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4276) : cplan_fun.sql(14872) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
   End If;
   
   --trace_printf("h_available = %d, h_planned_work = %d h_sched_work = %d, h_effort = %d\n",h_available,h_planned_work,h_sched_work,h_effort);
   --trace_printf("h_julian_day = %d, h_available = %d, h_planned_work = %d h_sched_work = %d, h_effort = %d\n",h_julian_day,h_available,h_planned_work,h_sched_work,h_effort);
   
   If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
             h_julian_day, 0) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4287+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4288) : cplan_fun.sql(14884) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 120;
   ElsIf Coalesce(h_julian_day, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4292+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4293) : cplan_fun.sql(14889) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 120;
   End If;
   /*  else if ( (h_type == 0)  ((h_available - h_planned_work - h_sched_work) < h_effort) ) // Not available
   		return 110;*/
   
   --trace_printf("h_type = %d, h_start_date = %s, h_finish_date = %s, h_effort = %d\nh_assign_string: %s\n", h_type,h_start_date, h_finish_date, h_effort,h_assign_string);
   If Coalesce(h_type, 0) > 0 Then
      -- CALL  U_TSKRES2
      h_and_roles := and_roles;
      h_and_skills := and_skills;
      
      If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) 
             AND (SubStr(h_res_type, 1, 1) = '0') Then
         If Coalesce(h_rollup, 0) = 4 Then
            h_update := 4;
            -- re_etc schedule
         ElsIf Coalesce(h_update, 0) = 0 Then
            h_update := (PMO_FUNC.BIT_OR(2, PMO_CNST.SC_BIT_30));
            h_work_type := 2;
         Else
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_30));
            h_work_type := 0;
         End If;
         
         If Coalesce(h_rollup, 0) = 2 Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_29));
         End If;
      Else
         If Coalesce(h_rollup, 0) = 3 Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_29_30));
         End If;
         If Coalesce(h_rollup, 0) = 1 Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_30));
         End If;
         If Coalesce(h_rollup, 0) = 2 Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_28));
         ElsIf Coalesce(h_rollup, 0) = 4 Then
            h_update := 4;
         End If;
         -- re_etc schedule
      End If;
      --trace_printf("h_rollup = %d, h_update = %d, h_contour = %d, h_bitflag = %d\n", h_rollup, h_update, h_contour, h_bitflag);
      
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_date) - PMO_ORASTD.DAYS(h_start_date)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4347+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4348) : cplan_fun.sql(14944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("============> h_total = %d\n", h_total);
      
      If Coalesce(h_total, 0) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4355+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4356) : cplan_fun.sql(14952) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
            --trace_printf("LEVEL: h_start_date = %s, h_finish_date = %s, h_effort: %d\n", h_start_date, h_finish_date, h_effort);
            --trace_printf("i_lev_etc: %d, h_tsk_assignment_id: %s, h_assign_string: %s\n", i_lev_etc,h_tsk_assignment_id,h_assign_string);
            If (BITAND(Coalesce(h_update, 0), 256)) = 256 Then
               --                  if ( (strcmp(h_start_date,h_ta_start_DT) == 0)
               --                            (strcmp(h_finish_date,h_ta_finish_DT) == 0) )
               If (i_lev_etc = 0) AND (i_has_actual = 0) AND (PMO_ORASTD.strcmp(h_start_date, h_ta_start_DT) = 0) 
                     AND (PMO_ORASTD.strcmp(h_assign_string, h_lev_assign_string) = 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     CONSIDER                = 0,
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4387+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4388) : cplan_fun.sql(14984) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4404+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4405) : cplan_fun.sql(15001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  If i_lev_etc = 1 Then
                     BEGIN
                        SELECT TO_DATE(SubStr(h_start_date, 1, 10)) - 1      INTO h_finish_DT FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4418+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4419) : cplan_fun.sql(15015) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 2);
                  ElsIf i_has_actual = 1 Then
                     h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 1);
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4447+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4448) : cplan_fun.sql(15044) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     BEGIN
                        INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4465+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4466) : cplan_fun.sql(15062) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            Else
               If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
                  h_proj_assign_flag := 0;
               Else
                  h_proj_assign_flag := 1;
                  
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_date, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
                     INTO h_start_date
                     FROM LEVEL_TASK_DAY_OF_YEAR
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4488+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4489) : cplan_fun.sql(15085) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("LEVEL_PA: SC_STATE: %d, h_start_date = %s, h_finish_date = %s\n", SC_STATE,h_start_date, h_finish_date);
               End If;
               
               --trace_printf("LEVEL: i_has_actual: %d, i_lev_etc: %d, h_start_date = %s, h_act_start_DT = %s, h_finish_DT = %s\n", i_has_actual,i_lev_etc,h_start_date,h_act_start_DT, h_finish_date);
               If i_lev_etc = 1 Then
                  BEGIN
                     SELECT  PMO_ORASTD.DB2TIMESTAMP(h_start_date) - 1     INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4505+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4506) : cplan_fun.sql(15102) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 2);
                  
                  BEGIN
                     INSERT INTO PMO_TMP_TDY_LEVEL (
                     PROJECT_ID,
                     RESOURCE_ID,
                     PRIORITY,
                     ACTUAL_FLAG,
                     TSK_ASSIGNMENT_ID,
                     DAY_YEAR,
                     SCHED_MIN,
                     BIT_FLAG,
                     BIT_SCHED,
                     PROJ_ASSIGN_FLAG)
                     SELECT
                     PROJECT_ID,
                     RESOURCE_ID,
                     h_priority,
                     0,
                     TSK_ASSIGNMENT_ID,
                     DAY_YEAR,
                     SCHED_MIN,
                     BIT_FLAG,
                     BIT_SCHED,
                     h_proj_assign_flag
                     FROM TASK_DAY_OF_YEAR
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_act_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4548+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4549) : cplan_fun.sql(15145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("LEVEL_???: SC_STATE: %d, h_act_start_DT: %s, h_finish_DT: %s\n", SC_STATE,h_act_start_DT,h_finish_DT);
               ElsIf i_has_actual = 1 Then
                  h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 1);
               End If;
               
               BEGIN
                  INSERT INTO PMO_TMP_TDY_LEVEL (
                  PROJECT_ID,
                  RESOURCE_ID,
                  PRIORITY,
                  ACTUAL_FLAG,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  SCHED_MIN,
                  BIT_FLAG,
                  BIT_SCHED,
                  PROJ_ASSIGN_FLAG)
                  SELECT
                  PROJECT_ID,
                  RESOURCE_ID,
                  h_priority,
                  0,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  SCHED_MIN,
                  BIT_FLAG,
                  BIT_SCHED,
                  h_proj_assign_flag
                  FROM PMO_TMP_TDOY;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4592+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4593) : cplan_fun.sql(15189) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("LEVEL_UPDATE: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
               --trace_printf("LEVEL: SC_STATE: %d, h_priority: %d, h_proj_assign_flag = %d\n", SC_STATE,h_priority,h_proj_assign_flag);
               
               --                  if ( (strcmp(h_start_date,h_ta_start_DT) == 0)
               --                            (strcmp(h_finish_date,h_ta_finish_DT) == 0) )
               If (i_lev_etc = 0) AND (i_has_actual = 0) AND ((PMO_ORASTD.strcmp(h_start_date
                     , h_ta_start_DT) <> 0) OR (PMO_ORASTD.strcmp(h_assign_string
                     , h_lev_assign_string) <> 0)) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     CONSIDER                = 6,
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4622+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4623) : cplan_fun.sql(15219) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4644+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4645) : cplan_fun.sql(15241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("LEVEL_UPDATE: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
               --trace_printf("h_start_date: %s, h_finish_date: %s, h_assign_string: %s\n", h_start_date, h_finish_date,h_assign_string);
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  BEGIN
                     INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4665+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4666) : cplan_fun.sql(15262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            
            --trace_printf("LEVEL: h_start_date = %s, h_finish_date = %s, h_act_start_DT: %s\n", h_start_date, h_finish_date, h_act_start_DT);
            
            start_date001 := h_start_date;
            finish_date001 := h_finish_date;
            --timeLog (file,"EXIT LEVELING_1:");
            --AEL: if this level exit
            If PMO_DB2M.SC_STATE = 100 Then
               PMO_DB2M.SC_STATE := 0;
            End If;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4683+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4684) : cplan_fun.sql(15280) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_DB2M.SC_STATE;
         Else
            BEGIN
               INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4699+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4700) : cplan_fun.sql(15296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
      Else
         If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
            /*              if ( (h_update  0x00100) == 0x100 )    // Calculate
            				{
            					if ( (i_lev_etc == 0)  (i_has_actual == 0) 
            						 (strcmp(h_start_date,h_ta_start_DT) == 0) 
            						 (strcmp(h_assign_string,h_lev_assign_string) == 0) )
            //                       (strcmp(h_finish_date,h_ta_finish_DT) == 0) )
            					{
            						BEGIN_SQL
            							UPDATE TASK_ASSIGNMENTS SET
            							CONSIDER                = 0,
            							LEVEL_START             = SC_NULL,
            							LEVEL_FINISH            = SC_NULL,
            							LEVEL_EFFORT            = 0,
            							LEVEL_ASSIGN_STRING     = SC_NULL
            							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
            						END_SQL;
            
            						BEGIN_SQL
            							DELETE FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_assign_id);
            						END_SQL;
            					}
            					else
            					{
            						BEGIN_SQL
            							UPDATE TASK_ASSIGNMENTS SET
            							LEVEL_START             = HOST(h_start_date),
            							LEVEL_FINISH            = HOST(h_finish_date),
            							LEVEL_EFFORT            = HOST(h_effort),
            							LEVEL_ASSIGN_STRING     = HOST(h_assign_string)
            							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
            						END_SQL;
            					}
            				}
            				else
            				{
            					if ( (i_lev_etc == 0)  (i_has_actual == 0) 
            						 ((strcmp(h_start_date,h_ta_start_DT) != 0) ||
            						 (strcmp(h_assign_string,h_lev_assign_string) != 0)) )
            //                       (strcmp(h_finish_date,h_ta_finish_DT) != 0)) )
            					{
            						BEGIN_SQL
            							UPDATE TASK_ASSIGNMENTS SET
            							CONSIDER                = 6,
            							LEVEL_START             = HOST(h_start_date),
            							LEVEL_FINISH            = HOST(h_finish_date),
            							LEVEL_EFFORT            = HOST(h_effort),
            							LEVEL_ASSIGN_STRING     = HOST(h_assign_string)
            							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
            						END_SQL;
            					}
            					else
            					{*/
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               LEVEL_START             = h_start_date,
               LEVEL_FINISH            = h_finish_date,
               LEVEL_EFFORT            = h_effort,
               LEVEL_ASSIGN_STRING     = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4777+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4778) : cplan_fun.sql(15374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --                  }
            --              }
         End If;
         
         h_tsk_assignment_id := assign_id;
      End If;
      
      --trace_printf("** q.3 h_bitflag: %d, h_percent_units: %d, h_work: %d\n",h_bitflag,h_percent_units,h_work);
      If Coalesce(h_percent_units, 0) > 0 Then
         /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
         			{
         				InitializeHost(SC_PARAM(h_unit), 101);
         				InitializeHost(SC_PARAM(h_eunit), calculate_flag);
         			}
         			else
         			{*/
         h_unit := 100;
         --}
         
         h_work := Coalesce(h_percent_units, 0);
      ElsIf Coalesce(h_unit, 0) = 100 Then
         h_unit := 0;
      End If;
      /*      else if ( (h_work == 0)  (i_max_min > 0)   (i_max_min != h_per_day))
      		{
      			InitializeHost(SC_PARAM(h_unit), 100);
      			h_work = ((double) i_max_min / (double) h_per_day) * 10000.00;
      		}*/
      --t_start = clock();
      --trace_printf("U_TSKRES2: h_start_date: %s, h_finish_date: %s, h_effort: %d, h_unit: %d, h_work: %d, h_update: %d\n", h_start_date, h_finish_date,h_effort, h_unit, h_work,h_update);
      PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_and_roles
                               , h_and_skills, h_start_date, h_finish_date, h_effort, h_work
                               , h_eunit, h_unit, h_res_type, h_work_type, h_update, h_contour
                               , h_bitflag, h_occurrence, h_consider, h_assign_string, h_rec_user
                               , h_per_day, p_reg_sale, p_spec_sale);
      --trace_printf("U_TSKRES2: SC_STATE = %d, h_res_id  =%s, h_update = %d, h_bitflag = %d\n", SC_STATE, h_res_id, h_update, h_bitflag);
      
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         If i_lev_etc = 1 Then
            start_date001 := h_act_start_DT;
         Else
            start_date001 := h_start_date;
         End If;
         finish_date001 := h_finish_date;
         --timeLog (file,"EXIT LEVELING:");
         --AEL: if this level exit
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4828+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4829) : cplan_fun.sql(15425) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      ElsIf PMO_DB2M.SC_STATE < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4833+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4834) : cplan_fun.sql(15430) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      ElsIf (PMO_DB2M.SC_STATE > 0) OR (Coalesce(h_rollup, 0) = 4) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 4838+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(4839) : cplan_fun.sql(15435) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
   End If;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PROFILE Then
      h_start_date := '1899-12-30 8:00:00';
      h_finish_date := '1899-12-30 8:00:00';
   End If;
   
   If iSF = 0 Then
      h_effort := 0;
   End If;
   
   If (Coalesce(Length(h_and_roles), 0) = 0) AND (Coalesce(Length(h_and_skills), 0) = 0) Then
      i_noskill := 1;
   Else
      i_noskill := 0;
   End If;
   
   If (Coalesce(h_julian_day, 0) > 0) OR (Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT) Then
      If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
         --          Nullify(SC_PARAM(h_start_date));
         --          Nullify(SC_PARAM(h_finish_date));
         h_start_date := '1899-12-30 8:00:00';
         h_finish_date := '1899-12-30 8:00:00';
         h_planned_work := 0;
         --h_planned_work_Ind = 0;
         h_sched_work := 0;
         --h_sched_work_Ind = 0;
         h_available := 0;
         --h_available_Ind = 0;
         h_assign_string := '';
         --h_assign_string_Ind = 0;
      Else
         h_planned_work := Coalesce(h_planned_work, 0);
         --h_planned_work_Ind = 0;
         h_sched_work := Coalesce(h_sched_work, 0);
         --h_sched_work_Ind = 0;
         h_available := Coalesce(h_available, 0);
         --h_available_Ind = 0;
         h_assign_string := h_assign_string;
         --h_assign_string_Ind = 0;
      End If;
      --trace_printf("strlen(h_assign_string): %d, h_assign_string: %s\n", strlen(h_assign_string), h_assign_string);
      --trace_printf("h_bitflag = %d, h_consider = %d, h_res_id = %s, i_noskill = %d\n", h_bitflag,h_consider,h_res_id,i_noskill);
      
      If i_noskill = 1 Then
         l_data_flag := 0;
         h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, h_and_roles, h_and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
         
         --trace_printf("\n (1)---->h_reg_cost:%f,h_spec_cost:%f,h_reg_sale:%f,h_spec_sale:%f\n", h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale);
         
         h_uniqueID := SYS_GUID();
         
         If l_data_flag = 0 Then
            BEGIN
               INSERT INTO PMO_TMP_SEARCH (RS_ID, RES_ID, RES_SALE_RATE, START_DATE, FINISH_DATE, ASSIGNABLE, PROPOSED, PLANNED, SCHED, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, CONTOUR, DEFAULT_FLAG)
               VALUES(h_uniqueID, h_res_id, h_res_cost, h_start_date, h_finish_date, h_available, h_effort, h_planned_work, h_sched_work, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale, h_assign_string, '0');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4909+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4910) : cplan_fun.sql(15506) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf l_data_flag = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_SEARCH (RS_ID, RES_ID, RES_SALE_RATE, START_DATE, FINISH_DATE, ASSIGNABLE, PROPOSED, PLANNED, SCHED, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, CONTOUR, DEFAULT_FLAG)
               VALUES(h_uniqueID, h_res_id, h_res_cost, h_start_date, h_finish_date, h_available, h_effort, h_planned_work, h_sched_work, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale, h_assign_string, '1');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4927+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4928) : cplan_fun.sql(15524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_SEARCH (RS_ID, RES_ID, RES_SALE_RATE, START_DATE, FINISH_DATE, ASSIGNABLE, PROPOSED, PLANNED, SCHED, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, CONTOUR, DEFAULT_FLAG)
               VALUES(h_uniqueID, h_res_id, h_res_cost, h_start_date, h_finish_date, h_available, h_effort, h_planned_work, h_sched_work, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale, h_assign_string, '2');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 4945+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(4946) : cplan_fun.sql(15542) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("(2)---->SC_STATE: %d, l_data_flag: %d\n", SC_STATE,l_data_flag);
      Else
         h_and_roles := and_roles;
         h_and_skills := and_skills;
         
         token := SubStr(token, 1, 32);
         --strcpy(token[32], "\0");
         
         --trace_printf("h_and_roles = %s, h_and_skills = %s\n", h_and_roles,h_and_skills);
         
         --Parse Roles list
         If Coalesce(Length(h_and_roles), 0) < 32 Then
            numItems := 0;
         Else
            numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
         End If;
         
         If numItems > 0 Then
            stmt := '';
            stmt := 
                  'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,ROLE_ID,ROLE_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) SELECT SYS_GUID(), ''' 
                  || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') 
                  || ',''' || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
                  h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' 
                  || TO_CHAR(Coalesce(h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(
                  h_sched_work, 0)) || ',''' || h_assign_string || ''',';
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               stwhere := 
                     't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t2.LAST_USED_DATETIME),'''') || ''"''),'
                      || 'COALESCE(r.REGULAR_COST,0), COALESCE(r.REGULAR_SALE,0), COALESCE(r.SPECIAL_COST,0), COALESCE(r.SPECIAL_SALE,0),''0'' '
                      || 'FROM TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID '
                      || 'WHERE t2.TYPE_ID=73 AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                      || ' ( t3.RANK >= SUBSTR(t2.RANK, 1, 8) AND t3.RANK <= t2.RANK  '
                      || '   AND substr(t2.RANK,1, case when length(t3.RANK) > length(t2.RANK) then 1 else length (t3.RANK) end) = t3.RANK )'
                      || ' AND t3.ELEMENT_ID IN (''';
            Else
               stwhere := 
                     't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"''),'
                      || ' CASE WHEN (R1.REGULAR_COST = 0 OR R1.REGULAR_COST IS NULL) AND (R.REGULAR_COST = 0 OR R.REGULAR_COST IS NULL) THEN 0 '
                      || ' WHEN (R1.REGULAR_COST = 0 OR  R1.REGULAR_COST IS NULL) THEN R.REGULAR_COST '
                      || ' ELSE R1.REGULAR_COST END AS REGULAR_COST, '
                      || ' CASE WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) AND (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL)  THEN 0 '
                      || ' WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) THEN R.REGULAR_SALE '
                      || ' ELSE R1.REGULAR_SALE END AS REGULAR_SALE, '
                      || ' CASE WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) AND (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL)  THEN 0 '
                      || ' WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) THEN R.SPECIAL_COST '
                      || ' ELSE R1.SPECIAL_COST END AS SPECIAL_COST, '
                      || ' CASE WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) AND (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL)  THEN 0 '
                      || ' WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) THEN R.SPECIAL_SALE ELSE R1.SPECIAL_SALE END AS SPECIAL_SALE, '
                      || ' CASE WHEN r1.RATE_ID IS NULL OR R1.REGULAR_COST = 0 THEN ''0'' ELSE t1.DEFAULT_FLAG END '
                      || 'FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,'
                      || 'TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
               stwhere := stwhere || h_res_id;
               stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                      || ' ( t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
                      || '   AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                      || ' AND t3.ELEMENT_ID IN (''';
            End If;
            i := 0;
            
            WHILE i < numItems LOOP
               If i > 0 Then
                  stwhere := stwhere || ''',''';
               End If;
               
               -- Get next role_id
               --                  pmo_substr(token, h_and_roles, i*32, 32); //strncpy( token, h_and_roles[i*32], 32);
               token := SubStr(h_and_roles, i * 32 + 1, 32);
               
               stwhere := stwhere || token;
               i := i + 1;
            END LOOP;
            stwhere := stwhere || ''')';
            stmt := stmt || stwhere;
            --trace_printf("***** ROLES *****\n%s\n\n",(char *) stmt);
            
            BEGIN
               EXECUTE IMMEDIATE stmt;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 5039+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(5040) : cplan_fun.sql(15636) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
                  END IF; 
               END;
            END;
         End If;
         
         --Parse Skills list
         If Coalesce(Length(h_and_skills), 0) < 32 Then
            numItems := 0;
         Else
            numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
         End If;
         If numItems > 0 Then
            stmt := '';
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               stmt := 
                     'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SKILL_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                      || 'SELECT SYS_GUID(), ''' || h_res_id || ''',' 
                           || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
                           || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
                           h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' 
                           || TO_CHAR(Coalesce(h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(
                           h_sched_work, 0)) || ',''' || h_assign_string || ''',';
               stwhere := 
                     '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t3.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t3.LAST_USED_DATETIME),'''') || ''"''), '
                      || ' COALESCE(r.REGULAR_COST,0), COALESCE(r.REGULAR_SALE,0), COALESCE(r.SPECIAL_COST,0), COALESCE(r.SPECIAL_SALE,0), ''0'' '
                      || 'FROM TMT_COMPETENCIES t4,TMT_COMPETENCIES t3 LEFT OUTER JOIN RATES r ON t3.RATE_ID=r.RATE_ID WHERE t4.ELEMENT_ID=t3.PARENT_ID AND t3.ELEMENT_ID IN (''';
            Else
               stmt := 
                     'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SKILL_ID,SKILL_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                      || 'SELECT SYS_GUID(), ''' || h_res_id || ''', ' 
                           || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
                           || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
                           h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' 
                           || TO_CHAR(Coalesce(h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(
                           h_sched_work, 0)) || ',''' || h_assign_string || ''',';
               
               stwhere := 
                     't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"''), '
                      || 'case when (r1.REGULAR_COST is null or r1.REGULAR_COST=0) then COALESCE(r.REGULAR_COST,0) else r1.REGULAR_COST end AS REGULAR_COST, '
                      || 'case when (r1.REGULAR_SALE is null or r1.REGULAR_SALE=0) then COALESCE(r.REGULAR_SALE,0) else r1.REGULAR_SALE end AS REGULAR_SALE, '
                      || 'case when (r1.SPECIAL_COST is null or r1.SPECIAL_COST=0) then COALESCE(r.SPECIAL_COST,0) else r1.SPECIAL_COST end AS SPECIAL_COST, '
                      || 'case when (r1.SPECIAL_SALE is null or r1.SPECIAL_SALE=0) then COALESCE(r.SPECIAL_SALE,0) else r1.SPECIAL_SALE end AS SPECIAL_SALE, ''0'' '
                      || 'FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
               
               stwhere := stwhere || h_res_id;
               
               stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                      || ' (t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK  '
                      || '  AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                      || ' AND t3.ELEMENT_ID IN (''';
            End If;
            
            i := 0;
            WHILE i < numItems LOOP
               If i > 0 Then
                  stwhere := stwhere || ''',''';
               End If;
               
               -- Get next role_id
               --                  pmo_substr(token, h_and_skills, i*32, 32); //strncpy( token, h_and_skills[i*32], 32 );
               token := SubStr(h_and_skills, i * 32 + 1, 32);
               
               stwhere := stwhere || token;
               i := i + 1;
            END LOOP;
            stwhere := stwhere || ''')';
            stmt := stmt || stwhere;
            --trace_printf("***** SKILLS *****\n%s\n\n",(char *) stmt);
            
            BEGIN
               EXECUTE IMMEDIATE stmt;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 5122+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(5123) : cplan_fun.sql(15719) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   --Parse Show Roles list
   If Coalesce(Length(h_shw_roles), 0) < 32 Then
      numItems := 0;
   Else
      numItems := Trunc((Coalesce(Length(h_shw_roles), 0) + 1) / 32);
   End If;
   
   If numItems > 0 Then
      token := SubStr(token, 1, 32);
      stmt := '';
      stmt := 
            'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SHOW_ROLE) SELECT SYS_GUID(),''' 
            || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
            || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
            h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' || TO_CHAR(Coalesce(
            h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(h_sched_work, 0)) || ',''' 
            || h_assign_string || ''',';
      stwhere := 
            '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"'') FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID '
             || ' AND (t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK  '
             || '       AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK)end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         -- Get next role_id
         --          pmo_substr(token, h_shw_roles, i*32, 32);     //strncpy( token, h_shw_roles[i*32], 32);
         token := SubStr(h_shw_roles, i * 32 + 1, 32);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      --trace_printf("***** SHOW ***** h_res_cost: %f\n%s\n\n",h_res_cost,(char *) stmt);
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 5185+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(5186) : cplan_fun.sql(15782) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
   End If;
   
   --Parse Show Skills list
   If Coalesce(Length(h_shw_skills), 0) < 32 Then
      numItems := 0;
   Else
      numItems := Trunc((Coalesce(Length(h_shw_skills), 0) + 1) / 32);
   End If;
   If numItems > 0 Then
      token := SubStr(token, 1, 32);
      stmt := '';
      stmt := 
            'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SHOW_SKILL) SELECT SYS_GUID(),''' 
            || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
            || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
            h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' || TO_CHAR(Coalesce(
            h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(h_sched_work, 0)) || ',''' 
            || h_assign_string || ''',';
      stwhere := 
            '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"'') FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID '
             || ' AND (t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
             || '      AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         -- Get next role_id
         --          pmo_substr(token, h_shw_skills, i*32, 32);    //    strncpy( token, h_shw_skills[i*32], 32);
         token := SubStr(h_shw_skills, i * 32 + 1, 32);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 5244+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(5245) : cplan_fun.sql(15841) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   --trace_printf("END: h_start_date: %s, h_finish_date: %s, h_effort: %d\n", h_start_date, h_finish_date,h_effort);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 5255+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(5256) : cplan_fun.sql(15852) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEARCH_2;
ALTER FUNCTION FC_SEARCH_2 COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_ACTUAL_DAY ==
CREATE OR REPLACE FUNCTION FC_ACTUAL_DAY
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
   , sh_h_minutes PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bit_available RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_task_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_available PLS_INTEGER := 0;
   h_ta_sched_min PLS_INTEGER := 0;
   h_pa_sched_min PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_ta_total_day PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_actual_flag PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CAV1 IS
         SELECT BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND (TSK_ASSIGNMENT_ID = h_prj_assignment_id OR TSK_ASSIGNMENT_ID = h_tsk_assignment_id) 
         AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));

      CURSOR CNODATA1 IS
         SELECT * FROM (SELECT AVAILABLE_MIN, BIT_SCHED FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)) 
         ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACTUAL_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACTUAL_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_ACTUAL_DAY(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 51+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACTUAL_DAY(52) : cplan_fun.sql(15914) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_CHAR(tempStr, SC_H_DESCRIPTION);
   
   --trace_printf("h_tsk_assignment_id = %s, h_dayofyear = %s, h_minutes = %d,h_bitflag: %d\n",h_tsk_assignment_id,h_dayofyear,h_minutes,h_bitflag);
   
   h_prj_assignment_id := NULL;
   
   BEGIN
      SELECT PROJECT_ID, TASK_ID, RESOURCE_ID, PROJECT_ASSIGNMENT_ID
      INTO h_project_id, h_task_id, h_res_id, h_prj_assignment_id
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 70+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(71) : cplan_fun.sql(15933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("a.1__ SC_STATE = %d, h_prj_assignment_id = %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   BEGIN
      SELECT SCHED_MIN, ACTUAL_MIN, ACTUAL_FLAG, BIT_SCHED, BIT_SCHED
      INTO h_ta_sched_min, h_actual_min, h_actual_flag, h_bitID, h_bitID1
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 87+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(88) : cplan_fun.sql(15950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("a.2__ SC_STATE = %d, h_ta_sched_min: %d\n", SC_STATE,h_ta_sched_min);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA AND (Coalesce(h_minutes, 0) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 95+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(96) : cplan_fun.sql(15958) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID1.h_value,tempStr);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If (Coalesce(h_actual_min, 0) = Coalesce(h_minutes, 0)) AND (Coalesce(h_actual_flag, 0) = 1) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 105+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(106) : cplan_fun.sql(15968) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      If (h_prj_assignment_id IS NOT NULL) Then
         If Coalesce(h_minutes, 0) = 0 Then
            h_temp_var := PMO_STPU.getNumberOfBits(h_bitID) * 5;
            If Coalesce(h_temp_var, 0) < Coalesce(h_ta_sched_min, 0) Then
               h_ta_sched_min := Coalesce(h_temp_var, 0);
            End If;
         End If;
         
         If Coalesce(h_ta_sched_min, 0) = 0 Then
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_pa_sched_min, h_bitID
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 131+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(132) : cplan_fun.sql(15994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SCHED_MIN, BIT_SCHED INTO h_pa_sched_min, h_bit_available 
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
               AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 148+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(149) : cplan_fun.sql(16011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bit_available);
               h_pa_sched_min := Coalesce(h_pa_sched_min, 0) + Coalesce(h_ta_sched_min, 0);
            End If;
         End If;
         
         --trace_printf("1.1__ SC_STATE = %d, h_pa_sched_min = %d\n", SC_STATE,h_pa_sched_min);
         --if ( SC_NODATA )
         --{
         -- BEGIN_SQL
         --  SELECT AVAILABLE_MIN,BIT_SCHED INTO HOST(h_available), HOST(h_bitID)
         --  FROM CALENDAR_DAY_OF_YEAR
         --  WHERE ELEMENT_ID=HOST(h_res_id) AND DAY_YEAR=DATE(HOST(h_dayofyear));
         -- END_SQL;
         
         -- //h_pa_sched_min = h_available;
         --}
         --else if ( (h_pa_sched_min > 0)  (h_pa_sched_min < h_minutes) )  // resource calendar
         --{
         -- BEGIN_SQL
         --  SELECT AVAILABLE_MIN, BIT_OR_DAY(BIT_SCHED,HOST(h_bitID)) INTO HOST(h_available), HOST(h_bitID)
         --  FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID=HOST(h_res_id) AND DAY_YEAR=DATE(HOST(h_dayofyear));
         -- END_SQL;
         --}
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA OR ((Coalesce(h_pa_sched_min, 0) > 0) AND (Coalesce(
               h_pa_sched_min, 0) < Coalesce(h_minutes, 0))) Then
            h_available := 0;
            h_bitID := PMO_CNST.SC_ZERO_BIT;
            
            OPEN CAV1;
            
            LOOP
               FETCH CAV1 INTO h_bit_available;
               IF CAV1%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               h_available := Coalesce(h_available, 0) + PMO_STPU.getNumberOfBits(h_bit_available) * 5;
               h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bit_available);
            END LOOP;
            
            BEGIN
               CLOSE CAV1 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 209+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(210) : cplan_fun.sql(16072) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         h_bitID2 := h_bitID;
      ElsIf Coalesce(h_ta_sched_min, 0) < Coalesce(h_minutes, 0) Then
         --removed the subtraction of ASSIGNED_MIN from AVAILABLE_MIN and the whole case statement
         BEGIN
            SELECT BIT_SCHED, AVAILABLE_MIN
            INTO h_bitID, h_available
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 228+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(229) : cplan_fun.sql(16091) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --orDays( (unsigned char *)h_bitID.h_value, (unsigned char *)h_bitID1.h_value, (unsigned char *)h_bitID.h_value);
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_ta_sched_min := Coalesce(h_available, 0);
         End If;
      Else
         BEGIN
            SELECT AVAILABLE_MIN INTO h_available
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 248+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(249) : cplan_fun.sql(16111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_ta_sched_min := Coalesce(h_available, 0);
         End If;
      End If;
      
      --trace_printf("1.a__ h_available = %d, h_ta_sched_min = %d, h_pa_sched_min = %d\n", h_available,h_ta_sched_min,h_pa_sched_min);
   ElsIf (h_prj_assignment_id IS NOT NULL) Then
      BEGIN
         SELECT SCHED_MIN, BIT_SCHED INTO h_pa_sched_min, h_bitID
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 269+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(270) : cplan_fun.sql(16132) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         --removed the case statement where assigned_min bit_assigned were used
         BEGIN
            SELECT AVAILABLE_MIN, BIT_SCHED
            INTO h_available, h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 286+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(287) : cplan_fun.sql(16149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available, h_bitID
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 302+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(303) : cplan_fun.sql(16165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   
   --trace_printf("1.2__ SC_STATE = %d, h_available = %d, tempStr: %s\n", SC_STATE,h_available,tempStr);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      If (h_prj_assignment_id IS NULL) Then
         h_duration := 0;
         h_temp_sc_state := EXTEND_PRCAL(h_res_id, h_dayofyear, h_dayofyear, h_duration
                                , PMO_CNST.SC_RESOURCE);
         
         OPEN CNODATA1;
         
         FETCH CNODATA1 INTO h_available, h_bitID;
         IF CNODATA1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CNODATA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 336+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(337) : cplan_fun.sql(16199) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_available, 0) = 0 Then
         BEGIN
            SELECT  (PMO_CNST.SC_BIT_SCHED8H)  INTO  h_bitID FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 351+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(352) : cplan_fun.sql(16214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --strcpy(tempStr,"");
   --printTimes((unsigned char *)h_bitID.h_value,tempStr);
   --trace_printf("2__ SC_STATE: %d, h_ta_sched_min: %d, h_pa_sched_min: %d, h_available: %d, tempStr = %s\n", SC_STATE,h_ta_sched_min,h_pa_sched_min,h_available,tempStr);//trace_flush();
   If Coalesce(h_minutes, 0) = 0 Then
      If (Coalesce(h_ta_sched_min, 0) > 0) OR ((Coalesce(h_pa_sched_min, 0) > 0)) Then
         If h_bitflag = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = 0, ACTUAL_FLAG = 0
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 377+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(378) : cplan_fun.sql(16240) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("%1__ SC_STATE = %d, h_dayofyear = %s\n",SC_STATE,h_dayofyear);
         Else
            If (h_prj_assignment_id IS NOT NULL) Then
               If Coalesce(h_pa_sched_min, 0) = 0 Then
                  If Coalesce(h_available, 0) = 0 Then
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 398+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(399) : cplan_fun.sql(16261) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id, SCHED_MIN = h_ta_sched_min, ACTUAL_MIN = 0, ACTUAL_FLAG = 0, TASK_ID = h_project_id
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 416+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(417) : cplan_fun.sql(16279) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("3.1__ SC_STATE = %d, h_dayofyear = %s, h_pa_sched_min = %d, h_tsk_assignment_id = %s\n",SC_STATE,h_dayofyear,h_pa_sched_min,h_tsk_assignment_id);
               ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 435+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(436) : cplan_fun.sql(16298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("4__ SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_ta_sched_min, ACTUAL_MIN = 0, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 455+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(456) : cplan_fun.sql(16318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     --trace_printf("5__ SC_STATE = %d, h_dayofyear = %s,h_ta_sched_min = %d\n",SC_STATE,h_dayofyear,h_ta_sched_min);
                  End If;
               End If;
            Else
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 475+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(476) : cplan_fun.sql(16338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   ElsIf Coalesce(h_ta_sched_min, 0) = Coalesce(h_minutes, 0) Then
      If h_bitflag = 1 Then
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 497+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(498) : cplan_fun.sql(16360) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_minutes, ACTUAL_MIN = h_minutes, BIT_SCHED = h_bitID, ACTUAL_FLAG = 1, TASK_ID = h_task_id
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 515+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(516) : cplan_fun.sql(16378) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      If Coalesce(h_ta_sched_min, 0) = 0 Then
         If Coalesce(h_pa_sched_min, 0) > Coalesce(h_minutes, 0) Then
            h_bit_minutes := Coalesce(h_pa_sched_min, 0) - Coalesce(h_minutes, 0);
         ElsIf Coalesce(h_available, 0) > 0 Then
            h_bit_minutes := Coalesce(h_available, 0) - Coalesce(h_minutes, 0);
         End If;
         
         PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
      Else
         h_bit_minutes := Coalesce(h_ta_sched_min, 0) - Coalesce(h_minutes, 0);
         If Coalesce(h_bit_minutes, 0) > 0 Then
            PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
            PMO_STPU.clearStartDayStruct(Coalesce(h_minutes, 0), h_bitID1);
            h_bitID2 := PMO_STPU.BIT_ANDNOT_DAY(h_bitID2, h_bitID);
         End If;
         --trace_printf("h_minutes = %d, h_bit_minutes = %d, h_bitID1 = %s\n",h_minutes,h_bit_minutes,tempStr);
      End If;
      
      --strcpy(tempStr,"");
      --printTimes((unsigned char *)h_bitID.h_value,tempStr);
      --trace_printf("h_bit_minutes = %d, h_pa_sched_min: %d, h_ta_sched_min = %d, h_bitID = %s\n",h_bit_minutes,h_pa_sched_min,h_ta_sched_min,tempStr);
      --trace_printf("h_prj_assignment_id = %d, %s\n",h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      If Coalesce(h_ta_sched_min, 0) = 0 Then
         If Coalesce(h_pa_sched_min, 0) > 0 Then
            If Coalesce(h_pa_sched_min, 0) = Coalesce(h_minutes, 0) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_tsk_assignment_id, ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1, TASK_ID = h_task_id
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 560+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(561) : cplan_fun.sql(16423) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, ACTUAL_MIN, BIT_SCHED, BIT_FLAG, ACTUAL_FLAG)
                  VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_prj_assignment_id, h_dayofyear, h_minutes, h_minutes, h_bitID, '2', 1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 578+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(579) : cplan_fun.sql(16441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_bit_minutes, 0) <= 0 Then
                  PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
               End If;
               --strcpy(tempStr,"");
               --printTimes((unsigned char *)h_bitID.h_value,tempStr);
               --trace_printf("--> h_minutes = %d, h_bitID = %s\n",h_minutes,tempStr);
               
               If Coalesce(h_pa_sched_min, 0) > Coalesce(h_minutes, 0) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = CASE WHEN SCHED_MIN >= h_minutes THEN SCHED_MIN - h_minutes ELSE 0 END, BIT_SCHED = PMO_STPU.BIT_ANDNOT_DAY(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 604+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(605) : cplan_fun.sql(16467) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 621+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(622) : cplan_fun.sql(16484) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         Else
            BEGIN
               SELECT SCHED_MIN INTO h_bit_minutes
               FROM TASK_DAY_OF_YEAR 
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 638+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(639) : cplan_fun.sql(16501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If h_bitflag = 1 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 657+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(658) : cplan_fun.sql(16520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_minutes, BIT_SCHED = h_bitID, ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1, TASK_ID = h_task_id
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 675+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(676) : cplan_fun.sql(16538) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, ACTUAL_MIN, BIT_SCHED, BIT_FLAG, ACTUAL_FLAG)
                  VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_tsk_assignment_id, h_dayofyear, h_minutes, h_minutes, h_bitID, '2', 1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 694+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(695) : cplan_fun.sql(16557) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("INSERT: SC_STATE = %d, h_dayofyear: %s, h_minutes: %d\n", SC_STATE,h_dayofyear,h_minutes);
         End If;
      Else
         If Coalesce(h_bit_minutes, 0) < 0 Then
            PMO_STPU.clearFinishDayStruct(Coalesce(h_minutes, 0), h_bitID);
         End If;
         
         --strcpy(tempStr,"");
         --printTimes((unsigned char *)h_bitID.h_value,tempStr);
         --trace_printf("h_minutes = %d, h_bit_minutes = %d, h_bitID = %s\n",h_minutes,h_bit_minutes,tempStr);
         
         If h_bitflag = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 724+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(725) : cplan_fun.sql(16587) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_minutes, BIT_SCHED = h_bitID, ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1, TASK_ID = h_task_id
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 742+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(743) : cplan_fun.sql(16605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("TA__ SC_STATE = %d, h_minutes = %d\n", SC_STATE,h_minutes);
            
            If (h_prj_assignment_id IS NOT NULL) Then
               If Coalesce(h_pa_sched_min, 0) > 0 Then
                  h_bit_minutes := Coalesce(h_pa_sched_min, 0) - Coalesce(h_minutes, 0);
                  
                  If Coalesce(h_bit_minutes, 0) > 0 Then
                     PMO_STPU.clearStartDayStruct(Coalesce(h_minutes, 0), h_bitID2);
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_bit_minutes, BIT_SCHED = h_bitID2
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 767+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(768) : cplan_fun.sql(16630) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 784+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(785) : cplan_fun.sql(16647) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("PA_2: SC_STATE = %d, h_pa_sched_min = %d, h_bit_minutes = %d\n", SC_STATE,h_pa_sched_min,h_bit_minutes);
               ElsIf Coalesce(h_bit_minutes, 0) > 0 Then
                  BEGIN
                     INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, ACTUAL_MIN, BIT_SCHED, BIT_FLAG, ACTUAL_FLAG)
                     VALUES(h_project_id, h_project_id, h_res_id, h_prj_assignment_id, h_prj_assignment_id, h_dayofyear, h_bit_minutes, 0, h_bitID1, '2', 0);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 804+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(805) : cplan_fun.sql(16667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("PA_3: SC_STATE = %d, h_bit_minutes = %d\n", SC_STATE,h_bit_minutes);
               End If;
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If Coalesce(h_minutes, 0) > Coalesce(h_available, 0) Then
                  h_minutes := Coalesce(h_available, 0);
               End If;
               
               h_bit_minutes := Coalesce(h_minutes, 0) - Coalesce(h_ta_sched_min, 0);
            End If;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 822+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(823) : cplan_fun.sql(16685) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACTUAL_DAY;
ALTER FUNCTION FC_ACTUAL_DAY COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_FIX_FF ==
CREATE OR REPLACE FUNCTION FC_FIX_FF
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_finish_D IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_finish_time VARCHAR2(8);
   h_dayofyear DATE;
   h_SD DATE;
   h_days PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_state VARCHAR2(1);
   h_actual_flag PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR CFF IS
         SELECT * FROM (SELECT PARENT_ID, TASK_ID, DAY_YEAR, RESOURCE_ID, ACTUAL_FLAG, SCHED_MIN, ACTUAL_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id --AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR DESC ) WHERE ROWNUM<=1;

      CURSOR C_FF IS
         SELECT DAY_YEAR, SCHED_MIN 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10)) AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_FF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_FF'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_FF(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_finish_D || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 46+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_FF(47) : cplan_fun.sql(16742) : ' || SYSTIMESTAMP);
   END IF; 

   --get the bit_flag to determine the state of the project in order to update the correct info
   BEGIN
      SELECT RESOURCE_ID, BIT_FLAG INTO h_res_id, h_state
      FROM TASK_ASSIGNMENTS
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 60+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(61) : cplan_fun.sql(16756) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CFF;
   
   FETCH CFF INTO h_id, h_task_id, h_dayofyear, h_res_id, h_actual_flag, h_temp_var, h_minute, h_bitID;
   IF CFF%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_D) - PMO_ORASTD.DAYS(h_dayofyear)
         INTO h_days
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(86) : cplan_fun.sql(16781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_count := PMO_ORASTD.strcmp(h_id, h_tsk_assignment_id);
      -- project assignment ?
      
      If Coalesce(h_days, 0) > 0 Then
         If Coalesce(h_count, 0) = 0 Then
            If Coalesce(h_actual_flag, 0) = 0 Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 110+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(111) : cplan_fun.sql(16806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_temp_var, 0) > 0) AND (Coalesce(h_temp_var, 0) > Coalesce(h_minute, 0)) Then
               h_prj_assign_id := NULL;
               h_temp_sc_state := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                        , h_tsk_assignment_id, h_dayofyear, h_minute, 0);
               
               h_temp_var := Coalesce(h_temp_var, 0) - Coalesce(h_minute, 0);
               PMO_STPU.clearStartDayStruct(Coalesce(h_minute, 0), h_bitID);
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
                  VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_id, h_finish_D, h_temp_var, h_bitID, '2');
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 134+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(135) : cplan_fun.sql(16830) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 140+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_FF(141) : cplan_fun.sql(16836) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN -1;
            End If;
         Else
            If Coalesce(h_actual_flag, 0) = 1 Then
               h_prj_assign_id := NULL;
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_dayofyear, h_minute, 0);
            End If;
            
            --put back the effort to project assignment
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_temp_var, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 165+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(166) : cplan_fun.sql(16861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               --assign task assignmnet on that day
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_finish_D;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 185+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(186) : cplan_fun.sql(16881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --put back the effort to project assignment
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_id, TASK_ID = PROJECT_ID
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 204+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(205) : cplan_fun.sql(16900) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               --move task assignmnet to FF
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 223+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(224) : cplan_fun.sql(16919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --remove the assighned effort from project assignmnet of that day
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN - h_temp_var, BIT_SCHED = PMO_STPU.BIT_ANDNOT_DAY(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_finish_D;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 242+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(243) : cplan_fun.sql(16938) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         --if (checkFlagStatus(h_state, '2'))  //scheduled
         --{
         -- BEGIN_SQL
         --  UPDATE TASK_ASSIGNMENTS SET
         --  SCHED_FINISH = TIMESTAMP(DATE(HOST(h_finish_D)),TIME(SCHED_FINISH))
         --  WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
         -- END_SQL;
         --}
         --else
         --{
         -- BEGIN_SQL             //planned
         --  UPDATE TASK_ASSIGNMENTS SET
         --  PLANNED_FINISH = TIMESTAMP(DATE(HOST(h_finish_D)),TIME(PLANNED_FINISH))
         --  WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
         -- END_SQL;
         --}
      ElsIf Coalesce(h_actual_flag, 0) = 0 Then
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 280+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(281) : cplan_fun.sql(16976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_effort := 0;
         
         OPEN C_FF;
         
         LOOP
            FETCH C_FF INTO h_dayofyear, h_minute;
            IF C_FF%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               EXIT;
            End If;
            
            h_effort := Coalesce(h_effort, 0) + Coalesce(h_minute, 0);
            
            h_temp_var := 0;
            h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE C_FF ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 315+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(316) : cplan_fun.sql(17011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --BEGIN_SQL
         -- SELECT LAST_AVAIL_TIME(BIT_SCHED) INTO HOST(h_finish_time)
         -- FROM TASK_DAY_OF_YEAR
         -- WHERE PROJECT_ID = HOST(h_project_id)
         --  AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
         --  AND DAY_YEAR = HOST(h_finish_D);
         --END_SQL;
         
         --if (SC_NODATA)
         --{
         -- InitializeHost(SC_PARAM(h_finish_time), "17:00");
         --}
         
         --if (checkFlagStatus(h_state, '2'))  //scheduled
         --{
         -- BEGIN_SQL
         --  UPDATE TASK_ASSIGNMENTS SET
         --  SCHED_WORK = SCHED_WORK - HOST(h_effort), ETC = SCHED_WORK - HOST(h_effort) - WORK_TODATE,
         --  SCHED_FINISH = TIMESTAMP(DATE(HOST(h_finish_D)),TIME(HOST(h_finish_time)))
         --  WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
         -- END_SQL;
         --}
         --else
         --{
         -- BEGIN_SQL             //planned
         --  UPDATE TASK_ASSIGNMENTS SET
         --  PLANNED_WORK = PLANNED_WORK - HOST(h_effort), ETC = PLANNED_WORK - HOST(h_effort) - WORK_TODATE,
         --  PLANNED_FINISH = TIMESTAMP(DATE(HOST(h_finish_D)),TIME(HOST(h_finish_time)))
         --  WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
         -- END_SQL;
         --}
      End If;
   End If;
   
   BEGIN
      CLOSE CFF ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 362+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(363) : cplan_fun.sql(17058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 368+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_FF(369) : cplan_fun.sql(17064) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_FF;
ALTER FUNCTION FC_FIX_FF COMPILE DEBUG;

/***************************************************************************************/

-- Delete with Actual,

-- deletes schedule (ACTUAL_FLAG = 0) from TASK_DAY_OF_YEAR

-- where DAY_YEAR < last submitted (h_start_D)

-- returns:

-- 0 when the last scheduled day in TASK_DAY_OF_YEAR > then last submitted actual (don't skip)

-- 1 when the last scheduled day in TASK_DAY_OF_YEAR = then last submitted actual (skip first day)

/***************************************************************************************/

Prompt == FUNCTION FC_DEL_WITHACTUAL ==
CREATE OR REPLACE FUNCTION FC_DEL_WITHACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_sched_work PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_work PLS_INTEGER := sh_h_work;
   h_type PLS_INTEGER := sh_h_type;
   h_dayofyear DATE;
   h_minutes PLS_INTEGER := 0;
   h_work1 PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CDWA0 IS
         SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id
         AND ACTUAL_FLAG = 0 AND DAY_YEAR < TO_DATE(SubStr(h_start_date, 1, 10));

      CURSOR CDWA2 IS
         SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR DESC;

      CURSOR CDWA1 IS
         SELECT * FROM (SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id
         AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ORDER BY DAY_YEAR DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DEL_WITHACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DEL_WITHACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_DEL_WITHACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 52+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DEL_WITHACTUAL(53) : cplan_fun.sql(17141) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("0__* h_type: %d, h_tsk_assignment_id = %s, h_start_date = %s\n",h_type,h_tsk_assignment_id,h_start_date);
   
   BEGIN
      SELECT UNITS_MSP INTO h_percent_units
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID=h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 67+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(68) : cplan_fun.sql(17156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("3.2.0__ SC_STATE = %d, h_percent_units = %d, h_prj_assignment_id: %d %s\n",SC_STATE,h_percent_units, h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   OPEN CDWA0;
   
   LOOP
      FETCH CDWA0 INTO h_dayofyear;
      IF CDWA0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      h_temp_var := 0;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
   END LOOP;
   
   BEGIN
      CLOSE CDWA0 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 100+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(101) : cplan_fun.sql(17189) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If ((h_prj_assignment_id IS NOT NULL)) AND (Coalesce(h_type, 0) = -1) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 107+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_WITHACTUAL(108) : cplan_fun.sql(17196) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   BEGIN
      SELECT COALESCE(SUM(SCHED_MIN),0) - h_sched_work INTO h_work1
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 122+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(123) : cplan_fun.sql(17211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("1__ h_sched_work = %d, h_work1 = %d\n",h_sched_work,h_work1);
   If Coalesce(h_work1, 0) > 0 Then
      OPEN CDWA2;
      
      LOOP
         FETCH CDWA2 INTO h_dayofyear, h_minutes;
         IF CDWA2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_temp_var := 0;
         h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         h_work1 := Coalesce(h_work1, 0) - Coalesce(h_minutes, 0);
         If Coalesce(h_work1, 0) <= 0 Then
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CDWA2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 158+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(159) : cplan_fun.sql(17247) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("2__ h_work1 = %d,h_start_date = %s\n",h_work1,h_start_date);
   
   -- fix the last scheduled day
   --if ( h_work1 < 0 )
   If Coalesce(h_work1, 0) <= 0 Then
      OPEN CDWA1;
      
      FETCH CDWA1 INTO h_dayofyear, h_minutes;
      IF CDWA1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      h_temp_var := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CDWA1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(189) : cplan_fun.sql(17277) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("3__ h_work1 = %d, h_work = %d, h_minutes = %d, h_dayofyear = %s\nh_prj_assignment_id = %s, h_prj_assignment_id_Ind = %d\n",h_work1,h_work,h_minutes,h_dayofyear,h_prj_assignment_id,h_prj_assignment_id_Ind);
      If Coalesce(h_temp_var, 0) = 0 Then
         h_start_date := h_dayofyear;
         If Coalesce(h_work, 0) > 0 Then
            If Coalesce(h_work, 0) > Coalesce(h_minutes, 0) Then
               h_temp_var := 0;
               h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
               PMO_DB2M.SC_STATE := 0;
            Else
               PMO_DB2M.SC_STATE := 1;
            End If;
         Else
            If (h_prj_assignment_id IS NOT NULL) Then
               If Coalesce(h_type, 0) = 0 Then
                  BEGIN
                     SELECT SCHED_MIN INTO h_work1 FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 216+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(217) : cplan_fun.sql(17305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("3.1__ SC_STATE = %d, h_work1 = %d\n",SC_STATE,h_work1);
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_work1, 0) > 0) Then
                     h_temp_var := 0;
                     h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
                     PMO_DB2M.SC_STATE := 0;
                  Else
                     PMO_DB2M.SC_STATE := 1;
                  End If;
               Else
                  BEGIN
                     SELECT AVAILABLE_MIN INTO h_work1 FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 239+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(240) : cplan_fun.sql(17328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("3.2__ SC_STATE = %d, h_work1 = %d\n",SC_STATE,h_work1);
                  
                  If (Coalesce(h_percent_units, 0) <> 10000) AND (Coalesce(h_percent_units, 0) <> 0) Then
                     h_work1 := Trunc(((Coalesce(h_work1, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.49);
                  End If;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_work1, 0) <> Coalesce(
                        h_minutes, 0)) Then
                     h_temp_var := 0;
                     
                     h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
                     PMO_DB2M.SC_STATE := 0;
                  Else
                     PMO_DB2M.SC_STATE := 1;
                  End If;
               End If;
            End If;
            
            --trace_printf("4__ SC_STATE = %d, h_work1 = %d, h_minutes = %d\n",SC_STATE,h_work1,h_minutes);
         End If;
      Else
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_start_date FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 273+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(274) : cplan_fun.sql(17362) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         PMO_DB2M.SC_STATE := 1;
      End If;
   ElsIf Coalesce(h_work1, 0) > 0 Then
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_start_date FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(291) : cplan_fun.sql(17379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := 1;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 299+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_WITHACTUAL(300) : cplan_fun.sql(17388) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DEL_WITHACTUAL;
ALTER FUNCTION FC_DEL_WITHACTUAL COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_RESCHEDULE ==
CREATE OR REPLACE FUNCTION FC_RESCHEDULE
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_delay PLS_INTEGER := 0
   , sh_h_units PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_type PLS_INTEGER := sh_h_type;
   h_contour PLS_INTEGER := sh_h_contour;
   h_delay PLS_INTEGER := Coalesce(sh_h_delay, 0);
   h_units PLS_INTEGER := Coalesce(sh_h_units, 0);
   h_prj_assignment_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_active VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_last_submitted DATE;
   h_every PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_bitflag1 PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_temp_res_cost NUMBER := 0;
   h_temp_type_id PLS_INTEGER := 0;
   h_temp_cri PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_temp_occurrence PLS_INTEGER := 0;
   h_temp_priority PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_update PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_dow PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_RESCHEDULE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_RESCHEDULE'), '-- ' || SYSTIMESTAMP, ':r := FC_RESCHEDULE(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_units), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 72+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_RESCHEDULE(73) : cplan_fun.sql(17471) : ' || SYSTIMESTAMP);
   END IF; 

   h_finish_date := '';
   h_active := '';
   h_every := 0;
   h_temp_occurrence := 0;
   
   BEGIN
      SELECT PROJECT_ID, PROJECT_ASSIGNMENT_ID, TASK_ID, RESOURCE_ID, RESOURCE_TYPE,
      C_EFFORT, T_UNIT, C_UNIT, Trunc(BIT_FLAG), LAST_SUBMITTED, CONTOUR_TYPE
      INTO h_project_id, h_prj_assignment_id, h_task_id, h_res_id, h_res_type,
      h_work, h_eunit, h_unit, h_work_type, h_last_submitted, h_bitflag
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 92+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(93) : cplan_fun.sql(17491) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_unit: %d, h_project_id = %d %s, h_prj_assignment_id: %d %s\n", SC_STATE,h_unit,h_project_id_Ind,h_project_id,h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 99+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(100) : cplan_fun.sql(17498) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480), TYPE_ID INTO h_per_day,h_type_id
      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 113+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(114) : cplan_fun.sql(17512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE: %d, h_work = %d, h_per_day = %d, h_type_id: %d\n", SC_STATE,h_work,h_per_day,h_type_id);//trace_flush();
   If SubStr(h_res_type, 1, 1) = '0' Then
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
         h_consider := 1;
         -- Personal Task
      Else
         h_consider := 7;
         -- none
      End If;
   Else
      h_consider := PMO_CNST.SC_SRCH_PROFILE;
      -- none
   End If;
   
   If Coalesce(h_work, 0) = 0 OR Coalesce(h_contour, 0) = 0 Then
      h_bitflag := 0;
   ElsIf Coalesce(h_work, 0) = 0 AND Coalesce(h_unit, 0) = 100 Then
      h_unit := 0;
   ElsIf Coalesce(h_bitflag, 0) >= 90 Then
      If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) <> 0 Then
         h_bitflag := 2;
      Else
         h_bitflag := 1;
      End If;
   End If;
   
   h_eunit := 0;
   -- when reschedule effort is always in minutes
   h_temp_sc_state := FC_CALC_DURATION(h_project_id, h_start_DT, h_finish_DT, h_effort, h_work, h_eunit
                                    , h_unit, h_consider, h_bitflag, h_contour, h_temp_occurrence
                                    , h_per_day, h_per_week, h_work, h_effort, h_duration);
   
   If Coalesce(h_temp_sc_state, 0) <> 0 OR Coalesce(h_duration, 0) = 0 Then
      h_duration := 1200;
   End If;
   If h_units < 104 Then
      h_units := 104;
   End If;
   
   --trace_printf("h_duration = %d, h_start_date = %s, h_finish_date = %s\n",h_duration,h_start_date,h_finish_date);//trace_flush();
   
   -- Check for project calendar when no finish date
   h_start_DT := h_start_date;
   
   If SubStr(h_res_type, 1, 1) = '1' Then
      If FC_CREATE_CDY(h_project_id, h_start_date, h_finish_date, h_duration) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 165+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(166) : cplan_fun.sql(17564) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
      
      h_start_date := h_start_DT;
   ElsIf (Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30) AND (PMO_ORASTD.strncmp(h_start_date
                , h_start_DT, 10) <> 0) Then
      h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
      -- don't skip
   End If;
   --trace_printf("h_contour: %d, h_start_DT: %s\n",h_contour,h_start_DT);//trace_flush();
   --trace_printf("h_type = %d, h_duration = %d, h_start_date = %s, h_finish_date = %s, h_effort = %d\n",h_type,h_duration,h_start_date,h_finish_date,h_effort);//trace_flush();
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   If Coalesce(h_work, 0) = 0 AND h_units > 0 Then
      h_work := h_units;
      h_unit := 100;
   ElsIf Coalesce(h_bitflag, 0) = 0 Then
      h_work := 0;
      h_unit := 0;
   End If;
   
   --trace_printf("*** h_units = %d, h_work = %d, h_unit: %d\n", h_units,h_work,h_unit);//trace_flush();
   
   If (h_prj_assignment_id IS NOT NULL) Then
      --1__*
      -- get start sched work
      BEGIN
         SELECT COALESCE(SUM(SCHED_MIN),0), MIN(DAY_YEAR), MAX(DAY_YEAR) INTO h_sched_work, h_start_DT, h_finish_DT
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 203+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(204) : cplan_fun.sql(17602) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("*** SC_STATE = %d, h_sched_work = %d, h_start_DT = %d %s, h_finish_DT = %s\n",SC_STATE, h_sched_work, h_start_DT_Ind, h_start_DT, h_finish_DT);//trace_flush();
      
      If (Coalesce(h_type, 0) = 2) AND (Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 211+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(212) : cplan_fun.sql(17610) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_ASSIGN_FROM_PROJECT_AFF;
      End If;
      
      If (Coalesce(h_sched_work, 0) = 0) OR ((h_start_DT IS NULL)) Then
         --h_bitflag1 = (h_bitflag | SC_BIT_27);     // don't update TASK_ASSIGNMENTS
         If Coalesce(h_type, 0) = 1 Then
            h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
            -- update TASK_ASSIGNMENTS
            h_update := 1;
         Else
            h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
            -- don't update TASK_ASSIGNMENTS
            h_update := 0;
         End If;
         --trace_printf("*** all from Res CAL (PA: remaining = 0):h_bitflag1 = %d, h_consider = %d, h_update = %d\n", h_bitflag1,h_consider,h_update);//trace_flush();
         h_temp_res_cost := 0.00;
         h_temp_type_id := 3;
         h_temp_cri := 0;
         h_temp_priority := 0;
         
         h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_date, h_finish_date, h_duration, h_effort, h_work
                               , h_eunit, h_unit, h_temp_cri, h_consider, h_res_type, h_work_type
                               , h_bitflag1, h_contour, h_temp_occurrence, h_update, h_per_day
                               , h_temp_priority, h_rec_user, Coalesce(h_per_day, 0), h_delay, 0.00
                               , 0.00);
         --trace_printf("*** all from Res CAL (PA: remaining = 0): h_temp_sc_state = %d, h_start_date = %s, h_finish_date = %s, h_effort = %d\n",h_temp_sc_state,h_start_date,h_finish_date,h_effort);//trace_flush();
      ElsIf Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
         --2__**
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_29_30));
         
         h_temp_res_cost := 0.00;
         h_temp_type_id := 3;
         h_temp_cri := 0;
         h_temp_consider := 10;
         h_temp_update := 0;
         h_temp_priority := 0;
         
         -- Assign from PA
         h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_date, h_finish_date, h_duration, h_sched_work
                               , h_work, h_eunit, h_unit, h_temp_cri, h_temp_consider, h_res_type
                               , h_work_type, h_bitflag1, h_contour, h_temp_occurrence
                               , h_temp_update, h_per_day, h_temp_priority, h_rec_user, Coalesce(
                               h_per_day, 0), h_delay, 0.00, 0.00);
         --trace_printf("*** from PA and Res CAL (h_effort > h_sched_work)\n1__sched_work from PA: h_temp_sc_state = %d, h_start_date = %s, h_finish_date = %s, h_sched_work = %d\n",h_temp_sc_state,h_start_date,h_finish_date,h_sched_work);//trace_flush();
         
         If Coalesce(h_temp_sc_state, 0) = 0 Then
            --Schedule the h_remaining for task assignment
            h_remaining := Coalesce(h_effort, 0) - Coalesce(h_sched_work, 0);
            
            -- get start sched work
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_start_DT FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 276+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(277) : cplan_fun.sql(17675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("*** SC_STATE = %d, h_start_date = %s, h_start_DTt = %s\n",SC_STATE, h_start_date, h_start_DT);//trace_flush();
            
            h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
            -- don't update TASK_ASSIGNMENTS
            If Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30 Then
               h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
            End If;
            -- don't skip
            
            h_finish_DT := '';
            -- Task assignment for remaining
            
            h_temp_res_cost := 0.00;
            h_temp_type_id := 3;
            h_temp_cri := 0;
            h_temp_update := 0;
            h_temp_priority := 0;
            
            h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_DT, h_finish_DT, h_duration, h_remaining, h_work
                               , h_eunit, h_unit, h_temp_cri, h_consider, h_res_type, h_work_type
                               , h_bitflag1, h_contour, h_temp_occurrence, h_temp_update, h_per_day
                               , h_temp_priority, h_rec_user, Coalesce(h_per_day, 0), h_delay, 0.00
                               , 0.00);
            
            --trace_printf("2__ h_remaining from Res Cal: h_temp_sc_state = %d, h_start_DT = %s, h_finish_DT = %s, h_remaining = %d\n",h_temp_sc_state,h_start_DT,h_finish_DT,h_remaining);//trace_flush();
         End If;
      Else
         --h_bitflag1 = (h_bitflag | SC_BIT_27);     // don't update TASK_ASSIGNMENTS
         If Coalesce(h_type, 0) = 1 Then
            h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
            -- don't update TASK_ASSIGNMENTS
         Else
            h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
            -- update TASK_ASSIGNMENTS
         End If;
         
         If (Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30) AND (PMO_ORASTD.strncmp(h_start_date, h_start_DT
               , 10) <> 0) Then
            h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
            -- don't skip
         End If;
         
         -- assign from PA
         
         h_temp_res_cost := 0.00;
         h_temp_type_id := 3;
         h_temp_cri := 0;
         h_temp_consider := 10;
         h_temp_priority := 0;
         
         If Coalesce(h_work_type, 0) = 2 Then
            h_update := 0;
         Else
            h_update := 3;
         End If;
         --trace_printf("h_effort_2: h_start_date = %s, h_finish_date = %s, h_per_day: %d, h_delay: %d\n",h_start_date,h_finish_date,h_per_day,h_delay);//trace_flush();
         
         h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_DT, h_finish_DT, h_duration, h_effort, h_work
                               , h_eunit, h_unit, h_temp_cri, h_temp_consider, h_res_type
                               , h_work_type, h_bitflag1, h_contour, h_temp_occurrence, h_update
                               , h_per_day, h_temp_priority, h_rec_user, Coalesce(h_per_day, 0)
                               , h_delay, 0.00, 0.00);
         --trace_printf("*** all from PA: h_temp_sc_state = %d, h_start_DT = %s, h_finish_DT = %s, h_effort = %d\n",h_temp_sc_state,h_start_DT,h_finish_DT,h_effort);//trace_flush();
      End If;
   Else
      --h_bitflag1 = (h_bitflag | SC_BIT_27);     // don't update TASK_ASSIGNMENTS
      If Coalesce(h_type, 0) = 1 Then
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
         -- update TASK_ASSIGNMENTS
         h_update := 1;
      Else
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
         -- don't update TASK_ASSIGNMENTS
         h_update := 0;
      End If;
      
      --trace_printf("h_effort_2: h_contour: %d, h_start_date = %s, h_finish_date = %s, h_effort = %d\n",h_contour,h_start_date,h_finish_date,h_effort);//trace_flush();
      
      h_temp_res_cost := 0.00;
      h_temp_type_id := 3;
      h_temp_cri := 0;
      h_temp_priority := 0;
      
      h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_date, h_finish_date, h_duration, h_effort, h_work
                               , h_eunit, h_unit, h_temp_cri, h_consider, h_res_type, h_work_type
                               , h_bitflag1, h_contour, h_temp_occurrence, h_update, h_per_day
                               , h_temp_priority, h_rec_user, Coalesce(h_per_day, 0), h_delay, 0.00
                               , 0.00);
      --trace_printf("*** all from Res CAL (PA_Ind = -1): h_temp_sc_state = %d, h_start_date = %s, h_finish_date = %s, h_effort = %d\n",h_temp_sc_state,h_start_date,h_finish_date,h_effort);//trace_flush();
   End If;
   
   If (Coalesce(h_temp_sc_state, 0) = 0) AND ((h_prj_assignment_id IS NOT NULL)) Then
      BEGIN
         SELECT WORK_TODATE INTO h_work FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 386+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(387) : cplan_fun.sql(17785) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_work: %d\n",SC_STATE,h_work);//trace_flush();
      
      -- CALL UPDATE assign string function
      If Coalesce(h_work, 0) = 0 Then
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
         -- fix ACTUAL_START  SCHED
      Else
         h_temp_var := 0;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
      End If;
   End If;
   
   If (Coalesce(h_temp_sc_state, 0) > 0) AND (Coalesce(h_temp_sc_state, 0) <> 100) AND (Coalesce(
             h_temp_sc_state, 0) < 200000) Then
      h_temp_sc_state := PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 411+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(412) : cplan_fun.sql(17810) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_RESCHEDULE;
ALTER FUNCTION FC_RESCHEDULE COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_ETC_SCHEDULE ==
CREATE OR REPLACE FUNCTION FC_ETC_SCHEDULE
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type PLS_INTEGER := sh_h_type;
   h_prj_assignment_id VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_active VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_week_of DATE;
   h_day_year DATE;
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_task_start_DT TIMESTAMP;
   h_task_finish_DT TIMESTAMP;
   h_bit_flag VARCHAR2(1);
   h_type_id PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_proposed_work PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_work_type PLS_INTEGER := 0;
   h_temp_contour_type PLS_INTEGER := 0;
   h_temp_occurrence PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_resched PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_AS TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_forecast_start TIMESTAMP;
   h_forecast_finish TIMESTAMP;
   h_FF_flag PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_accpt_aut PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_has_actual PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR CR_WOF IS
         SELECT * FROM (SELECT WEEK_OF FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF <= TO_DATE(SubStr(h_start_date, 1, 10))
         ORDER BY WEEK_OF DESC ) WHERE ROWNUM<=1;

      CURSOR CFD IS
         SELECT * FROM (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ETC_SCHEDULE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ETC_SCHEDULE'), '-- ' || SYSTIMESTAMP, ':r := FC_ETC_SCHEDULE(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 90+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ETC_SCHEDULE(91) : cplan_fun.sql(17911) : ' || SYSTIMESTAMP);
   END IF; 

   
   h_prj_assignment_id := '';
   
   h_start_DT := '';
   h_resched := 0;
   h_accpt_aut := 1;
   h_tph_delete := 0;
   
   --trace_printf("h_type = %d, h_tsk_assignment_id: %s\n", h_type,h_tsk_assignment_id); trace_flush();
   If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 2) Then
      BEGIN
         SELECT ra.ELEMENT_TYPE, ta.PROJECT_ID, ta.PROJECT_ASSIGNMENT_ID, ta.TSK_ASSIGNMENT_ID, ta.RES_ASSIGNMENT_ID, ta.TASK_ID,
         ta.RESOURCE_ID, ta.ACTUAL_START, ta.LAST_SUBMITTED, ta.PROPOSED_WORK, ta.SCHED_WORK, ta.WORK_TODATE, ta.ETC,
         ta.C_EFFORT, ta.T_UNIT, ta.C_UNIT, ta.C_CONTOUR, ta.UNITS_MSP, ta.LAST_SCHED_MIN, ta.CONTOUR_TYPE, 
         ta.UNRESOLVED_START, ta.UNRESOLVED_FINISH, ta.BILLABLE, ta.CAPITAL_EXPENSE, ta.REGULAR_COST, ta.REGULAR_RATE,
         ta.SCHED_START, ta.FORECAST_START, ta.FORECAST_FINISH, CASE WHEN ta.FORECAST_FINISH IS NOT NuLL AND ta.FORECAST_START IS NuLL AND ta.FORECAST_FINISH BETWEEN ta.SCHED_START AND ta.SCHED_FINISH THEN 1 ELSE 0 END,
         ta.RESOURCE_TYPE, ta.BIT_FLAG
         INTO h_type_id, h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_assign_id, h_task_id,
         h_res_id, h_AS, h_start_date, h_proposed_work, h_sched_work, h_work_todate, h_etc,
         h_work, h_eunit, h_unit, h_contour, h_percent_units, h_last_day, h_bitflag,
         h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale,
         h_sched_start, h_forecast_start, h_forecast_finish,
         h_FF_flag, h_res_type, h_bit_flag
         FROM TASK_ASSIGNMENTS ta JOIN RESOURCE_ASSIGNMENTS ra ON ra.RES_ASSIGNMENT_ID = ta.RES_ASSIGNMENT_ID
         WHERE ta.TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 124+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(125) : cplan_fun.sql(17945) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("0_SC_STATE = %d, h_type_id: %d, h_task_id: %s, h_etc: %d, h_proposed_work: %d\n", SC_STATE,h_type_id,h_task_id,h_etc,h_proposed_work); trace_flush();
      --trace_printf("h_prj_assignment_id: %d %s, h_contour: %d, h_work: %d, h_unit: %d, h_percent_units: %d\n", h_prj_assignment_id_Ind,h_prj_assignment_id,h_contour,h_work,h_unit,h_percent_units); trace_flush();
      --trace_printf("h_ta_start_DT: %s, h_ta_finish_DT: %s, h_start_date: %s\n", h_ta_start_DT,h_ta_finish_DT,h_start_date);
      
      h_SD := SubStr(h_ta_start_DT, 1, 10);
      h_FD := SubStr(h_ta_finish_DT, 1, 10);
      h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
      h_effort := Coalesce(h_sched_work, 0);
      --trace_printf("====> h_effort = %d, h_work_todate: %d, h_remaining: %d\n", h_effort,h_work_todate,h_remaining); trace_flush();
      If Coalesce(h_work_todate, 0) = 0 Then
         --this is for when we decrease etc without any actual, accept automatically
         --otherwise when we remove the actual go to else
         If Coalesce(h_etc, 0) > Coalesce(h_remaining, 0) Then
            If Coalesce(h_type, 0) = 2 Then
               If (h_prj_assignment_id IS NOT NULL) Then
                  If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_start_date
                        , h_etc) <> 0 Then
                     --trace_printf("1_PROJECT_ASSIGNMENT_REMAINING = %d\n", PROJECT_ASSIGNMENT_REMAINING); trace_flush();
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 147+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(148) : cplan_fun.sql(17968) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
                  End If;
               End If;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 153+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(154) : cplan_fun.sql(17974) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            Else
               h_effort := Coalesce(h_etc, 0);
            End If;
         ElsIf Coalesce(h_etc, 0) < Coalesce(h_remaining, 0) Then
            h_effort := Coalesce(h_etc, 0);
         Else
            h_effort := Coalesce(h_proposed_work, 0);
            If Coalesce(h_type, 0) = 0 Then
               h_etc := Coalesce(h_proposed_work, 0);
            End If;
         End If;
      Else
         If ((Coalesce(h_etc, 0) = 0) AND (Coalesce(h_remaining, 0) < 0)) OR ((Coalesce(
               h_work_todate, 0) > Coalesce(h_effort, 0)) AND (Coalesce(h_etc, 0) > 0)) Then
            If (h_prj_assignment_id IS NOT NULL) Then
               If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_start_date
                     , h_etc) <> 0 Then
                  --trace_printf("2_PROJECT_ASSIGNMENT_REMAINING = %d\n", PROJECT_ASSIGNMENT_REMAINING); trace_flush();
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 175+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(176) : cplan_fun.sql(17996) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
               End If;
               
               h_temp_var := -1;
            Else
               h_temp_var := 0;
            End If;
            
            h_effort := Coalesce(h_work_todate, 0);
            h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort, h_work
                            , h_prj_assignment_id, h_temp_var);
            
            h_temp_var := 1;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_var := 0;
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
            End If;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 198+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(199) : cplan_fun.sql(18019) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         ElsIf Coalesce(h_remaining, 0) = Coalesce(h_etc, 0) Then
            If Coalesce(h_etc, 0) + Coalesce(h_work_todate, 0) < Coalesce(h_proposed_work, 0) Then
               h_effort := Coalesce(h_remaining, 0) + Coalesce(h_work_todate, 0);
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET SCHED_WORK = h_effort, PROPOSED_WORK = h_effort
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 217+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(218) : cplan_fun.sql(18038) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_etc, 0) + Coalesce(h_work_todate, 0) > Coalesce(h_proposed_work, 0) Then
               If Coalesce(h_work_todate, 0) >= Coalesce(h_proposed_work, 0) Then
                  h_effort := Coalesce(h_work_todate, 0);
               Else
                  h_effort := Coalesce(h_proposed_work, 0);
               End If;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET SCHED_WORK = h_effort
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 241+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(242) : cplan_fun.sql(18062) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf Coalesce(h_etc, 0) < Coalesce(h_remaining, 0) Then
            h_effort := Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0);
            
            h_accpt_aut := 2;
            --flag for accept etc and Forecast in U_assign_string
         ElsIf (Coalesce(h_etc, 0) > Coalesce(h_remaining, 0)) AND (Coalesce(h_effort, 0) > Coalesce(
                                     h_work_todate, 0)) Then
            -- reschedule, don't accept etc.
            NULL;
         ElsIf (Coalesce(h_type, 0) = 2) OR (Coalesce(h_type, 0) = 0) OR (Coalesce(
                h_type_id, 0) = PMO_CNST.SC_PERSONAL_TASK) Then
            --trace_printf("h_res_id = %s, h_rec_user: %s\n", h_res_id,h_rec_user); trace_flush();
            If (Coalesce(h_type_id, 0) = PMO_CNST.SC_PERSONAL_TASK) OR (PMO_ORASTD.strcmp(h_res_id
                  , h_rec_user) <> 0) Then
               --if it's personal task or manager accept etc and forecast automatically
               --check if forecast change
               
               If ((h_forecast_finish IS NOT NULL)) OR ((h_forecast_start IS NOT NULL)) Then
                  If (h_forecast_start IS NULL) Then
                     h_forecast_start := h_sched_start;
                  End If;
                  
                  If ((h_forecast_finish IS NOT NULL)) AND (SubStr(h_res_type, 1, 1) = '0') Then
                     BEGIN
                        SELECT COALESCE(SUM(AVAILABLE_MIN),0) INTO h_effort FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = h_res_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_forecast_start, 1, 10)) AND TO_DATE(SubStr(h_forecast_finish, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 278+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(279) : cplan_fun.sql(18099) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  --trace_printf("Forecast change---> h_effort: %d\n", h_effort);
                  h_type := 3;
                  --                      h_etc    = h_remaining;
                  h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
                  
                  h_accpt_aut := 2;
                  --flag for accept etc and Forecast in U_assign_string
               Else
                  h_effort := Coalesce(h_work_todate, 0) + Coalesce(h_remaining, 0);
                  --                      h_etc    = h_remaining;
                  h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
                  h_type := 1;
               End If;
               --trace_printf("h_etc: %d, h_remaining: %d, h_sched_work: %d\n", h_etc, h_remaining, h_sched_work);
               
               h_temp_sc_state := FC_ACPT_ETC_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id
                                      , h_prj_assignment_id, h_forecast_start, h_forecast_finish
                                      , h_start_date, h_bit_flag, h_effort, h_remaining
                                      , h_sched_work, h_etc, h_contour, h_FF_flag, h_type
                                      , h_ta_start_DT, h_ta_finish_DT, h_reg_cost, h_reg_sale
                                      , h_billable, h_capital_expense, h_rec_user, 0);
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 306+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(307) : cplan_fun.sql(18127) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            ElsIf ((h_start_date IS NULL)) OR (PMO_ORASTD.strncmp(h_ta_start_DT, h_start_date, 10) = 0) Then
               If (h_start_date IS NULL) Then
                  h_temp_var := 0;
               Else
                  h_temp_var := 1;
               End If;
               
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
               
               If (h_prj_assignment_id IS NOT NULL) Then
                  h_temp_var := 0;
                  h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
               End If;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 324+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(325) : cplan_fun.sql(18145) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            End If;
         End If;
      End If;
      --trace_printf("h_effort = %d, h_remaining: %d, h_etc: %d, h_last_day: %d, h_start_date: %d %s\n", h_effort,h_remaining,h_etc,h_last_day,h_start_date_Ind,h_start_date); trace_flush();
      
      If (h_prj_assignment_id IS NOT NULL) Then
         --trace_printf("h_FD: %s, h_start_date: %s\n", h_FD,h_start_date);
         If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_start_date
               , h_etc) <> 0 Then
            --trace_printf("1_h_billable: %d, h_capital_expense =  %d, h_SD: %s, h_FD: %s\n",h_billable,h_capital_expense, h_SD, h_FD); trace_flush();
            If Coalesce(h_tph_delete, 0) = 0 Then
               h_tph_delete := -1;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            End If;
            
            h_temp_var := -1;
            h_temp_sc_state := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort
                                     , h_work, h_prj_assignment_id, h_temp_var);
            --trace_printf("2_h_billable: %d, h_capital_expense =  %d, h_SD: %s, h_FD: %s\n",h_billable,h_capital_expense, h_SD, h_FD); trace_flush();
            
            If Coalesce(h_tph_delete, 0) = -1 Then
               h_tph_delete := 0;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET BEST = -1 WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 368+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(369) : cplan_fun.sql(18189) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_temp_var := 1;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            h_temp_var := 1;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
            
            --trace_printf("*** ETC_SCHEDULE: Can't reschedule from Project assignments\n");//trace_flush();
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 381+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(382) : cplan_fun.sql(18202) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
         End If;
      End If;
   ElsIf Coalesce(h_type, 0) = 1 Then
      BEGIN
         SELECT PROJECT_ID, PROJECT_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, RESOURCE_ID, UNITS_MSP,
         CASE WHEN LAST_SUBMITTED IS NOT NuLL AND TO_DATE(SubStr(BAK_START_DT, 1, 10)) > TO_DATE(SubStr(LAST_SUBMITTED, 1, 10)) THEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) ELSE LAST_SUBMITTED END,
         ACTUAL_START, LAST_SUBMITTED, CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END, WORK_TODATE, ETC, C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR,
         UNRESOLVED_START, UNRESOLVED_FINISH, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE, LAST_SCHED_MIN, CONTOUR_TYPE
         INTO h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_assign_id, h_task_id, h_res_id, h_percent_units,
         h_start_DT, h_AS, h_start_date, h_sched_work, h_work_todate, h_etc, h_work, h_eunit, h_unit, h_contour,
         h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale, h_last_day, h_bitflag
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 402+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(403) : cplan_fun.sql(18223) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1_SC_STATE = %d, h_effort = %d, h_work_todate: %d, h_etc: %d\n", SC_STATE,h_effort,h_work_todate,h_etc);
      
      h_SD := SubStr(h_ta_start_DT, 1, 10);
      h_FD := SubStr(h_ta_finish_DT, 1, 10);
      h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
      h_effort := Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0);
      --trace_printf("h_remaining: %d, h_prj_assignment_id: %d:%s, h_start_date: %s\n",h_remaining,h_prj_assignment_id_Ind,h_prj_assignment_id,h_start_date); trace_flush();
      If (h_prj_assignment_id IS NOT NULL) Then
         If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_start_date
               , h_etc) <> 0 Then
            --trace_printf("3_PROJECT_ASSIGNMENT_REMAINING = %d\n", PROJECT_ASSIGNMENT_REMAINING); trace_flush();
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 418+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(419) : cplan_fun.sql(18239) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
         End If;
      End If;
   End If;
   
   --trace_printf("SC_STATE = %d, h_type = %d, h_tsk_assignment_id = %s, h_effort = %d, h_remaining = %d\nh_work_todate = %d, h_start_date = %s, h_start_date_Ind = %d\n", SC_STATE,h_type,h_tsk_assignment_id,h_effort,h_remaining,h_work_todate,h_start_date,h_start_date_Ind); trace_flush();
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR (PMO_ORASTD.strcmp(h_project_id
         , h_task_id) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 429+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(430) : cplan_fun.sql(18250) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   
   BEGIN
      SELECT TASK_TYPE, MINUTE_PER_DAY, UNRESOLVED_START, UNRESOLVED_FINISH, UNRESOLVED_DURATION, ASSIGN_COUNT
      INTO h_task_type, h_per_day, h_task_start_DT, h_task_finish_DT, h_duration, h_assign_count
      FROM TMT_WBS
      WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 445+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(446) : cplan_fun.sql(18266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_remaining > 0) )
   	{
   		if (h_type == 1)    // reschedule etc
   		{
   			BEGIN_SQL
   				SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   				WHERE PROJECT_ID = HOST(h_project_id)
   					AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   					AND ACTUAL_FLAG = 0;
   			END_SQL_NOCHECK;
   //trace_printf("ETC_*_FD: SC_STATE: %d, h_tsk_assignment_id: %s, h_duration: %d\n", SC_STATE,h_tsk_assignment_id,h_duration);
   
   			if ( h_duration > 0 )
   			{
   				InitializeHost(SC_PARAM(h_tph_delete), -1);
   				h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   						SC_PARAM(h_project_id),
   						SC_PARAM(h_task_id),
   						SC_PARAM(h_tsk_assignment_id),
   						SC_PARAM(h_SD),
   						SC_PARAM(h_FD),
   						SC_PARAM(h_reg_cost),
   						SC_PARAM(h_reg_sale),
   						SC_PARAM(h_tph_delete),
   						SC_PARAM(h_capital_expense),
   						SC_PARAM(h_billable));
   
   				h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.49);
   				h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   										  SC_PARAM(h_tsk_assignment_id),
   										  SC_PARAM(h_start_date),
   										  SC_PARAM(h_per_day),
   										  SC_PARAM(h_minutes));
   
   				InitializeHost(SC_PARAM(h_tph_delete), 0);
   				h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   						SC_PARAM(h_project_id),
   						SC_PARAM(h_task_id),
   						SC_PARAM(h_tsk_assignment_id),
   						SC_PARAM(h_SD),
   						SC_PARAM(h_FD),
   						SC_PARAM(h_reg_cost),
   						SC_PARAM(h_reg_sale),
   						SC_PARAM(h_tph_delete),
   						SC_PARAM(h_capital_expense),
   						SC_PARAM(h_billable));
   
   				h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
   				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   			}
   		}
   		else if ( h_work_todate > 0 )
   		{
   //trace_printf("h_AS: %s, h_SD: %s, h_duration: %d\n", h_AS,h_SD,h_duration);
   			Nullify(SC_PARAM(h_finish_date));
   			if ( strncmp(h_AS,h_SD,10) != 0)
   			{
   				BEGIN_SQL
   					DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_MIN_MAX (
   						MIN_DATE  DATE,
   						MAX_DATE  DATE,
   						MINUTE_PER_DAY SMALLINT,
   						START_DATE  TIMESTAMP,
   						FINISH_DATE  TIMESTAMP,
   						DURATION   INTEGER
   					)
   					WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
   				END_SQL_NOCHECK;
   
   				BEGIN_SQL
   					DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_CDY (
   						ELEMENT_ID  CHAR(32),
   						DAY_YEAR  DATE,
   						DAY_WEEK  SMALLINT,
   						AVAILABLE_MIN SMALLINT,
   						BIT_SCHED  CHAR(36) FOR BIT DATA,
   						//removed from here
   						WEEK_OF   SMALLINT,
   						MONTH_OF  SMALLINT,
   						YEAR_OF   SMALLINT
   					)
   					WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
   				END_SQL_NOCHECK;
   
   				h_temp_var = int (((double) h_duration / (double) h_per_day) + 0.5);
   				if (h_temp_var < 10)
   					h_temp_var = 20;
   				else
   					h_temp_var *= 2;
   
   				BEGIN_SQL
   					SELECT DATE(HOST(h_AS)) + HOST(h_temp_var) DAYS INTO HOST(h_finish_date) FROM DUAL ;
   				END_SQL;
   //trace_printf("h_duration: %d, h_AS: %d %s, h_finish_date: %d %s\n", h_duration,h_start_date_Ind,h_AS,h_finish_date_Ind,h_finish_date);
   
   				BEGIN_SQL
   					INSERT INTO session.PMO_TMP_CDY
   						SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
   						FROM CALENDAR_DAY_OF_YEAR
   						WHERE ELEMENT_ID = HOST(h_project_id) AND DAY_YEAR BETWEEN DATE(HOST(h_AS)) AND DATE(HOST(h_finish_date));
   				END_SQL;
   
   				BEGIN_SQL
   					INSERT INTO session.PMO_TMP_MIN_MAX
   					SELECT MIN(DAY_YEAR), MAX(DAY_YEAR), HOST(h_per_day), SC_1899_12_30_8_00_00,SC_1899_12_30_8_00_00, 0
   					FROM session.PMO_TMP_CDY;
   				END_SQL;
   
   				BEGIN_SQL
   					SELECT TIMESTAMP(DATE(HOST(h_AS)), TIME(FIRST_AVAIL_TIME(BIT_SCHED))) INTO HOST(h_AS)
   					FROM session.PMO_TMP_CDY WHERE DAY_YEAR = HOST(h_AS);
   				END_SQL;
   				InitializeHost(SC_PARAM(h_finish_date), "");
   
   //trace_printf("A_* h_duration: %d, h_AS: %d %s, h_finish_date: %d %s\n", h_duration,h_start_date_Ind,h_AS,h_finish_date_Ind,h_finish_date);
   				h_temp_var = GET_FD_DURATION(SC_PARAM(h_project_id), SC_PARAM(h_AS), SC_PARAM(h_finish_date), SC_PARAM(h_duration));
   //trace_printf("B_* h_temp_var: %d, h_AS: %d %s, h_finish_date: %d %s\n", h_temp_var,h_start_date_Ind,h_AS,h_finish_date_Ind,h_finish_date);
   
   				BEGIN_SQL
   					SELECT COUNT('X') INTO HOST(h_bitflag) FROM DUAL WHERE EXISTS (
   						SELECT 'X' FROM TASK_ASSIGNMENTS
   						WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
   							AND TSK_ASSIGNMENT_ID <> HOST(h_tsk_assignment_id) AND ACTUAL_START IS NOT NuLL);
   				END_SQL_NOCHECK;
   //trace_printf("SC_STATE: %d, h_finish_date: %d %s, h_bitflag: %d\n", SC_STATE,h_finish_date_Ind,h_finish_date,h_bitflag);
   
   				if (h_bitflag == 0)
   				{
   
   					h_temp_var = FC_CHANGE_TA_DATES (SC_PARAM(h_project_id),
   													 SC_PARAM(h_task_id),
   													 SC_PARAM(h_task_start_DT),
   													 SC_PARAM(h_task_finish_DT),
   													 SC_PARAM(h_AS),
   													 SC_PARAM(h_finish_date),
   													 SC_PARAM(h_per_day),
   													 SC_PARAM(h_rec_user), h_task_type );
   				}
   
   				BEGIN_SQL
   					SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   					WHERE PROJECT_ID = HOST(h_project_id)
   						AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   						AND ACTUAL_FLAG = 0;
   				END_SQL_NOCHECK;
   //trace_printf("0.*** h_duration: %d, h_project_id: %s, h_tsk_assignment_id: %s\n", h_duration,h_project_id,h_tsk_assignment_id);
   
   				if ( h_duration > 0 )
   				{
   					if (h_tph_delete == 0)
   					{
   						InitializeHost(SC_PARAM(h_tph_delete), -1);
   						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   					}
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.49);
   //trace_printf("1.*** h_minutes: %d, h_remaining: %d, h_duration: %d\n", h_minutes,h_remaining,h_duration);
   //trace_printf("2.*** h_project_id: %s, h_SD: %s, h_FD: %d %s\n", h_project_id,h_SD,h_FD_Ind,h_FD);
   //trace_printf("3.*** h_AS: %s, h_finish_date: %d %s\n", h_AS,h_finish_date_Ind,h_finish_date);
   
   					if(IsNotNullParam(h_finish_date))
   					{
   						if ( (IsNotNullParam(h_AS))  (strncmp(h_AS,h_FD,10) > 0) )
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0;
   							END_SQL;
   //trace_printf("1.ACTUAL *** DELETE: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_AS)) AND r.DAY_YEAR <= DATE(HOST(h_finish_date));
   							END_SQL;
   						}
   						else if ( strncmp(h_finish_date,h_FD,10) > 0)
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0 AND DAY_YEAR < DATE(HOST(h_start_date));
   							END_SQL;
   //trace_printf("1.3 *** DELETE h_finish_date > h_FD: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_FD)) AND r.DAY_YEAR <= DATE(HOST(h_finish_date));
   							END_SQL;
   						}
   						else if ( strncmp(h_finish_date,h_FD,10) < 0)
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0 AND DAY_YEAR > DATE(HOST(h_finish_date));
   							END_SQL;
   //trace_printf("1.4 *** DELETE h_finish_date < h_FD: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_start_date)) AND r.DAY_YEAR < DATE(HOST(h_SD));
   							END_SQL;
   //trace_printf("1.4 *** INSERT h_finish_date < h_FD: SC_STATE: %d\n", SC_STATE);
   
   						}
   					}
   
   					BEGIN_SQL
   						SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   						WHERE PROJECT_ID = HOST(h_project_id)
   							AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   							AND ACTUAL_FLAG = 0;
   					END_SQL_NOCHECK;
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.499);
   //trace_printf("0.2*** h_minutes: %d, h_remaining: %d, h_duration: %d\n", h_minutes,h_remaining,h_duration);
   
   					h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   											  SC_PARAM(h_tsk_assignment_id),
   											  SC_PARAM(h_start_date),
   											  SC_PARAM(h_per_day),
   											  SC_PARAM(h_minutes));
   
   					if (h_tph_delete == -1)
   					{
   						InitializeHost(SC_PARAM(h_tph_delete), 0);
   						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   					}
   
   					h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_29);
   					h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   				}
   			}
   		}
   		else
   		{
   			BEGIN_SQL
   				SELECT COUNT('X') INTO HOST(h_bitflag) FROM DUAL WHERE EXISTS (
   					SELECT 'X' FROM TASK_ASSIGNMENTS
   					WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
   						AND TSK_ASSIGNMENT_ID <> HOST(h_tsk_assignment_id) AND ACTUAL_START IS NOT NuLL);
   			END_SQL_NOCHECK;
   //trace_printf("1_*_FD: SC_STATE: %d, h_finish_date: %d %s, h_bitflag: %d\n", SC_STATE,h_finish_date_Ind,h_finish_date,h_bitflag);
   
   			if (h_bitflag == 0)
   			{
   				BEGIN_SQL
   					SELECT BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING
   					INTO HOST(h_start_date), HOST(h_finish_date), HOST(h_effort), HOST(h_assign_string)
   					FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
   				END_SQL_NOCHECK;
   
   //trace_printf("2_*_FD: h_effort %d, h_start_date %s, h_finish_date %s, h_assign_string: %s\n",h_effort, h_start_date, h_finish_date,h_assign_string);
   				h_bitflag = (1 | SC_BIT_28);
   
   				InitializeHost(SC_PARAM(h_temp_work_type), 2);
   				InitializeHost(SC_PARAM(h_temp_contour_type), 0);
   				InitializeHost(SC_PARAM(h_temp_occurrence), 0);
   				InitializeHost(SC_PARAM(h_temp_consider), 7);
   
   				if ( strncmp(h_start_date,h_SD,10) != 0)
   				{
   					BEGIN_SQL
   						SELECT MIN(BAK_START_DT), MAX(BAK_FINISH_DT) INTO HOST(h_ta_start_DT), HOST(h_ta_finish_DT)
   						FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id);
   					END_SQL_NOCHECK;
   //trace_printf("T1_*: h_ta_start_DT %s, h_ta_finish_DT %s\n",h_ta_start_DT, h_ta_finish_DT);
   //trace_printf("h_task_start_DT %s, h_task_finish_DT %s\n",h_task_start_DT, h_task_finish_DT);
   
   					h_temp_var = FC_CHANGE_TA_DATES (SC_PARAM(h_project_id),
   													 SC_PARAM(h_task_id),
   													 SC_PARAM(h_task_start_DT),
   													 SC_PARAM(h_task_finish_DT),
   													 SC_PARAM(h_ta_start_DT),
   													 SC_PARAM(h_ta_finish_DT),
   													 SC_PARAM(h_per_day),
   													 SC_PARAM(h_rec_user), h_task_type );
   				}
   
   				// Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
   				h_temp_var = U_TSKRES2(SC_PARAM(h_tsk_assignment_id),  SC_PARAM(h_project_id),         SC_PARAM(h_task_id),
   									 SC_PARAM(h_res_id),               SC_PARAM(h_active),             SC_PARAM(h_active),
   									 SC_PARAM(h_start_date),           SC_PARAM(h_finish_date),        SC_PARAM(h_effort),
   									 SC_PARAM(h_work),                 SC_PARAM(h_eunit),              SC_PARAM(h_unit),
   									 SC_PARAM(h_res_type),             SC_PARAM(h_temp_work_type),     SC_PARAM(h_bitflag),
   									 SC_PARAM(h_contour),              SC_PARAM(h_temp_contour_type),  SC_PARAM(h_temp_occurrence),
   									 SC_PARAM(h_temp_consider),        SC_PARAM(h_assign_string),      SC_PARAM(h_rec_user),
   									 SC_PARAM(h_per_day));
   
   				InitializeHost(SC_PARAM(h_temp_var), SC_BIT_30);
   				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   			}
   			else
   			{
   				BEGIN_SQL
   					SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   					WHERE PROJECT_ID = HOST(h_project_id)
   						AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   						AND ACTUAL_FLAG = 0;
   				END_SQL_NOCHECK;
   //trace_printf("3_*_FD: SC_STATE: %d, h_tsk_assignment_id: %s, h_duration: %d\n", SC_STATE,h_tsk_assignment_id,h_duration);
   
   				if ( h_duration > 0 )
   				{
   					InitializeHost(SC_PARAM(h_tph_delete), -1);
   					h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.499);
   					h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   											  SC_PARAM(h_tsk_assignment_id),
   											  SC_PARAM(h_start_date),
   											  SC_PARAM(h_per_day),
   											  SC_PARAM(h_minutes));
   
   					InitializeHost(SC_PARAM(h_tph_delete), 0);
   					h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   
   					h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
   					h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   				}
   			}
   		}
   
   		return 0;
   	}*/
   
   --trace_printf("h_tph_delete: %d, h_effort = %d, h_remaining: %d, h_etc: %d, h_work_todate: %d, h_start_date: %d %s\n", h_tph_delete,h_effort,h_remaining,h_etc,h_work_todate,h_start_date_Ind,h_start_date); trace_flush();
   --  if ( (h_work_todate != 0) || (h_type != 0) || (h_etc != h_remaining) )
   --  {
   If Coalesce(h_tph_delete, 0) = 0 Then
      h_tph_delete := -1;
      h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
      --trace_printf("TPHASE_1: h_temp_sc_state: %d\n",h_temp_sc_state);
   End If;
   --  }
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_start_date IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assignment_id
                                   , h_tsk_assignment_id, h_start_date, h_last_day, 0);
   End If;
   
   If Coalesce(h_etc, 0) = 0 Then
      If Coalesce(h_work_todate, 0) = 0 Then
         --Update SUMMARY_TIMESHEETS
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = 10000,
            FORECAST_START    = NULL,
            FORECAST_FINISH   = NULL,
            FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 919+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(920) : cplan_fun.sql(18740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_prj_assignment_id IS NOT NULL) Then
            h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
         ElsIf Coalesce(h_effort, 0) = 0 Then
            h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
         Else
            h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
         End If;
         
         BEGIN
            UPDATE PROJECT_ASSIGNMENTS SET ROLLUP_BITS = 1 WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 944+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(945) : cplan_fun.sql(18765) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 961+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(962) : cplan_fun.sql(18782) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM SUMMARY_TIMESHEETS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 978+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(979) : cplan_fun.sql(18799) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET SECURITY_ID = CASE WHEN SECURITY_ID = 'SCTEAMMEMBER' THEN 'SCPARTICIPANTS' ELSE SECURITY_ID END,
            PARTICIPANT = CASE WHEN SECURITY_ID = 'SCTEAMMEMBER' THEN PMO_CNST.SC_PARTICIPANT ELSE BITAND(PARTICIPANT, PMO_CNST.SC_CLEAR_TEAMMEMBER) END
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 997+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(998) : cplan_fun.sql(18818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
         
         If (h_prj_assignment_id IS NOT NULL) Then
            h_temp_var := 0;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
         End If;
         
         If Coalesce(h_tph_delete, 0) = -1 Then
            h_tph_delete := 0;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            --trace_printf("TPHASE_2: h_temp_sc_state: %d\n",h_temp_sc_state);
         End If;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1017+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1018) : cplan_fun.sql(18838) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      Else
         If (h_start_date IS NULL) Then
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_start_date FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1031+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1032) : cplan_fun.sql(18852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_temp_var := 0;
         h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort, h_work
                            , h_prj_assignment_id, h_temp_var);
         
         -- get week of
         
         OPEN CR_WOF;
         
         FETCH CR_WOF INTO h_day_year;
         IF CR_WOF%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               CLOSE CR_WOF ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1061+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1062) : cplan_fun.sql(18882) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               CLOSE CR_WOF ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1074+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1075) : cplan_fun.sql(18895) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --Update SUMMARY_TIMESHEETS
            BEGIN
               UPDATE SUMMARY_TIMESHEETS SET ETC = 0, EV_ETC = 0, MILESTONE_PERCENT = 10000, PDA_FLAG = 'P',
               TOTAL_SCHED_EFFORT = h_work_todate,
               EAC = ACWP,
               BASELINE_WORK_TODATE = h_work_todate,
               BCWP = BAC,
               REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF >= h_day_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1098+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1099) : cplan_fun.sql(18919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            /*              BEGIN_SQL
            					UPDATE TASK_ASSIGNMENTS SET REC_STATUS = 'U', REC_USER = HOST(h_rec_user), REC_DATETIME = SC_NOW
            					WHERE RESOURCE_ID = HOST(h_res_id) AND PROJECT_ID = HOST(h_project_id);
            				END_SQL;*/
         End If;
         
         h_temp_var := 1;
         -- it was 3
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
         
         If (h_prj_assignment_id IS NOT NULL) Then
            h_temp_var := 0;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
         End If;
         
         If Coalesce(h_tph_delete, 0) = -1 Then
            h_tph_delete := 0;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            --trace_printf("TPHASE_3: h_temp_sc_state: %d\n",h_temp_sc_state);
         End If;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1126+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1127) : cplan_fun.sql(18947) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   End If;
   
   If Coalesce(h_bitflag, 0) >= 90 Then
      If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) <> 0 Then
         h_bitflag := 2;
      Else
         h_bitflag := 1;
      End If;
   End If;
   
   --trace_printf("h_work_todate: %d, h_bitflag: %d, h_eunit: %d, h_unit: %d, h_contour: %d\n", h_work_todate, h_bitflag, h_eunit, h_unit, h_contour);
   --trace_printf("h_prj_assignment_id = %s, h_prj_assignment_id_Ind = %d\n",h_prj_assignment_id,h_prj_assignment_id_Ind);
   --trace_printf("h_type: %d, h_effort: %d, h_etc: %d, h_remaining: %d, h_start_date =  %d, %s\n",h_type,h_effort,h_etc,h_remaining,h_start_date_Ind,h_start_date);
   
   If (Coalesce(h_work_todate, 0) = 0) AND (Coalesce(h_type, 0) = 0) AND (Coalesce(
             h_etc, 0) = Coalesce(h_remaining, 0)) Then
      h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
      
      BEGIN
         SELECT BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING
         INTO h_start_date, h_finish_date, h_effort, h_assign_string
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1158+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1159) : cplan_fun.sql(18979) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("*** h_effort %d, h_start_date %s, h_finish_date %s, h_assign_string: %s\n",h_effort, h_start_date, h_finish_date,h_assign_string);
      
      If ((h_assign_string IS NOT NULL)) AND (Coalesce(Length(h_assign_string), 0) > 0) Then
         If (h_prj_assignment_id IS NOT NULL) Then
            h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_29_30));
         Else
            h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_28));
         End If;
         
         h_temp_work_type := 2;
         h_temp_contour_type := 0;
         h_temp_occurrence := 0;
         h_temp_consider := 7;
         
         -- Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
         PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_active
                               , h_active, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_res_type, h_temp_work_type, h_bitflag, h_contour
                               , h_temp_contour_type, h_temp_occurrence, h_temp_consider
                               , h_assign_string, h_rec_user, h_per_day, 0.00, 0.00);
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_resched := 1;
            h_tph_delete := 0;
         End If;
         
         h_start_date := NULL;
      Else
         h_finish_date := '';
         
         BEGIN
            SELECT PROPOSED_START,PROPOSED_WORK INTO h_start_date, h_effort
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1200+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1201) : cplan_fun.sql(19021) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("NO ACTUAL: SC_STATE = %d,h_start_date = %d:%s,h_effort: %d\n",SC_STATE,h_start_date_Ind,h_start_date,h_effort);
         
         h_accpt_aut := 3;
         h_skip := 0;
      End If;
   Else
      --trace_printf("0__ h_type = %d, h_effort = %d, h_etc = %d, h_work_todate = %d, h_remaining = %d\n", h_type,h_effort,h_etc,h_work_todate,h_remaining);
      If (h_start_date IS NULL) Then
         --strcpy (h_rec_user,"");
         h_assign_string := '';
         h_finish_date := '';
         h_res_type := '0';
         h_active := '';
         
         OPEN CFD;
         
         FETCH CFD INTO h_start_date;
         IF CFD%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("SC_STATE = %d,h_start_date = %d:%s,h_prj_assignment_id: %d:%s\n",SC_STATE,h_start_date_Ind,h_start_date,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               CLOSE CFD ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1237+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1238) : cplan_fun.sql(19058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
            Else
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
            End If;
            
            --trace_printf("1__ h_type = %d, h_effort = %d, h_project_id = %s, h_tsk_assignment_id = %s\n", h_type,h_effort,h_project_id,h_tsk_assignment_id);
            If Coalesce(h_etc, 0) > Coalesce(h_remaining, 0) Then
               h_effort := Coalesce(h_etc, 0);
            Else
               BEGIN
                  SELECT h_effort - COALESCE(SUM(SCHED_MIN),0) INTO h_effort
                  FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1261+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1262) : cplan_fun.sql(19082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               CLOSE CFD ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1276) : cplan_fun.sql(19096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
            
            BEGIN
               SELECT BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING
               INTO h_start_date, h_finish_date, h_effort, h_assign_string
               FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1292+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1293) : cplan_fun.sql(19113) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("*** h_effort %d, h_start_date %s, h_finish_date %s\n",h_effort, h_start_date, h_finish_date);
            
            If ((h_assign_string IS NOT NULL)) AND (Coalesce(Length(h_assign_string), 0) > 0) Then
               If (h_prj_assignment_id IS NOT NULL) Then
                  h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_29_30));
               Else
                  h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_27));
               End If;
               
               h_temp_work_type := 2;
               h_temp_contour_type := 0;
               h_temp_occurrence := 0;
               h_temp_consider := 7;
               
               -- Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
               PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_active
                               , h_active, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_res_type, h_temp_work_type, h_bitflag, h_contour
                               , h_temp_contour_type, h_temp_occurrence, h_temp_consider
                               , h_assign_string, h_rec_user, h_per_day, 0.00, 0.00);
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_resched := 3;
               End If;
               
               h_start_date := NULL;
            Else
               h_finish_date := '';
            End If;
         End If;
      Else
         h_start_date := SubStr(h_start_date, 1, 10);
         If (PMO_ORASTD.strncmp(h_start_date, h_SD, 10) < 0) OR (Coalesce(h_contour, 0) > 0) Then
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
            Else
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
            End If;
            
            h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
         Else
            If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 0) Then
               h_skip := Coalesce(h_work, 0);
            Else
               h_skip := 0;
            End If;
            
            --trace_printf("A__* h_prj_assignment_id: %d %s\n", h_prj_assignment_id_Ind,h_prj_assignment_id);
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_var := 0;
               h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort, h_skip
                            , h_prj_assignment_id, h_temp_var);
            Else
               h_temp_var := -2;
               h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort, h_skip
                            , h_res_id, h_temp_var);
            End If;
            
            If Coalesce(h_skip, 0) = 1 Then
               h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
            End If;
         End If;
      End If;
      
      BEGIN
         UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 1 AND SCHED_MIN <> ACTUAL_MIN;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1372+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1373) : cplan_fun.sql(19193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT h_effort - COALESCE(SUM(SCHED_MIN),0) INTO h_effort
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1386+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1387) : cplan_fun.sql(19207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("*** h_effort = %d, h_type: %d, h_accpt_aut: %d, h_skip = %d, h_contour = %d, h_start_date = %d, %s\n", h_effort,h_type,h_accpt_aut,h_skip,h_contour,h_start_date_Ind,h_start_date);
   
   If ((h_start_date IS NOT NULL)) AND (Coalesce(h_effort, 0) > 0) Then
      h_week_of := NULL;
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1405+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1406) : cplan_fun.sql(19226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("TIMESHEET_APPROVAL_STATUS: SC_STATE: %d, h_week_of: %d %s\n", SC_STATE,h_week_of_Ind,h_week_of);//trace_flush();
      
      If (h_week_of IS NOT NULL) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1420+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1421) : cplan_fun.sql(19241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strncmp(h_start_date, h_week_of, 10) < 0 Then
            h_start_date := h_week_of;
            
            If Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30 Then
               h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
               -- remove don't replace with profile if actual
            End If;
         End If;
         --trace_printf("*** h_start_date: %s\n", h_start_date);//trace_flush();
      End If;
      
      If Coalesce(h_type, 0) = 1 Then
         --trace_printf("h_start_date = %s, h_start_DT = %s, h_start_DT_Ind = %d\n", h_start_date,h_start_DT,h_start_DT_Ind);
         If ((h_start_DT IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) < 0) Then
            If Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30 Then
               h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
               -- remove don't replace with profile if actual
            End If;
            
            h_temp_var := 2;
            h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_DT, h_effort, h_temp_var, h_contour
                                 , h_res_id, 0, Coalesce(h_percent_units, 0));
            --trace_printf("1_FC_RESCHEDULE: h_temp_sc_state: %d\n",h_temp_sc_state);
            If Coalesce(h_temp_sc_state, 0) = 0 Then
               h_resched := 1;
            End If;
         Else
            h_temp_var := 2;
            h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_res_id, 0, Coalesce(h_percent_units, 0));
            --trace_printf("2_FC_RESCHEDULE: h_temp_sc_state: %d\n",h_temp_sc_state);
            h_resched := 1;
         End If;
      Else
         --H-remaining contains the changes value of etc, but h_etc = h_effort - h_work_todate
         -- accept/keep etc is controlled in the function FC_RESCHEDULE: accept less etc only.
         h_temp_var := 2;
         h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_res_id, 0, Coalesce(h_percent_units, 0));
         --trace_printf("3_FC_RESCHEDULE: h_temp_sc_state: %d\n",h_temp_sc_state);
         
         If Coalesce(h_accpt_aut, 0) = 2 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET BAK_ASSIGN_STRING = ASSIGN_STRING,
               PROPOSED_START = SCHED_START, PROPOSED_FINISH = SCHED_FINISH, PROPOSED_WORK = SCHED_WORK,
               BAK_START_DT = SCHED_START, BAK_FINISH_DT = SCHED_FINISH, BAK_WORK = SCHED_WORK, BEST = SCHED_WORK
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1482+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1483) : cplan_fun.sql(19303) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_accpt_aut));
         h_resched := 1;
      End If;
   Else
      h_resched := 1;
      If Coalesce(h_type, 0) = 1 Then
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
      Else
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_accpt_aut, 0, 0);
      End If;
      
      --trace_printf("3_* h_tph_delete: %d, h_billable: %d, h_capital_expense =  %d, h_SD: %s, h_FD: %s\n",h_tph_delete,h_billable,h_capital_expense, h_SD, h_FD);
      If Coalesce(h_tph_delete, 0) = -1 Then
         h_tph_delete := 0;
         h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
      End If;
   End If;
   
   If ((h_prj_assignment_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_prj_assignment_id
         , h_tsk_assignment_id) <> 0) Then
      --trace_printf("3_*** U_ASSIGN_STRING, h_temp_sc_state: %d\n",h_temp_sc_state);
      --Tudor ??? here we need to check if h_temp_state contains error number then return error, otherwise continue
      If Coalesce(h_temp_sc_state, 0) > 100 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1514+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1515) : cplan_fun.sql(19335) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      Else
         h_temp_var := 0;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
      End If;
   End If;
   
   If Coalesce(h_resched, 0) = 0 Then
      PMO_DB2M.SC_STATE := 110;
   Else
      PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   End If;
   --trace_printf("END__FC_ETC_SCHEDULE: SC_STATE: %d, h_resched: %d\n",SC_STATE,h_resched);//trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1531+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1532) : cplan_fun.sql(19352) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ETC_SCHEDULE;
ALTER FUNCTION FC_ETC_SCHEDULE COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_D_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_D_ACTUAL
(
     sh_h_table_type PLS_INTEGER
   , sh_h_type_id PLS_INTEGER
   , h_assign_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_tsk_prj PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_profile_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_donot_replace PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_bitflag1 PLS_INTEGER
   , sh_h_min_per_day PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_tsk_prj PLS_INTEGER := sh_h_tsk_prj;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_donot_replace PLS_INTEGER := sh_h_donot_replace;
   h_effort PLS_INTEGER := sh_h_effort;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_work PLS_INTEGER := sh_h_work;
   h_unit PLS_INTEGER := sh_h_unit;
   h_bitflag1 PLS_INTEGER := sh_h_bitflag1;
   h_min_per_day PLS_INTEGER := sh_h_min_per_day;
   h_priority PLS_INTEGER := sh_h_priority;
   h_contour PLS_INTEGER := sh_h_contour;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_uniqueID VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_day_year DATE;
   h_dayofyear DATE;
   h_week_of DATE;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_res_type VARCHAR2(1);
   h_prj_assignment_id VARCHAR2(32);
   h_duration PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_temp_every PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mls_percent PLS_INTEGER := 0;
   h_reg_sale NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_cost NUMBER := 0;
   h_temp_cri PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_work_type PLS_INTEGER := 0;
   h_temp_update PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_pa_units PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   i_return PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER;
   j PLS_INTEGER;
   ii PLS_INTEGER;
   i_decommission PLS_INTEGER NOT NULL := 0;
   i_continue PLS_INTEGER NOT NULL := 0;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CWOF IS
         SELECT * FROM (SELECT WEEK_OF FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF <= h_day_year
         ORDER BY WEEK_OF DESC ) WHERE ROWNUM<=1;

      CURSOR CCALS IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR > h_day_year
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_DACT1 IS
         SELECT ta.DAY_YEAR, ta.SCHED_MIN, ta.BIT_SCHED
         FROM TASK_DAY_OF_YEAR ta JOIN TASK_DAY_OF_YEAR pa ON ta.PROJECT_ID = pa.PROJECT_ID AND ta.PARENT_ID = pa.TSK_ASSIGNMENT_ID
         WHERE ta.PROJECT_ID = h_project_id AND ta.TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ta.DAY_YEAR > h_day_year;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_D_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_D_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_D_ACTUAL(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_assign_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_tsk_prj), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_profile_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_donot_replace), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_min_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 126+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_D_ACTUAL(127) : cplan_fun.sql(19489) : ' || SYSTIMESTAMP);
   END IF; 

   h_temp_var := 1;
   h_mls_percent := 1;
   h_finish_DT := '';
   h_res_assignment_id := '';
   h_dayofyear := '';
   --trace_printf("h_tsk_assignment_id: %d:%s, h_donot_replace = %d, h_bitflag = %d, h_bitflag1 = %d, h_effort = %d, h_work = %d\n", h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_donot_replace, h_bitflag, h_bitflag1, h_effort,h_work);
   
   -- get last submitted date
   BEGIN
      SELECT PROJECT_ASSIGNMENT_ID, TO_DATE(SubStr(ACTUAL_START, 1, 10)), LAST_SUBMITTED, WORK_TODATE, MILESTONE_PERCENT,
      LAST_SCHED_MIN, PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, UNITS_MSP
      INTO h_prj_assignment_id, h_dayofyear, h_day_year,h_work_todate, h_mls_percent,
      h_last_day, h_prj_assign_id, h_billable, h_capital_expense, h_percent_units
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 149+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(150) : cplan_fun.sql(19512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_day_year: %s, h_prj_assignment_id: %d:%s, h_mls_percent: %d\n", SC_STATE,h_day_year,h_prj_assignment_id_Ind,h_prj_assignment_id,h_mls_percent); trace_flush();
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 157+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(158) : cplan_fun.sql(19520) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27 Then
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
      i_decommission := 1;
      -- keep assignments with actual and remove the remaining
   End If;
   
   BEGIN
      SELECT PROF_NUMBER INTO h_temp_update FROM PROFILES
      WHERE PROJECT_ID = h_project_id AND PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 176+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(177) : cplan_fun.sql(19539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      h_res_type := '1';
   Else
      h_res_type := '0';
   End If;
   --trace_printf("SC_STATE: %d, h_res_type: %s, h_type_id: %d\n", SC_STATE,h_res_type,h_type_id); trace_flush();
   
   h_temp_sc_state := 0;
   If Coalesce(h_donot_replace, 0) = 0 Then
      If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
         h_temp_security_id := 'SCPROJECTASSIGNMENT';
         h_temp_participant := PMO_CNST.SC_AFFECTED;
      Else
         h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
         h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
      End If;
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         If Coalesce(h_effort, 0) > 0 Then
            --Get a unique id
            h_uniqueID := SYS_GUID();
            h_temp_bitflag := 1;
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_profile_id, h_tsk_prj, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,
               RESOURCE_TYPE, PROFILE_ID, PARTICIPANT, SECURITY_ID, REC_STATUS, REC_DATETIME)
               SELECT h_uniqueID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               '1', h_profile_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, 'I',SYSTIMESTAMP
               FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 223+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(224) : cplan_fun.sql(19586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 230+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(231) : cplan_fun.sql(19593) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_DB2M.SC_STATE;
            End If;
         End If;
      Else
         h_temp_sc_state := FC_MERGE_ASSIGN(h_bitflag, h_project_id, h_task_id, h_profile_id, h_assign_id
                                   , h_res_assignment_id, h_rec_user, h_effort, 1);
         --trace_printf("MERGE_ASSIGN_* h_temp_sc_state: %d, h_assign_id: %d:%s\n", h_temp_sc_state,h_assign_id_Ind,h_assign_id); trace_flush();
         
         If Coalesce(h_temp_sc_state, 0) = 200 Then
            --Get a unique id
            h_uniqueID := SYS_GUID();
            h_temp_bitflag := 1;
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_profile_id, h_tsk_prj, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_STATUS, REC_DATETIME)
               SELECT h_uniqueID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               '0', h_profile_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, 'I',SYSTIMESTAMP
               FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 263+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(264) : cplan_fun.sql(19626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 270+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(271) : cplan_fun.sql(19633) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_DB2M.SC_STATE;
            End If;
         Else
            i_continue := 1;
         End If;
      End If;
      --trace_printf("SC_STATE: %d, h_effort: %d\n", SC_STATE,h_effort); trace_flush();
   End If;
   
   -- get week of
   
   OPEN CWOF;
   
   FETCH CWOF INTO h_week_of;
   IF CWOF%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   --trace_printf("SC_STATE: %d, h_week_of: %s, h_day_year: %s\n",SC_STATE,h_week_of,h_day_year);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         CLOSE CWOF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 302+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(303) : cplan_fun.sql(19665) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         CLOSE CWOF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 315+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(316) : cplan_fun.sql(19678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Update SUMMARY_TIMESHEETS
      BEGIN
         UPDATE SUMMARY_TIMESHEETS SET ETC = 0, EV_ETC = 0, MILESTONE_PERCENT = 10000, PDA_FLAG = 'P',
         TOTAL_SCHED_EFFORT = h_work_todate,
         EAC = ACWP,
         BASELINE_WORK_TODATE = h_work_todate,
         BCWP = BAC,
         REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF >= h_week_of;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 339+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(340) : cplan_fun.sql(19702) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*      BEGIN_SQL
      			UPDATE TASK_ASSIGNMENTS SET REC_STATUS = 'U', REC_USER = HOST(h_rec_user), REC_DATETIME = SC_NOW
      			WHERE RESOURCE_ID = HOST(h_res_id) AND PROJECT_ID = HOST(h_project_id);
      		END_SQL;*/
   End If;
   
   --trace_printf("*** i_continue: %d, h_day_year = %s\n", i_continue,h_day_year);
   If i_continue = 1 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 353+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(354) : cplan_fun.sql(19716) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_day_year IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_day_year, h_last_day, 0);
   End If;
   
   If (h_prj_assignment_id IS NOT NULL) AND i_decommission = 0 AND Coalesce(h_bitflag, 0) <> 2 
         AND Coalesce(h_bitflag, 0) <> 1 Then
      If Coalesce(h_donot_replace, 0) = 0 Then
         ora_temp_sc_state := PMO_DB2M.SC_STATE;
         null_return := FC_U_PA_WORK(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, Coalesce(
                            h_min_per_day, 0), 1);
         PMO_DB2M.SC_STATE := ora_temp_sc_state;
      Else
         ora_temp_sc_state := PMO_DB2M.SC_STATE;
         null_return := FC_U_PA_WORK(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, Coalesce(
                            h_min_per_day, 0), 0);
         PMO_DB2M.SC_STATE := ora_temp_sc_state;
      End If;
   End If;
   
   BEGIN
      DELETE FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
      AND ACTUAL_FLAG = 0;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 393+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(394) : cplan_fun.sql(19756) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("DELETE: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
   
   -- get finish datetime
   BEGIN
      SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED)) INTO h_finish_date
      FROM TASK_DAY_OF_YEAR WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_day_year;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 409+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(410) : cplan_fun.sql(19772) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_day_year = %s, h_finish_date = %s\n", SC_STATE,h_day_year,h_finish_date);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET ETC = 0, FD = PMO_CNST.SC_AF, ACTUAL_FINISH = h_finish_date,
         MILESTONE_PERCENT = 10000, SCHED_WORK = WORK_TODATE, SCHED_FINISH = h_finish_date,
         PROPOSED_START = SCHED_START, PROPOSED_WORK = WORK_TODATE, PROPOSED_FINISH = h_finish_date,
         FORECAST_START = NULL, FORECAST_FINISH = NULL, FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 432+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(433) : cplan_fun.sql(19795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
      
      -- CALL UPDATE assign string function
      h_temp_var := 3;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(
                                   h_min_per_day, 0), 0);
      
      If (h_prj_assign_id IS NOT NULL) Then
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assign_id, h_temp_var, Coalesce(
                                   h_min_per_day, 0), 0);
      End If;
   End If;
   
   If (SubStr(h_res_type, 1, 1) = '1') OR (Coalesce(h_donot_replace, 0) = 0) Then
      If (Coalesce(h_donot_replace, 0) = 0) AND (Coalesce(h_effort, 0) > 0) Then
         OPEN CCALS;
         
         FETCH CCALS INTO h_start_DT;
         IF CCALS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CCALS ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 469+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(470) : cplan_fun.sql(19832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_duration := Trunc(Coalesce(h_effort, 0) / Coalesce(h_min_per_day, 0));
         
         If Coalesce(h_contour, 0) = 0 Then
            h_work := 0;
         ElsIf Coalesce(h_work, 0) > 0 Then
            h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0001F000)), 12);
            If Coalesce(h_every, 0) > 0 Then
               If ((Coalesce(h_bitflag1, 0) = 2) OR (Coalesce(h_bitflag1, 0) = 5)) AND ((BITAND(Coalesce(
                     h_contour, 0), PMO_CNST.SC_0000007F)) > 0) Then
                  i := 1;
                  ii := 7;
                  h_days := 0;
                  WHILE ii > 0 LOOP
                     If (BITAND(Coalesce(h_contour, 0), i)) = i Then
                        h_days := Coalesce(h_days, 0) + 1;
                     End If;
                     i := PMO_FUNC.ShiftLeft(i, 1);
                     ii := ii - 1;
                  END LOOP;
               Else
                  h_days := 1;
               End If;
               
               h_duration := Coalesce(h_duration, 0) + 1;
               
               If Coalesce(h_bitflag1, 0) = 1 Then
                  h_duration := Coalesce(h_duration, 0) * Coalesce(h_every, 0);
               ElsIf Coalesce(h_bitflag1, 0) = 2 Then
                  h_duration := Coalesce(h_duration, 0) * (Trunc((Coalesce(h_every, 0) * 7) / Coalesce(h_days, 0)));
               ElsIf Coalesce(h_bitflag1, 0) < 6 Then
                  h_duration := Trunc(Coalesce(h_duration, 0) * ((Coalesce(h_every, 0) * 31) / (4.5 * Coalesce(
                             h_days, 0))));
               End If;
            Else
               h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_1FFE0000)), 17);
               -- July, August, Decembre
               
               If Coalesce(h_every, 0) > 0 Then
                  i := 0;
                  j := 0;
                  WHILE j < 12 LOOP
                     h_temp_every := (BITAND(Coalesce(h_every, 0), 1));
                     If Coalesce(h_temp_every, 0) <> 0 Then
                        i := i + 1;
                     End If;
                     
                     h_every := (PMO_FUNC.ShiftRight(Coalesce(h_every, 0), 1));
                     
                     j := j + 1;
                  END LOOP;
                  
                  h_duration := Trunc(Coalesce(h_duration, 0) / i) + 1;
                  h_duration := Coalesce(h_duration, 0) * 365;
               End If;
            End If;
         End If;
         
         If Coalesce(h_duration, 0) < 30 Then
            h_duration := 50;
         ElsIf Coalesce(h_duration, 0) > 800 Then
            h_duration := 1200;
         Else
            h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
         End If;
         
         --trace_printf("1__ h_duration = %d, h_start_DT = %s, h_finish_DT = %s\n", h_duration,h_start_DT,h_finish_DT);
         -- Check for project calendar when no finish date
         If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 543+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(544) : cplan_fun.sql(19906) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
         --trace_printf("2__ h_duration = %d, h_start_DT = %s, h_finish_DT = %s\n", h_duration,h_start_DT,h_finish_DT);
         
         --      h_bitflag1 = h_bitflag1 | SC_BIT_28;
         --trace_printf("***: h_res_type: %s, h_profile_id: %s, h_bitflag1: %d, h_effort: %d, h_tsk_prj: %d\n", h_res_type,h_profile_id,h_bitflag1,h_effort,h_tsk_prj);
         
         h_temp_res_cost := 0.00;
         h_temp_cri := 0;
         
         h_temp_work_type := 0;
         h_temp_update := 0;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_temp_consider := 12;
         Else
            h_temp_consider := 0;
         End If;
         
         If Coalesce(h_work, 0) = 0 Then
            h_work := Coalesce(h_percent_units, 0);
            h_unit := 100;
         End If;
         
         -- get resource selling rates
         BEGIN
            SELECT REGULAR_RATE, SPECIAL_RATE INTO h_reg_sale, h_spec_sale
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 579+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(580) : cplan_fun.sql(19942) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s, h_reg_sale = %.4f, h_spec_sale = %.4f\n",SC_STATE, h_tsk_assignment_id, h_reg_sale, h_spec_sale);
         
         -- update profile selling rates
         /*BEGIN_SQL
         				UPDATE TASK_ASSIGNMENTS SET REGULAR_RATE = HOST(h_reg_sale), SPECIAL_RATE = HOST(h_spec_sale)
         				WHERE RES_ASSIGNMENT_ID = HOST(h_uniqueID);
         			END_SQL;*/
         
         i_return := FC_SEARCH_2(h_uniqueID, h_project_id, h_task_id, h_profile_id, h_temp_res_cost
                        , h_tsk_prj, h_finish_DT, h_finish_DT, h_finish_DT, h_finish_DT, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_temp_cri
                        , h_temp_consider, h_res_type, h_temp_work_type, h_bitflag1, h_contour
                        , h_occurrence, h_temp_update, h_min_per_day, h_priority, h_rec_user, 0, 0
                        , Coalesce(h_reg_sale, 0), Coalesce(h_spec_sale, 0));
         --trace_printf("***: i_return = %d, h_donot_replace = %d, h_tsk_assignment_id = %s, h_day_year = %s\n", i_return,h_donot_replace,h_tsk_assignment_id,h_day_year);
         
         PMO_DB2M.SC_STATE := i_return;
      Else
         If Coalesce(h_donot_replace, 0) = 2 Then
            If Coalesce(h_bitflag, 0) = 3 Then
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR > h_day_year;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 616+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(617) : cplan_fun.sql(19979) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               OPEN C_DACT1;
               
               LOOP
                  FETCH C_DACT1 INTO h_dayofyear,h_temp_var,h_bitID;
                  IF C_DACT1%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
                  --trace_printf("h_dayofyear: %s, h_temp_var: %d\n", h_dayofyear,h_temp_var);
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_temp_var, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 648+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(649) : cplan_fun.sql(20011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 666+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(667) : cplan_fun.sql(20029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE C_DACT1 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 680+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(681) : cplan_fun.sql(20043) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR > h_day_year;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 698+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(699) : cplan_fun.sql(20061) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   End If;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 712+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(713) : cplan_fun.sql(20075) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_D_ACTUAL;
ALTER FUNCTION FC_D_ACTUAL COMPILE DEBUG;

/**********************************************************/

Prompt == FUNCTION FC_U_RATES ==
CREATE OR REPLACE FUNCTION FC_U_RATES
(
     h_id IN OUT NOCOPY VARCHAR2
   , sh_h_reg_cost NUMBER
   , sh_h_spec_cost NUMBER
   , sh_h_reg_rate NUMBER
   , sh_h_spec_rate NUMBER
   , sh_h_rate_adjustment PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , sh_h_flag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_spec_cost NUMBER := sh_h_spec_cost;
   h_reg_rate NUMBER := sh_h_reg_rate;
   h_spec_rate NUMBER := sh_h_spec_rate;
   h_rate_adjustment PLS_INTEGER := sh_h_rate_adjustment;
   h_flag PLS_INTEGER := sh_h_flag;
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_pa_timephase VARCHAR2(1);
   h_planned_effort PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_rates_edited PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_old_reg_cost NUMBER := 0;
   h_old_reg_sale NUMBER := 0;
   h_old_spec_cost NUMBER := 0;
   h_old_spec_rate NUMBER := 0;
   i_pa PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_RATES'), '-- ' || SYSTIMESTAMP, ':r := FC_U_RATES(' || 
       '''' || h_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_spec_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_rate_adjustment), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_flag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_RATES', 50+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_RATES(51) : cplan_fun.sql(20136) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT PROJECT_ID, TASK_ID, RESOURCE_ID,
      PLANNED_WORK, SCHED_WORK, TO_DATE(SubStr(PROPOSED_START, 1, 10)), TO_DATE(SubStr(PROPOSED_FINISH, 1, 10)),
      BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE,
      SPECIAL_COST, SPECIAL_RATE, RATES_EDITED
      INTO   h_project_id, h_task_id, h_resource_id,
      h_planned_effort, h_sched_effort, h_SD, h_FD,
      h_billable, h_capital_expense, h_old_reg_cost, h_old_reg_sale ,
      h_old_spec_cost,h_old_spec_rate,h_rates_edited
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID=h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 69+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(70) : cplan_fun.sql(20155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If Coalesce(h_flag, 0) > 0 Then
         If (Coalesce(h_old_reg_cost, 0) <> Coalesce(h_reg_cost, 0)) OR (Coalesce(
                                            h_old_spec_cost, 0) <> Coalesce(h_spec_cost, 0)) Then
            h_rates_edited := (PMO_FUNC.BIT_OR(Coalesce(h_rates_edited, 0), 1));
         End If;
         If (Coalesce(h_old_reg_sale, 0) <> Coalesce(h_reg_rate, 0)) OR (Coalesce(
                                            h_old_spec_rate, 0) <> Coalesce(h_spec_rate, 0)) Then
            h_rates_edited := (PMO_FUNC.BIT_OR(Coalesce(h_rates_edited, 0), 2));
         End If;
      End If;
      
      h_tph_delete := -1;
      i_pa := PMO_ORASTD.strcmp(h_project_id, h_task_id);
      --trace_printf("0_* i_pa: %d:, h_task_id: %s\n", i_pa,h_task_id);
      If i_pa = 0 Then
         BEGIN
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 98+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(99) : cplan_fun.sql(20184) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);
         
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD
                                            , h_old_reg_cost, h_old_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD
                                            , h_old_reg_cost, h_old_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
      End If;
      
      --Execute update Statement
      If Coalesce(h_billable, 0) = 0 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET REGULAR_COST = h_reg_cost,
            SPECIAL_COST = h_spec_cost,
            REGULAR_RATE = h_reg_rate,
            SPECIAL_RATE = h_spec_rate,
            RATE_ADJUSTMENT = h_rate_adjustment,
            RATES_EDITED = h_rates_edited,
            SCHED_LABOR_COST = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            PLANNED_LABOR_COST = (PLANNED_WORK/60.00) * h_reg_cost,
            FORECAST_LABOR_COST = (ETC/60.00) * h_reg_cost + ACTUAL_LABOR_COST
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 139+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(140) : cplan_fun.sql(20225) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET REGULAR_COST = h_reg_cost,
            SPECIAL_COST = h_spec_cost,
            REGULAR_RATE = h_reg_rate,
            SPECIAL_RATE = h_spec_rate,
            RATE_ADJUSTMENT = h_rate_adjustment,
            RATES_EDITED = h_rates_edited,
            SCHED_LABOR_COST = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            PLANNED_LABOR_COST = (PLANNED_WORK/60.00) * h_reg_cost,
            SCHED_LABOR_REVENUE = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_rate + ACTUAL_LABOR_REVENUE,
            PLANNED_LABOR_REVENUE = (PLANNED_WORK/60.00) * h_reg_rate,
            FORECAST_LABOR_COST = (ETC/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            FORECAST_LABOR_REVENUE = (ETC/60.00) * h_reg_rate + ACTUAL_LABOR_REVENUE
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(170) : cplan_fun.sql(20255) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("2_* i_pa: %d, h_pa_timephase: %s\n", i_pa,h_pa_timephase);
      
      h_tph_delete := 0;
      If i_pa = 0 Then
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                            , h_reg_rate, h_tph_delete, h_capital_expense
                                            , h_billable);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                            , h_reg_rate, h_tph_delete, h_capital_expense
                                            , h_billable);
      End If;
      
      If i_pa = 0 Then
         If Coalesce(h_billable, 0) = 0 Then
            If Coalesce(h_planned_effort, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_SCHED_COST = (h_sched_effort/60.00) * h_reg_cost
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 206+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(207) : cplan_fun.sql(20292) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_COST = (h_planned_effort/60.00) * h_reg_cost
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 225+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(226) : cplan_fun.sql(20311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_planned_effort, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_SCHED_COST = (h_sched_effort/60.00) * h_reg_cost,
                  ESTIMATED_SCHED_REVENUE = (h_sched_effort/60.00) * h_reg_rate
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 247+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(248) : cplan_fun.sql(20333) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_COST = (h_planned_effort/60.00) * h_reg_cost,
                  ESTIMATED_PLANNED_REVENUE = (h_planned_effort/60.00) * h_reg_rate
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 267+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(268) : cplan_fun.sql(20353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      --trace_printf("SC_STATE: %d, h_planned_effort: %d, h_old_reg_cost: %.2f\n", SC_STATE,h_planned_effort,h_old_reg_cost);
      --trace_printf("h_project_id: %s, h_resource_id: %s\n", h_project_id,h_resource_id);
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASSIGNMENT_BIT) WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 289+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(290) : cplan_fun.sql(20375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Rollup Dates
      h_temp_sc_state := FC_Rollup(h_project_id);
      -- New Rollup
   Else
      --Execute update Statement
      BEGIN
         UPDATE PROJECT_ASSIGNMENTS
         SET REGULAR_COST = h_reg_cost,
         SPECIAL_COST = h_spec_cost,
         REGULAR_RATE = h_reg_rate,
         SPECIAL_RATE = h_spec_rate
         WHERE PROJECT_ID = h_res_id AND RESOURCE_ID = h_id; -- Resource ID (h_id) is element id (unique) in the broker;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 316+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(317) : cplan_fun.sql(20402) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_RATES', 323+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_RATES(324) : cplan_fun.sql(20409) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_RATES;
ALTER FUNCTION FC_U_RATES COMPILE DEBUG;

